Package: RESI
Type: Package
Title: Robust Effect Size Index (RESI) Estimation
Version: 1.3.2
Authors@R: 
  c(person('Megan', 'Jones', email='megan.n.taylor@vanderbilt.edu', role=c('aut', 'cre')),
  person('Kaidi', 'Kang', email = 'kaidi.kang@vanderbilt.edu', role = c('aut')),
  person('Simon', 'Vandekar', email='simon.vandekar@vanderbilt.edu', role=c('aut')),
  person('Gina', 'Yu', email='gina.l.yu@vanderbilt.edu', role=c('ctb')),
  person('Xinyu', 'Zhang', email='xinyu.zhang@vanderbilt.edu', role=c('ctb')))
Description: Summarize model output using a robust effect size index. The index is introduced in Vandekar, Tao, & Blume (2020, <doi:10.1007/s11336-020-09698-2>). Software paper available at <doi:10.18637/jss.v112.i03>.
License: GPL-3
Encoding: UTF-8
LazyData: true
Collate: 'internal_functions.R' 'summary.resi.R' 'resi.R' 'resi_pe.R'
        'Anova.resi.R' 'data.R' 'chisq2S.R' 'f2S.R' 't2S.R' 'z2S.R'
        'Rsq2S.R' 'S2Rsq.R' 'S2d.R' 'S2fsq.R' 'S2z.R' 'S2chisq.R'
        'd2S.R' 'fsq2S.R' 'plot.R' 'ggplot.R' 'print.R' 'omnibus.R'
Imports: aod, boot, car, clubSandwich, ggplot2, lmtest, nlme, sandwich
Suggests: gee, geepack, glmmTMB, glmtoolbox, knitr, lme4, pscl, R.rsp,
        regtools, rmarkdown, splines, survival, tibble, testthat (>=
        3.0.0)
RoxygenNote: 7.3.2
Depends: R (>= 2.10)
Config/testthat/edition: 3
URL: https://statimagcoll.github.io/RESI/,
        https://github.com/statimagcoll/RESI
VignetteBuilder: R.rsp
BugReports: https://github.com/statimagcoll/RESI/issues
NeedsCompilation: no
Packaged: 2025-07-28 17:12:35 UTC; megan
Author: Megan Jones [aut, cre],
  Kaidi Kang [aut],
  Simon Vandekar [aut],
  Gina Yu [ctb],
  Xinyu Zhang [ctb]
Maintainer: Megan Jones <megan.n.taylor@vanderbilt.edu>
Repository: CRAN
Date/Publication: 2025-07-29 09:20:21 UTC
Built: R 4.6.0; ; 2026-04-25 14:05:41 UTC; unix

Package: RESI
Type: Package
Title: Robust Effect Size Index (RESI) Estimation
Version: 1.4.2
Authors@R: 
  c(person('Megan', 'Jones', email='megan.n.taylor@vanderbilt.edu', role=c('aut')),
  person('Kaidi', 'Kang', email = 'kaidi.kang@vanderbilt.edu', role = c('aut')),
  person('Simon', 'Vandekar', email='simon.vandekar@vumc.org', role=c('aut', 'cre')),
  person('Gina', 'Yu', email='gina.l.yu@vanderbilt.edu', role=c('ctb')),
  person('Xinyu', 'Zhang', email='xinyu.zhang@vanderbilt.edu', role=c('ctb')))
Description: Summarize model output using a robust effect size index. The index is introduced in Vandekar, Tao, & Blume (2020, <doi:10.1007/s11336-020-09698-2>). Software paper available at <doi:10.18637/jss.v112.i03>.
License: GPL-3
Encoding: UTF-8
LazyData: true
Collate: 'internal_functions.R' 'summary.resi.R' 'resi.R' 'resi_pe.R'
        'Anova.resi.R' 'data.R' 'chisq2S.R' 'f2S.R' 't2S.R' 'z2S.R'
        'Rsq2S.R' 'S2Rsq.R' 'S2d.R' 'S2fsq.R' 'S2z.R' 'S2chisq.R'
        'd2S.R' 'fsq2S.R' 'plot.R' 'ggplot.R' 'print.R' 'omnibus.R'
        'simulations.R' 'resi_asymptotic.R'
Imports: aod, boot, car, clubSandwich, CompQuadForm, ggplot2, lmtest,
        nlme, parallel, sandwich, splines
Suggests: gee, geepack, robustbase, glmmTMB, glmtoolbox, knitr, lme4,
        pscl, R.rsp, regtools, rmarkdown, survival, tibble, testthat
        (>= 3.0.0)
RoxygenNote: 7.3.3
Depends: R (>= 2.10)
Config/testthat/edition: 3
URL: https://statimagcoll.github.io/RESI/,
        https://github.com/statimagcoll/RESI
BugReports: https://github.com/statimagcoll/RESI/issues
NeedsCompilation: no
Packaged: 2026-07-24 15:54:48 UTC; simon
Author: Megan Jones [aut],
  Kaidi Kang [aut],
  Simon Vandekar [aut, cre],
  Gina Yu [ctb],
  Xinyu Zhang [ctb]
Maintainer: Simon Vandekar <simon.vandekar@vumc.org>
Repository: CRAN
Date/Publication: 2026-07-24 21:20:08 UTC

# Generated by roxygen2: do not edit by hand

S3method(Anova,resi)
S3method(anova,resi)
S3method(ggplot,anova_resi)
S3method(ggplot,resi)
S3method(ggplot,summary_resi)
S3method(plot,anova_resi)
S3method(plot,resi)
S3method(plot,summary_resi)
S3method(print,omnibus_resi)
S3method(print,resi)
S3method(print,summary_resi)
S3method(resi,coxph)
S3method(resi,default)
S3method(resi,gee)
S3method(resi,geeglm)
S3method(resi,glm)
S3method(resi,glmgee)
S3method(resi,glmmTMB)
S3method(resi,hurdle)
S3method(resi,lm)
S3method(resi,lme)
S3method(resi,lmerMod)
S3method(resi,nls)
S3method(resi,survreg)
S3method(resi,zeroinfl)
S3method(resi_pe,coxph)
S3method(resi_pe,default)
S3method(resi_pe,emmGrid)
S3method(resi_pe,gee)
S3method(resi_pe,geeglm)
S3method(resi_pe,glm)
S3method(resi_pe,glmgee)
S3method(resi_pe,glmmTMB)
S3method(resi_pe,hurdle)
S3method(resi_pe,lm)
S3method(resi_pe,lme)
S3method(resi_pe,lmerMod)
S3method(resi_pe,nls)
S3method(resi_pe,survreg)
S3method(resi_pe,zeroinfl)
S3method(summary,resi)
export(Rsq2S)
export(S2Rsq)
export(S2chisq)
export(S2d)
export(S2fsq)
export(S2z)
export(chisq2S)
export(d2S)
export(f2S)
export(fsq2S)
export(omnibus)
export(resi)
export(resi_pe)
export(t2S)
export(z2S)
import(ggplot2)
importFrom(aod,wald.test)
importFrom(boot,boot)
importFrom(car,Anova)
importFrom(clubSandwich,vcovCR)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,strwidth)
importFrom(lmtest,waldtest)
importFrom(nlme,getGroups)
importFrom(sandwich,vcovHC)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,df.residual)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(utils,capture.output)
importFrom(utils,methods)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# RESI <img src="man/figures/logo.png" align="right" height="138"/>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/RESI)](https://CRAN.R-project.org/package=RESI)
[![Website](https://img.shields.io/badge/Website-RESI-blue)](https://statimagcoll.github.io/RESI/)

<!-- badges: end -->

RESI is an R package designed to implement the Robust Effect Size Index
(RESI, denoted as S) described in Vandekar, Tao, & Blume (2020). The
RESI is a versatile effect size measure that can be easily computed and
added to common reports (such as summary and ANOVA tables). This package
currently supports <code>lm</code>, <code>glm</code>, <code>nls</code>,
<code>survreg</code>, <code>coxph</code>, <code>hurdle</code>,
<code>zeroinfl</code>, <code>gee</code>, <code>geeglm</code>,
<code>lme</code>, <code>lmerMod</code>, <code>lmrob</code>, and
<code>glmrob</code> models. Confidence intervals are now computed using
the bootstrap or one of three asymptotic methods: a profiled quadratic
form, Cornish-Fisher expansion, or normal approximation. A Bayesian
bootstrap is also available for <code>lm</code> and <code>nls</code>
models. In addition to the main <code>resi</code> function, the package
also includes a point-estimate-only function (<code>resi_pe</code>),
conversions from S to other common effect size measures and vice versa,
print methods, plot methods, summary methods, and Anova/anova methods.

If you would like to contribute to the package, please branch off of our
[GitHub](https://github.com/statimagcoll/RESI) and submit a pull request
describing the contribution. Please use the [GitHub
Issues](https://github.com/statimagcoll/RESI/issues) page to report any
problems and the
[Discussions](https://github.com/statimagcoll/RESI/discussions/categories/q-a)
page to seek additional support.

## References

Jones M, Kang K, Vandekar S (2025). RESI: An R Package for Robust Effect
Sizes. <i> Journal of Statistical Software, 112(3), 1–27. <i>
<https://doi.org/10.18637/jss.v112.i03>.

Kang K, Jones MT, Armstrong K, Avery S, McHugo M, Heckers S, & Vandekar
S. Accurate Confidence and Bayesian Interval Estimation for
Non-centrality Parameters and Effect Size Indices. <i>Psychometrika.</i>
2023. <https://doi.org/10.1007/s11336-022-09899-x>.

Kang K, Seidlitz J, Bethlehem RAI, et al. Study design features increase
replicability in brain-wide association studies. <i>Nature.</i> 2024.
<https://doi.org/10.1038/s41586-024-08260-9>.

Vandekar S, Tao R, Blume J. A Robust Effect Size Index.
<i>Psychometrika.</i> 2020;85(1):232–246.
<https://doi.org/10.1007/s11336-020-09698-2>.

Zhang X, Muscatello R, Jones M, Corbett B, Vandekar S. Asymptotic
Distribution of Robust Effect Size Index. <i>arXiv:2601.19004.</i> 2026.
<https://doi.org/10.48550/arXiv.2601.19004>.

# Asymptotic confidence intervals for RESI
# Implements the normal approximation and quadratic-form (Imhof) CIs described in
# Zhang et al. (2025) "On the Asymptotic Distribution of the Robust Effect Size Index"
# and the quadratic-form notes by Vandekar.
#
# Exported:
#   resi_pe_asymptotic()   -- point estimates + asymptotic CIs (both methods)
#
# Internal pipeline:
#   .resi_precompute()     -- model-level sandwich ingredients (once per model)
#   .resi_contrast()       -- per-contrast whitened vector + Sigma_R
#   .resi_ci_normal_signed()  -- signed normal CI  (coefficients, m1=1)
#   .resi_ci_normal_unsigned() -- unsigned truncated normal CI (anova)
#   .resi_ci_qf()          -- quadratic-form Imhof CI (anova)
#   .get_L_coef()          -- contrast matrix for a single coefficient
#   .get_L_anova()         -- contrast matrix for an anova term (Type 2)


# ============================================================
#  Model-level precomputation
# ============================================================

#' @noRd
.resi_precompute <- function(model, type = "HC3",
                             deriv_method = c("corrected", "original", "population", "population2", "zeroB", "zero_phi_cross")) {

  deriv_method <- match.arg(deriv_method)

  type <- match.arg(type, c("HC3", "const", "HC", "HC0", "HC1", "HC2",
                             "HC4", "HC4m", "HC5"))
  if (type == "HC") type <- "HC0"

  # ---- basic model objects ----
  X <- model.matrix(model)
  if (any(alias <- is.na(coef(model)))) X <- X[, !alias, drop = FALSE]
  n <- nrow(X)
  p <- ncol(X)
  e <- residuals(model, "response")

  # ---- HC leverage weights (applied to mean-parameter rows of psi only) ----
  h <- hatvalues(model)
  one_m_h <- pmax(1 - h, .Machine$double.eps)
  p_int <- max(1L, as.integer(round(sum(h))))
  sqrtw <- switch(toupper(type),
    "HC0"  = rep(1, n),
    "HC1"  = rep(sqrt(n / max(n - p, 1L)), n),
    "HC2"  = 1 / sqrt(one_m_h),
    "HC3"  = 1 / one_m_h,
    "HC4"  = {
      nhp <- (n / p_int) * h
      one_m_h^(-pmin(4, nhp) / 2)
    },
    "HC4M" = {
      nhp <- (n / p_int) * h
      delta <- pmin(1, nhp) + pmin(1.5, nhp)
      one_m_h^(-delta / 2)
    },
    "HC5"  = {
      nhp <- (n / p_int) * h
      k <- 0.7
      deltaCap <- max(4, (n / p_int) * k * max(h))
      delta <- pmin(nhp, deltaCap)
      one_m_h^(-delta / 4)
    },
    rep(1, n)   # fallback (const handled via is_const below)
  )
  is_const <- (type == "const")

  # ============================================================
  #  lm branch: theta = (phi, beta),  m = p+1
  # ============================================================
  if (inherits(model, "lm") && !inherits(model, "glm")) {

    phi <- summary(model)$sigma^2
    m   <- p + 1L

    # --- per-observation psi and psi' ---
    psi_list <- lapply(seq_len(n), function(i) {
      xi <- X[i, , drop = FALSE]        # 1 x p
      ei <- e[i]
      psi_phi  <- (ei^2 - phi) / (2 * phi^2)
      psi_beta <- (ei / phi) * xi
      cbind(psi_phi, psi_beta)           # 1 x m
    })

    psiprime_list <- lapply(seq_len(n), function(i) {
      xi <- X[i, , drop = FALSE]
      ei <- e[i]
      d_phi_phi  <- (phi - 2 * ei^2) / (2 * phi^3)
      d_phi_beta <- -ei * xi / phi^2
      d_beta_phi <- t(d_phi_beta)
      d_beta_beta <- -crossprod(xi) / phi
      rbind(c(d_phi_phi, d_phi_beta),
            cbind(d_beta_phi, d_beta_beta))
    })

    # --- A and B matrices ---
    A_full <- -.resi_mean_list(psiprime_list)
    A_full <- .resi_sym(A_full)
    A_inv  <- .resi_safe_inv(A_full)

    # B: HC weights applied to mean-parameter (beta) rows only
    B_full <- if (is_const) {
      .resi_mean_outer(psi_list, rep(1, n), lm_phi_row = TRUE)
    } else {
      .resi_mean_outer(psi_list, sqrtw, lm_phi_row = TRUE)
    }
    B_full <- .resi_sym(B_full)

    cov_theta <- .resi_sym(A_inv %*% B_full %*% A_inv)

    # --- analytic dA/dtheta and dB/dtheta ---
    XtX_n    <- crossprod(X) / n
    XtX_n_hc <- if (is_const) XtX_n else crossprod(X * sqrtw) / n

    # dA/d_phi: phi column shared base (same starting point for v1 and v2).
    # The (phi,phi) element: population approx gives -1/phi^3 (from A_pp=1/(2phi^2));
    # sample version gives (phi - 3*mean(e^2))/phi^4, which is corrected in v2 below.
    # The (beta,beta) block -XtX_n/phi^2 is exact for both (X'X/phi doesn't depend on phi via e).
    dA_phi <- rbind(
      cbind(-1 / phi^3,     matrix(0, 1, p)),
      cbind(matrix(0, p, 1), -XtX_n / phi^2)
    )
    dA_dtheta <- matrix(0, m * m, m)
    dA_dtheta[, 1] <- as.vector(dA_phi)

    # dB/dtheta: two implementations selected by deriv_method.
    # "original" (v1): original code before corrections -- kept for comparison.
    # "corrected" (v2): corrected dB/dphi off-diagonals + nonzero dB/dbeta columns.
    e4mean <- mean(e^4)
    dB_dtheta <- matrix(0, m * m, m)

    if (deriv_method == "original") {
      # ---- v1: original formulas (kept for comparison) ----
      # Errors: (a) dB_phi-phi used (phi^2-2*e4mean) [wrong factor]; (b) dB_phi-beta
      # factored as colMeans(X)*e3mean [wrong]; (c) dB_beta-beta used -XtX_hc/phi^2
      # [wrong power/missing e^2]; (d) all dB/dbeta_l = 0.
      e3mean <- mean(e^3)
      dB_phi_v1 <- rbind(
        cbind((phi^2 - 2 * e4mean) / phi^5,
              -(3 / (2 * phi^4)) * t(colMeans(X) * e3mean)),
        cbind(-(3 / (2 * phi^4)) * colMeans(X) * e3mean,
              -XtX_n_hc / phi^2)
      )
      dB_dtheta[, 1] <- as.vector(dB_phi_v1)
      # dB/dbeta columns: all zero (v1 approximation)

    } else {
      # ---- v2+: corrected formulas ----
      # 'corrected'    : sample-level derivatives for all blocks (exact FD match).
      # 'population'   : beta-beta block of dB/dbeta_l = 0.
      # 'population2'  : all beta columns of dB = 0.
      # 'zeroB'          : dB_dtheta = 0 entirely (only A-chain contributes).
      #                    Diagnostic: removes B-chain; unrealistic since B depends on phi.
      # 'zero_phi_cross' : population + phi-beta cross block of dB/dphi set to zero.
      #                    Tests whether E[e^3 x_j] cross terms in dB/dphi matter.

      if (deriv_method != "zeroB") {
        # ---- dB/dphi (phi column of dB_dtheta) ----
        # phi-beta cross block: zeroed for zero_phi_cross, sample-level otherwise
        e3_X_w <- if (deriv_method == "zero_phi_cross") {
          rep(0, p)          # phi-beta cross = 0
        } else {
          if (is_const) colMeans(e^3 * X)
          else          colMeans(sqrtw * e^3 * X)
        }
        B_beta <- B_full[2:m, 2:m, drop = FALSE]   # p x p beta-beta block of B

        dB_phi_v2 <- rbind(
          cbind((phi^2 - e4mean) / phi^5,
                -(3 / (2 * phi^4)) * t(e3_X_w)),
          cbind(-(3 / (2 * phi^4)) * e3_X_w,
                -2 / phi * B_beta)
        )
        dB_dtheta[, 1] <- as.vector(dB_phi_v2)

        # ---- dB/dbeta_l columns (l = 1,...,p; column index 1+l) ----
        # e2_m_phi_e <- (e^2 - phi) * e  
        e2_m_phi_e <- e^3
        # had a minus phi, but that is pre expectation                                # n-vector
        v_3e2      <- if (is_const) 3 * e^2  - phi                    # no HC weight for const
                      else sqrtw * (3 * e^2 - phi)                      # HC-weighted
        # d_phi_phi_betas[l] = dB_{phi,phi} / d_beta_l
        d_phi_phi_betas  <- -colMeans(e2_m_phi_e * X) / phi^4          # p-vector
        # dB_phi_beta_beta[j,l] = dB_{phi,beta_j} / d_beta_l
        dB_phi_beta_beta <- -crossprod(X, v_3e2 * X) / (2 * n * phi^3) # p x p

        # Precompute HC squared weights for the beta-beta block
        tau_eff <- if (is_const) e else sqrtw^2 * e   # tau_i * e_i (n-vector)

        for (l in seq_len(p)) {
          dB_l <- matrix(0, m, m)
          if (deriv_method == "population2") {
            # entire beta column zeroed: phi-phi, phi-beta, and beta-beta blocks all 0
          } else {
            dB_l[1L, 1L]  <- d_phi_phi_betas[l]
            dB_l[1L, 2:m] <- dB_phi_beta_beta[, l]
            dB_l[2:m, 1L] <- dB_phi_beta_beta[, l]   # B is symmetric
            if (deriv_method == "corrected") {
              v_l             <- tau_eff * X[, l]       # n-vector: tau_i e_i x_il
              dB_l[2:m, 2:m] <- -2 / phi^2 * crossprod(X, v_l * X) / n
            }
            # population, zero_phi_cross: beta-beta block left as 0
          }
          dB_dtheta[, 1L + l] <- as.vector(dB_l)
        }
      }
      # zeroB: dB_dtheta remains all zeros (initialized above)

      # Sample-level dA/dphi phi-phi element:
      # d(A_pp)/dphi = (phi - 3*mean(e^2))/phi^4  [exact sample derivative]
      # The population approx A_pp = 1/(2phi^2) gives d/dphi = -1/phi^3;
      # at E[e^2]=phi these differ by factor 2.
      e2mean <- mean(e^2)
      dA_dtheta[1L, 1L] <- (phi - 3 * e2mean) / phi^4

      # Sample-level dA/d_beta_l: phi-beta cross block is -(X'X/n)_{jl}/phi^2.
      # d(A_pb_j)/d_beta_l = d/d_beta_l [mean(e*x_j/phi^2)] = -mean(x_j*x_l)/phi^2
      #                     = -(X'X/n)_{jl}/phi^2  (nonzero even at MLE)
      # d(A_pp)/d_beta_l   = -2*mean(e*x_l)/phi^3  (= 0 exactly at OLS MLE)
      # d(A_bb_jk)/d_beta_l = 0  (X'X/n/phi doesn't depend on beta)
      e_X_means <- colMeans(e * X)  # mean(e*x_l) -- p-vector, 0 exactly at OLS MLE
      for (l in seq_len(p)) {
        dA_l <- matrix(0, m, m)
        dA_l[1L, 1L]  <- -2 * e_X_means[l] / phi^3  # 0 at OLS MLE
        dA_l[1L, 2:m] <- -XtX_n[, l] / phi^2        # phi-beta cross
        dA_l[2:m, 1L] <- -XtX_n[l, ] / phi^2        # symmetric
        dA_dtheta[, 1L + l] <- as.vector(dA_l)
      }
    }

    theta_hat <- c(phi, coef(model))
    lm_model  <- TRUE

  # ============================================================
  #  GLM branch: theta = beta,  m = p
  # ============================================================
  } else if (inherits(model, "glm")) {

    m <- p
    lm_model <- FALSE
    phi <- NULL

    ef    <- sandwich::estfun(model)
    ef    <- ef[, colnames(X), drop = FALSE]
    w_vec <- weights(model, type = "working")
    mu_hat <- pmin(pmax(fitted(model), .Machine$double.eps),
                   1 - .Machine$double.eps)

    psi_list      <- lapply(seq_len(n), function(i)
      matrix(ef[i, ], nrow = 1))
    psiprime_list <- lapply(seq_len(n), function(i)
      -w_vec[i] * crossprod(X[i, , drop = FALSE]))

    A_full <- -.resi_mean_list(psiprime_list)
    A_full <- .resi_sym(A_full)
    A_inv  <- .resi_safe_inv(A_full)

    B_full <- if (is_const) {
      .resi_mean_outer(psi_list, rep(1, n), lm_phi_row = FALSE)
    } else {
      .resi_mean_outer(psi_list, sqrtw, lm_phi_row = FALSE)
    }
    B_full <- .resi_sym(B_full)

    cov_theta <- .resi_sym(A_inv %*% B_full %*% A_inv)

    # analytic dA/dtheta (= dB/dtheta under canonical link)
    dA_dtheta <- matrix(0, m * m, m)
    for (j in seq_len(m)) {
      vj <- w_vec * (1 - 2 * mu_hat) * X[, j]
      dA_dtheta[, j] <- as.vector(crossprod(X, vj * X) / n)
    }
    dB_dtheta <- dA_dtheta

    theta_hat <- coef(model)

  } else {
    stop(".resi_precompute: model class not supported (lm or glm only)")
  }

  list(
    model        = model,
    X            = X,
    n            = n,
    p            = p,
    m            = m,
    lm_model     = lm_model,
    phi          = phi,
    theta_hat    = theta_hat,
    A_inv        = A_inv,
    B_full       = B_full,
    cov_theta    = cov_theta,
    dA_dtheta    = dA_dtheta,
    dB_dtheta    = dB_dtheta,
    deriv_method = deriv_method,
    is_const     = is_const,
    type         = type,
    psi_list     = psi_list,
    sqrtw        = sqrtw
  )
}


# ============================================================
#  Per-contrast computation
# ============================================================

#' @noRd
#' @param precomp output of .resi_precompute()
#' @param L_model contrast matrix in *beta* (coefficient) space (m1 x p)
#' @param vcovmat_n Optional matrix n * vcovfunc(model) in beta space (p x p).
#'   When supplied, used for Sigma_beta so that Stilde targets the same population
#'   quantity as the vcovfunc-based point estimate. cov_theta (from precomp) is
#'   still used for Sigma_R via the delta method.
#' @return list with R_beta, Stilde, dR_dtheta (m1 x m), Sigma_R (m1 x m1),
#'         beta_hat, Sigma_beta, m1
.resi_contrast <- function(precomp, L_model, vcovmat_n = NULL) {

  m1     <- nrow(L_model)
  m      <- precomp$m
  n      <- precomp$n
  A_inv  <- precomp$A_inv
  Sig    <- precomp$cov_theta     # m x m  (used for Sigma_R)
  dA     <- precomp$dA_dtheta     # m^2 x m
  dB     <- precomp$dB_dtheta     # m^2 x m

  # Extend L to theta space: prepend column of zeros for phi if lm
  L <- if (precomp$lm_model) cbind(0, L_model) else L_model  # m1 x m

  # ---- point estimate ----
  beta_hat <- drop(L %*% precomp$theta_hat)           # m1

  # Sigma_beta: use vcovmat_n (vcovfunc-based, in beta space) when supplied so
  # that Stilde is consistent with the vcovfunc-based RESI point estimate.
  # Note: cov_theta[beta,beta] ≈ n * vcovHC, so for the default HC3 vcovfunc
  # this is numerically transparent; it matters when vcovfunc != type.
  Sigma_beta <- if (!is.null(vcovmat_n)) {
    .resi_sym(L_model %*% vcovmat_n %*% t(L_model))
  } else {
    .resi_sym(L %*% Sig %*% t(L))
  }

  # ---- EVD of Sigma_beta ----
  eig <- eigen(Sigma_beta, symmetric = TRUE)
  d   <- pmax(eig$values, .Machine$double.eps)         # eigenvalues (m1)
  V   <- eig$vectors                                   # m1 x m1

  sqrtd <- sqrt(d)
  Phat  <- V %*% diag(1 / sqrtd, m1) %*% t(V)         # Sigma_beta^{-1/2}

  R_beta <- Phat %*% beta_hat                          # m1-vector, whitened RESI
  Stilde <- sqrt(sum(R_beta^2))                        # ||R_beta||

  # ---- Lyapunov weight matrix ----
  # W[j,k] = 1 / (sqrt(d[j]) * sqrt(d[k]) * (sqrt(d[j]) + sqrt(d[k])))
  W <- outer(sqrtd, sqrtd, function(a, b) 1 / (a * b * (a + b)))  # m1 x m1

  VT_beta <- drop(t(V) %*% beta_hat)   # m1-vector (V-basis projection of beta_hat)

  # ---- dR_dtheta: m1 x m matrix ----
  direct <- Phat %*% L                   # (a) direct term: m1 x m

  Achain <- matrix(0, m1, m)
  Bchain <- matrix(0, m1, m)
  for (k in seq_len(m)) {
    # (b) A-chain
    dA_k     <- matrix(dA[, k], m, m)
    dSig_A   <- -(A_inv %*% dA_k %*% Sig + Sig %*% dA_k %*% A_inv)
    dSigB_A  <- L %*% dSig_A %*% t(L)
    M_A      <- t(V) %*% dSigB_A %*% V  # m1 x m1 in V-basis
    Achain[, k] <- -V %*% (W * M_A) %*% VT_beta

    # (c) B-chain (zero for parametric / const)
    if (!precomp$is_const) {
      dB_k     <- matrix(dB[, k], m, m)
      dSig_B   <- A_inv %*% dB_k %*% A_inv
      dSigB_B  <- L %*% dSig_B %*% t(L)
      M_B      <- t(V) %*% dSigB_B %*% V
      Bchain[, k] <- -V %*% (W * M_B) %*% VT_beta
    }
  }

  dR_dtheta <- direct + Achain + Bchain   # m1 x m

  # ---- Sigma_R = Cov(sqrt(n) R_hat) ----
  Sigma_R <- dR_dtheta %*% Sig %*% t(dR_dtheta)   # m1 x m1
  Sigma_R <- .resi_sym(Sigma_R)

  # ---- HC-weighted influence values (for CF skewness; m1=1 only) ----
  # phi_tilde_i = (dR_dtheta %*% A_inv) %*% psi_tilde_i  (scalar when m1=1)
  # where psi_tilde applies HC weights consistent with B_full.
  phi_tilde <- NULL
  if (m1 == 1L && !is.null(precomp$psi_list)) {
    c_vec <- drop(dR_dtheta %*% A_inv)   # length-m row vector
    phi_tilde <- vapply(seq_len(n), function(i) {
      psi_i <- drop(precomp$psi_list[[i]])   # length m
      if (precomp$lm_model) {
        # phi (element 1) unweighted; beta rows (2:m) weighted
        psi_i[seq(2L, precomp$m)] <- psi_i[seq(2L, precomp$m)] * precomp$sqrtw[i]
      } else {
        psi_i <- psi_i * precomp$sqrtw[i]
      }
      sum(c_vec * psi_i)
    }, numeric(1L))
  }

  list(
    m1         = m1,
    n          = n,
    beta_hat   = beta_hat,
    Sigma_beta = Sigma_beta,
    R_beta     = R_beta,
    Stilde     = Stilde,
    dR_dtheta  = dR_dtheta,
    dR_direct  = direct,      # m1 x m: direct (Phat*L) contribution
    dR_Achain  = Achain,      # m1 x m: A-chain contribution
    dR_Bchain  = Bchain,      # m1 x m: B-chain contribution
    Sigma_R    = Sigma_R,
    phi_tilde  = phi_tilde
  )
}


# ============================================================
#  CI functions
# ============================================================

#' Signed Cornish-Fisher CI for a single coefficient (m1 = 1)
#'
#' Uses the closed-form CF inversion: since q_p(s) is linear in s, the CI
#' S_pm ± se * w_p inverts to exact bounds without root-finding.
#' gamma1 and gamma2 are estimated from the HC-weighted influence functions
#' phi_tilde stored in `contrast$phi_tilde`.
#' @noRd
.resi_ci_cf_signed <- function(contrast, alpha = 0.05) {
  Sigma_R   <- as.numeric(contrast$Sigma_R)   # scalar
  n         <- contrast$n
  R_beta    <- drop(contrast$R_beta)           # signed scalar S_pm
  phi_tilde <- contrast$phi_tilde              # n-vector, NULL if unavailable

  se <- sqrt(max(Sigma_R, .Machine$double.eps) / n)

  # if (is.null(phi_tilde) || length(phi_tilde) != n) {
  #   # phi_tilde not available (shouldn't happen in normal use)
  #   return(.resi_ci_normal_signed(contrast, alpha = alpha))
  # }

  # Standardized cumulants of Z_n = sqrt(n) * (S_pm - S_true) / sqrt(Sigma_R)
  # where phi_tilde are the HC-weighted influence values and Sigma_R = mean(phi^2).
  #
  # gamma1 = kappa_3(Z_n) = n^{-1/2} * mean(phi^3) / Sigma_R^{3/2}
  # gamma2 = kappa_4(Z_n) = n^{-1}   * (mean(phi^4) / Sigma_R^2 - 3)
  #
  # Both are O(n^{-1/2}) and O(n^{-1}) respectively, and -> 0 as n -> inf
  # for any distribution with finite fourth moment (CLT rate).
  m3     <- mean(phi_tilde^3)
  m4     <- mean(phi_tilde^4)
  gamma1 <- m3 / (sqrt(n) * Sigma_R^(3 / 2))
  gamma2 <- (m4 / Sigma_R^2 - 3) / n

  cf_w <- function(p) {
    z <- qnorm(p)
    z +
      (gamma1      / 6 ) * (z^2 - 1) +
      (gamma2      / 24) * (z^3 - 3 * z) -
      (gamma1^2    / 36) * (2 * z^3 - 5 * z)
  }

  # These are the correct intervals
  # Closed-form: LCI = S_pm - se * w_{1-alpha/2},  UCI = S_pm - se * w_{alpha/2}
  LCI <- R_beta - se * cf_w(1 - alpha / 2)
  UCI <- R_beta - se * cf_w(    alpha / 2)
  # these are incorrect, but more align with the percentile bootstrap, which seems to do better for some reason
  # this code was just to investigate if this is more similar to that.
  # UCI <- R_beta + se * cf_w(1 - alpha / 2)
  # LCI <- R_beta + se * cf_w(    alpha / 2)

  c(LCI = LCI, UCI = UCI)
}

#' Signed normal CI for a single coefficient (m1 = 1)
#' @noRd
.resi_ci_normal_signed <- function(contrast, alpha = 0.05) {
  Sigma_R <- contrast$Sigma_R  # scalar (1x1)
  n       <- contrast$n
  Stilde  <- contrast$Stilde   # |Z|/sqrt(n) sign preserved via R_beta sign
  R_beta  <- contrast$R_beta   # signed (scalar)

  # signed: S_pm = R_beta (= Z/sqrt(n) when m1=1, Sigma_beta=1)
  S_signed <- R_beta   # scalar
  # Floor SE at 1/sqrt(n): asymptotically Sigma_R -> 1 under H0, but near-null
  # sandwich estimates can collapse below 1, shrinking the CI inappropriately.
  # se       <- max(sqrt(as.numeric(Sigma_R) / n), 1 / sqrt(n))
  se       <- sqrt(as.numeric(Sigma_R) / n)
  z        <- qnorm(1 - alpha / 2)
  c(LCI = S_signed - z * se, UCI = S_signed + z * se)
}

#' Unsigned truncated normal CI (anova, m1 >= 1)
#' @noRd
.resi_ci_normal_unsigned <- function(contrast, alpha = 0.05) {
  Sigma_R <- contrast$Sigma_R   # m1 x m1
  R_beta  <- contrast$R_beta    # m1
  Stilde  <- contrast$Stilde
  n       <- contrast$n
  m1      <- contrast$m1

  # df-corrected center: Shat = sqrt(max(0, Stilde^2 - m1/n))
  # Aligns CI center with chisq2S point estimator; same correction as QF CI.
  Shat <- sqrt(max(0, Stilde^2 - m1 / n))

  # Null upper bound: (1-alpha) quantile of Stilde under H0.
  # Under H0, sqrt(n)*||R_hat|| ~ ||N(0, I_m1)|| = chi(m1) because Sigma_R -> I
  # asymptotically. So P(Stilde >= UCI | S=0) = alpha gives
  # UCI = sqrt(qchisq(1-alpha, df=m1) / n).
  # For m1=1: sqrt(qchisq(1-alpha, 1)/n) = qnorm(1-alpha/2)/sqrt(n).
  UCI_null <- sqrt(qchisq(1 - alpha, df = m1) / n)

  if (Shat <= 0) {
    return(c(LCI = 0, UCI = UCI_null))
  }

  u       <- R_beta / Stilde                              # unit direction
  sigma2S <- as.numeric(t(u) %*% Sigma_R %*% u)         # scalar variance
  # Floor SE at sqrt(m1/n): asymptotically tr(Sigma_R) -> m1 under H0, but
  # near-null sandwich estimates can underestimate, shrinking the CI.
  # se      <- max(sqrt(pmax(sigma2S, 0) / n), sqrt(m1 / n))
  se      <- sqrt(pmax(sigma2S, 0) / n)

  # truncate_ci handles boundary; centered at Shat
  bounds <- .resi_truncate_ci(Shat, se, n, m1, alpha)

  # When LCI clips to 0, the directional delta-method UCI can underestimate
  # spread for m1>1 (single-direction 1D normal vs chi(m1) null distribution).
  # Use the chi(m1) null upper as a floor.
  if (bounds[1] == 0) {
    bounds[2] <- max(bounds[2], UCI_null)
  }

  c(LCI = bounds[1], UCI = bounds[2])
}

#' Quadratic-form Imhof CI (anova, m1 >= 1)
#' @noRd
.resi_ci_qf <- function(contrast, alpha = 0.05) {
  Sigma_R <- contrast$Sigma_R
  R_beta  <- contrast$R_beta
  Stilde  <- contrast$Stilde
  n       <- contrast$n
  m1      <- contrast$m1

  # T2_obs  <- max(0, n * Stilde^2 - m1)    # df-corrected test statistic (Shat^2 * n)
  # Shat    <- sqrt(T2_obs / n)              # df-corrected point estimate (for search bounds only)
  T2_obs <- n * Stilde^2                  # raw test statistic
  Shat   <- sqrt(max(0, Stilde^2 - m1/n)) # (for search bounds only)

  # EVD of Sigma_R
  eigR       <- eigen(Sigma_R, symmetric = TRUE)
  lambda     <- pmax(eigR$values, .Machine$double.eps)
  U          <- eigR$vectors

  # Estimated non-centrality direction (unit length in U-basis)
  delta_unit <- if (Stilde > 0) drop(t(U) %*% R_beta) / Stilde else
    rep(0, length(lambda))

  # Normal-approx SE of Stilde (for search interval sizing).
  # Delta-method: se_S = sqrt(u^T Sigma_R u / n), u = R_beta/Stilde.
  # Under H0, n*Stilde^2 ~ chi^2_{m1}, so Var(Stilde) -> m1/(2n) * (2/m1) = 1/n
  # giving SD(Stilde) ~ sqrt(m1/n) / sqrt(2) ... more precisely the SD of
  # chi(m1)/sqrt(n) is sqrt(m1/n) (since Var(chi(m1)) ~ m1 for large m1).
  # Use sqrt(m1/n) as a floor so the bracket is never degenerate near null.
  u_dir        <- if (Stilde > 0) R_beta / Stilde else eigR$vectors[, 1]
  sigma2S      <- as.numeric(t(u_dir) %*% Sigma_R %*% u_dir)
  se_S_delta   <- sqrt(pmax(sigma2S / n, .Machine$double.eps))
  se_S_floor   <- sqrt(m1 / n)
  se_S         <- max(se_S_delta, se_S_floor)

  # nc_k = n * S_b^2 * delta_unit_k^2 / lambda_k  (non-centrality for chi^2_1 k-th term)
  nc_fun <- function(S_b) n * S_b^2 * delta_unit^2 / lambda

  # Probability function P(Q(S_b) >= T2_obs)
  # For m1=1: use pchisq (no imhof precision issues)
  # For m1>1: use imhof
  if (m1 == 1L) {
    q <- T2_obs / lambda[1]
    # For large q or ncp, use the exact identity chi^2(1,lambda) = (Z + sqrt(lambda))^2:
    #   P(chi^2(1,lambda) >= q) = pnorm(sqrt(lambda) - sqrt(q)) + pnorm(-sqrt(lambda) - sqrt(q))
    # This avoids pnchisq convergence failures (which arise when q and ncp are both large,
    # e.g. ~1e12, as happens in ill-conditioned GLMs at small n).  The formula is
    # algebraically exact and pnorm is numerically stable at arbitrarily large arguments.
    prob_fn <- function(S_b) {
      nc <- nc_fun(S_b)
      if (!is.finite(nc)) return(1.0)
      if (q > 1e4 || nc > 1e4) {
        sq  <- sqrt(max(q,  0))
        snc <- sqrt(max(nc, 0))
        return(pnorm(snc - sq, lower.tail = FALSE) + pnorm(-snc - sq))
      }
      pchisq(q, df = 1, ncp = nc, lower.tail = FALSE)
    }
  } else {
    prob_fn <- function(S_b) {
      nc <- nc_fun(S_b)
      # Cap individual non-centrality components to avoid imhof overflow
      nc <- pmin(nc, 1e10 * max(T2_obs, 1))
      suppressWarnings(CompQuadForm::imhof(T2_obs, lambda = lambda, delta = nc)$Qq)
    }
  }

  # ---- Lower bound ----
  # P(Q(0)) = P(central weighted chi-sq >= T2_obs)
  p0 <- prob_fn(0)
  if (is.na(p0) || p0 < 0) p0 <- 0   # imhof can return negative for extreme cases

  if (p0 >= alpha / 2) {
    LCI <- 0
  } else {
    lower_candidates <- Shat * c(0.9, 0.7, 0.5, 0.3, 0.1)
    bracket_lo <- NA_real_
    for (lo in lower_candidates) {
      plo <- tryCatch(prob_fn(lo), error = function(e) NA_real_)
      if (!is.na(plo) && is.finite(plo) && plo >= 0 && plo < alpha / 2) {
        bracket_lo <- lo
        break
      }
    }
    if (is.na(bracket_lo)) bracket_lo <- 0

    LCI <- tryCatch(
      uniroot(function(s) prob_fn(s) - alpha / 2,
              lower = bracket_lo, upper = Stilde,
              tol = se_S * 1e-3, extendInt = "upX")$root,
      error = function(e) {
        warning("QF lower CI bound search failed; using 0.")
        0
      }
    )
    LCI <- max(LCI, 0)
  }

  # ---- Upper bound ----
  # When p0 = P(Q(0) >= T2_obs) >= 1-alpha/2, T2_obs falls in the lower alpha/2
  # tail of the null distribution. Strict test inversion gives an empty upper set
  # because prob_fn is increasing and never drops to 1-alpha/2. Fallback: use the
  # (1-alpha) quantile of Stilde under H0: n*Stilde^2 ~ chi^2_{m1}, so
  # UCI_null = sqrt(qchisq(1-alpha, df=m1) / n). Same formula as the normal CI.
  # The same fallback is used when the bracket search fails.
  # Null upper bound: (1-alpha) quantile of Stilde under H0.
  # Under H0, n*Stilde^2 ~ chi^2_{m1} asymptotically, so
  # UCI_null = sqrt(qchisq(1-alpha, df=m1) / n).
  UCI_null <- sqrt(qchisq(1 - alpha, df = m1) / n)

  if (p0 >= 1 - alpha / 2) {
    # T2_obs is in the lower alpha/2 tail of the null distribution;
    # strict test inversion gives an empty upper set. Use the null upper bound.
    UCI <- UCI_null
  } else {
    # Normal case: uniroot in (Shat, bracket_up) where prob_fn crosses 1-alpha/2
    upper_search <- Shat + se_S * c(3, 6, 12, 25, 50, 100) * qnorm(1 - alpha / 2)
    bracket_up   <- NA_real_
    for (up in upper_search) {
      pup <- tryCatch(prob_fn(up), error = function(e) NA_real_)
      if (is.na(pup) || !is.finite(pup) || pup >= 1 - alpha / 2) {
        bracket_up <- up; break
      }
    }
    if (is.na(bracket_up)) {
      UCI <- UCI_null
    } else {
      UCI <- tryCatch(
        uniroot(function(s) {
          p <- prob_fn(s)
          if (is.na(p) || !is.finite(p)) return(1 - (1 - alpha / 2))
          p - (1 - alpha / 2)
        },
                lower = Stilde, upper = bracket_up,
                tol = se_S * 1e-3)$root,
        error = function(e) UCI_null
      )
    }
  }

  c(LCI = max(LCI, 0), UCI = UCI)
}

#' Cornish-Fisher test-inversion CI (anova, m1 >= 1)
#'
#' Inverts the Cornish-Fisher approximation to the generalized non-central
#' chi-square distribution of T^2 = n*Stilde^2.  Uses the exact cumulants of
#' sum_k lambda_k (Z_k + delta_k)^2, with lambda_k the eigenvalues of
#' Sigma_R and delta_k scaled so that sum delta_k^2 = n*S_b^2.
#' @noRd
.resi_ci_cf <- function(contrast, alpha = 0.05) {
  Sigma_R <- contrast$Sigma_R
  R_beta  <- contrast$R_beta
  Stilde  <- contrast$Stilde
  n       <- contrast$n
  m1      <- contrast$m1

  T2_obs <- n * Stilde^2
  Shat   <- sqrt(max(0, Stilde^2 - m1 / n))   # df-corrected; used for bracket sizing only

  # ---- Cumulant ingredients (matrix powers, no EVD needed) ----
  # c_r = tr(Sigma_R^r),  v_r = R' Sigma_R^{r-1} R / Stilde^2
  SigR2 <- Sigma_R %*% Sigma_R          # Sigma_R^2 (m1 x m1)
  c1 <- sum(diag(Sigma_R))              # tr(Sigma_R)
  c2 <- sum(Sigma_R * Sigma_R)          # tr(Sigma_R^2) = ||Sigma_R||_F^2 (symmetric)
  c3 <- sum(diag(SigR2 %*% Sigma_R))   # tr(Sigma_R^3)
  c4 <- sum(SigR2 * SigR2)             # tr(Sigma_R^4) = ||Sigma_R^2||_F^2

  if (Stilde > 0) {
    SigR_Rb  <- drop(Sigma_R %*% R_beta)       # Sigma_R   R
    SigR2_Rb <- drop(SigR2   %*% R_beta)       # Sigma_R^2 R
    SigR3_Rb <- drop(SigR2   %*% SigR_Rb)      # Sigma_R^3 R
    v2 <- sum(R_beta * SigR_Rb)  / Stilde^2    # R' Sigma   R / Stilde^2
    v3 <- sum(R_beta * SigR2_Rb) / Stilde^2    # R' Sigma^2 R / Stilde^2
    v4 <- sum(R_beta * SigR3_Rb) / Stilde^2    # R' Sigma^3 R / Stilde^2
  } else {
    v2 <- v3 <- v4 <- 0
  }

  # ---- SE for bracket sizing ----
  u_dir   <- if (Stilde > 0) R_beta / Stilde else c(1, rep(0, m1 - 1))
  sigma2S <- as.numeric(t(u_dir) %*% Sigma_R %*% u_dir)
  se_S    <- max(sqrt(pmax(sigma2S / n, .Machine$double.eps)), sqrt(m1 / n))

  # ---- Cornish-Fisher quantile: q_p(S_b) ----
  # p-th quantile of T^2 under S_true = S_b, using the 4-term CF expansion
  # through the gamma1^2 correction (standard second-order accurate expansion).
  #   kappa_r = 2^{r-1}(r-1)! * [c_r + r * n*S_b^2 * v_r]
  #   gamma1 = kappa3 / sigma^3,  gamma2 = kappa4 / sigma^4
  #   q_p = mu + sigma * [z + gamma1/6*(z^2-1) + gamma2/24*(z^3-3z) - gamma1^2/36*(2z^3-5z)]
  cf_quant <- function(S_b, p) {
    nSb2  <- n * S_b^2
    mu    <- c1 + nSb2
    var_q <- 2 * (c2 + 2 * nSb2 * v2)
    if (!is.finite(var_q) || var_q <= 0) return(NA_real_)
    sigma  <- sqrt(var_q)
    kappa3 <- 8  * (c3 + 3 * nSb2 * v3)
    kappa4 <- 48 * (c4 + 4 * nSb2 * v4)
    gamma1 <- kappa3 / sigma^3
    gamma2 <- kappa4 / sigma^4
    z <- qnorm(p)
    w <- z +
      (gamma1 / 6)      * (z^2 - 1) +
      (gamma2 / 24)     * (z^3 - 3 * z) -
      (gamma1^2 / 36)   * (2 * z^3 - 5 * z)
    mu + sigma * w
  }

  # ---- Null upper bound and boundary conditions ----
  # Use chi-square tail probability for the boundary checks.  The CF series
  # can diverge near S_b = 0 (gamma1 is large for the central distribution),
  # so we use the exact chi-square approximation there -- the same approach
  # as the QF CI.  CF is only applied for the inversion at nonzero S_b.
  UCI_null <- sqrt(qchisq(1 - alpha, df = m1) / n)

  # P(T2 >= T2_obs | S = 0) under chi^2_{m1} approximation
  p0 <- pchisq(T2_obs, df = m1, lower.tail = FALSE)

  # Degenerate: no signal at all
  if (T2_obs == 0) return(c(LCI = 0, UCI = UCI_null))

  # ---- Lower bound ----
  # LCI = 0 when the null distribution already has >= alpha/2 probability above T2_obs.
  if (p0 >= alpha / 2) {
    LCI <- 0
  } else {
    # Before calling uniroot, verify that the CF quantile at S_b = 0 is actually
    # below T2_obs.  The chi-square boundary check (p0 < alpha/2) guarantees this
    # for the exact chi-square distribution, but the CF expansion can overestimate
    # the null quantile (large gamma1 for small m1 / small n), giving
    # cf_quant(0, 1-alpha/2) > T2_obs even when p0 < alpha/2.  In that case the
    # root does not exist in [0, Stilde] and LCI = 0 by the CF approximation.
    q0_cf <- tryCatch(cf_quant(0, 1 - alpha / 2), error = function(e) NA_real_)
    if (is.na(q0_cf) || q0_cf >= T2_obs) {
      LCI <- 0
    } else {
      # CF inversion: solve q_{1-alpha/2}(S_L) = T2_obs in (0, Stilde).
      # At S_b = Stilde: q_{1-alpha/2}(Stilde) = c1 + T2_obs + sigma*(positive) > T2_obs.
      LCI <- tryCatch(
        uniroot(function(s) cf_quant(s, 1 - alpha / 2) - T2_obs,
                lower = 0, upper = Stilde,
                tol = se_S * 1e-3, extendInt = "upX")$root,
        error = function(e) {
          warning("CF lower CI bound search failed; using 0.")
          0
        }
      )
      LCI <- max(LCI, 0)
    }
  }

  # ---- Upper bound ----
  # UCI = UCI_null when T2_obs falls in the lower tail of the null distribution.
  if (p0 >= 1 - alpha / 2) {
    UCI <- UCI_null
  } else {
    # Bracket upper: find S_b where q_{alpha/2}(S_b) first exceeds T2_obs
    upper_search <- Shat + se_S * c(3, 6, 12, 25, 50, 100) * qnorm(1 - alpha / 2)
    bracket_up   <- NA_real_
    for (up in upper_search) {
      q_up <- tryCatch(cf_quant(up, alpha / 2), error = function(e) NA_real_)
      if (!is.na(q_up) && is.finite(q_up) && q_up >= T2_obs) {
        bracket_up <- up; break
      }
    }
    if (is.na(bracket_up)) {
      UCI <- UCI_null
    } else {
      UCI <- tryCatch(
        uniroot(function(s) cf_quant(s, alpha / 2) - T2_obs,
                lower = Stilde, upper = bracket_up,
                tol = se_S * 1e-3)$root,
        error = function(e) UCI_null
      )
    }
  }

  c(LCI = max(LCI, 0), UCI = UCI)
}


# ============================================================
#  Contrast-matrix helpers
# ============================================================

#' @noRd
.get_L_coef <- function(model, coef_name) {
  coefs <- names(coef(model))
  coefs <- coefs[!is.na(coef(model))]
  idx   <- which(coefs == coef_name)
  if (length(idx) == 0) stop("Coefficient not found: ", coef_name)
  L <- matrix(0, 1, length(coefs))
  L[1, idx] <- 1
  L
}

#' Type-2 Anova contrast matrix for a single term (in beta space)
#' Adapted from get_L_anova2 in functions.R
#' @noRd
.get_L_anova_term <- function(model, term, vcovmat) {
  coefs       <- names(coef(model))
  not_aliased <- !is.na(coef(model))
  names_terms <- labels(terms(model))
  which_term  <- which(term == names_terms)
  factors     <- attr(terms(model), "factors")
  asgn        <- attr(model.matrix(model), "assign")
  asgn[!not_aliased] <- NA

  subs_term <- which(asgn == which_term)

  # relatives: terms that contain 'term' as a lower-order effect
  relatives <- setdiff(seq_along(names_terms), which_term)
  relatives <- relatives[sapply(names_terms[relatives], function(t2)
    all(factors[, term] <= factors[, t2]))]
  subs_rel  <- unlist(lapply(relatives, function(r) which(asgn == r)))

  Ip <- diag(sum(not_aliased))
  hyp1 <- Ip[subs_rel, , drop = FALSE]
  hyp2 <- Ip[c(subs_rel, subs_term), , drop = FALSE]

  if (nrow(hyp1) == 0) {
    L_out <- hyp2
  } else {
    # Orthogonalize: columns of hyp2 orthogonal to span(hyp1) under vcovmat metric
    L_out <- t(.resi_conjcomp(t(hyp1), t(hyp2), vcovmat))
  }
  L_out <- L_out[!apply(L_out, 1, function(x) all(x == 0)), , drop = FALSE]
  L_out
}

#' @noRd
.resi_conjcomp <- function(X, Z, ip = diag(nrow(X))) {
  xq <- qr(t(Z) %*% ip %*% X)
  if (xq$rank == 0) return(Z)
  Z %*% qr.Q(xq, complete = TRUE)[, -(seq_len(xq$rank)), drop = FALSE]
}


# ============================================================
#  Main exported function
# ============================================================

#' Robust Effect Size Index with Asymptotic Confidence Intervals
#'
#' Computes RESI point estimates and asymptotic confidence intervals using
#' either a normal approximation (Zhang et al., 2025) or a quadratic-form
#' (Imhof/Davies) approach.
#'
#' @param model.full Fitted \code{lm} or \code{glm} model object.
#' @param data Data frame of model data.
#' @param vcovfunc Variance estimator for RESI point estimates. Default:
#'   \code{sandwich::vcovHC}.
#' @param coefficients Logical; include coefficient table. Default \code{TRUE}.
#' @param anova Logical; include anova table. Default \code{TRUE}.
#' @param alpha Numeric; significance level. Default \code{0.05}.
#' @param ci.method Character; \code{"qf"} (quadratic-form Imhof, default),
#'   \code{"normal"} (truncated normal),
#'   or \code{"cf"} (Cornish-Fisher test inversion).
#' @param type Character; HC type for the sandwich variance used in CI
#'   construction (controls tau_i weights in SigmaXw). Default \code{"HC3"}.
#'   Supported: \code{"HC0"}–\code{"HC5"}, \code{"const"}.
#' @param unbiased Logical; use bias-corrected RESI point estimate. Default
#'   \code{TRUE}.
#' @param Anova.args List; additional arguments passed to \code{car::Anova}.
#' @param vcov.args List; additional arguments passed to \code{vcovfunc}.
#' @param ... Ignored.
#' @return A list of class \code{"resi"} with \code{coefficients} and/or
#'   \code{anova} tables containing RESI point estimates and CIs.
#' @importFrom sandwich vcovHC estfun
#' @importFrom CompQuadForm imhof
#' @importFrom car Anova
#' @importFrom lmtest coeftest
#' @importFrom stats coef hatvalues residuals fitted weights qnorm qchisq pchisq pnorm
#'   model.matrix terms uniroot
#' @export
resi_pe_asymptotic <- function(model.full,
                                data,
                                vcovfunc  = sandwich::vcovHC,
                                coefficients = TRUE,
                                anova        = TRUE,
                                alpha        = 0.05,
                                ci.method    = c("qf", "normal", "cf"),
                                type         = "HC3",
                                unbiased     = TRUE,
                                Anova.args   = list(),
                                vcov.args    = list(),
                                ...) {

  ci.method    <- match.arg(ci.method)
  model     <- model.full
  n         <- nrow(model.matrix(model))

  if (missing(data)) {
    data <- if (!is.null(model$model)) model$model else
      stop("data argument required")
  }

  # ---- vcovfunc with extra args ----
  if (length(vcov.args) > 0) {
    vcovfunc2 <- function(x) {
      args <- c(list(x), vcov.args)
      do.call(vcovfunc, args)
    }
  } else {
    vcovfunc2 <- vcovfunc
  }

  # ---- model-level precomputation (extended framework) ----
  # Treats SigmaX = X'X/n and SigmaXw = X'diag(tau*e^2)X/n as separate
  # estimators; phi cancels in A^{-1}BA^{-1}, giving a pure influence-function
  # variance estimate with no chain-rule approximation errors.
  precomp     <- .resi_precompute_ext(model, type = type)
  contrast_fn <- .resi_contrast_ext

  # ---- vcov matrix for point estimates (same as resi_pe) ----
  vcovmat <- tryCatch(vcovfunc2(model), error = function(e)
    stop("vcovfunc failed: ", conditionMessage(e)))

  output <- list()

  # ============================================================
  #  Coefficients table  (signed normal CI always)
  # ============================================================
  if (coefficients) {
    coef_names <- names(coef(model))[!is.na(coef(model))]
    is_lm      <- inherits(model, "lm") && !inherits(model, "glm")
    rdf        <- if (is_lm) model$df.residual else NULL

    # point estimates via coeftest
    ctab <- tryCatch(
      lmtest::coeftest(model, vcov. = vcovmat),
      error = function(e) stop("coeftest failed")
    )
    torZ <- if ("z value" %in% colnames(ctab)) "z" else "t"

    coef_df <- data.frame(
      Estimate    = ctab[, "Estimate"],
      `Std. Error` = ctab[, "Std. Error"],
      Statistic   = ctab[, paste(torZ, "value")],
      `p-value`   = ctab[, paste0("Pr(>|", torZ, "|)")],
      check.names = FALSE,
      row.names   = rownames(ctab)
    )
    colnames(coef_df)[3:4] <- c(paste(torZ, "value"),
                                  paste0("Pr(>|", torZ, "|)"))
    if (torZ == "z") {
      coef_df$RESI <- suppressWarnings(z2S(coef_df[[3]], n, unbiased))
    } else {
      coef_df$RESI <- suppressWarnings(t2S(coef_df[[3]], rdf, n, unbiased))
    }

    # asymptotic signed CIs
    ci_mat <- do.call(rbind, lapply(coef_names, function(cn) {
      L_mod  <- .get_L_coef(model, cn)
      contr  <- tryCatch(contrast_fn(precomp, L_mod, vcovmat_n = n * vcovmat),
                         error = function(e) NULL)
      if (is.null(contr)) return(c(LCI = NA_real_, UCI = NA_real_))
      if (ci.method == "cf") {
        .resi_ci_cf_signed(contr, alpha = alpha)
      } else {
        .resi_ci_normal_signed(contr, alpha = alpha)
      }
    }))

    coef_df[, paste0(c(alpha / 2, 1 - alpha / 2) * 100, "%")] <- ci_mat
    output$coefficients <- coef_df
  }

  # ============================================================
  #  Anova table  (unsigned CI; method depends on ci.method)
  # ============================================================
  if (anova) {
    is_lm <- inherits(model, "lm") && !inherits(model, "glm")

    # point estimates via car::Anova (matches resi_pe behaviour)
    if (is_lm) {
      anova_tab <- tryCatch(
        suppressMessages(do.call(car::Anova,
          c(list(mod = model, vcov. = vcovmat), Anova.args))),
        error = function(e) stop("car::Anova failed")
      )
      anova_tab <- anova_tab[rownames(anova_tab) != "Residuals", , drop = FALSE]
      rdf        <- model$df.residual
      anova_tab$RESI <- f2S(anova_tab[, "F"], anova_tab[, "Df"], rdf, n)
    } else {
      anova_tab <- tryCatch(
        suppressMessages(do.call(car::Anova,
          c(list(mod = model, test.statistic = "Wald",
                 vcov. = vcovmat), Anova.args))),
        error = function(e) stop("car::Anova failed")
      )
      anova_tab <- anova_tab[rownames(anova_tab) != "Residuals", , drop = FALSE]
      anova_tab$RESI <- chisq2S(anova_tab[, "Chisq"], anova_tab[, "Df"], n)
    }

    term_names <- rownames(anova_tab)

    # CI for each anova term
    ci_mat <- do.call(rbind, lapply(term_names, function(term) {
      L_mod <- tryCatch(
        .get_L_anova_term(model, term, vcovmat),
        error = function(e) NULL
      )
      if (is.null(L_mod) || nrow(L_mod) == 0)
        return(c(LCI = NA_real_, UCI = NA_real_))

      contr <- tryCatch(contrast_fn(precomp, L_mod, vcovmat_n = n * vcovmat),
                        error = function(e) NULL)
      if (is.null(contr)) return(c(LCI = NA_real_, UCI = NA_real_))

      if (ci.method == "qf") {
        tryCatch(.resi_ci_qf(contr, alpha = alpha),
                 error = function(e) c(LCI = NA_real_, UCI = NA_real_))
      } else if (ci.method == "cf") {
        tryCatch(.resi_ci_cf(contr, alpha = alpha),
                 error = function(e) c(LCI = NA_real_, UCI = NA_real_))
      } else {
        .resi_ci_normal_unsigned(contr, alpha = alpha)
      }
    }))

    anova_tab[, paste0(c(alpha / 2, 1 - alpha / 2) * 100, "%")] <- ci_mat
    class(anova_tab) <- c("anova_resi", class(anova_tab))
    output$anova <- anova_tab
  }

  output$alpha     <- alpha
  output$ci.method <- ci.method
  output$type      <- type
  class(output)    <- c("resi", "list")
  output
}


# ============================================================
#  Extended framework: Sigma_X and Sigma_Xw as separate estimators
# ============================================================

#' Extended precompute: adds SigmaXA, SigmaXB ingredients
#'
#' Generalizes to lm and glm (both parametric and robust).
#' The sandwich V_beta = SigmaXA^{-1} SigmaXB SigmaXA^{-1} is decomposed into
#' two moment-matrix estimators treated as independent:
#'   SigmaXA = -(1/n) sum_i c_i c_i' (bread moments; lm: X_i, glm: sqrt(w_i)*X_i)
#'   SigmaXB = (1/n) sum_i tau_i r_i^2 X_i X_i'  (HC-weighted meat)
#' where r_i are the appropriate residuals and tau_i the HC squared weights.
#' @noRd
.resi_precompute_ext <- function(model, type = "HC3") {
  is_glm <- inherits(model, "glm")
  is_lm  <- inherits(model, "lm") && !is_glm
  if (!is_lm && !is_glm)
    stop(".resi_precompute_ext: only lm or glm models supported")

  # For CI computation the extended framework always uses a consistent (robust)
  # variance estimator.  The 'const' type signals the parametric RESI point
  # estimate but must not carry over to the CI variance: upgrade to HC0 so that
  # tau_i = 1 (HC0) is used explicitly and 'is_const' does not affect B_full.
  ci_type <- if (type == "const") "HC0" else type

  precomp <- .resi_precompute(model, type = ci_type, deriv_method = "zeroB")
  X       <- precomp$X
  n       <- precomp$n
  sqrtw   <- precomp$sqrtw
  tau     <- sqrtw^2          # HC squared weights: tau_i = sqrtw_i^2

  if (is_lm) {
    # lm: SigmaXA = X'X/n,  SigmaXB = X'diag(tau*e^2)X/n
    # c_i = X_i (bread contribution),  r_i = e_i (residuals)
    r   <- residuals(model, "response")   # OLS residuals
    w_A <- rep(1, n)                       # bread weights: 1 for lm
    SigmaXA <- crossprod(X) / n
    SigmaXB <- crossprod(X, tau * r^2 * X) / n
  } else {
    # glm: SigmaXA = X'WX/n,  SigmaXB = X'diag(tau*(y-mu)^2)X/n
    # c_i = sqrt(w_i)*X_i (bread contribution),  r_i = y_i - mu_i
    w_A <- weights(model, type = "working")  # working weights w_i = mu_i(1-mu_i)
    r   <- residuals(model, type = "response")  # y - mu(beta_hat)
    SigmaXA <- crossprod(X * sqrt(w_A), X * sqrt(w_A)) / n   # X'WX/n
    SigmaXB <- crossprod(X, tau * r^2 * X) / n
  }

  precomp$SigmaXA     <- SigmaXA
  precomp$SigmaXA_inv <- .resi_safe_inv(SigmaXA)
  precomp$SigmaXB     <- SigmaXB
  precomp$r           <- r          # residuals for direct term
  precomp$w_A         <- w_A        # bread weights (1 for lm, w_i for glm)
  precomp$tau         <- tau
  precomp$deriv_method <- "extended"
  precomp
}

#' Extended contrast: Sigma_R via per-observation influence functions
#'
#' General formulation for lm and glm (parametric and robust).
#' The sandwich decomposes as V_beta = SigmaXA^{-1} SigmaXB SigmaXA^{-1} where:
#'   SigmaXA = bread moment matrix (lm: X'X/n; glm: X'WX/n)
#'   SigmaXB = HC-weighted meat    (lm: X'diag(tau*e^2)X/n; glm: X'diag(tau*r^2)X/n)
#'
#' Per-observation influence function (m1-vector):
#'   phi_i = phi_direct_i + z_XB_i + z_XA_i
#' where (H = L SigmaXA^{-1}, d = eigenvalues of Sigma_beta_L = H SigmaXB H'):
#'   phi_direct_i: sqrt(tau_i)*r_i * Sigma_beta_L^{-1/2} H X_i    [direct beta]
#'   z_XB_i: -V*(W*(tau_i*r_i^2*atilde_i*atilde_i' - D))*VT_beta  [SigmaXB variance]
#'   z_XA_i:  V*(W*(atilde_i*delta_i' + delta_i*atilde_i' - 2D))*VT_beta [SigmaXA variance]
#' where atilde_i = V'H(sqrt(w_A_i)*X_i) and delta_i = V'G(sqrt(w_A_i)*X_i),
#' G = F*SigmaXB*SigmaXA^{-1}.  For lm: w_A_i = 1; for glm: w_A_i = w_i.
#'
#' Centering: E[tau_i r_i^2 atilde_i atilde_i^T]_eig = D  => const_XB = VT_beta/(2*sqrtd)
#'            E[atilde_i delta_i^T + delta_i atilde_i^T]_eig = 2D => const_XA = 2*const_XB
#' @noRd
.resi_contrast_ext <- function(precomp_ext, L_model, vcovmat_n = NULL) {
  stopifnot(identical(precomp_ext$deriv_method, "extended"))

  m1          <- nrow(L_model)
  n           <- precomp_ext$n
  X           <- precomp_ext$X
  r           <- precomp_ext$r           # residuals (e for lm, y-mu for glm)
  tau         <- precomp_ext$tau
  w_A         <- precomp_ext$w_A         # bread weights (1 for lm, w_i for glm)
  SigmaXA_inv <- precomp_ext$SigmaXA_inv
  SigmaXB     <- precomp_ext$SigmaXB

  # H = L_model SigmaXA^{-1}  (m1 x p)
  H <- L_model %*% SigmaXA_inv

  # Sigma_beta_L = H SigmaXB H'  (m1 x m1)
  # General sandwich: V_beta = SigmaXA^{-1} SigmaXB SigmaXA^{-1}
  # For lm: phi cancels in A^{-1}BA^{-1} giving SigmaXA=X'X/n, SigmaXB=SigmaXw.
  # For glm: A_beta = X'WX/n (SigmaXA), B_beta = HC-weighted meat (SigmaXB).
  Sigma_beta_L <- .resi_sym(H %*% SigmaXB %*% t(H))

  # EVD
  eig   <- eigen(Sigma_beta_L, symmetric = TRUE)
  d     <- pmax(eig$values, .Machine$double.eps)
  V_eig <- eig$vectors           # m1 x m1
  sqrtd <- sqrt(d)
  Phat  <- V_eig %*% diag(1/sqrtd, m1) %*% t(V_eig)   # Sigma_beta_L^{-1/2}
  W     <- outer(sqrtd, sqrtd, function(a, b) 1/(a*b*(a+b)))

  # Point estimate
  beta_hat <- as.vector(L_model %*% coef(precomp_ext$model))  # m1-vector
  R_beta   <- as.vector(Phat %*% beta_hat)   # m1-vector
  Stilde   <- sqrt(sum(R_beta^2))
  VT_beta  <- as.vector(t(V_eig) %*% beta_hat)   # m1-vector, eigenspace

  # Projected matrices (m1 x p)
  # F = V'H,  G = F*SigmaXB*SigmaXA^{-1}
  F_mat <- t(V_eig) %*% H
  G_mat <- F_mat %*% SigmaXB %*% SigmaXA_inv

  # Per-obs projections (m1 x n)
  # FX_tilde: for SigmaXB and direct — uses X_i directly (SigmaXB = X'diag(tau*r^2)X/n)
  #   E[tau_i r_i^2 FX_tilde_i FX_tilde_i^T]_eig = F SigmaXB F^T = D  => const_XB = VT_beta/(2*sqrtd) ✓
  # A_tilde: for SigmaXA — uses c_i = sqrt(w_A_i)*X_i (SigmaXA = sum_i w_A_i X_i X_i^T / n)
  #   E[A_tilde_i A_tilde_i^T]_eig = F SigmaXA F^T = D  => const_XA = 2*VT_beta/(2*sqrtd) ✓
  FX_tilde <- F_mat %*% t(X)           # m1 x n: F X_i  (unweighted, for B and direct)
  C_mat    <- sweep(t(X), 2, sqrt(w_A), '*')  # p x n: sqrt(w_A_i)*X_i
  A_tilde  <- F_mat %*% C_mat           # m1 x n: F * sqrt(w_A_i)*X_i  (for SigmaXA)
  D_tilde  <- G_mat %*% C_mat           # m1 x n: G * sqrt(w_A_i)*X_i  (for SigmaXA)

  # Lyapunov-weighted projections: Wvb[j,k] = W[j,k]*VT_beta[k]
  Wvb       <- W * matrix(VT_beta, m1, m1, byrow = TRUE)   # m1 x m1
  Q_B_mat   <- Wvb %*% FX_tilde   # m1 x n: for z_XB (uses unweighted FX)
  Q_mat     <- Wvb %*% A_tilde    # m1 x n: for z_XA (uses w_A-weighted atilde)
  Q_D_mat   <- Wvb %*% D_tilde    # m1 x n: for z_XA delta term

  # Centering constants (m1-vectors, eigenspace):
  # E[tau_i r_i^2 FX_tilde_i FX_tilde_i^T]_eig = F SigmaXB F^T = D => const_XB = VT_beta/(2*sqrtd)
  # E[A_tilde_i delta_i^T + delta_i A_tilde_i^T]_eig = 2D           => const_XA = 2*const_XB
  const_XB <- VT_beta / (2 * sqrtd)   # m1-vector
  const_XA <- 2 * const_XB

  # Assemble per-obs influence functions (m1 x n)
  # (1) Direct beta: sqrt(tau_i)*r_i * Sigma_beta_L^{-1/2} * H * X_i  (uses FX_tilde)
  sc <- sqrt(tau) * r   # n-vector: HC-weighted residuals
  phi_direct <- (V_eig %*% sweep(FX_tilde, 1, sqrtd, '/')) *
                matrix(sc, m1, n, byrow = TRUE)

  # (2) SigmaXB: -V*(W*(tau_i*r_i^2*FX_tilde_i FX_tilde_i' - D))*VT_beta  (uses FX_tilde)
  XB_core  <- sweep(FX_tilde * Q_B_mat, 2, tau * r^2, '*') -
               matrix(const_XB, m1, n)
  phi_XB_m <- -V_eig %*% XB_core

  # (3) SigmaXA: V*(W*(A_tilde_i delta_i' + delta_i A_tilde_i' - 2D))*VT_beta  (uses A_tilde)
  XA_core  <- A_tilde * Q_D_mat + D_tilde * Q_mat - matrix(const_XA, m1, n)
  phi_XA_m <- V_eig %*% XA_core

  phi_ext <- phi_direct + phi_XB_m + phi_XA_m   # m1 x n

  # Sigma_R_ext = (1/n) * phi_ext %*% t(phi_ext)  (m1 x m1)
  Sigma_R_ext <- .resi_sym(tcrossprod(phi_ext) / n)

  list(
    m1         = m1,
    n          = n,
    beta_hat   = beta_hat,
    Sigma_beta = Sigma_beta_L,
    R_beta     = R_beta,
    Stilde     = Stilde,
    dR_dtheta  = NULL,
    dR_direct  = NULL,
    dR_Achain  = NULL,
    dR_Bchain  = NULL,
    Sigma_R    = Sigma_R_ext,
    phi_tilde  = if (m1 == 1L) drop(phi_ext) else NULL
  )
}


# ============================================================
#  Utility helpers (all unexported)
# ============================================================

#' @noRd
.resi_mean_list <- function(lst) {
  Reduce("+", lst) / length(lst)
}

#' @noRd
.resi_mean_outer <- function(psi_list, sqrtw, lm_phi_row = FALSE) {
  n <- length(psi_list)
  m <- ncol(psi_list[[1]])
  B <- matrix(0, m, m)
  for (i in seq_len(n)) {
    psi_i <- psi_list[[i]]   # 1 x m
    if (lm_phi_row) {
      # phi row unweighted, beta rows weighted
      psi_w          <- psi_i
      psi_w[1, 2:m]  <- psi_i[1, 2:m] * sqrtw[i]
    } else {
      psi_w <- psi_i * sqrtw[i]
    }
    B <- B + crossprod(psi_w)
  }
  B / n
}

#' @noRd
.resi_sym <- function(M) (M + t(M)) / 2

#' @noRd
.resi_safe_inv <- function(M) {
  tryCatch(chol2inv(chol(M)), error = function(e) solve(M))
}

#' Truncated CI for unsigned RESI (Algorithm 1, Zhang et al. 2025)
#' @noRd
.resi_truncate_ci <- function(Shat, se, n, m1, alpha = 0.05) {
  z1 <- qnorm(1 - alpha / 2)
  SL <- Shat - z1 * se
  SU <- Shat + z1 * se

  if (SL > 0) return(c(SL, SU))

  # probability mass at/below zero under H0: P(T^2 > n*Shat^2 | S=0)
  # Under H0, T^2 ~ chi^2_{m1}, so P(T^2 > m1 + n*Shat^2) (since Shat^2 = (T^2-m1)/n)
  gamma_p <- pchisq(m1 + n * Shat^2, df = m1, lower.tail = FALSE)

  if (gamma_p < alpha / 2) {
    SU <- Shat + qnorm(1 - (alpha - gamma_p)) * se
  } else {
    SU <- Shat + qnorm(1 - alpha) * se
  }
  c(0, SU)
}

## Tests for asymptotic CI functions (resi_pe_asymptotic, resi ci.method)
## These are fast (no bootstrap) tests checking mathematical properties.

library(sandwich)
library(car)

data <- RESI::insurance

# Fit a few standard models used throughout
mod.lm  <- lm(charges ~ region * age + bmi + sex, data = data)
mod.glm <- glm(smoker ~ age + region, data = data, family = "binomial")


# ---------------------------------------------------------------------------
# Helper: run both ci.method values and return the output
# ---------------------------------------------------------------------------
asym_n <- resi_pe_asymptotic(mod.lm,  ci.method = "normal")
asym_q <- resi_pe_asymptotic(mod.lm,  ci.method = "qf")
asym_g <- resi_pe_asymptotic(mod.glm, ci.method = "normal")


# ===========================================================================
# 1. Output structure
# ===========================================================================
test_that("resi_pe_asymptotic returns expected structure", {
  expect_s3_class(asym_n, "resi")

  # Both tables present
  expect_true(!is.null(asym_n$coefficients))
  expect_true(!is.null(asym_n$anova))

  # CI columns named correctly for default alpha = 0.05
  ci_cols <- c("2.5%", "97.5%")
  expect_true(all(ci_cols %in% colnames(asym_n$coefficients)))
  expect_true(all(ci_cols %in% colnames(asym_n$anova)))
  expect_true(all(ci_cols %in% colnames(asym_q$anova)))

  # RESI column present
  expect_true("RESI" %in% colnames(asym_n$coefficients))
  expect_true("RESI" %in% colnames(asym_n$anova))

  # custom alpha produces correct column names
  out_01 <- resi_pe_asymptotic(mod.lm, alpha = 0.01, ci.method = "normal")
  expect_true(all(c("0.5%", "99.5%") %in% colnames(out_01$coefficients)))

  # GLM also works
  expect_s3_class(asym_g, "resi")
  expect_true(!is.null(asym_g$anova))
})


# ===========================================================================
# 2. Point estimates match resi_pe()
# ===========================================================================
test_that("resi_pe_asymptotic RESI point estimates match resi_pe()", {
  pe_lm  <- resi_pe(mod.lm)
  pe_glm <- resi_pe(mod.glm)

  # lm: coefficients
  expect_equal(asym_n$coefficients[, "RESI"],
               pe_lm$coefficients[, "RESI"],
               tolerance = 1e-6,
               label = "lm coefficients RESI")

  # lm: anova (compare on non-Residuals rows)
  anova_rows <- rownames(asym_n$anova)
  expect_equal(asym_n$anova[anova_rows, "RESI"],
               pe_lm$anova[anova_rows, "RESI"],
               tolerance = 1e-6,
               label = "lm anova RESI")

  # glm: anova
  anova_rows_g <- rownames(asym_g$anova)
  expect_equal(asym_g$anova[anova_rows_g, "RESI"],
               pe_glm$anova[anova_rows_g, "RESI"],
               tolerance = 1e-6,
               label = "glm anova RESI")
})


# ===========================================================================
# 3. RESI is contained within its own CI (unsigned / anova)
# ===========================================================================
test_that("RESI point estimate is within asymptotic CIs (anova, unsigned)", {
  # normal method
  resi_col  <- asym_n$anova[, "RESI"]
  lo_col    <- asym_n$anova[, "2.5%"]
  hi_col    <- asym_n$anova[, "97.5%"]
  expect_true(all(lo_col <= resi_col + 1e-10),
              label = "normal: LCI <= RESI (anova)")
  expect_true(all(resi_col <= hi_col + 1e-10),
              label = "normal: RESI <= UCI (anova)")

  # qf method
  resi_col_q <- asym_q$anova[, "RESI"]
  lo_col_q   <- asym_q$anova[, "2.5%"]
  hi_col_q   <- asym_q$anova[, "97.5%"]
  expect_true(all(lo_col_q <= resi_col_q + 1e-10),
              label = "qf: LCI <= RESI (anova)")
  expect_true(all(resi_col_q <= hi_col_q + 1e-10),
              label = "qf: RESI <= UCI (anova)")
})


# ===========================================================================
# 4. Non-negativity of unsigned CIs (anova table)
# ===========================================================================
test_that("Asymptotic anova CIs are non-negative", {
  expect_true(all(asym_n$anova[, "2.5%"]  >= -1e-10))
  expect_true(all(asym_n$anova[, "97.5%"] >  0))
  expect_true(all(asym_q$anova[, "2.5%"]  >= -1e-10))
  expect_true(all(asym_q$anova[, "97.5%"] >  0))
})


# ===========================================================================
# 5. Monotonicity: wider alpha gives narrower CI
# ===========================================================================
test_that("Wider alpha gives narrower asymptotic CI", {
  out_05 <- resi_pe_asymptotic(mod.lm, alpha = 0.05, ci.method = "normal")
  out_10 <- resi_pe_asymptotic(mod.lm, alpha = 0.10, ci.method = "normal")

  width_05_coef <- out_05$coefficients[, "97.5%"] - out_05$coefficients[, "2.5%"]
  width_10_coef <- out_10$coefficients[, "95%"]   - out_10$coefficients[, "5%"]
  expect_true(all(width_10_coef <= width_05_coef + 1e-10),
              label = "coefficients: alpha=0.10 width <= alpha=0.05 width")

  width_05_an <- out_05$anova[, "97.5%"] - out_05$anova[, "2.5%"]
  width_10_an <- out_10$anova[, "95%"]   - out_10$anova[, "5%"]
  expect_true(all(width_10_an <= width_05_an + 1e-10),
              label = "anova: alpha=0.10 width <= alpha=0.05 width")
})


# ===========================================================================
# 6. Normal and QF CIs have similar widths for lm (large n = 1338)
# ===========================================================================
test_that("Normal and QF anova CIs are close for large n (lm)", {
  # With n=1338, both methods should agree within ~30% of CI width.
  width_n <- asym_n$anova[, "97.5%"] - asym_n$anova[, "2.5%"]
  width_q <- asym_q$anova[, "97.5%"] - asym_q$anova[, "2.5%"]

  # Relative difference in width should be < 30% for each term
  rel_diff <- abs(width_q - width_n) / pmax(width_n, 1e-6)
  expect_true(all(rel_diff < 0.30),
              info  = paste("Relative width differences:", round(rel_diff, 3)),
              label = "normal vs QF CI widths agree within 30%")
})


# ===========================================================================
# 7. CI bounds agree on coefficients: normal and QF both use signed normal
# ===========================================================================
test_that("QF and normal coefficient CIs are identical (both use signed normal)", {
  out_qf <- resi_pe_asymptotic(mod.lm, ci.method = "qf")
  # Coefficients CIs are always from the signed normal path regardless of ci.method
  expect_equal(asym_n$coefficients[, "2.5%"],
               out_qf$coefficients[, "2.5%"],
               tolerance = 1e-8,
               label = "LCI: normal == qf for coefficients")
  expect_equal(asym_n$coefficients[, "97.5%"],
               out_qf$coefficients[, "97.5%"],
               tolerance = 1e-8,
               label = "UCI: normal == qf for coefficients")
})


# ===========================================================================
# 8. resi() with ci.method="normal" / "qf" skips bootstrap and returns CIs
# ===========================================================================
test_that("resi() ci.method='normal' returns CIs without bootstrap", {
  # Should complete quickly (no boot) and have CI columns
  out_r <- resi(mod.lm, ci.method = "normal")
  expect_s3_class(out_r, "resi")
  expect_true("ci.method" %in% names(out_r))
  expect_equal(out_r$ci.method, "normal")
  expect_true(all(c("2.5%", "97.5%") %in% colnames(out_r$coefficients)))
  expect_true(all(c("2.5%", "97.5%") %in% colnames(out_r$anova)))

  out_rq <- resi(mod.lm, ci.method = "qf")
  expect_s3_class(out_rq, "resi")
  expect_equal(out_rq$ci.method, "qf")
})


# ===========================================================================
# 9. Correctly handles anova = FALSE or coefficients = FALSE
# ===========================================================================
test_that("resi_pe_asymptotic respects anova/coefficients flags", {
  out_no_an  <- resi_pe_asymptotic(mod.lm, anova = FALSE)
  out_no_co  <- resi_pe_asymptotic(mod.lm, coefficients = FALSE)

  expect_null(out_no_an$anova)
  expect_true(!is.null(out_no_an$coefficients))

  expect_null(out_no_co$coefficients)
  expect_true(!is.null(out_no_co$anova))
})


# ===========================================================================
# 10. Under a known-null predictor, QF lower bound should be 0
# ===========================================================================
test_that("QF LCI = 0 for near-null predictor", {
  # region:age interaction is near-null in lm (RESI ~ 0)
  out_q <- resi_pe_asymptotic(mod.lm, ci.method = "qf")
  interaction_lci <- out_q$anova["region:age", "2.5%"]
  expect_equal(interaction_lci, 0, tolerance = 1e-6,
               label = "region:age QF lower CI should be 0")
})


# ===========================================================================
# 11. Signed coefficient CIs: negative RESI can have negative LCI
# ===========================================================================
test_that("Signed coefficient CIs allow negative values", {
  resi_signed <- asym_n$coefficients[, "RESI"]
  lci_signed  <- asym_n$coefficients[, "2.5%"]

  # If any RESI is negative, its LCI should also be negative
  neg_resi <- resi_signed[resi_signed < 0]
  if (length(neg_resi) > 0) {
    neg_lci <- lci_signed[resi_signed < 0]
    expect_true(all(neg_lci < 0),
                label = "Negative RESI should have negative LCI")
  }
})


# ===========================================================================
# 12. RESI point estimates are identical across all CI methods
# ===========================================================================
test_that("RESI point estimates are identical across ci.method values", {
  out_norm <- resi(mod.lm, ci.method = "normal")
  out_qf   <- resi(mod.lm, ci.method = "qf")
  set.seed(1L)
  out_boot <- resi(mod.lm, nboot = 5L)

  # anova table
  expect_equal(out_norm$anova[, "RESI"], out_qf$anova[, "RESI"],
               label = "anova RESI: normal == qf")
  expect_equal(out_norm$anova[, "RESI"], out_boot$anova[, "RESI"],
               label = "anova RESI: normal == boot")

  # coefficients table
  expect_equal(out_norm$coefficients[, "RESI"], out_qf$coefficients[, "RESI"],
               label = "coeff RESI: normal == qf")
  expect_equal(out_norm$coefficients[, "RESI"], out_boot$coefficients[, "RESI"],
               label = "coeff RESI: normal == boot")

  # GLM
  out_gnorm <- resi(mod.glm, ci.method = "normal")
  out_gqf   <- resi(mod.glm, ci.method = "qf")
  expect_equal(out_gnorm$anova[, "RESI"], out_gqf$anova[, "RESI"],
               label = "GLM anova RESI: normal == qf")
  expect_equal(out_gnorm$coefficients[, "RESI"], out_gqf$coefficients[, "RESI"],
               label = "GLM coeff RESI: normal == qf")
})

#' Convert RESI (S) estimate to Z statistic
#'
#' Converts the robust effect size index (S) to Z statistic.
#' Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
#' @param S The value of the RESI estimate.
#' @param n Number of independent samples.
#' @param unbiased Logical, whether the unbiased or alternative estimator was used to compute RESI estimate. Default is TRUE.
#' @return Returns a scalar or vector argument of the Chi-square statistic.
#' @details The formula for converting a RESI estimate to a corresponding Z statistic depends on which estimator
#' was used to compute the RESI estimate (unbiased vs. alternative, see \code{\link{z2S}}). For the unbiased estimator,
#' the RESI can be positive or negative and there is a 1-1 transformation from S to Z. The formula for converting S (unbiased) to the
#' Z statistic is:
#'
#' \eqn{\sqrt(n)*S}
#'
#' For the alternative formula, if the RESI estimate is 0, the Z statistic is only known within an interval, [-1, 1].
#' For a non-zero S, the formula is:
#'
#' \eqn{\sqrt{S^2}/S\sqrt(n*abs(S) + 1)}
#'
#' @examples
#' # convert S estimates with corresponding degrees of freedom to
#' # Z statistics estimates (using unbiased formula)
#' S_ests = c(-0.2, 0, 0.1)
#' S2z(S = S_ests, n = 300, unbiased = TRUE)
#'
#' # convert S estimates with corresponding degrees of freedom to
#' # Z statistics estimates (using alernative formula)
#' S_ests = c(-0.2, 0, 0.1)
#' S2z(S = S_ests, n = 300, unbiased = FALSE)
#' @export
S2z = function(S, n, unbiased = TRUE){
  if (unbiased){
    z = sqrt(n)*S
  } else{
    z = sqrt(S^2)/S * sqrt(n*abs(S) + 1)
    if (any(S == 0)){
      if(length(S) == 1){
        stop(paste0("Function is not 1-1 for S = 0, Z statistic is between -1 and 1"))
      } else{
        z[S == 0]= NA
        warning("Function is not 1-1 for S = 0, Z statistic is between -1 and 1")
      }
    }
  }
  return(z)
}

# [AI-generated: Claude Sonnet 4.6, 2026-06-15]
# [Prompt summary: Plasmode simulation pipeline for Goals 1 and 2 - RESI bootstrap CI evaluation]

# ============================================================
#  Internal helpers
# ============================================================

# Extract aligned RESI point estimates and CI bounds from one resi object
# for a specified set of row names (terms).
.simExtractRows <- function(resi_obj, table_name, terms, ci_lo, ci_hi) {
  tab <- resi_obj[[table_name]]
  if (is.null(tab)) {
    na_vec <- setNames(rep(NA_real_, length(terms)), terms)
    return(list(resi = na_vec, lo = na_vec, hi = na_vec))
  }
  idx <- match(terms, rownames(tab))
  list(
    resi = setNames(tab[idx, "RESI"],  terms),
    lo   = setNames(if (ci_lo %in% colnames(tab)) tab[idx, ci_lo] else
                      rep(NA_real_, length(terms)), terms),
    hi   = setNames(if (ci_hi %in% colnames(tab)) tab[idx, ci_hi] else
                      rep(NA_real_, length(terms)), terms)
  )
}

# Overwrite RESI column in a resi_pe object with direct (population) formulas:
#   T-stat  -> S = t / sqrt(n)          (signed)
#   Z-stat  -> S = z / sqrt(n)          (signed)
#   F-stat  -> S = sqrt(F * df / n)     (no rdf or df correction)
#   Chisq   -> S = sqrt(chisq / n)      (no df subtraction)
# This treats the full-dataset test statistics as the true population parameters
# rather than using small-sample-corrected estimators from resi_pe.
.simDirectRESI <- function(pe_obj, n, rdf = NULL) {
  if (!is.null(pe_obj$coefficients)) {
    tab <- pe_obj$coefficients
    if ("t value" %in% colnames(tab)) {
      # For parametric lm, rdf = n-p is passed so that S_true = t/sqrt(rdf),
      # which targets beta/sqrt(diag(n*(X'X)^{-1}*mean(resid^2))) -- the
      # population RESI using sigma^2 = mean(resid^2) with no df correction.
      denom <- if (!is.null(rdf)) sqrt(rdf) else sqrt(n)
      tab[, "RESI"] <- tab[, "t value"] / denom
    } else if ("z value" %in% colnames(tab)) {
      tab[, "RESI"] <- tab[, "z value"] / sqrt(n)
    }
    pe_obj$coefficients <- tab
  }
  if (!is.null(pe_obj$anova)) {
    tab <- pe_obj$anova
    if ("F" %in% colnames(tab)) {
      # Same logic: S_true = sqrt(F * df / rdf) when rdf is supplied.
      denom <- if (!is.null(rdf)) rdf else n
      tab[, "RESI"] <- sqrt(tab[, "F"] * tab[, "Df"] / denom)
    } else if ("Chisq" %in% colnames(tab)) {
      tab[, "RESI"] <- sqrt(tab[, "Chisq"] / n)
    }
    pe_obj$anova <- tab
  }
  pe_obj
}

# Compute per-term simulation metrics from a list of resi objects.
# Returns a data.frame with rows = terms and columns = bias, mse, coverage, width.
.simComputeMetrics <- function(reps, table_name, true_resi, ci_lo, ci_hi,
                                exclude_rows = NULL) {
  terms <- rownames(true_resi)
  if (!is.null(exclude_rows)) terms <- setdiff(terms, exclude_rows)
  if (length(terms) == 0L) return(NULL)

  extracted <- lapply(reps, .simExtractRows,
                      table_name = table_name, terms = terms,
                      ci_lo = ci_lo, ci_hi = ci_hi)

  resi_mat <- do.call(rbind, lapply(extracted, `[[`, "resi"))
  lo_mat   <- do.call(rbind, lapply(extracted, `[[`, "lo"))
  hi_mat   <- do.call(rbind, lapply(extracted, `[[`, "hi"))

  tv       <- true_resi[terms, "RESI"]
  diff_mat <- sweep(resi_mat, 2L, tv, "-")
  in_ci    <- sweep(lo_mat,   2L, tv, "<=") & sweep(hi_mat, 2L, tv, ">=")

  # upper_coverage: P(hi >= tv) -- CI upper bound is at or above the true value
  # lower_coverage: P(lo <= tv) -- CI lower bound is at or below the true value
  upper_cov_mat <- sweep(hi_mat, 2L, tv, ">=")
  lower_cov_mat <- sweep(lo_mat, 2L, tv, "<=")

  data.frame(
    bias           = colMeans(diff_mat,           na.rm = TRUE),
    mse            = colMeans(diff_mat ^ 2L,      na.rm = TRUE),
    empirical_sd   = apply(resi_mat, 2L, sd,      na.rm = TRUE),
    coverage       = colMeans(in_ci,              na.rm = TRUE),
    upper_coverage = colMeans(upper_cov_mat,      na.rm = TRUE),
    lower_coverage = colMeans(lower_cov_mat,      na.rm = TRUE),
    width          = colMeans(hi_mat - lo_mat,    na.rm = TRUE),
    row.names = terms,
    stringsAsFactors = FALSE
  )
}


# ============================================================
#  insurancePlasmodeSim
# ============================================================

#' Insurance Plasmode Simulation for RESI Evaluation
#'
#' Runs a plasmode simulation study using the \code{\link{insurance}} dataset to
#' evaluate RESI confidence interval performance. In each replicate, \code{n}
#' observations are resampled with replacement from the full insurance dataset
#' (\emph{N} = 1338). The RESI point estimates from \code{\link{resi_pe}} applied
#' to the full dataset are treated as the true parameter values for computing bias,
#' MSE, CI coverage, and CI width.
#'
#' Two models are evaluated:
#' \itemize{
#'   \item \strong{lm}: \code{log10(charges) ~ ns(age, df=3) * sex + bmi + smoker + region}
#'   \item \strong{glm}: \code{I(charges > 10000) ~ ns(age, df=3) * sex + bmi + smoker + region}
#'     with \code{family = binomial()}
#' }
#' Each model is evaluated under both parametric (\code{vcovfunc = stats::vcov}) and
#' robust (\code{vcovfunc = sandwich::vcovHC}) variance settings, yielding four
#' simulation conditions.
#'
#' Parallelization is via \code{\link[parallel]{mclapply}}, which uses forking and is
#' not supported on Windows (falls back to sequential evaluation on Windows).
#'
#' @param nsim Integer, number of simulation replicates per (setting, \code{n}) cell.
#'   Default 1000. Use 10 for initial testing.
#' @param n.vec Integer vector of sample sizes. Default
#'   \code{c(50, 100, 200, 500, 1000, 2000, 5000)}.
#' @param nboot Integer, bootstrap replicates per internal \code{\link{resi}} call.
#'   Default 500. Use 10 for initial testing. Ignored when \code{ci.method != "boot"}.
#' @param alpha Numeric, CI significance level. Default 0.05.
#' @param ci.method Character, CI method passed to \code{\link{resi}}. One of
#'   \code{"boot"} (bootstrap, default), \code{"normal"} (asymptotic truncated-normal),
#'   or \code{"qf"} (asymptotic quadratic-form / Imhof). When \code{ci.method != "boot"},
#'   \code{nboot} is ignored.
#' @param output.dir Character, path to the directory where all results are saved.
#'   Created if it does not exist. Defaults to \code{"resiBootSim"},
#'   \code{"resiAsympNormalSim"}, or \code{"resiAsympQFSim"} based on
#'   \code{ci.method} when \code{NULL}.
#' @param fixed.knots Logical. If \code{TRUE}, spline knots are fixed at the
#'   empirical tertiles of \code{age} in the full insurance dataset rather than
#'   re-selected by \code{df = 3} in each bootstrap sample. Default \code{FALSE}.
#' @param mc.cores.settings Integer, cores for the outer
#'   \code{mclapply} over (setting \eqn{\times} sample size) combinations. Default 1.
#' @param mc.cores.reps Integer, cores for the inner \code{mclapply} over simulation
#'   replicates within each (setting, \code{n}) cell. Default 1.
#'
#' @return Invisibly returns the summary metrics \code{data.frame}. Side effects:
#' \itemize{
#'   \item \code{output.dir/sim_raw/<setting>_n<n>.rds}: list of per-replicate
#'     \code{anova} and \code{coefficients} tables.
#'   \item \code{output.dir/summary_table.rds}: combined metrics table with columns
#'     \code{model, vcov, n, n_success, table, term, bias, mse, coverage, width}.
#' }
#' @seealso \code{\link{simFigures}}, \code{\link{simCompareMethodsFigures}},
#'   \code{\link{resi}}, \code{\link{resi_pe}}
#' @importFrom parallel mclapply
#' @importFrom splines ns
#' @importFrom sandwich vcovHC
#' @importFrom stats lm glm vcov binomial
#' @export
insurancePlasmodeSim <- function(nsim              = 1000L,
                                  n.vec             = c(50, 100, 200, 500, 1000, 2000, 5000),
                                  nboot             = 500L,
                                  alpha             = 0.05,
                                  ci.method         = c("boot", "normal", "qf", "cf"),
                                  output.dir        = NULL,
                                  fixed.knots       = FALSE,
                                  mc.cores.settings = 1L,
                                  mc.cores.reps     = 1L) {

  ci.method <- match.arg(ci.method)
  if (is.null(output.dir)) {
    output.dir <- switch(ci.method,
      boot   = "resiBootSim",
      normal = "resiAsympNormalSim",
      qf     = "resiAsympQFSim",
      cf     = "resiAsympCFSim"
    )
  }

  insurance <- RESI::insurance

  # Pre-compute expected factor levels for plasmode resampling check
  .fvars   <- c("sex", "smoker", "region")
  .flevels <- lapply(.fvars, function(v) unique(insurance[[v]]))
  names(.flevels) <- .fvars

  ci_lo <- paste0(alpha / 2 * 100, "%")
  ci_hi <- paste0((1 - alpha / 2) * 100, "%")

  if (fixed.knots) {
    .age_knots <- quantile(insurance$age, c(1/3, 2/3))
    .age_bk    <- range(insurance$age)
    lm_formula  <- eval(bquote(log10(charges) ~ splines::ns(age, knots = .(.age_knots), Boundary.knots = .(.age_bk)) * sex + bmi + smoker + region))
    glm_formula <- eval(bquote(I(charges > 15000) ~ splines::ns(age, knots = .(.age_knots), Boundary.knots = .(.age_bk)) * sex + bmi + smoker + region))
  } else {
    lm_formula  <- log10(charges) ~ splines::ns(age, df = 3) * sex + bmi + smoker + region
    glm_formula <- I(charges > 15000) ~ splines::ns(age, df = 3) * sex + bmi + smoker + region
  }

  model_settings <- list(
    list(type = "lm",  label = "lm_parametric",
         formula = lm_formula,  family = NULL,
         vcov_name = "parametric", vcovfunc = stats::vcov),
    list(type = "lm",  label = "lm_robust",
         formula = lm_formula,  family = NULL,
         vcov_name = "robust",     vcovfunc = sandwich::vcovHC),
    list(type = "glm", label = "glm_parametric",
         formula = glm_formula, family = stats::binomial(),
         vcov_name = "parametric", vcovfunc = stats::vcov),
    list(type = "glm", label = "glm_robust",
         formula = glm_formula, family = stats::binomial(),
         vcov_name = "robust",     vcovfunc = sandwich::vcovHC)
  )

  # --- True RESI values from the full dataset ------------------------------------
  message("Computing true RESI values from full insurance dataset (n = ",
          nrow(insurance), ")...")

  true_vals <- lapply(model_settings, function(s) {
    .formula <- s$formula
    .family  <- s$family
    full_mod <- if (s$type == "lm") {
      m <- lm(.formula, data = insurance)
      m$call[["formula"]] <- .formula
      m
    } else {
      m <- glm(.formula, data = insurance, family = .family)
      m$call[["formula"]] <- .formula
      m$call[["family"]]  <- .family
      m
    }
    # Robust true values: use HC0 (no hat-value correction) so that the
    # small-sample inflation from HC3 hat weights does not enter S_true.
    # Parametric lm true values: pass rdf = n-p so that S_true = t/sqrt(rdf),
    # targeting sigma^2 = mean(resid^2) (population value, no df correction).
    true_vcovfunc <- if (s$vcov_name == "robust") {
      function(x) sandwich::vcovHC(x, type = "HC0")
    } else {
      s$vcovfunc
    }
    rdf_true <- if (s$type == "lm" && s$vcov_name == "parametric") {
      full_mod$df.residual
    } else {
      NULL
    }
    .simDirectRESI(resi_pe(full_mod, data = insurance, vcovfunc = true_vcovfunc),
                   n = nrow(insurance), rdf = rdf_true)
  })
  names(true_vals) <- sapply(model_settings, `[[`, "label")

  # --- Output directories --------------------------------------------------------
  sim_raw_dir <- file.path(output.dir, "sim_raw")
  dir.create(sim_raw_dir,                      recursive = TRUE, showWarnings = FALSE)
  dir.create(file.path(output.dir, "figures"), recursive = TRUE, showWarnings = FALSE)

  # --- (setting x n) grid -------------------------------------------------------
  grid <- expand.grid(
    setting_idx = seq_along(model_settings),
    n           = n.vec,
    stringsAsFactors = FALSE
  )

  message("Running ", nsim, " replicates for each of ", nrow(grid),
          " (setting x sample size) combinations ...")

  # --- Outer mclapply over (setting x n) ----------------------------------------
  all_metrics <- parallel::mclapply(seq_len(nrow(grid)), function(g) {

    s_idx <- grid$setting_idx[g]
    n     <- grid$n[g]
    s     <- model_settings[[s_idx]]
    tv    <- true_vals[[s_idx]]

    setting_label <- paste0(s$label, "_n", n)

    # --- Inner mclapply over replicates -----------------------------------------
    reps_raw <- parallel::mclapply(seq_len(nsim), function(i) {
      # Resample with replacement until the plasmode sample is well-conditioned:
      # (a) all factor levels present; (b) for GLM, both binary outcome values
      #     must appear within each smoker stratum (prevents complete separation).
      repeat {
        sample_data <- insurance[sample(nrow(insurance), n, replace = TRUE), ]
        ok <- all(vapply(.fvars, function(v)
          all(.flevels[[v]] %in% sample_data[[v]]), logical(1L)))
        if (ok && s$type == "glm") {
          biny <- as.integer(sample_data$charges > 15000)
          ok   <- length(unique(biny)) > 1L &&
            all(tapply(biny, sample_data$smoker,
                       function(x) length(unique(x)) > 1L))
        }
        if (ok) break
      }

      # Embed the evaluated formula (and family for glm) directly into mod$call
      # so that resi()'s internal update() calls can resolve them without needing
      # 's' to be in scope (it is not accessible inside boot::boot / resi_stat).
      .formula <- s$formula
      .family  <- s$family
      mod <- tryCatch({
        m <- if (s$type == "lm") {
          lm(.formula, data = sample_data)
        } else {
          glm(.formula, data = sample_data, family = .family)
        }
        m$call[["formula"]] <- .formula
        if (s$type == "glm") m$call[["family"]] <- .family
        m
      }, error = function(e) NULL)
      if (is.null(mod)) return(NULL)

      tryCatch(
        resi(mod, data = sample_data, nboot = nboot, vcovfunc = s$vcovfunc,
             alpha = alpha, store.boot = FALSE, ci.method = ci.method),
        error = function(e) NULL
      )
    }, mc.cores = mc.cores.reps)

    reps <- Filter(Negate(is.null), reps_raw)
    n_success <- length(reps)

    # Save minimal per-replicate output (anova + coefficients tables only)
    reps_to_save <- lapply(reps, function(r) {
      list(anova = r$anova, coefficients = r$coefficients)
    })
    # Retry loop: on macOS, SIGCHLD from mclapply child processes can interrupt
    # gzfile() with "Interrupted system call"; a brief pause + retry resolves it.
    for (.attempt in seq_len(5L)) {
      saved <- tryCatch({
        Sys.sleep(0.05 * .attempt)
        saveRDS(reps_to_save,
                file = file.path(sim_raw_dir, paste0(setting_label, ".rds")))
        TRUE
      }, error = function(e) FALSE)
      if (saved) break
      if (.attempt == 5L)
        warning("saveRDS failed after 5 attempts for ", setting_label)
    }

    if (n_success == 0L) {
      warning("All replicates failed for setting: ", setting_label)
      return(NULL)
    }

    # --- Compute metrics --------------------------------------------------------
    anova_metrics <- .simComputeMetrics(reps, "anova",
                                         tv$anova, ci_lo, ci_hi,
                                         exclude_rows = "Residuals")
    coef_metrics  <- .simComputeMetrics(reps, "coefficients",
                                         tv$coefficients, ci_lo, ci_hi,
                                         exclude_rows = "(Intercept)")

    rows <- list()
    if (!is.null(anova_metrics)) {
      rows[["anova"]] <- data.frame(
        model     = s$type,
        vcov      = s$vcov_name,
        n         = n,
        n_success = n_success,
        table     = "anova",
        term      = rownames(anova_metrics),
        anova_metrics,
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }
    if (!is.null(coef_metrics)) {
      rows[["coef"]] <- data.frame(
        model     = s$type,
        vcov      = s$vcov_name,
        n         = n,
        n_success = n_success,
        table     = "coefficients",
        term      = rownames(coef_metrics),
        coef_metrics,
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }
    do.call(rbind, rows)

  }, mc.cores = mc.cores.settings)

  # --- Combine and save summary table -------------------------------------------
  summary_table <- do.call(rbind, Filter(Negate(is.null), all_metrics))
  saveRDS(summary_table, file = file.path(output.dir, "summary_table.rds"))

  message("Done. Results saved to: ", output.dir)
  invisible(summary_table)
}


# ============================================================
#  simRecomputeSummary
# ============================================================

#' Recompute Summary Table from Raw Simulation Output
#'
#' Reads the per-replicate raw \code{.rds} files saved by
#' \code{\link{insurancePlasmodeSim}} and recomputes the summary metrics table
#' (bias, MSE, coverage, upper_coverage, lower_coverage, width).  Use this
#' whenever \code{.simComputeMetrics} has been updated (e.g. new metrics added)
#' without needing to rerun the full simulation.
#'
#' True RESI values are re-estimated from the full \code{\link{insurance}} dataset
#' using the same formulae and variance functions as the original simulation.
#'
#' @param output.dir Character, directory containing simulation output.
#'   Default \code{"resiBootSim"}.
#' @param alpha Numeric, CI significance level used in the original simulation.
#'   Default 0.05.
#' @param fixed.knots Logical. Must match the value used in the original
#'   \code{\link{insurancePlasmodeSim}} call so that the true RESI values are
#'   computed from the same model formula. Default \code{FALSE}.
#'
#' @return Invisibly returns the updated summary \code{data.frame}.
#'   Overwrites \code{output.dir/summary_table.rds}.
#' @seealso \code{\link{insurancePlasmodeSim}}, \code{\link{simFigures}}
#' @importFrom splines ns
#' @importFrom sandwich vcovHC
#' @importFrom stats lm glm vcov binomial
#' @export
simRecomputeSummary <- function(output.dir  = "resiBootSim",
                                 alpha       = 0.05,
                                 fixed.knots = FALSE) {

  insurance <- RESI::insurance

  ci_lo <- paste0(alpha / 2 * 100, "%")
  ci_hi <- paste0((1 - alpha / 2) * 100, "%")

  if (fixed.knots) {
    .age_knots <- quantile(insurance$age, c(1/3, 2/3))
    .age_bk    <- range(insurance$age)
    lm_formula  <- eval(bquote(log10(charges) ~ splines::ns(age, knots = .(.age_knots), Boundary.knots = .(.age_bk)) * sex + bmi + smoker + region))
    glm_formula <- eval(bquote(I(charges > 15000) ~ splines::ns(age, knots = .(.age_knots), Boundary.knots = .(.age_bk)) * sex + bmi + smoker + region))
  } else {
    lm_formula  <- log10(charges) ~ splines::ns(age, df = 3) * sex + bmi + smoker + region
    glm_formula <- I(charges > 15000) ~ splines::ns(age, df = 3) * sex + bmi + smoker + region
  }

  model_settings <- list(
    list(type = "lm",  label = "lm_parametric",
         formula = lm_formula,  family = NULL,
         vcov_name = "parametric", vcovfunc = stats::vcov),
    list(type = "lm",  label = "lm_robust",
         formula = lm_formula,  family = NULL,
         vcov_name = "robust",     vcovfunc = sandwich::vcovHC),
    list(type = "glm", label = "glm_parametric",
         formula = glm_formula, family = stats::binomial(),
         vcov_name = "parametric", vcovfunc = stats::vcov),
    list(type = "glm", label = "glm_robust",
         formula = glm_formula, family = stats::binomial(),
         vcov_name = "robust",     vcovfunc = sandwich::vcovHC)
  )

  message("Recomputing true RESI values from full insurance dataset...")
  true_vals <- lapply(model_settings, function(s) {
    .formula <- s$formula
    .family  <- s$family
    full_mod <- if (s$type == "lm") {
      m <- lm(.formula, data = insurance)
      m$call[["formula"]] <- .formula
      m
    } else {
      m <- glm(.formula, data = insurance, family = .family)
      m$call[["formula"]] <- .formula
      m$call[["family"]]  <- .family
      m
    }
    true_vcovfunc <- if (s$vcov_name == "robust") {
      function(x) sandwich::vcovHC(x, type = "HC0")
    } else {
      s$vcovfunc
    }
    rdf_true <- if (s$type == "lm" && s$vcov_name == "parametric") {
      full_mod$df.residual
    } else {
      NULL
    }
    .simDirectRESI(resi_pe(full_mod, data = insurance, vcovfunc = true_vcovfunc),
                   n = nrow(insurance), rdf = rdf_true)
  })
  names(true_vals) <- sapply(model_settings, `[[`, "label")

  sim_raw_dir <- file.path(output.dir, "sim_raw")
  raw_files   <- list.files(sim_raw_dir, pattern = "\\.rds$", full.names = TRUE)
  if (length(raw_files) == 0L)
    stop("No raw .rds files found in: ", sim_raw_dir)

  setting_labels <- sapply(model_settings, `[[`, "label")

  all_metrics <- lapply(raw_files, function(f) {
    fname <- sub("\\.rds$", "", basename(f))
    n     <- suppressWarnings(as.integer(sub(".*_n", "", fname)))
    label <- sub("_n[0-9]+$", "", fname)

    s_idx <- which(setting_labels == label)
    if (length(s_idx) == 0L || is.na(n)) {
      warning("Could not parse setting/n from filename: ", basename(f))
      return(NULL)
    }
    s  <- model_settings[[s_idx]]
    tv <- true_vals[[s_idx]]

    reps_raw  <- readRDS(f)
    n_success <- length(reps_raw)
    if (n_success == 0L) return(NULL)

    anova_metrics <- .simComputeMetrics(reps_raw, "anova",
                                         tv$anova, ci_lo, ci_hi,
                                         exclude_rows = "Residuals")
    coef_metrics  <- .simComputeMetrics(reps_raw, "coefficients",
                                         tv$coefficients, ci_lo, ci_hi,
                                         exclude_rows = "(Intercept)")

    rows <- list()
    if (!is.null(anova_metrics)) {
      rows[["anova"]] <- data.frame(
        model     = s$type,
        vcov      = s$vcov_name,
        n         = n,
        n_success = n_success,
        table     = "anova",
        term      = rownames(anova_metrics),
        anova_metrics,
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }
    if (!is.null(coef_metrics)) {
      rows[["coef"]] <- data.frame(
        model     = s$type,
        vcov      = s$vcov_name,
        n         = n,
        n_success = n_success,
        table     = "coefficients",
        term      = rownames(coef_metrics),
        coef_metrics,
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }
    do.call(rbind, rows)
  })

  summary_table <- do.call(rbind, Filter(Negate(is.null), all_metrics))
  saveRDS(summary_table, file = file.path(output.dir, "summary_table.rds"))
  message("Done. Updated summary_table.rds saved to: ", output.dir)
  invisible(summary_table)
}


# ============================================================
#  simEstimatorFigures
# ============================================================

#' Estimator Comparison Figures: t2S/f2S vs z2S/chisq2S
#'
#' Reads the per-replicate raw \code{.rds} files from one simulation directory and
#' produces Bias + MSE comparison figures contrasting the RESI estimators actually
#' used for \code{lm} models against the simpler z-statistic / chi-squared alternatives:
#' \itemize{
#'   \item \strong{Coefficients table}: \code{\link{t2S}} (used) vs \code{\link{z2S}} (alternative)
#'   \item \strong{Anova table}: \code{\link{f2S}} (used) vs \code{\link{chisq2S}} (alternative)
#' }
#' Only \code{lm} models are included; GLM models are skipped because z2S and
#' chisq2S are the natural estimators there.
#'
#' @param sim.dir Character, directory containing simulation output with a
#'   \code{sim_raw/} sub-directory.  Default \code{"resiBootSim"}.
#' @param figures.dir Character, output directory for PDFs.
#'   Default \code{file.path(sim.dir, "figures", "estimator_compare")}.
#' @param alpha Numeric, nominal level (used only in figure titles). Default 0.05.
#' @param fixed.knots Logical. Must match the value used in the original
#'   \code{\link{insurancePlasmodeSim}} call. Default \code{FALSE}.
#'
#' @return Invisibly returns the combined estimator-comparison \code{data.frame}.
#' @seealso \code{\link{simCompareMethodsFigures}}, \code{\link{simRecomputeSummary}}
#' @importFrom grDevices pdf dev.off
#' @importFrom graphics plot lines points abline legend par axis plot.new layout
#' @importFrom stats lm vcov
#' @importFrom sandwich vcovHC
#' @importFrom splines ns
#' @export
simEstimatorFigures <- function(
    sim.dir     = "resiBootSim",
    figures.dir = NULL,
    alpha       = 0.05,
    fixed.knots = FALSE
) {
  if (is.null(figures.dir))
    figures.dir <- file.path(sim.dir, "figures", "estimator_compare")
  dir.create(figures.dir, recursive = TRUE, showWarnings = FALSE)

  insurance <- RESI::insurance

  # ---- Formula ---------------------------------------------------------------
  if (fixed.knots) {
    .age_knots <- quantile(insurance$age, c(1/3, 2/3))
    .age_bk    <- range(insurance$age)
    lm_formula <- eval(bquote(
      log10(charges) ~ splines::ns(age, knots = .(.age_knots),
                                   Boundary.knots = .(.age_bk)) * sex + bmi + smoker + region))
  } else {
    lm_formula <- log10(charges) ~ splines::ns(age, df = 3) * sex + bmi + smoker + region
  }

  # ---- lm settings only ------------------------------------------------------
  model_settings <- list(
    list(label = "lm_parametric", vcov_name = "parametric",
         vcovfunc = stats::vcov),
    list(label = "lm_robust",     vcov_name = "robust",
         vcovfunc = sandwich::vcovHC)
  )

  # ---- True S from full insurance dataset ------------------------------------
  message("Computing true RESI values from full insurance dataset...")
  true_vals <- lapply(model_settings, function(s) {
    m <- lm(lm_formula, data = insurance)
    m$call[["formula"]] <- lm_formula
    tv_vcov <- if (s$vcov_name == "robust") {
      function(x) sandwich::vcovHC(x, type = "HC0")
    } else {
      s$vcovfunc
    }
    rdf_tv <- if (s$vcov_name == "parametric") m$df.residual else NULL
    .simDirectRESI(resi_pe(m, data = insurance, vcovfunc = tv_vcov),
                   n = nrow(insurance), rdf = rdf_tv)
  })
  names(true_vals) <- sapply(model_settings, `[[`, "label")

  # ---- Colours / layout constants --------------------------------------------
  est_cols <- c("t2S / f2S" = "#1B6CA8", "z2S / chisq2S" = "#CC5500")
  row_h    <- 3.2
  leg_h    <- 0.55
  raw_dir  <- file.path(sim.dir, "sim_raw")

  # ---- Helper: extract one column from anova/coef table per replicate --------
  .extract_col <- function(reps, tbl_name, terms, col) {
    m <- matrix(NA_real_, nrow = length(reps), ncol = length(terms),
                dimnames = list(NULL, terms))
    for (ri in seq_along(reps)) {
      tab <- reps[[ri]][[tbl_name]]
      idx <- match(terms, rownames(tab))
      ok  <- !is.na(idx)
      if (any(ok)) m[ri, ok] <- tab[idx[ok], col]
    }
    m
  }

  # ---- Loop over lm settings -------------------------------------------------
  all_combined <- list()

  for (s in model_settings) {
    lbl   <- s$label
    vtype <- s$vcov_name
    tv    <- true_vals[[lbl]]

    raw_files <- list.files(raw_dir,
                            pattern = paste0("^", lbl, "_n[0-9]+\\.rds$"),
                            full.names = TRUE)
    if (length(raw_files) == 0L) {
      message("No raw files found for: ", lbl, " -- skipping")
      next
    }

    # -- Per-n summaries -------------------------------------------------------
    per_n <- lapply(raw_files, function(f) {
      n_sim <- as.integer(sub(".*_n([0-9]+)\\.rds$", "\\1", basename(f)))
      reps  <- readRDS(f)
      if (length(reps) == 0L) return(NULL)

      rows <- list()

      # Anova: f2S (RESI col) vs chisq2S(F*Df, Df, n)
      a_terms <- setdiff(rownames(tv$anova), "Residuals")
      if (length(a_terms) > 0L) {
        resi_f <- .extract_col(reps, "anova", a_terms, "RESI")
        resi_c <- matrix(NA_real_, nrow = length(reps), ncol = length(a_terms),
                         dimnames = list(NULL, a_terms))
        for (ri in seq_along(reps)) {
          tab <- reps[[ri]]$anova
          idx <- match(a_terms, rownames(tab))
          ok  <- !is.na(idx)
          if (any(ok)) {
            F_i  <- tab[idx[ok], "F"]
            Df_i <- tab[idx[ok], "Df"]
            resi_c[ri, ok] <- chisq2S(F_i * Df_i, Df_i, n_sim)
          }
        }
        tv_a <- tv$anova[a_terms, "RESI"]
        diff_f <- sweep(resi_f, 2L, tv_a, "-")
        diff_c <- sweep(resi_c, 2L, tv_a, "-")
        rows$anova <- rbind(
          data.frame(model = "lm", vcov = vtype, n = n_sim, table = "anova",
                     term = a_terms, estimator = "t2S / f2S",
                     bias = colMeans(diff_f, na.rm = TRUE),
                     mse  = colMeans(diff_f^2, na.rm = TRUE),
                     row.names = NULL, stringsAsFactors = FALSE),
          data.frame(model = "lm", vcov = vtype, n = n_sim, table = "anova",
                     term = a_terms, estimator = "z2S / chisq2S",
                     bias = colMeans(diff_c, na.rm = TRUE),
                     mse  = colMeans(diff_c^2, na.rm = TRUE),
                     row.names = NULL, stringsAsFactors = FALSE)
        )
      }

      # Coefficients: t2S (RESI col) vs z2S(t, n)
      c_terms <- setdiff(rownames(tv$coefficients), "(Intercept)")
      if (length(c_terms) > 0L) {
        coef_col <- "t value"
        resi_t <- .extract_col(reps, "coefficients", c_terms, "RESI")
        t_mat  <- .extract_col(reps, "coefficients", c_terms, coef_col)
        resi_z <- matrix(z2S(t_mat, n_sim), nrow = nrow(t_mat), ncol = ncol(t_mat),
                         dimnames = dimnames(t_mat))
        tv_c   <- tv$coefficients[c_terms, "RESI"]
        diff_t <- sweep(resi_t, 2L, tv_c, "-")
        diff_z <- sweep(resi_z, 2L, tv_c, "-")
        rows$coef <- rbind(
          data.frame(model = "lm", vcov = vtype, n = n_sim, table = "coefficients",
                     term = c_terms, estimator = "t2S / f2S",
                     bias = colMeans(diff_t, na.rm = TRUE),
                     mse  = colMeans(diff_t^2, na.rm = TRUE),
                     row.names = NULL, stringsAsFactors = FALSE),
          data.frame(model = "lm", vcov = vtype, n = n_sim, table = "coefficients",
                     term = c_terms, estimator = "z2S / chisq2S",
                     bias = colMeans(diff_z, na.rm = TRUE),
                     mse  = colMeans(diff_z^2, na.rm = TRUE),
                     row.names = NULL, stringsAsFactors = FALSE)
        )
      }

      do.call(rbind, rows)
    })

    combined <- do.call(rbind, Filter(Negate(is.null), per_n))
    if (is.null(combined) || nrow(combined) == 0L) next
    all_combined[[lbl]] <- combined

    n_vals <- sort(unique(combined$n))

    # -- Figures ---------------------------------------------------------------
    for (tbl_name in c("anova", "coefficients")) {
      sub <- combined[combined$table == tbl_name, ]
      if (nrow(sub) == 0L) next

      terms   <- unique(sub$term)
      n_terms <- length(terms)
      tv_tbl  <- if (tbl_name == "anova") tv$anova else tv$coefficients

      fig_path <- file.path(figures.dir,
                            paste0("estimator_compare_", lbl, "_", tbl_name, ".pdf"))
      grDevices::pdf(fig_path, width = 7, height = row_h * n_terms + leg_h)

      # Layout: n_terms rows x 2 cols + 1 legend row
      n_panels <- 2L * n_terms
      lay_mat  <- matrix(c(seq_len(n_panels),
                           rep(n_panels + 1L, 2L)),
                         nrow = n_terms + 1L, ncol = 2L, byrow = TRUE)
      graphics::layout(lay_mat,
                       widths  = c(3.2, 3.2),
                       heights = c(rep(row_h, n_terms), leg_h))

      for (ti in seq_along(terms)) {
        term      <- terms[[ti]]
        tdata     <- sub[sub$term == term, ]
        true_s    <- if (term %in% rownames(tv_tbl)) tv_tbl[term, "RESI"] else NA_real_
        term_title <- if (is.finite(true_s)) sprintf("%s: S=%.3f", term, true_s) else term

        for (metric in c("bias", "mse")) {
          ylab <- if (metric == "bias") "Bias" else "MSE"
          ylim <- range(tdata[[metric]], na.rm = TRUE)
          if (metric == "bias") ylim <- range(c(ylim, 0), na.rm = TRUE)
          ylim <- ylim + diff(ylim) * c(-0.05, 0.05)

          graphics::par(mar = c(3.2, 3.0, 2.2, 0.5), mgp = c(1.8, 0.5, 0))
          graphics::plot(NULL, xlim = range(n_vals), ylim = ylim,
                         xlab = "Sample Size", ylab = ylab,
                         main = if (metric == "bias") term_title else "",
                         log = "x", xaxt = "n", bty = "l")
          graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                         mgp = c(1.7, 0.35, 0))
          if (metric == "bias") graphics::abline(h = 0, lty = 2L, col = "gray40")

          for (est in names(est_cols)) {
            ed <- tdata[tdata$estimator == est, ]
            ed <- ed[order(ed$n), ]
            if (nrow(ed) == 0L) next
            graphics::lines(ed$n, ed[[metric]], col = est_cols[[est]], lwd = 2L)
            graphics::points(ed$n, ed[[metric]], col = est_cols[[est]], pch = 16L, cex = 0.9)
          }
        }
      }

      # Bottom legend
      est_labels <- if (tbl_name == "anova") c("f2S (used)", "chisq2S") else c("t2S (used)", "z2S")
      graphics::par(mar = c(0.2, 0.2, 0.2, 0.2))
      graphics::plot.new()
      graphics::legend("center",
                       legend = paste0(est_labels, "    "),
                       col    = unname(est_cols),
                       pch    = 16L, lwd = 2L, lty = 1L,
                       bty    = "n", cex = 0.9, ncol = 2L,
                       title  = "Estimator", title.font = 2L)
      grDevices::dev.off()
      message("Saved: ", fig_path)
    }
  }

  message("Estimator comparison figures saved to: ", figures.dir)
  invisible(do.call(rbind, all_combined))
}


# ============================================================
#  simFigures
# ============================================================

#' Simulation Performance Figures for RESI Evaluation
#'
#' Produces performance figures from the output of \code{\link{insurancePlasmodeSim}}.
#' Creates one PDF figure per (model type) \eqn{\times} (variance estimator) combination
#' (4 figures total by default). Each figure is a 2 \eqn{\times} 4 panel grid:
#' \itemize{
#'   \item \strong{Top row}: Anova-table RESI metrics (Bias, MSE, CI Coverage, CI Width)
#'   \item \strong{Bottom row}: Coefficients-table RESI metrics (same metrics; intercept excluded)
#'   \item \strong{Colors}: one colored line per model term (high-contrast matte palette)
#'   \item \strong{x-axis}: sample size on a log scale
#' }
#' Dashed reference lines are drawn at zero for Bias and at \eqn{1 - \alpha} for CI Coverage.
#'
#' @param output.dir Character, directory containing simulation output from
#'   \code{\link{insurancePlasmodeSim}}. Default \code{"resiBootSim"}.
#' @param alpha Numeric, nominal CI level used for the coverage reference line.
#'   Default 0.05.
#' @param ci.label Character, label for the CI method used in figure titles and file
#'   names. Default \code{NULL}, which auto-detects from \code{output.dir}:
#'   \code{"boot"} if the directory name contains "Boot"/"boot",
#'   \code{"normal"} if it contains "Normal"/"normal",
#'   \code{"qf"} if it contains "QF"/"qf", otherwise \code{basename(output.dir)}.
#'
#' @return Invisibly returns the summary metrics \code{data.frame}. Saves PDF figures
#'   to \code{file.path(output.dir, "figures")}, named
#'   \code{sim_<model>_<vcov>_<ci.label>.pdf}.
#' @seealso \code{\link{insurancePlasmodeSim}}
#' @importFrom grDevices pdf dev.off
#' @importFrom graphics plot lines points abline legend par axis plot.new layout
#' @export
simFigures <- function(output.dir = "resiBootSim",
                        alpha      = 0.05,
                        ci.label   = NULL) {

  # Auto-detect ci.label from output.dir when not supplied
  if (is.null(ci.label)) {
    ci.label <- if (grepl("[Bb]oot",   output.dir)) "boot" else
                if (grepl("[Nn]ormal", output.dir)) "normal" else
                if (grepl("[Qq][Ff]",  output.dir)) "qf" else
                basename(output.dir)
  }

  # High-contrast matte palette (matplotlib tab10 + extensions)
  .sim_pal <- c(
    "#1F77B4", "#D62728", "#2CA02C", "#FF7F0E", "#9467BD",
    "#8C564B", "#E377C2", "#17BECF", "#BCBD22", "#7F7F7F",
    "#AEC7E8", "#98DF8A"
  )

  summary_table <- readRDS(file.path(output.dir, "summary_table.rds"))
  if (is.null(summary_table) || nrow(summary_table) == 0L)
    stop("summary_table.rds in '", output.dir, "' is empty or NULL. ",
         "Re-run insurancePlasmodeSim() (all simulation replicates may have ",
         "failed -- check for 'All replicates failed' warnings).")

  figures_dir   <- file.path(output.dir, "figures")
  dir.create(figures_dir, recursive = TRUE, showWarnings = FALSE)

  n_vals        <- sort(unique(summary_table$n))
  metrics       <- c("bias", "mse", "coverage", "width")
  metric_labels <- c("Bias", "MSE", "CI Coverage", "CI Width")

  for (mtype in c("lm", "glm")) {
    for (vtype in c("parametric", "robust")) {

      sub <- summary_table[summary_table$model == mtype &
                             summary_table$vcov  == vtype, ]
      if (is.null(sub) || nrow(sub) == 0L) next

      fig_path <- file.path(
        figures_dir,
        paste0("sim_", mtype, "_", vtype, "_", ci.label, ".pdf")
      )

      anova_terms <- unique(sub$term[sub$table == "anova"])
      coef_terms  <- unique(sub$term[sub$table == "coefficients"])

      anova_cols <- setNames(
        .sim_pal[seq_along(anova_terms)], anova_terms)
      coef_cols  <- setNames(
        .sim_pal[seq_along(coef_terms)],  coef_terms)

      row_info <- list(
        list(table       = "anova",
             terms       = anova_terms,
             cols        = anova_cols,
             main_prefix = "Anova"),
        list(table       = "coefficients",
             terms       = coef_terms,
             cols        = coef_cols,
             main_prefix = "Coef")
      )

      # Layout: 4 rows x 5 cols; coverage column (col 3) is split into two
      # half-height panels (upper coverage on top, lower coverage on bottom).
      # Row 1:  1   2   3   4   5   (anova bias, mse, upper cov, width, legend)
      # Row 2:  1   2   6   4   5   (spans continue; anova lower cov)
      # Row 3:  7   8   9  10  11   (coef bias, mse, upper cov, width, legend)
      # Row 4:  7   8  12  10  11   (spans continue; coef lower cov)
      grDevices::pdf(fig_path, width = 15, height = 7)
      graphics::layout(
        matrix(c( 1,  2,  3,  4,  5,
                  1,  2,  6,  4,  5,
                  7,  8,  9, 10, 11,
                  7,  8, 12, 10, 11), nrow = 4L, byrow = TRUE),
        widths  = c(rep(3, 4), 2.2),
        heights = c(1, 1, 1, 1)
      )

      for (ri in row_info) {
        sub_tbl <- sub[sub$table == ri$table, ]
        cex_leg <- min(1.0, 9 / length(ri$terms))

        # Helper: draw lines/points for all terms for a given metric
        draw_lines <- function(metric) {
          for (term in ri$terms) {
            td <- sub_tbl[sub_tbl$term == term, ]
            td <- td[order(td$n), ]
            graphics::lines(td$n,  td[[metric]], col = ri$cols[term], lwd = 2L)
            graphics::points(td$n, td[[metric]], col = ri$cols[term], pch = 16L,
                             cex = 0.8)
          }
        }

        # Shared y-range for both coverage sub-panels
        cov_ylim <- range(
          c(sub_tbl[["upper_coverage"]], sub_tbl[["lower_coverage"]],
            1 - alpha/2 - 0.02, 1.01),
          na.rm = TRUE
        )

        # --- Panel: Bias ---
        bias_rows <- if (mtype == "glm") sub_tbl$n >= 500 else rep(TRUE, nrow(sub_tbl))
        ylim <- range(c(sub_tbl[bias_rows, "bias"], 0), na.rm = TRUE)
        graphics::par(mar = c(2.8, 2.8, 1.8, 0.4), mgp = c(1.7, 0.45, 0))
        graphics::plot(NULL, xlim = range(n_vals), ylim = ylim,
                       xlab = "Sample Size", ylab = "Bias",
                       main = paste(toupper(mtype), ri$main_prefix, "Bias",
                                    paste0("(", vtype, ")")),
                       log = "x", xaxt = "n", bty = "l")
        graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                       mgp = c(1.7, 0.35, 0))
        graphics::abline(h = 0, lty = 2L, col = "gray40")
        draw_lines("bias")

        # --- Panel: MSE ---
        mse_rows <- if (mtype == "glm") sub_tbl$n >= 500 else rep(TRUE, nrow(sub_tbl))
        ylim <- if (any(is.finite(sub_tbl[mse_rows, "mse"]))) range(sub_tbl[mse_rows, "mse"], na.rm = TRUE) else c(0, 1)
        graphics::par(mar = c(2.8, 2.8, 1.8, 0.4), mgp = c(1.7, 0.45, 0))
        graphics::plot(NULL, xlim = range(n_vals), ylim = ylim,
                       xlab = "Sample Size", ylab = "MSE",
                       main = paste(toupper(mtype), ri$main_prefix, "MSE",
                                    paste0("(", vtype, ")")),
                       log = "x", xaxt = "n", bty = "l")
        graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                       mgp = c(1.7, 0.35, 0))
        draw_lines("mse")

        # --- Panel: Upper Coverage (top half-panel, no x-axis) ---
        graphics::par(mar = c(0.3, 2.8, 1.8, 0.4), mgp = c(1.7, 0.45, 0))
        graphics::plot(NULL, xlim = range(n_vals), ylim = cov_ylim,
                       xlab = "", ylab = "Upper Cov.",
                       main = paste(toupper(mtype), ri$main_prefix, "Coverage",
                                    paste0("(", vtype, ")")),
                       log = "x", xaxt = "n", bty = "l")
        graphics::abline(h = 1 - alpha/2, lty = 2L, col = "gray40")
        draw_lines("upper_coverage")

        # --- Panel: Width ---
        width_rows <- if (mtype == "glm") sub_tbl$n >= 500 else rep(TRUE, nrow(sub_tbl))
        ylim <- if (any(is.finite(sub_tbl[width_rows, "width"]))) range(sub_tbl[width_rows, "width"], na.rm = TRUE) else c(0, 1)
        graphics::par(mar = c(2.8, 2.8, 1.8, 0.4), mgp = c(1.7, 0.45, 0))
        graphics::plot(NULL, xlim = range(n_vals), ylim = ylim,
                       xlab = "Sample Size", ylab = "CI Width",
                       main = paste(toupper(mtype), ri$main_prefix, "CI Width",
                                    paste0("(", vtype, ")")),
                       log = "x", xaxt = "n", bty = "l")
        graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                       mgp = c(1.7, 0.35, 0))
        draw_lines("width")

        # --- Legend panel (spans both coverage half-rows) ---
        graphics::par(mar = c(0.5, 0.3, 0.5, 0.3))
        graphics::plot.new()
        graphics::legend(
          "center",
          legend = ri$terms,
          col    = ri$cols[ri$terms],
          lwd    = 2L,
          pch    = 16L,
          bty    = "n",
          cex    = cex_leg,
          title  = paste(ri$main_prefix, "terms"),
          title.font = 2L
        )

        # --- Panel: Lower Coverage (bottom half-panel, with x-axis) ---
        graphics::par(mar = c(2.8, 2.8, 0.3, 0.4), mgp = c(1.7, 0.45, 0))
        graphics::plot(NULL, xlim = range(n_vals), ylim = cov_ylim,
                       xlab = "Sample Size", ylab = "Lower Cov.",
                       main = "",
                       log = "x", xaxt = "n", bty = "l")
        graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                       mgp = c(1.7, 0.35, 0))
        graphics::abline(h = 1 - alpha/2, lty = 2L, col = "gray40")
        draw_lines("lower_coverage")
      }

      grDevices::dev.off()
      message("Saved: ", fig_path)
    }
  }

  invisible(summary_table)
}


# ============================================================
#  simCompareMethodsFigures
# ============================================================

# Internal: full delta-method sigma2S (including A/B chain-rule terms) for each
# coefficient in coef_idx (1-based index into coef(model)), using HC-type B.
# Returns named vector sigma2S_Th1_k = quad_k / S_k^2  (= n * resiSE_k^2).
.sigma2S_Th1 <- function(model, coef_idx, type = "HC0") {
  is_lm  <- inherits(model, "lm") && !inherits(model, "glm")
  X      <- model.matrix(model)
  if (any(al <- is.na(coef(model)))) X <- X[, !al, drop = FALSE]
  ef     <- sandwich::estfun(model)
  ef     <- ef[, colnames(X), drop = FALSE]
  p  <- ncol(X); n <- nrow(X)
  e  <- residuals(model, "response")
  sym_fn <- function(M) (M + t(M)) / 2

  if (is_lm) {
    phi   <- summary(model)$sigma^2
    m     <- p + 1L
    theta <- c(phi, coef(model))
    psi_list <- lapply(seq_len(n), function(i) {
      xi <- X[i, , drop = FALSE]; ei <- e[i]
      cbind((ei^2 - phi) / (2*phi^2), matrix(ei * xi / phi, 1))
    })
    psiprime_list <- lapply(seq_len(n), function(i) {
      xi <- X[i, , drop = FALSE]; ei <- e[i]
      rbind(c((phi - 2*ei^2) / (2*phi^3), -ei * as.vector(xi) / phi^2),
            cbind(matrix(-ei * as.vector(xi) / phi^2, p, 1),
                  -crossprod(xi) / phi))
    })
  } else {
    m     <- p
    theta <- coef(model)
    w_vec <- weights(model, type = "working")
    psi_list <- lapply(seq_len(n), function(i)
      matrix(ef[i, , drop = TRUE], 1))
    psiprime_list <- lapply(seq_len(n), function(i)
      -w_vec[i] * crossprod(X[i, , drop = FALSE]))
  }

  A_full <- sym_fn(-Reduce("+", psiprime_list) / n)
  A_inv  <- tryCatch(chol2inv(chol(A_full)), error = function(e2) solve(A_full))

  h    <- hatvalues(model)
  omh  <- pmax(1 - h, .Machine$double.eps)
  sqrtw <- switch(toupper(type),
    "HC0" = rep(1, n),
    "HC1" = rep(sqrt(n / max(n - p, 1L)), n),
    "HC2" = 1 / sqrt(omh),
    "HC3" = 1 / omh,
    rep(1, n)
  )
  B_full <- sym_fn(Reduce("+", lapply(seq_len(n), function(i) {
    psi_i <- psi_list[[i]]
    if (is_lm) psi_i[1L, seq(2L, m)] <- psi_i[1L, seq(2L, m)] * sqrtw[i]
    else       psi_i[1L, ]           <- psi_i[1L, ] * sqrtw[i]
    crossprod(psi_i)
  })) / n)
  cov_th <- sym_fn(A_inv %*% B_full %*% A_inv)

  XtXn    <- crossprod(X) / n
  XtXn_hc <- crossprod(X * sqrtw) / n

  if (is_lm) {
    dA_phi <- rbind(
      c(-1 / phi^3, numeric(p)),
      cbind(numeric(p), -XtXn / phi^2))
    dA_dth <- matrix(0, m * m, m); dA_dth[, 1L] <- as.vector(dA_phi)
    dB_phi <- rbind(
      c((phi^2 - 2 * mean(e^4)) / phi^5,
        -(3 / (2 * phi^4)) * colMeans(X) * mean(e^3)),
      cbind(matrix(-(3 / (2 * phi^4)) * colMeans(X) * mean(e^3), p, 1),
            -XtXn_hc / phi^2))
    dB_dth <- matrix(0, m * m, m); dB_dth[, 1L] <- as.vector(dB_phi)
  } else {
    mu_hat <- fitted(model)
    dA_dth <- matrix(0, m * m, m)
    for (j in seq_len(m)) {
      vj <- w_vec * (1 - 2 * mu_hat) * X[, j]
      dA_dth[, j] <- as.vector(crossprod(X, vj * X) / n)
    }
    dB_dth <- dA_dth
  }

  mcov_n <- n * sandwich::vcovHC(model, type = type)
  nms    <- names(coef(model))[coef_idx]
  result <- setNames(numeric(length(coef_idx)), nms)

  for (ki in seq_along(coef_idx)) {
    k <- coef_idx[ki]
    if (is_lm) {
      L  <- matrix(0, 1, m); L[1L, k + 1L] <- 1  # +1: phi occupies position 1
      Lm <- matrix(0, 1, p); Lm[1L, k]     <- 1
    } else {
      L  <- matrix(0, 1, m); L[1L, k] <- 1
      Lm <- L
    }
    beta_k   <- as.numeric(L %*% theta)
    cov_beta <- as.numeric(L %*% cov_th %*% t(L))
    Ssq      <- beta_k^2 / as.numeric(Lm %*% mcov_n %*% t(Lm))
    if (!is.finite(Ssq) || Ssq <= 0 || !is.finite(cov_beta) || cov_beta <= 0) {
      result[ki] <- NA_real_; next
    }
    La_row <- L %*% A_inv  # 1xm
    Lc_row <- L %*% cov_th # 1xm
    sc <- beta_k / cov_beta
    deriv_th <- sc * L
    deriv_A  <- (sc^2 / 2) *
      (kronecker(La_row, Lc_row) + kronecker(Lc_row, La_row)) %*% dA_dth
    deriv_B  <- -(sc^2 / 2) * kronecker(La_row, La_row) %*% dB_dth
    d_tot    <- deriv_th + deriv_A + deriv_B
    quad     <- max(0, as.numeric(d_tot %*% cov_th %*% t(d_tot)))
    result[ki] <- quad / Ssq
  }
  result
}

# ============================================================
#  simCalibrationFigures
# ============================================================

#' Asymptotic Calibration Check for RESI Variance Estimates
#'
#' Runs a plasmode simulation to verify that the asymptotic normal CI machinery
#' is correctly calibrated for all four model settings (lm/glm x
#' parametric/robust).  For each (setting, sample size) cell the function
#' checks:
#' \enumerate{
#'   \item \strong{Bias(theta)}: \eqn{\hat\theta \to \theta_{\rm true}} --
#'     raw coefficients (and \eqn{\phi = \hat\sigma^2} for \code{lm}) converge
#'     to the full-dataset values.
#'   \item \strong{vcov check A} (estimator consistency):
#'     \eqn{n \cdot \bar V_{\rm analytic} / (n_{\rm full} \cdot V_{\rm true}) \to 1}.
#'   \item \strong{vcov check B} (calibration):
#'     \eqn{n \cdot \widehat{\rm Var}(\hat\beta) / (n_{\rm full} \cdot V_{\rm true}) \to 1}.
#'   \item \strong{Bias(R)}: RESI point estimates converge to the full-dataset
#'     values.
#'   \item \strong{sigma2S check A} (estimator consistency):
#'     \eqn{\bar{\hat\sigma}^2_S / \sigma^2_{S,\rm true} \to 1}.
#'   \item \strong{sigma2S check B} (CI calibration):
#'     \eqn{n \cdot \widehat{\rm Var}(\hat R) / \sigma^2_{S,\rm true} \to 1}.
#' }
#' True values are taken from the full \code{\link{insurance}} dataset using the
#' same definitions as \code{\link{insurancePlasmodeSim}}.  \eqn{\sigma^2_S} is
#' extracted from the asymptotic-normal CI half-width:
#' \eqn{\hat\sigma^2_S = n \cdot ({\rm hw}/z_{\alpha/2})^2}.
#'
#' @param nsim Integer, replicates per (setting, \eqn{n}) cell. Default 500.
#' @param n.vec Integer vector of sample sizes.
#'   Default \code{c(100, 200, 500, 1000, 2000, 5000)}.
#' @param alpha Numeric, nominal CI level. Default 0.05.
#' @param output.dir Character, directory for raw per-cell RDS files.
#'   Default \code{"resiCalibrationSim"}.
#' @param fixed.knots Logical. Fix spline knots at full-dataset quantiles.
#'   Default \code{FALSE}.
#' @param mc.cores.reps Integer, cores for within-cell parallelism. Default 1.
#' @param deriv_method Character, type of derivative being evaluated. Package now only includes "extended" in resi implementation.

#'
#' @return Invisibly returns the combined metrics \code{data.frame}.
#' @seealso \code{\link{insurancePlasmodeSim}}, \code{\link{simCompareMethodsFigures}}
#' @importFrom parallel mclapply
#' @importFrom splines ns
#' @importFrom sandwich vcovHC
#' @importFrom stats lm glm vcov binomial qnorm coef setNames sd
#' @importFrom grDevices pdf dev.off
#' @importFrom graphics plot lines points abline legend par axis plot.new layout
#' @export
simCalibrationSim <- function(
    nsim          = 500L,
    n.vec         = c(100L, 200L, 500L, 1000L, 2000L, 5000L),
    alpha         = 0.05,
    output.dir    = "resiCalibrationSim",
    fixed.knots   = FALSE,
    deriv_method  = c("corrected", "original", "population", "population2", "zeroB", "zero_phi_cross", "extended"),
    mc.cores.reps = 1L
) {
  deriv_method <- match.arg(deriv_method)
  raw_dir <- file.path(output.dir, "raw", deriv_method)
  dir.create(raw_dir,      recursive = TRUE, showWarnings = FALSE)

  insurance <- RESI::insurance
  n_full    <- nrow(insurance)
  z_ref     <- stats::qnorm(1 - alpha / 2)
  ci_lo_col <- paste0(alpha / 2 * 100,       "%")
  ci_hi_col <- paste0((1 - alpha / 2) * 100, "%")

  .fvars   <- c("sex", "smoker", "region")
  .flevels <- lapply(.fvars, function(v) unique(insurance[[v]]))
  names(.flevels) <- .fvars

  # ---- Formulas ---------------------------------------------------------------
  if (fixed.knots) {
    .age_knots <- stats::quantile(insurance$age, c(1/3, 2/3))
    .age_bk    <- range(insurance$age)
    lm_formula  <- eval(bquote(log10(charges) ~
      splines::ns(age, knots = .(.age_knots), Boundary.knots = .(.age_bk)) *
        sex + bmi + smoker + region))
    glm_formula <- eval(bquote(I(charges > 15000) ~
      splines::ns(age, knots = .(.age_knots), Boundary.knots = .(.age_bk)) *
        sex + bmi + smoker + region))
  } else {
    lm_formula  <- log10(charges) ~
      splines::ns(age, df = 3) * sex + bmi + smoker + region
    glm_formula <- I(charges > 15000) ~
      splines::ns(age, df = 3) * sex + bmi + smoker + region
  }

  model_settings <- list(
    list(type = "lm",  label = "lm_parametric",
         formula = lm_formula,  family = NULL,
         vcov_name = "parametric", vcovfunc = stats::vcov),
    list(type = "lm",  label = "lm_robust",
         formula = lm_formula,  family = NULL,
         vcov_name = "robust",     vcovfunc = sandwich::vcovHC),
    list(type = "glm", label = "glm_parametric",
         formula = glm_formula, family = stats::binomial(),
         vcov_name = "parametric", vcovfunc = stats::vcov),
    list(type = "glm", label = "glm_robust",
         formula = glm_formula, family = stats::binomial(),
         vcov_name = "robust",     vcovfunc = sandwich::vcovHC)
  )

  # ---- True values from full dataset ------------------------------------------
  message("Computing true values from full dataset (n = ", n_full, ")...")
  true_vals <- lapply(model_settings, function(s) {
    .formula <- s$formula; .family <- s$family
    full_mod <- if (s$type == "lm") {
      m <- stats::lm(.formula, data = insurance)
      m$call[["formula"]] <- .formula; m
    } else {
      m <- stats::glm(.formula, data = insurance, family = .family)
      m$call[["formula"]] <- .formula
      m$call[["family"]]  <- .family; m
    }

    # theta_true: phi = sigma^2 = summary()$sigma^2 (same as .resi_precompute)
    phi_true  <- if (s$type == "lm") summary(full_mod)$sigma^2 else NULL
    beta_true <- stats::coef(full_mod)

    # n_full * vcov_true: full covariance matrix (all non-aliased coefficients)
    vcov_full_true_raw <- n_full * s$vcovfunc(full_mod)
    vcov_diag_true     <- diag(vcov_full_true_raw)
    # Submatrix for non-intercept terms (computed later once coef_terms_true is known)

    # R_true using the same population definition as insurancePlasmodeSim:
    #   - robust: HC0 (no hat-value correction)
    #   - parametric lm: divide by sqrt(rdf) = sqrt(n-p), not sqrt(n)
    true_vcovfunc_r <- if (s$vcov_name == "robust") {
      function(x) sandwich::vcovHC(x, type = "HC0")
    } else {
      s$vcovfunc
    }
    rdf_true_r <- if (s$type == "lm" && s$vcov_name == "parametric") {
      full_mod$df.residual
    } else {
      NULL
    }
    pe_true_raw <- resi_pe(full_mod, data = insurance, vcovfunc = true_vcovfunc_r)
    pe_true     <- .simDirectRESI(pe_true_raw, n = n_full, rdf = rdf_true_r)
    coef_tab_true   <- pe_true$coefficients
    coef_terms_true <- rownames(coef_tab_true)[rownames(coef_tab_true) != "(Intercept)"]
    R_true <- coef_tab_true[coef_terms_true, "RESI"]
    names(R_true) <- coef_terms_true   # data.frame[rows,col] drops names

    # sigma2S_true: Sigma_R[k,k] from the delta-method variance of R_hat,
    #   evaluated at the population values (full-model coefficients + HC0 Sigma_theta).
    #   HC0 is used as the "true" population covariance (consistent estimator).
    #   For parametric: vcov_is_model=TRUE triggers the parametric derivative;
    #     type="HC0" supplies HC0-based Sigma_theta via the B matrix.
    #   For robust: vcovfunc=HC0 triggers the robust derivative; type="HC0" same.
    #   sigma2S_true_k = [dR_dtheta * Sigma_theta_HC0 * dR_dtheta^T]_{kk}
    #                  = n_full * (hw / z_ref)^2 from the normal CI
    asym_true <- tryCatch(
      resi_pe_asymptotic(full_mod, vcovfunc = true_vcovfunc_r,
                         ci.method = "normal", type = "HC0",
                         deriv_method = deriv_method),
      error = function(e) NULL
    )
    if (!is.null(asym_true)) {
      tab_a <- asym_true$coefficients
      tab_a <- tab_a[rownames(tab_a) != "(Intercept)", , drop = FALSE]
      hw_true <- (tab_a[, ci_hi_col] - tab_a[, ci_lo_col]) / 2
      names(hw_true) <- rownames(tab_a)   # data.frame[,col] drops names
      sigma2S_true <- n_full * (hw_true / z_ref)^2
    } else {
      sigma2S_true <- setNames(rep(NA_real_, length(R_true)), names(R_true))
    }

    # sigma2S_true_Th1: full delta-method variance (including A/B chain rules)
    #   evaluated at population values with HC0 Sigma_theta.
    #   sigma2S_Th1_k = quad_k / S_k^2 where quad_k uses deriv_theta + deriv_A + deriv_B.
    coef_idx_th1 <- match(coef_terms_true, names(coef(full_mod)))
    sigma2S_true_Th1 <- tryCatch(
      .sigma2S_Th1(full_mod, coef_idx_th1, type = "HC0"),
      error = function(e) setNames(rep(NA_real_, length(coef_terms_true)),
                                    coef_terms_true)
    )

    # vcov_phi_phi_true and vcov_phi_beta_true: sandwich (phi,phi) and (phi,beta_k)
    # elements for the lm settings, using the per-setting HC type as the "true"
    # population reference (so the check targets convergence of the sample
    # estimator to the full-dataset sandwich, consistent with vcov_diag_true).
    # For GLM there is no phi: set to NA.
    precomp_type_tv  <- if (s$vcov_name == "robust") "HC3" else "const"
    precomp_tv       <- NULL   # initialise; assigned below for lm only
    vcov_phi_phi_true  <- NA_real_
    vcov_phi_beta_true <- setNames(rep(NA_real_, length(coef_terms_true)),
                                   coef_terms_true)
    if (s$type == "lm") {
      precomp_tv <- tryCatch(
        .resi_precompute(full_mod, type = precomp_type_tv),
        error = function(e) NULL
      )
      if (!is.null(precomp_tv)) {
        vcov_phi_phi_true <- precomp_tv$cov_theta[1L, 1L]
        beta_names_tv     <- names(coef(full_mod))[!is.na(coef(full_mod))]
        beta_pos_tv       <- match(coef_terms_true, beta_names_tv)
        vcov_phi_beta_true <- precomp_tv$cov_theta[1L, 1L + beta_pos_tv]
        names(vcov_phi_beta_true) <- coef_terms_true
      }
    } else {
      # GLM: precomp_tv needed for vcov_full_true_mat (beta-beta block)
      precomp_tv <- tryCatch(
        .resi_precompute(full_mod, type = precomp_type_tv),
        error = function(e) NULL
      )
    }

    # Chain decomposition of Sigma_R at true-value estimates (HC0 sandwich).
    # For 'extended': Sigma_R comes from influence-function approach; no chain decomp.
    # For other methods: standard J = J_direct + J_Achain + J_Bchain decomposition.
    precomp_hc0 <- if (deriv_method == "extended") {
      tryCatch(.resi_precompute_ext(full_mod, type = "HC0"), error = function(e) NULL)
    } else {
      tryCatch(.resi_precompute(full_mod, type = "HC0", deriv_method = deriv_method),
               error = function(e) NULL)
    }
    na_terms <- setNames(rep(NA_real_, length(coef_terms_true)), coef_terms_true)
    J_dir_list    <- list()
    J_Achain_list <- list()
    J_Bchain_list <- list()
    sig2S_true_dir    <- na_terms
    sig2S_true_Achain <- na_terms
    sig2S_true_Bchain <- na_terms
    theta_true_full   <- NULL
    if (!is.null(precomp_hc0)) {
      Sig_hc0 <- precomp_hc0$cov_theta   # m x m Sigma_theta at HC0
      theta_true_full <- precomp_hc0$theta_hat  # (phi, beta_1,...,beta_p)
      for (tm in coef_terms_true) {
        L_tm <- .get_L_coef(full_mod, tm)
        if (deriv_method == "extended") {
          # Extended: Sigma_R is the total; store in dir, Achain/Bchain = 0
          ct_tm <- tryCatch(.resi_contrast_ext(precomp_hc0, L_tm), error = function(e) NULL)
          if (!is.null(ct_tm)) {
            J_dir_list[[tm]]    <- matrix(0, 1, precomp_hc0$m)
            J_Achain_list[[tm]] <- matrix(0, 1, precomp_hc0$m)
            J_Bchain_list[[tm]] <- matrix(0, 1, precomp_hc0$m)
            sig2S_true_dir[tm]    <- as.numeric(ct_tm$Sigma_R)
            sig2S_true_Achain[tm] <- 0
            sig2S_true_Bchain[tm] <- 0
          }
        } else {
          ct_tm <- tryCatch(.resi_contrast(precomp_hc0, L_tm), error = function(e) NULL)
          if (!is.null(ct_tm)) {
            J_dir_list[[tm]]    <- ct_tm$dR_direct
            J_Achain_list[[tm]] <- ct_tm$dR_Achain
            J_Bchain_list[[tm]] <- ct_tm$dR_Bchain
            sig2S_true_dir[tm]    <- as.numeric(ct_tm$dR_direct  %*% Sig_hc0 %*% t(ct_tm$dR_direct))
            sig2S_true_Achain[tm] <- as.numeric(ct_tm$dR_Achain %*% Sig_hc0 %*% t(ct_tm$dR_Achain))
            sig2S_true_Bchain[tm] <- as.numeric(ct_tm$dR_Bchain %*% Sig_hc0 %*% t(ct_tm$dR_Bchain))
          }
        }
      }
    }

    list(deriv_method_used   = deriv_method,
         phi_true            = phi_true,
         beta_true           = beta_true,
         vcov_diag_true      = vcov_diag_true,
         # vcov_full_true_mat: full analytic Sigma_theta from precomp_tv (HC3 or const).
         # For lm: (p+1)x(p+1) with phi as first row/col; for glm: pxp.
         # Row/col names: c("phi", coef_terms_true) for lm, coef_terms_true for glm.
         vcov_full_true_mat  = if (!is.null(precomp_tv)) {
           if (s$type == "lm") {
             m_all  <- precomp_tv$m
             beta_names_all <- names(coef(full_mod))[!is.na(coef(full_mod))]
             beta_pos_all   <- match(coef_terms_true, beta_names_all)
             idx_all <- c(1L, 1L + beta_pos_all)
             nm_all  <- c("phi", coef_terms_true)
             mat_all <- precomp_tv$cov_theta[idx_all, idx_all, drop = FALSE]
             rownames(mat_all) <- colnames(mat_all) <- nm_all
             mat_all
           } else {
             beta_names_all <- names(coef(full_mod))[!is.na(coef(full_mod))]
             beta_pos_all   <- match(coef_terms_true, beta_names_all)
             mat_all <- precomp_tv$cov_theta[beta_pos_all, beta_pos_all, drop = FALSE]
             rownames(mat_all) <- colnames(mat_all) <- coef_terms_true
             mat_all
           }
         } else NULL,
         vcov_phi_phi_true   = vcov_phi_phi_true,
         vcov_phi_beta_true  = vcov_phi_beta_true,
         R_true              = R_true,
         sigma2S_true        = sigma2S_true,
         sigma2S_true_Th1    = sigma2S_true_Th1,
         J_dir_list          = J_dir_list,
         J_Achain_list       = J_Achain_list,
         J_Bchain_list       = J_Bchain_list,
         sig2S_true_dir      = sig2S_true_dir,
         sig2S_true_Achain   = sig2S_true_Achain,
         sig2S_true_Bchain   = sig2S_true_Bchain,
         theta_true_full     = theta_true_full,
         coef_terms          = coef_terms_true)
  })
  names(true_vals) <- sapply(model_settings, `[[`, "label")

  # ---- Per-setting, per-n simulations -----------------------------------------
  all_metrics <- list()

  for (s in model_settings) {
    lbl      <- s$label
    tv       <- true_vals[[lbl]]
    is_lm    <- (s$type == "lm")
    .formula <- s$formula; .family <- s$family
    coef_terms <- tv$coef_terms
    message("\nSetting: ", lbl)

    for (n_s in n.vec) {
      cell_label <- paste0(lbl, "_n", n_s)
      rds_path   <- file.path(raw_dir, paste0(cell_label, ".rds"))

      if (file.exists(rds_path)) {
        cell_data <- readRDS(rds_path)
        if (cell_data$n_success > 0L) {
          message("  n = ", n_s, " (loading cached)")
        } else {
          message("  n = ", n_s, " (cached n_success = 0; re-running)")
          file.remove(rds_path)
          cell_data <- NULL
        }
      } else {
        cell_data <- NULL
      }

      if (is.null(cell_data)) {
        message("  n = ", n_s, " ...")
        reps_raw <- parallel::mclapply(seq_len(nsim), function(i) {
          repeat {
            dat <- insurance[sample(n_full, n_s, replace = TRUE), ]
            ok  <- all(vapply(.fvars, function(v)
              all(.flevels[[v]] %in% dat[[v]]), logical(1L)))
            if (ok && !is_lm) {
              biny <- as.integer(dat$charges > 15000)
              ok   <- length(unique(biny)) > 1L &&
                all(tapply(biny, dat$smoker,
                           function(x) length(unique(x)) > 1L))
            }
            if (ok) break
          }

          mod <- tryCatch({
            m <- if (is_lm) stats::lm(.formula, data = dat)
                 else stats::glm(.formula, data = dat, family = .family)
            m$call[["formula"]] <- .formula
            if (!is_lm) m$call[["family"]] <- .family
            m
          }, error = function(e) NULL)
          if (is.null(mod)) return(NULL)

          tryCatch({
            # theta_hat
            phi_hat  <- if (is_lm) summary(mod)$sigma^2 else NULL
            beta_hat <- stats::coef(mod)[coef_terms]

            # n_s * diag(vcov_hat) for non-intercept terms
            vcov_hat_diag <- n_s * diag(s$vcovfunc(mod))[coef_terms]

            # R_hat and sigma2S_hat via asymptotic normal CI
            asym <- resi_pe_asymptotic(mod, vcovfunc = s$vcovfunc,
                                       ci.method = "normal",
                                       deriv_method = deriv_method)
            tab  <- asym$coefficients
            tab  <- tab[rownames(tab) != "(Intercept)", , drop = FALSE]
            hw   <- (tab[coef_terms, ci_hi_col] - tab[coef_terms, ci_lo_col]) / 2
            names(hw)   <- coef_terms   # data.frame[rows,col] drops names
            R_hat       <- tab[coef_terms, "RESI"]
            names(R_hat) <- coef_terms
            sigma2S_hat <- n_s * (hw / z_ref)^2
            names(sigma2S_hat) <- coef_terms

            # phi-variance and phi-beta covariances from the full sandwich
            # (lm only; requires model-level precomputation)
            vcov_phi_phi  <- NULL
            vcov_phi_beta <- NULL
            if (is_lm) {
              precomp_type_rep <- if (s$vcov_name == "robust") "HC3" else "const"
              precomp_rep <- tryCatch(
                .resi_precompute(mod, type = precomp_type_rep,
                                 deriv_method = deriv_method),
                error = function(e) NULL
              )
              if (!is.null(precomp_rep)) {
                # cov_theta[1,1] = Sigma_theta_{phi,phi} = asymptotic variance of
                # sqrt(n)(phi_hat - phi).  Do NOT multiply by n_s: cov_theta is
                # already the asymptotic (scaled) variance, analogous to
                # n_s * diag(vcovfunc(mod)) for the beta terms.
                vcov_phi_phi <- precomp_rep$cov_theta[1L, 1L]
                beta_names_rep <- names(coef(mod))[!is.na(coef(mod))]
                beta_pos_rep   <- match(coef_terms, beta_names_rep)
                vcov_phi_beta  <- precomp_rep$cov_theta[1L, 1L + beta_pos_rep]
                names(vcov_phi_beta) <- coef_terms
              }
            }

            list(phi_hat        = phi_hat,
                 beta_hat       = beta_hat,
                 theta_hat_full = NULL,     # removed: Jacobian-projection MC not used
                 vcov_hat_diag  = vcov_hat_diag,
                 R_hat          = R_hat,
                 sigma2S_hat    = sigma2S_hat,
                 vcov_phi_phi   = vcov_phi_phi,
                 vcov_phi_beta  = vcov_phi_beta)
          }, error = function(e) NULL)
        }, mc.cores = mc.cores.reps)

        reps <- Filter(Negate(is.null), reps_raw)

        # Store matrices
        cell_data <- list(
          n           = n_s,
          n_success   = length(reps),
          phi_vec     = if (is_lm) sapply(reps, `[[`, "phi_hat") else NULL,
          beta_mat    = do.call(rbind, lapply(reps, `[[`, "beta_hat")),
          vcov_mat    = do.call(rbind, lapply(reps, `[[`, "vcov_hat_diag")),
          R_mat       = do.call(rbind, lapply(reps, `[[`, "R_hat")),
          sig2S_mat   = do.call(rbind, lapply(reps, `[[`, "sigma2S_hat")),
          phi_var_vec = if (is_lm && !is.null(reps[[1L]]$vcov_phi_phi))
                          sapply(reps, `[[`, "vcov_phi_phi") else NULL,
          phi_cov_mat = if (is_lm && !is.null(reps[[1L]]$vcov_phi_beta))
                          do.call(rbind, lapply(reps, `[[`, "vcov_phi_beta")) else NULL
        )
        saveRDS(cell_data, rds_path)
      }

      if (cell_data$n_success == 0L) next

      # ---- Compute metrics ---------------------------------------------------
      n_s_eff <- cell_data$n  # actual n used

      # (1) Bias(phi) for lm
      phi_bias <- if (!is.null(cell_data$phi_vec))
        mean(cell_data$phi_vec, na.rm = TRUE) - tv$phi_true else NA_real_

      # (2) Bias(beta_k)
      beta_bias <- colMeans(cell_data$beta_mat, na.rm = TRUE) -
        tv$beta_true[coef_terms]

      # (3) vcov check A: mean(n_s * vcov_hat_kk) / (n_full * vcov_true_kk)
      vcov_A <- colMeans(cell_data$vcov_mat, na.rm = TRUE) /
        tv$vcov_diag_true[coef_terms]

      # (4) vcov check B: n_s * Var_empirical(beta_k) / (n_full * vcov_true_kk)
      vcov_B <- (n_s_eff * apply(cell_data$beta_mat, 2L, stats::var, na.rm = TRUE)) /
        tv$vcov_diag_true[coef_terms]

      # (5) Bias(R_k)
      R_bias <- colMeans(cell_data$R_mat, na.rm = TRUE) - tv$R_true[coef_terms]

      # (6) sigma2S check A: mean(sigma2S_hat_k) / sigma2S_true_k
      sig2S_A <- colMeans(cell_data$sig2S_mat, na.rm = TRUE) /
        tv$sigma2S_true[coef_terms]

      # (7) sigma2S check B: n_s * Var_empirical(R_k) / sigma2S_true_k
      sig2S_B <- (n_s_eff * apply(cell_data$R_mat, 2L, stats::var, na.rm = TRUE)) /
        tv$sigma2S_true[coef_terms]

      # (8) Coefficient of variation of the variance estimator V_hat_kk across
      #     replicates -- quantifies the "variance of the variance estimator".
      cv_Vhat <- apply(cell_data$vcov_mat, 2L, function(x)
        stats::sd(x, na.rm = TRUE) / mean(x, na.rm = TRUE))

      # (9) phi variance check A: mean(Sigma_theta_phi_phi_hat) / Sigma_theta_phi_phi_true
      #     cov_theta[1,1] IS Sigma_theta (asymptotic variance of sqrt(n)*phi_hat),
      #     so no n factor is needed -- analogous to n_s*diag(vcovfunc) for beta.
      #     (scalar, same for all terms in this cell; NA for GLM)
      phi_var_A <- if (!is.null(cell_data$phi_var_vec) &&
                        is.finite(tv$vcov_phi_phi_true) &&
                        tv$vcov_phi_phi_true > 0)
        mean(cell_data$phi_var_vec, na.rm = TRUE) / tv$vcov_phi_phi_true
      else NA_real_

      # (10) phi variance check B: n_s * Var_empirical(phi_hat) / Sigma_theta_phi_phi_true
      #      Empirical counterpart: n_s * var(phi_vec) estimates Sigma_theta_{phi,phi}.
      phi_var_B <- if (!is.null(cell_data$phi_vec) &&
                        is.finite(tv$vcov_phi_phi_true) &&
                        tv$vcov_phi_phi_true > 0)
        n_s_eff * stats::var(cell_data$phi_vec, na.rm = TRUE) / tv$vcov_phi_phi_true
      else NA_real_

      # (11) phi-beta covariance check A: per-term ratio (NA for GLM or near-zero truth)
      phi_cov_A <- if (!is.null(cell_data$phi_cov_mat) &&
                        all(is.finite(tv$vcov_phi_beta_true)))
        colMeans(cell_data$phi_cov_mat, na.rm = TRUE) / tv$vcov_phi_beta_true
      else setNames(rep(NA_real_, length(coef_terms)), coef_terms)

      # (12) Raw MC variances of the estimator components (no Jacobian involved).
      #   var_R_MC_k   = n_s * var(R_hat_k)     -- actual MC variance of RESI
      #   var_b_MC_k   = n_s * var(beta_hat_k)  -- MC variance of numerator
      #   var_se_MC_k  = n_s * var(vcov_mat_k)  -- MC variance of scaled sandwich vcov
      # Compare var_R_MC to sigma2S_true (analytic total) to get the total ratio.
      # Compare var_b_MC to vcov_diag_true to get the direct-channel ratio (= vcov_B).
      # If sig2S_B >> vcov_B, the A/B chain or nonlinear terms are responsible.
      var_R_MC <- n_s_eff * apply(cell_data$R_mat,    2L, stats::var, na.rm = TRUE)
      var_b_MC <- n_s_eff * apply(cell_data$beta_mat, 2L, stats::var, na.rm = TRUE)
      var_se_MC <- n_s_eff * apply(cell_data$vcov_mat, 2L, stats::var, na.rm = TRUE)

      row <- data.frame(
        model        = s$type,
        vcov         = s$vcov_name,
        deriv_method = deriv_method,
        n            = n_s_eff,
        n_success    = cell_data$n_success,
        term         = coef_terms,
        phi_bias     = phi_bias,
        beta_bias    = beta_bias,
        vcov_A       = vcov_A,
        vcov_B       = vcov_B,
        R_bias       = R_bias,
        sig2S_A      = sig2S_A,
        sig2S_B      = sig2S_B,
        cv_Vhat      = cv_Vhat,
        phi_var_A    = phi_var_A,
        phi_var_B    = phi_var_B,
        phi_cov_A    = phi_cov_A,
        var_R_MC     = var_R_MC,
        var_b_MC     = var_b_MC,
        var_se_MC    = var_se_MC,
        row.names    = NULL,
        stringsAsFactors = FALSE
      )
      all_metrics[[length(all_metrics) + 1L]] <- row
    }
  }

  summary_df <- do.call(rbind, all_metrics)
  saveRDS(summary_df, file.path(output.dir, paste0("calibration_summary_",  deriv_method, ".rds")))
  saveRDS(true_vals,  file.path(output.dir, paste0("calibration_truevals_", deriv_method, ".rds")))
  message("Simulation complete (deriv_method = '", deriv_method, "'). Results saved to: ", output.dir)
  invisible(summary_df)
}

#' Calibration Figures for RESI Variance Estimates
#'
#' Reads output from \code{\link{simCalibrationSim}} and produces one PDF per
#' model setting showing convergence of point estimates, covariance estimates,
#' RESI estimates, and RESI variance estimates to their population targets.
#'
#' @param output.dir Character, directory containing output from
#'   \code{\link{simCalibrationSim}}. Default \code{"resiCalibrationSim"}.
#' @param figures.dir Character, output directory for PDFs.
#'   Default \code{file.path(output.dir, "figures")}.
#' @param alpha Numeric, nominal CI level. Default 0.05.
#' @param deriv_method Character, type of derivative being evaluated. Package now only includes "extended" in resi implementation.
#'
#' @return Invisibly returns the summary data frame. Saves PDF figures to
#'   \code{figures.dir}.
#' @seealso \code{\link{simCalibrationSim}}, \code{\link{insurancePlasmodeSim}}
#' @importFrom splines ns
#' @importFrom sandwich vcovHC
#' @importFrom stats lm glm vcov binomial setNames
#' @importFrom grDevices pdf dev.off
#' @importFrom graphics plot lines points abline legend par axis plot.new layout
#' @export
simCalibrationFigures <- function(
    output.dir   = "resiCalibrationSim",
    figures.dir  = NULL,
    alpha        = 0.05,
    deriv_method = c("extended", "corrected", "original", "population", "population2", "zeroB", "zero_phi_cross")
) {
  deriv_method <- match.arg(deriv_method)
  if (is.null(figures.dir))
    figures.dir <- file.path(output.dir, "figures", deriv_method)
  dir.create(figures.dir, recursive = TRUE, showWarnings = FALSE)

  summary_path  <- file.path(output.dir, paste0("calibration_summary_",  deriv_method, ".rds"))
  truevals_path <- file.path(output.dir, paste0("calibration_truevals_", deriv_method, ".rds"))
  if (!file.exists(summary_path))
    stop("'", summary_path, "' not found. Run simCalibrationSim(deriv_method='", deriv_method, "') first.")
  if (!file.exists(truevals_path))
    stop("'", truevals_path, "' not found. Run simCalibrationSim(deriv_method='", deriv_method, "') first.")

  summary_df <- readRDS(summary_path)
  true_vals  <- readRDS(truevals_path)

  # Reconstruct minimal model_settings from true_vals names (label -> type/vcov_name)
  model_settings <- lapply(names(true_vals), function(lbl) {
    parts <- strsplit(lbl, "_", fixed = TRUE)[[1L]]
    list(type      = parts[1L],
         label     = lbl,
         vcov_name = parts[2L])
  })

  # ---- Figures ---------------------------------------------------------------
  # Abbreviate long term names and format for display in legends
  .abbrev <- function(x) {
    x <- sub("splines::ns\\(age, df = [0-9]+\\)", "ns(age)", x)
    x <- sub("regionnorthwest", "Reg:NW", x)
    x <- sub("regionsoutheast", "Reg:SE", x)
    x <- sub("regionsouthwest", "Reg:SW", x)
    x <- sub("regionnortheast", "Reg:NE", x)
    x <- sub(":sexmale$",       ":sex",   x)
    x
  }

  # Pretty-print setting label for titles (e.g. "lm_robust" -> "Robust LM")
  .fmt_lbl <- function(lbl) {
    parts <- strsplit(lbl, "_", fixed = TRUE)[[1L]]
    mod   <- toupper(parts[1L])
    vcov  <- if (length(parts) > 1L && parts[2L] == "robust") "Robust" else "Parametric"
    paste(vcov, mod)
  }

  .sim_pal <- c("#1F77B4", "#D62728", "#2CA02C", "#FF7F0E", "#9467BD",
                "#8C564B", "#E377C2", "#17BECF", "#BCBD22", "#7F7F7F",
                "#AEC7E8", "#98DF8A")
  n_vals_plot <- sort(unique(summary_df$n))

  for (s in model_settings) {
    lbl      <- s$label
    is_lm    <- (s$type == "lm")
    tv       <- true_vals[[lbl]]
    coef_terms <- tv$coef_terms
    n_terms    <- length(coef_terms)
    term_cols  <- setNames(.sim_pal[seq_len(n_terms)], coef_terms)

    sub <- summary_df[summary_df$model == s$type &
                        summary_df$vcov  == s$vcov_name, ]
    if (nrow(sub) == 0L) next

    fig_path <- file.path(figures.dir, paste0("calibration_", lbl, ".pdf"))

    # Layout: [bias_theta | vcov_ratios] / [bias_R | sig2S_ratios] / [legend]
    # Panel 1: Bias of theta (phi + beta_k lines; phi as a special dotted line)
    # Panel 2: vcov ratios  (solid=A, dashed=B, one color per term)
    # Panel 3: Bias of R_k
    # Panel 4: sigma2S ratios (solid=A, dashed=B, dotted=Th1)
    # Panel 5: legend (spans full width)
    is_glm <- !is_lm
    grDevices::pdf(fig_path, width = 9, height = 8.5)
    graphics::layout(
      matrix(c(1, 2, 3, 4, 5, 5), nrow = 3L, byrow = TRUE),
      widths  = c(4.0, 4.0),
      heights = c(3.0, 3.0, 2.0)
    )

    # --- Panel 1: Bias(theta) ---
    # Collect bias range for beta + phi
    all_bias_theta <- unlist(lapply(coef_terms, function(tm) {
      sub[sub$term == tm, "beta_bias"]
    }))
    if (is_lm) {
      phi_biases <- sub[sub$term == coef_terms[1L], "phi_bias"]
      all_bias_theta <- c(all_bias_theta, phi_biases)
    }
    ylim1 <- range(c(all_bias_theta, 0), na.rm = TRUE)
    ylim1 <- ylim1 + diff(ylim1) * c(-0.06, 0.06)
    if (is_glm) ylim1 <- c(-0.2, 0.2)   # clip glm bias panel

    graphics::par(mar = c(3.2, 3.5, 2.2, 0.5), mgp = c(2.0, 0.5, 0))
    graphics::plot(NULL, xlim = range(n_vals_plot), ylim = ylim1,
                   xlab = "n",
                   ylab = expression(paste("Calibration of ", theta)),
                   main = bquote(.(paste0(.fmt_lbl(lbl), ": Calibration of ")) * theta),
                   log = "x", xaxt = "n", bty = "l")
    graphics::axis(1L, at = n_vals_plot, labels = n_vals_plot, las = 2L,
                   mgp = c(1.7, 0.35, 0))
    graphics::abline(h = 0, lty = 2L, col = "gray40")

    for (tm in coef_terms) {
      d <- sub[sub$term == tm, ]
      d <- d[order(d$n), ]
      graphics::lines(d$n,  d$beta_bias, col = term_cols[[tm]], lwd = 1.5)
      graphics::points(d$n, d$beta_bias, col = term_cols[[tm]], pch = 16L, cex = 0.7)
    }
    if (is_lm) {
      d <- sub[sub$term == coef_terms[1L], ]
      d <- d[order(d$n), ]
      graphics::lines(d$n,  d$phi_bias, col = "#000000", lwd = 2L, lty = 3L)
      graphics::points(d$n, d$phi_bias, col = "#000000", pch = 17L, cex = 0.8)
    }

    # --- Panel 2: normalized-bias covariance calibration ---
    # For each n, load raw cell, build full theta matrix (phi + beta for lm),
    # and compute the normalized bias of every covariance element:
    #   diff[j,k] = (Sigma_analytic[j,k] - n_s*cov_MC[j,k]) /
    #               sqrt(n_s*var_MC[j,j] * n_s*var_MC[k,k])
    # = (Sigma_A[j,k] - Sigma_MC[j,k]) / sqrt(Sigma_MC[j,j] * Sigma_MC[k,k])
    # Diagonal: diff[k,k] = Sigma_A[k,k]/Sigma_MC[k,k] - 1 = 1/vcov_B_k - 1
    # Off-diagonal: normalized difference in covariance (target 0)
    # Includes phi row/col for lm; colored lines = diagonal, gray = off-diagonal.
    raw_dir_p   <- file.path(output.dir, "raw", deriv_method)
    Sig_A_mat   <- tv$vcov_full_true_mat    # analytic Sigma_theta (includes phi if lm)
    lm_has_phi  <- is_lm && !is.null(Sig_A_mat) && "phi" %in% rownames(Sig_A_mat)
    all_labels  <- if (!is.null(Sig_A_mat)) rownames(Sig_A_mat) else coef_terms
    n_all       <- length(all_labels)

    diag_diff_list <- vector("list", length(n_vals_plot))
    off_diff_list  <- vector("list", length(n_vals_plot))
    for (i_n in seq_along(n_vals_plot)) {
      nv <- n_vals_plot[i_n]
      rp <- file.path(raw_dir_p, paste0(lbl, "_n", nv, ".rds"))
      if (!file.exists(rp)) next
      cd <- readRDS(rp)
      if (is.null(cd$beta_mat) || nrow(cd$beta_mat) < 3L) next
      n_s_p <- cd$n
      bm    <- cd$beta_mat
      if (!all(coef_terms %in% colnames(bm))) next
      bm <- bm[, coef_terms, drop = FALSE]
      # Full theta matrix: prepend phi for lm
      theta_mc <- if (lm_has_phi && !is.null(cd$phi_vec))
        cbind(phi = cd$phi_vec, bm) else bm
      if (is.null(Sig_A_mat)) next
      Sig_MC <- n_s_p * stats::cov(theta_mc, use = "pairwise.complete.obs")
      # Normalized bias: (Sig_A - Sig_MC) / sqrt(Sig_MC_jj * Sig_MC_kk)
      sd_MC   <- sqrt(pmax(diag(Sig_MC), .Machine$double.eps))
      Sig_MC_nm <- Sig_A_mat  # same dimension; use A for name alignment
      if (nrow(Sig_MC) == nrow(Sig_A_mat)) {
        diff_mat <- (Sig_A_mat - Sig_MC) /
          outer(sd_MC, sd_MC)  # element-wise divide by product of MC SDs
      } else {
        diff_mat <- matrix(NA_real_, nrow(Sig_A_mat), ncol(Sig_A_mat))
      }
      rownames(diff_mat) <- colnames(diff_mat) <- all_labels
      diag_diff_list[[i_n]] <- diag(diff_mat)
      off_diff_list[[i_n]]  <- diff_mat[upper.tri(diff_mat)]
    }

    diag_diff_mat <- do.call(rbind, lapply(diag_diff_list, function(x)
      if (is.null(x)) rep(NA_real_, n_all) else x[all_labels]))
    off_diff_vals <- unlist(off_diff_list)
    all_diff_vals <- c(as.vector(diag_diff_mat), off_diff_vals)
    ylim2 <- range(c(all_diff_vals, 0), na.rm = TRUE)
    ylim2 <- ylim2 + diff(ylim2) * c(-0.06, 0.06)
    if (!is.finite(ylim2[1L])) ylim2 <- c(-1, 2)
    if (is_glm) ylim2[2L] <- min(ylim2[2L], 3)

    graphics::par(mar = c(3.2, 3.5, 2.2, 0.5), mgp = c(2.0, 0.5, 0))
    graphics::plot(NULL, xlim = range(n_vals_plot), ylim = ylim2,
                   xlab = "n",
                   ylab = expression(paste("(", Sigma[A] - Sigma[MC], ") / ",
                                           sqrt(Sigma[MC*","*jj] %*% Sigma[MC*","*kk]))),
                   main = paste0(.fmt_lbl(lbl), ":  Covariance calibration"),
                   log = "x", xaxt = "n", bty = "l", font.main = 1L)
    graphics::axis(1L, at = n_vals_plot, labels = n_vals_plot, las = 2L,
                   mgp = c(1.7, 0.35, 0))
    graphics::abline(h = 0, lty = 2L, col = "gray40")

    # Off-diagonal: thin gray
    n_pairs <- if (n_all > 1L) ncol(utils::combn(n_all, 2L)) else 0L
    for (pair in seq_len(n_pairs)) {
      vals <- vapply(off_diff_list, function(x)
        if (is.null(x) || length(x) < pair) NA_real_ else x[pair], numeric(1L))
      if (any(is.finite(vals)))
        graphics::lines(n_vals_plot, vals, col = "gray70", lwd = 0.8)
    }

    # Diagonal: colored per term (beta), black for phi
    for (k_idx in seq_len(n_all)) {
      lbl_k <- all_labels[k_idx]
      vals  <- diag_diff_mat[, k_idx]
      if (!any(is.finite(vals))) next
      if (lbl_k == "phi") {
        graphics::lines(n_vals_plot,  vals, col = "#000000", lwd = 2L)
        graphics::points(n_vals_plot, vals, col = "#000000", pch = 15L, cex = 0.8)
      } else if (lbl_k %in% names(term_cols)) {
        graphics::lines(n_vals_plot,  vals, col = term_cols[[lbl_k]], lwd = 1.5)
        graphics::points(n_vals_plot, vals, col = term_cols[[lbl_k]], pch = 16L, cex = 0.7)
      }
    }

    # --- Panel 3: Bias(R) ---
    all_R_bias <- unlist(lapply(coef_terms, function(tm) sub[sub$term == tm, "R_bias"]))
    ylim3 <- range(c(all_R_bias, 0), na.rm = TRUE)
    ylim3 <- ylim3 + diff(ylim3) * c(-0.06, 0.06)
    if (is_glm) ylim3 <- c(-0.1, 0.1)   # clip glm bias panel

    graphics::par(mar = c(3.2, 3.5, 2.2, 0.5), mgp = c(2.0, 0.5, 0))
    graphics::plot(NULL, xlim = range(n_vals_plot), ylim = ylim3,
                   xlab = "n",
                   ylab = expression(paste("Calibration of ", R)),
                   main = bquote(.(paste0(.fmt_lbl(lbl), ": Calibration of ")) * R),
                   log = "x", xaxt = "n", bty = "l")
    graphics::axis(1L, at = n_vals_plot, labels = n_vals_plot, las = 2L,
                   mgp = c(1.7, 0.35, 0))
    graphics::abline(h = 0, lty = 2L, col = "gray40")

    for (tm in coef_terms) {
      d <- sub[sub$term == tm, ]
      d <- d[order(d$n), ]
      graphics::lines(d$n,  d$R_bias, col = term_cols[[tm]], lwd = 1.5)
      graphics::points(d$n, d$R_bias, col = term_cols[[tm]], pch = 16L, cex = 0.7)
    }

    # --- Panel 4: sigma2S ratios (true-to-MC solid, init-to-MC dashed) ---
    # Compute ylim including BOTH sig2S_B and sig2S_Th1_vals
    all_sig2S <- unlist(lapply(coef_terms, function(tm) {
      d <- sub[sub$term == tm, ]
      r_Th1 <- tv$sigma2S_true[tm] / tv$sigma2S_true_Th1[tm]
      sig2S_Th1_v <- d$sig2S_B * r_Th1
      c(d$sig2S_B, sig2S_Th1_v)
    }))
    ylim4 <- range(c(1/all_sig2S, 1), na.rm = TRUE)
    ylim4 <- ylim4 + diff(ylim4) * c(-0.06, 0.10)
    if (is_glm) ylim4[2L] <- max(ylim4[2L], 3)

    graphics::par(mar = c(3.2, 3.5, 2.2, 0.5), mgp = c(2.0, 0.5, 0))
    graphics::plot(NULL, xlim = range(n_vals_plot), ylim = ylim4,
                   xlab = "n",
                   ylab = expression(paste(sigma[R]^2, " ratio")),
                   main = bquote(.(paste0(.fmt_lbl(lbl), ":  ")) *
                                   sigma[R]^2 ~ "calibration"),
                   log = "x", xaxt = "n", bty = "l")
    graphics::axis(1L, at = n_vals_plot, labels = n_vals_plot, las = 2L,
                   mgp = c(1.7, 0.35, 0))
    graphics::abline(h = 1, lty = 2L, col = "gray40")

    for (tm in coef_terms) {
      d    <- sub[sub$term == tm, ]
      d    <- d[order(d$n), ]
      # sig2S_Th1 = MC / sigma2S_true_Th1 = (sig2S_B * sigma2S_true) / sigma2S_true_Th1
      r_Th1 <- tv$sigma2S_true[tm] / tv$sigma2S_true_Th1[tm]
      sig2S_Th1_vals <- 1/(d$sig2S_B * r_Th1)
      # Solid: sig2S_B = sigma2S_true /sigma2S_MC
      # Dashed: sig2S_Th1 = sigma2S_true_init/sigma2S_MC   ("init-to-MC" ratio)
      graphics::lines(d$n, 1/d$sig2S_B,      col = term_cols[[tm]], lwd = 1.8, lty = 1L)
      graphics::lines(d$n, sig2S_Th1_vals, col = term_cols[[tm]], lwd = 1.5, lty = 2L)
      graphics::points(d$n, 1/d$sig2S_B,      col = term_cols[[tm]], pch = 16L, cex = 0.7)
      graphics::points(d$n, sig2S_Th1_vals, col = term_cols[[tm]], pch = 1L,  cex = 0.7)
    }

    # --- Panel 5: Legend ---
    short_terms <- .abbrev(coef_terms)
    leg_ncol <- ceiling(n_terms / 3L)

    graphics::par(mar = c(0.2, 0.5, 0.2, 0.5), cex = 0.90)
    graphics::plot.new()

    # Left: coefficient colour key (coefficients only, without phi)
    graphics::legend(x = 0.01, y = 1.0,
                     legend = short_terms,
                     col    = unname(term_cols[coef_terms]),
                     pch    = rep(16L, n_terms),
                     lty    = rep(1L, n_terms),
                     lwd    = 1.5,
                     bty    = "n", cex = 0.85, xjust = 0L, yjust = 1L,
                     ncol   = leg_ncol,
                     title  = "Coefficient (all panels)", title.font = 2L)

    # If lm: phi below coefficients
    if (is_lm) {
      graphics::legend(x = 0.01, y = 0.65,
                       legend = expression(phi ~ "(error variance)"),
                       col    = "#000000", pch = 15L, lty = 1L, lwd = 1.5,
                       bty    = "n", cex = 0.85, xjust = 0L, yjust = 1L)
    }

    # Right: line-style key
    graphics::legend(x = 0.60, y = 1.0,
                     legend = c(
                       "Diagonal: vcov ratio - 1",
                       "Off-diagonal: norm. bias (gray)",
                       expression(sigma[R]^2 ~ ": MC / true  (solid)"),
                       expression(sigma[R]^2 ~ ": MC / init  (dashed)")
                     ),
                     col    = c("black", "gray70", "gray30", "gray30"),
                     lty    = c(1L, 1L, 1L, 2L),
                     pch    = c(15L, NA_integer_, 16L, 1L),
                     lwd    = c(1.5, 0.8, 1.8, 1.5),
                     bty    = "n", cex = 0.85, xjust = 0L, yjust = 1L,
                     title  = "Line style key", title.font = 2L)

    grDevices::dev.off()
    message("Saved: ", fig_path)
  }

  message("Calibration figures saved to: ", figures.dir)

  # ---- Structured return value -----------------------------------------------
  # For each setting: matrices of size (n_vals x terms) for each metric,
  # plus a phi_bias vector (length n_vals) for lm settings.
  structured <- lapply(model_settings, function(s) {
    lbl <- s$label
    sub <- summary_df[summary_df$model == s$type &
                        summary_df$vcov  == s$vcov_name, ]
    if (nrow(sub) == 0L) return(NULL)
    tv         <- true_vals[[lbl]]
    terms_here <- tv$coef_terms
    n_here     <- sort(unique(sub$n))

    # Helper: reshape one metric column into a (n_vals x terms) matrix
    .mat <- function(col) {
      m <- matrix(NA_real_, nrow = length(n_here), ncol = length(terms_here),
                  dimnames = list(n = as.character(n_here), term = terms_here))
      for (ni in seq_along(n_here)) {
        for (tm in terms_here) {
          v <- sub[sub$n == n_here[ni] & sub$term == tm, col]
          if (length(v) == 1L) m[ni, tm] <- v
        }
      }
      m
    }

    out <- list(
      n              = n_here,
      n_success      = setNames(
        vapply(n_here, function(ni) {
          v <- sub[sub$n == ni & sub$term == terms_here[1L], "n_success"]
          if (length(v)) v[1L] else NA_integer_
        }, integer(1L)), as.character(n_here)),
      # (1) Bias of raw coefficients
      beta_bias      = .mat("beta_bias"),
      # (2) vcov: analytical estimator ratio
      vcov_A         = .mat("vcov_A"),
      # (3) vcov: empirical-variance calibration ratio
      vcov_B         = .mat("vcov_B"),
      # (4) Bias of R
      R_bias         = .mat("R_bias"),
      # (5) sigma2S: analytical estimator ratio
      sig2S_A          = .mat("sig2S_A"),
      # (6) sigma2S: empirical-variance calibration ratio (key CI check)
      sig2S_B          = .mat("sig2S_B"),
      # (7) sigma2S: empirical variance / Th1 full-delta-method reference
      sig2S_Th1        = local({
        r <- tv$sigma2S_true[terms_here] / tv$sigma2S_true_Th1[terms_here]
        m <- .mat("sig2S_B")
        sweep(m, 2L, r, "*")
      }),
      # Reference values used for normalisation
      R_true              = tv$R_true,
      sigma2S_true        = tv$sigma2S_true,
      sigma2S_true_Th1    = tv$sigma2S_true_Th1,
      vcov_diag_true      = tv$vcov_diag_true[terms_here]
    )
    if (s$type == "lm") {
      phi_row <- sub[sub$term == terms_here[1L], ]
      phi_row <- phi_row[order(phi_row$n), ]
      out$phi_bias <- setNames(phi_row$phi_bias, as.character(phi_row$n))
      out$phi_true <- tv$phi_true
    }
    out
  })
  names(structured) <- sapply(model_settings, `[[`, "label")
  structured        <- Filter(Negate(is.null), structured)

  invisible(list(summary_table = summary_df, by_setting = structured))
}


#' Per-Term CI Method Comparison Figures
#'
#' Reads simulation summary tables from multiple output directories (one per CI
#' method) and produces PDF figures comparing CI methods across sample sizes.
#' For each (model type x variance estimator x table) combination two PDFs are
#' saved: an estimator comparison (Bias, MSE) and a CI comparison (SE
#' calibration, coverage, width).
#'
#' @param output.dirs Named character vector mapping CI method labels to their
#'   simulation output directories. Default:
#'   \code{c(boot = "resiBootSim", normal = "resiAsympNormalSim",
#'   qf = "resiAsympQFSim", cf = "resiAsympCFSim")}.
#'   Directories that do not exist are silently skipped.
#' @param figures.dir Character, directory where comparison figures are saved.
#'   Default: \code{file.path(output.dirs[1], "figures", "method_comparison")}.
#' @param alpha Numeric, nominal CI level used for coverage reference lines.
#'   Default 0.05.
#' @param fixed.knots Logical. Must match the value used in the original
#'   \code{\link{insurancePlasmodeSim}} call. Default \code{FALSE}.
#'
#' @return Invisibly returns the combined summary \code{data.frame}. Saves
#'   \code{estimator_<model>_<vcov>_<table>.pdf},
#'   \code{compare_<model>_<vcov>_<table>.pdf}, and
#'   \code{covquant_<model>_<vcov>_<table>.pdf} to \code{figures.dir}.
#' @seealso \code{\link{insurancePlasmodeSim}}, \code{\link{simFigures}},
#'   \code{\link{simEstimatorFigures}}
#' @importFrom grDevices pdf dev.off
#' @importFrom graphics plot lines points abline legend par axis plot.new mtext
#' @importFrom splines ns
#' @importFrom sandwich vcovHC
#' @importFrom stats lm glm vcov binomial density setNames sd
#' @export
simCompareMethodsFigures <- function(
    output.dirs = c(Bootstrap   = "resiBootSim",
                    Normal = "resiAsympNormalSim",
                    QF    = "resiAsympQFSim",
                    CF     = "resiAsympCFSim"),
    figures.dir = NULL,
    alpha       = 0.05,
    fixed.knots = FALSE) {

  method_names <- names(output.dirs)
  if (is.null(method_names) || any(method_names == ""))
    stop("output.dirs must be a named vector (names = CI method labels)")

  if (is.null(figures.dir))
    figures.dir <- file.path(output.dirs[[1]], "figures", "method_comparison")
  dir.create(figures.dir, recursive = TRUE, showWarnings = FALSE)

  # Read and merge tables, skipping missing directories
  tables <- lapply(seq_along(output.dirs), function(i) {
    f <- file.path(output.dirs[[i]], "summary_table.rds")
    if (!file.exists(f)) {
      warning("summary_table.rds not found in: ", output.dirs[[i]], " -- skipping")
      return(NULL)
    }
    tbl <- readRDS(f)
    tbl$ci_method <- method_names[[i]]
    tbl
  })
  tables <- Filter(Negate(is.null), tables)
  if (length(tables) == 0L) stop("No valid summary tables found")
  combined <- do.call(rbind, tables)

  # Keep only methods that were successfully loaded, in supplied order
  avail_methods <- intersect(method_names, unique(combined$ci_method))

  # High-contrast palette and shapes per method
  base_cols   <- c("#1F77B4", "#D62728", "#2CA02C", "#FF7F0E", "#9467BD")
  method_cols <- setNames(base_cols[seq_along(avail_methods)], avail_methods)
  n_vals <- sort(unique(combined$n))
  z_ref  <- qnorm(1 - alpha / 2)

  # Compute true RESI from full insurance dataset for per-term title labels
  true_S_lookup <- tryCatch({
    insurance_full <- RESI::insurance
    if (fixed.knots) {
      .age_knots_ts <- stats::quantile(insurance_full$age, c(1/3, 2/3))
      .age_bk_ts    <- range(insurance_full$age)
      lm_form_ts    <- eval(bquote(
        log10(charges) ~ splines::ns(age, knots = .(.age_knots_ts),
                                     Boundary.knots = .(.age_bk_ts)) *
          sex + bmi + smoker + region))
      glm_form_ts   <- eval(bquote(
        I(charges > 15000) ~ splines::ns(age, knots = .(.age_knots_ts),
                                         Boundary.knots = .(.age_bk_ts)) *
          sex + bmi + smoker + region))
    } else {
      lm_form_ts  <- log10(charges) ~ splines::ns(age, df = 3) *
        sex + bmi + smoker + region
      glm_form_ts <- I(charges > 15000) ~ splines::ns(age, df = 3) *
        sex + bmi + smoker + region
    }
    ts_settings <- list(
      list(mtype="lm",  vtype="parametric", form=lm_form_ts,  fam=NULL,
           robust=FALSE),
      list(mtype="lm",  vtype="robust",     form=lm_form_ts,  fam=NULL,
           robust=TRUE),
      list(mtype="glm", vtype="parametric", form=glm_form_ts,
           fam=stats::binomial(), robust=FALSE),
      list(mtype="glm", vtype="robust",     form=glm_form_ts,
           fam=stats::binomial(), robust=TRUE)
    )
    result_ts <- list()
    for (s_ts in ts_settings) {
      key_ts <- paste(s_ts$mtype, s_ts$vtype, sep = "_")
      m_ts <- tryCatch({
        if (s_ts$mtype == "lm") {
          m <- stats::lm(s_ts$form, data = insurance_full)
          m$call[["formula"]] <- s_ts$form; m
        } else {
          m <- stats::glm(s_ts$form, data = insurance_full, family = s_ts$fam)
          m$call[["formula"]] <- s_ts$form
          m$call[["family"]]  <- s_ts$fam; m
        }
      }, error = function(e) NULL)
      if (is.null(m_ts)) next
      tv_ts <- if (s_ts$robust)
        function(x) sandwich::vcovHC(x, type = "HC0") else stats::vcov
      pe_ts <- tryCatch(
        resi_pe(m_ts, vcovfunc = tv_ts, unbiased = TRUE),
        error = function(e) NULL)
      if (!is.null(pe_ts)) result_ts[[key_ts]] <- pe_ts
    }
    result_ts
  }, error = function(e) {
    warning("Could not compute true RESI for titles: ", conditionMessage(e))
    list()
  })

  get_true_S <- function(mtype, vtype, tbl_name, term) {
    key_ts <- paste(mtype, vtype, sep = "_")
    pe_ts  <- true_S_lookup[[key_ts]]
    if (is.null(pe_ts)) return(NA_real_)
    tbl_ts <- pe_ts[[tbl_name]]
    if (is.null(tbl_ts) || !(term %in% rownames(tbl_ts))) return(NA_real_)
    tbl_ts[term, "RESI"]
  }

  row_h <- 3.2
  leg_h <- 0.55

  for (mtype in c("lm", "glm")) {
    for (vtype in c("parametric", "robust")) {
      for (tbl_name in c("anova", "coefficients")) {

        sub <- combined[
          combined$model == mtype &
          combined$vcov  == vtype &
          combined$table == tbl_name, ]
        if (nrow(sub) == 0L) next

        terms   <- unique(sub$term)
        n_terms <- length(terms)

        # Helper: draw method lines/points for a given metric column
        draw_method_lines <- function(term_data, metric) {
          for (meth in avail_methods) {
            md <- term_data[term_data$ci_method == meth, ]
            md <- md[order(md$n), ]
            if (nrow(md) == 0L) next
            graphics::lines(md$n,  md[[metric]],
                            col = method_cols[[meth]], lwd = 2L)
            graphics::points(md$n, md[[metric]],
                             col = method_cols[[meth]], pch = 16L, cex = 0.8)
          }
        }

        # ======================================================
        # PDF 1: Estimator figures (Bias + MSE only)
        # One row per term, legend at bottom spanning both columns
        # ======================================================
        fig_path_est <- file.path(
          figures.dir,
          paste0("estimator_", mtype, "_", vtype, "_", tbl_name, ".pdf")
        )
        lay_mat_est <- matrix(0L, nrow = n_terms + 1L, ncol = 2L)
        for (ti in seq_len(n_terms)) {
          lay_mat_est[ti, 1L] <- 2L * (ti - 1L) + 1L   # Bias
          lay_mat_est[ti, 2L] <- 2L * (ti - 1L) + 2L   # MSE
        }
        leg_id_est <- 2L * n_terms + 1L
        lay_mat_est[n_terms + 1L, ] <- leg_id_est

        grDevices::pdf(fig_path_est, width = 7,
                       height = row_h * n_terms + leg_h)
        graphics::layout(lay_mat_est,
                         widths  = c(3.2, 3.2),
                         heights = c(rep(row_h, n_terms), leg_h))

        for (ti in seq_along(terms)) {
          term      <- terms[[ti]]
          term_data <- sub[sub$term == term, ]
          true_s    <- get_true_S(mtype, vtype, tbl_name, term)
          term_title <- if (is.finite(true_s))
            sprintf("%s: S=%.3f", term, true_s) else term

          est_rows <- if (mtype == "glm") term_data$n >= 500 else
            rep(TRUE, nrow(term_data))

          # ---- Bias ----
          ylim <- range(c(term_data[est_rows, "bias"], -0.02, 0.01), na.rm = TRUE)
          graphics::par(mar = c(3.2, 3.0, 2.2, 0.5), mgp = c(1.8, 0.5, 0))
          graphics::plot(NULL, xlim = range(n_vals), ylim = ylim,
                         xlab = "Sample Size", ylab = "Bias",
                         main = term_title, log = "x", xaxt = "n", bty = "l")
          graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                         mgp = c(1.7, 0.35, 0))
          graphics::abline(h = 0, lty = 2L, col = "gray40")
          draw_method_lines(term_data, "bias")

          # ---- MSE ----
          ylim <- range(term_data[est_rows, "mse"], na.rm = TRUE)
          graphics::par(mar = c(3.2, 3.0, 2.2, 0.5), mgp = c(1.8, 0.5, 0))
          graphics::plot(NULL, xlim = range(n_vals), ylim = ylim,
                         xlab = "Sample Size", ylab = "MSE",
                         main = "", log = "x", xaxt = "n", bty = "l")
          graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                         mgp = c(1.7, 0.35, 0))
          draw_method_lines(term_data, "mse")
        }  # end term loop (estimator PDF)

        # ---- Bottom legend (flat/horizontal) ----
        graphics::par(mar = c(0.2, 0.2, 0.2, 0.2))
        graphics::plot.new()
        graphics::legend(
          "center",
          legend = paste0(avail_methods, "    "),
          col    = method_cols[avail_methods],
          pch    = 16L, lwd = 2L, lty = 1L,
          bty    = "n", cex = 0.9,
          ncol   = length(avail_methods),
          title  = "CI method", title.font = 2L
        )
        grDevices::dev.off()
        message("Saved: ", fig_path_est)

        # ======================================================
        # PDF 2: CI comparison (SE cal + stacked coverage + Width)
        # Two sub-rows per term (upper/lower cov stacked), legend at bottom
        # Layout per term:
        #   sub-row 1: [SE(span), UpperCov, Width(span)]
        #   sub-row 2: [SE(span), LowerCov, Width(span)]
        # Drawing order per layout reading: SE -> UpperCov -> Width -> LowerCov
        # ======================================================
        fig_path_ci <- file.path(
          figures.dir,
          paste0("compare_", mtype, "_", vtype, "_", tbl_name, ".pdf")
        )
        n_rows_ci  <- 2L * n_terms + 1L
        lay_mat_ci <- matrix(0L, nrow = n_rows_ci, ncol = 3L)
        panel_id   <- 1L
        for (ti in seq_len(n_terms)) {
          r1 <- 2L * ti - 1L; r2 <- 2L * ti
          se_id   <- panel_id
          ucov_id <- panel_id + 1L
          wid_id  <- panel_id + 2L
          lcov_id <- panel_id + 3L
          lay_mat_ci[r1, 1L] <- se_id;   lay_mat_ci[r2, 1L] <- se_id   # SE spans
          lay_mat_ci[r1, 2L] <- ucov_id                                  # upper cov
          lay_mat_ci[r2, 2L] <- lcov_id                                  # lower cov
          lay_mat_ci[r1, 3L] <- wid_id;  lay_mat_ci[r2, 3L] <- wid_id  # Width spans
          panel_id <- panel_id + 4L
        }
        leg_id_ci <- panel_id
        lay_mat_ci[n_rows_ci, ] <- leg_id_ci

        sub_h <- row_h / 2   # height of each coverage sub-row (half of a full row)
        grDevices::pdf(fig_path_ci, width = 11,
                       height = row_h * n_terms + leg_h)
        graphics::layout(lay_mat_ci,
                         widths  = c(3.5, 3.0, 3.0),
                         heights = c(rep(c(sub_h, sub_h), n_terms), leg_h))

        for (ti in seq_along(terms)) {
          term      <- terms[[ti]]
          term_data <- sub[sub$term == term, ]
          true_s    <- get_true_S(mtype, vtype, tbl_name, term)
          term_title <- if (is.finite(true_s))
            sprintf("%s: S=%.3f", term, true_s) else term

          # Shared coverage y-range (same scale for upper + lower)
          # No n-based clipping: use all sample sizes for both lm and glm.
          cov_vals <- c(term_data[["upper_coverage"]], term_data[["lower_coverage"]])
          cov_ylim <- range(c(cov_vals, 1 - alpha/2 - 0.02, 1.01), na.rm = TRUE)

          # Width y-range: clip extreme values (1.5 for glm, 10 for lm)
          max_wid  <- if (mtype == "glm") 1.5 else 10
          wid_vals <- term_data[["width"]]
          wid_ylim <- range(wid_vals[wid_vals <= max_wid], na.rm = TRUE)

          # Drawing order: SE (spans) -> UpperCov -> Width (spans) -> LowerCov

          # ---- Panel 1: SE calibration (spans both sub-rows, term title here) ----
          se_comp <- do.call(rbind, lapply(avail_methods, function(meth) {
            md <- term_data[term_data$ci_method == meth, ]
            md <- md[order(md$n), ]
            data.frame(
              ci_method  = meth,
              n          = md$n,
              emp_se     = sqrt(md$n) * md$empirical_sd,
              half_width = sqrt(md$n) * md$width / (2 * z_ref),
              stringsAsFactors = FALSE
            )
          }))
          se_vals <- c(se_comp$emp_se, se_comp$half_width)
          max_se  <- if (mtype == "glm") 3 else 10
          lim_se  <- range(se_vals[se_vals <= max_se], na.rm = TRUE)
          graphics::par(mar = c(3.2, 3.2, 2.2, 0.5), mgp = c(1.8, 0.5, 0))
          graphics::plot(
            NULL, xlim = lim_se, ylim = lim_se,
            xlab = expression(sqrt(n) %*% " Empirical SE"),
            ylab = expression(sqrt(n) %*% " CI Half-Width / " * z[alpha/2]),
            main = term_title, bty = "l", asp = 1
          )
          graphics::abline(0, 1, lty = 2L, col = "gray40")
          for (meth in avail_methods) {
            md_se <- se_comp[se_comp$ci_method == meth, ]
            md_se <- md_se[order(md_se$n), ]
            if (nrow(md_se) == 0L) next
            graphics::points(md_se$emp_se[1L], md_se$half_width[1L],
                             col = method_cols[[meth]], pch = 16L, cex = 1.1)
            if (nrow(md_se) >= 2L) {
              for (i in seq_len(nrow(md_se) - 1L)) {
                graphics::arrows(
                  md_se$emp_se[i],     md_se$half_width[i],
                  md_se$emp_se[i+1L],  md_se$half_width[i+1L],
                  col = method_cols[[meth]], lwd = 1.5, length = 0.07, angle = 20L
                )
              }
            }
          }

          # ---- Panel 2: Upper coverage (top sub-row, no x-axis) ----
          graphics::par(mar = c(0.3, 3.0, 2.2, 0.5), mgp = c(1.8, 0.45, 0))
          graphics::plot(NULL, xlim = range(n_vals), ylim = cov_ylim,
                         xlab = "", ylab = "Upper Cov.",
                         main = "", log = "x", xaxt = "n", bty = "l")
          graphics::abline(h = 1 - alpha/2, lty = 2L, col = "gray40")
          draw_method_lines(term_data, "upper_coverage")

          # ---- Panel 3: Width (spans both sub-rows) ----
          graphics::par(mar = c(3.2, 3.0, 2.2, 0.5), mgp = c(1.8, 0.5, 0))
          graphics::plot(NULL, xlim = range(n_vals), ylim = wid_ylim,
                         xlab = "Sample Size", ylab = "CI Width",
                         main = "", log = "x", xaxt = "n", bty = "l")
          graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                         mgp = c(1.7, 0.35, 0))
          draw_method_lines(term_data, "width")

          # ---- Panel 4: Lower coverage (bottom sub-row, with x-axis) ----
          graphics::par(mar = c(2.8, 3.0, 0.3, 0.5), mgp = c(1.8, 0.45, 0))
          graphics::plot(NULL, xlim = range(n_vals), ylim = cov_ylim,
                         xlab = "Sample Size", ylab = "Lower Cov.",
                         main = "", log = "x", xaxt = "n", bty = "l")
          graphics::axis(1, at = n_vals, labels = n_vals, las = 2L,
                         mgp = c(1.7, 0.35, 0))
          graphics::abline(h = 1 - alpha/2, lty = 2L, col = "gray40")
          draw_method_lines(term_data, "lower_coverage")
        }  # end term loop (CI PDF)

        # ---- Bottom legend (flat/horizontal) ----
        graphics::par(mar = c(0.2, 0.2, 0.2, 0.2))
        graphics::plot.new()
        graphics::legend(
          "center",
          legend = paste0(avail_methods, "    "),
          col    = method_cols[avail_methods],
          pch    = 16L, lwd = 2L, lty = 1L,
          bty    = "n", cex = 0.9,
          ncol   = length(avail_methods),
          title  = "CI method", title.font = 2L
        )
        grDevices::dev.off()
        message("Saved: ", fig_path_ci)
      }
    }
  }

  # ============================================================
  #  Coverage-quantile figures
  #  (S_true - LCI) / (UCI - LCI) density curves
  #  One PDF per (model x vcov x table): rows = terms, columns = n values
  # ============================================================
  message("Generating coverage-quantile figures...")

  insurance    <- RESI::insurance
  ci_lo_col_cq <- paste0(alpha / 2 * 100, "%")
  ci_hi_col_cq <- paste0((1 - alpha / 2) * 100, "%")

  if (fixed.knots) {
    .age_knots    <- stats::quantile(insurance$age, c(1/3, 2/3))
    .age_bk       <- range(insurance$age)
    lm_formula_cq  <- eval(bquote(
      log10(charges) ~ splines::ns(age, knots = .(.age_knots),
                                   Boundary.knots = .(.age_bk)) *
        sex + bmi + smoker + region))
    glm_formula_cq <- eval(bquote(
      I(charges > 15000) ~ splines::ns(age, knots = .(.age_knots),
                                       Boundary.knots = .(.age_bk)) *
        sex + bmi + smoker + region))
  } else {
    lm_formula_cq  <- log10(charges) ~ splines::ns(age, df = 3) *
      sex + bmi + smoker + region
    glm_formula_cq <- I(charges > 15000) ~ splines::ns(age, df = 3) *
      sex + bmi + smoker + region
  }

  cq_settings <- list(
    list(type = "lm",  key = "lm_parametric",
         formula = lm_formula_cq,  family = NULL,
         vcovfunc = stats::vcov,         robust = FALSE),
    list(type = "lm",  key = "lm_robust",
         formula = lm_formula_cq,  family = NULL,
         vcovfunc = sandwich::vcovHC,    robust = TRUE),
    list(type = "glm", key = "glm_parametric",
         formula = glm_formula_cq, family = stats::binomial(),
         vcovfunc = stats::vcov,         robust = FALSE),
    list(type = "glm", key = "glm_robust",
         formula = glm_formula_cq, family = stats::binomial(),
         vcovfunc = sandwich::vcovHC,    robust = TRUE)
  )

  # Compute true RESI from full insurance dataset
  cq_true <- lapply(cq_settings, function(s) {
    full_mod <- tryCatch({
      if (s$type == "lm") {
        m <- stats::lm(s$formula, data = insurance)
        m$call[["formula"]] <- s$formula; m
      } else {
        m <- stats::glm(s$formula, data = insurance, family = s$family)
        m$call[["formula"]] <- s$formula
        m$call[["family"]]  <- s$family; m
      }
    }, error = function(e) NULL)
    if (is.null(full_mod)) return(NULL)
    # Use HC0 for robust (no hat-value correction at full n)
    tv_vcov <- if (s$robust) function(x) sandwich::vcovHC(x, type = "HC0")
               else s$vcovfunc
    tryCatch(resi_pe(full_mod, vcovfunc = tv_vcov, unbiased = TRUE),
             error = function(e) NULL)
  })
  names(cq_true) <- vapply(cq_settings, `[[`, character(1L), "key")

  for (mtype_cq in c("lm", "glm")) {
    for (vtype_cq in c("parametric", "robust")) {
      key_cq  <- paste0(mtype_cq, "_", vtype_cq)
      pe_true <- cq_true[[key_cq]]
      if (is.null(pe_true)) next

      for (tbl_cq in c("anova", "coefficients")) {
        true_t <- pe_true[[tbl_cq]]
        if (is.null(true_t) || nrow(true_t) == 0L) next

        terms_cq <- rownames(true_t)
        terms_cq <- setdiff(terms_cq, c("(Intercept)", "Residuals"))
        if (length(terms_cq) == 0L) next

        n_vals_cq <- sort(unique(combined$n))

        # Load per-replicate (LCI, UCI) for every (method, n, term)
        cq_dat <- lapply(setNames(avail_methods, avail_methods), function(meth) {
          raw_dir <- file.path(output.dirs[[meth]], "sim_raw")
          if (!dir.exists(raw_dir)) return(NULL)
          lapply(setNames(n_vals_cq, as.character(n_vals_cq)), function(n_val) {
            f <- file.path(raw_dir, paste0(key_cq, "_n", n_val, ".rds"))
            if (!file.exists(f)) return(NULL)
            reps <- tryCatch(readRDS(f), error = function(e) NULL)
            if (is.null(reps) || length(reps) == 0L) return(NULL)
            lapply(setNames(terms_cq, terms_cq), function(term) {
              s_true <- true_t[term, "RESI"]
              lci_v <- vapply(reps, function(r) {
                tbl_r <- r[[tbl_cq]]
                if (is.null(tbl_r) || !(term %in% rownames(tbl_r))) return(NA_real_)
                if (!(ci_lo_col_cq %in% colnames(tbl_r)))           return(NA_real_)
                tbl_r[term, ci_lo_col_cq]
              }, numeric(1L))
              uci_v <- vapply(reps, function(r) {
                tbl_r <- r[[tbl_cq]]
                if (is.null(tbl_r) || !(term %in% rownames(tbl_r))) return(NA_real_)
                if (!(ci_hi_col_cq %in% colnames(tbl_r)))           return(NA_real_)
                tbl_r[term, ci_hi_col_cq]
              }, numeric(1L))
              cq_v <- (s_true - lci_v) / (uci_v - lci_v)
              cq_v[!is.finite(cq_v)] <- NA_real_
              cq_v
            })
          })
        })

        # Build PDF
        n_terms_cq  <- length(terms_cq)
        n_sizes_cq  <- length(n_vals_cq)
        n_panels_cq <- n_terms_cq * n_sizes_cq
        lay_cq      <- matrix(seq_len(n_panels_cq),
                              nrow = n_terms_cq, ncol = n_sizes_cq,
                              byrow = TRUE)
        lay_cq      <- cbind(lay_cq, n_panels_cq + 1L)

        col_w_cq <- 2.4
        row_h_cq <- 2.4

        fig_path_cq <- file.path(
          figures.dir,
          paste0("covquant_", mtype_cq, "_", vtype_cq, "_", tbl_cq, ".pdf")
        )
        grDevices::pdf(fig_path_cq,
                       width  = col_w_cq * n_sizes_cq + 2.2,
                       height = row_h_cq * n_terms_cq + 0.5)
        graphics::layout(lay_cq,
                         widths  = c(rep(col_w_cq, n_sizes_cq), 2.2),
                         heights = rep(row_h_cq, n_terms_cq))

        for (ti in seq_along(terms_cq)) {
          term_cq      <- terms_cq[[ti]]
          is_first_row <- (ti == 1L)
          is_last_row  <- (ti == n_terms_cq)

          for (ni in seq_along(n_vals_cq)) {
            n_cq         <- n_vals_cq[[ni]]
            is_first_col <- (ni == 1L)

            # Compute density for each method
            dens_list_cq <- lapply(setNames(avail_methods, avail_methods), function(meth) {
              vals <- cq_dat[[meth]][[as.character(n_cq)]][[term_cq]]
              vals <- vals[is.finite(vals)]
              if (length(vals) < 5L) return(NULL)
              tryCatch(stats::density(vals, n = 256L), error = function(e) NULL)
            })

            xlim_cq <- {
              all_x <- unlist(c(
                lapply(dens_list_cq, function(d) if (!is.null(d)) range(d$x)),
                list(c(0, 1))))
              c(min(all_x) - 0.05, max(all_x) + 0.05)
            }
            max_y_cq <- max(
              vapply(dens_list_cq,
                     function(d) if (!is.null(d)) max(d$y) else 0, numeric(1L)),
              1.0)
            ylim_cq <- c(0, max_y_cq * 1.1)

            t_mar <- if (is_first_row) 1.8 else 0.4
            b_mar <- if (is_last_row)  2.5 else 0.4
            l_mar <- if (is_first_col) 3.2 else 1.2

            graphics::par(mar = c(b_mar, l_mar, t_mar, 0.3), mgp = c(1.5, 0.4, 0))
            graphics::plot(NULL, xlim = xlim_cq, ylim = ylim_cq,
                           xlab = if (is_last_row)  "(S_true - LCI) / Width" else "",
                           ylab = if (is_first_col) "Density" else "",
                           main = if (is_first_row) paste0("n = ", n_cq) else "",
                           bty  = "l",
                           xaxt = if (is_last_row)  "s" else "n",
                           yaxt = if (is_first_col) "s" else "n")

            # Row label (term name) on left margin of first column
            if (is_first_col)
              graphics::mtext(term_cq, side = 2L, line = 2.0,
                              cex = 0.65, las = 0, font = 2L)

            # Reference lines: 0/1 = CI boundaries, 0.5 = midpoint, y=1 = uniform
            graphics::abline(v = c(0, 1), lty = 2L, col = "gray50", lwd = 0.8)
            graphics::abline(v = 0.5,     lty = 3L, col = "gray70", lwd = 0.8)
            graphics::abline(h = 1.0,     lty = 3L, col = "gray80", lwd = 0.8)

            for (meth in avail_methods) {
              d <- dens_list_cq[[meth]]
              if (is.null(d)) next
              graphics::lines(d, col = method_cols[[meth]], lwd = 1.8)
            }
          }
        }

        # Shared legend
        graphics::par(mar = c(0.3, 0.3, 0.3, 0.3))
        graphics::plot.new()
        graphics::legend("center",
                         legend = avail_methods,
                         col    = method_cols[avail_methods],
                         lwd = 2L, lty = 1L,
                         bty = "n", cex = 0.9,
                         title = "CI method", title.font = 2L)

        grDevices::dev.off()
        message("Saved: ", fig_path_cq)
      }
    }
  }

  message("Comparison figures saved to: ", figures.dir)
  invisible(combined)
}

#' Compute the robust effect size index estimate from t statistic (default)
#'
#' This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
#' Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
#' @param t The t statistic for the parameter of interest.
#' @param rdf Model residual degrees of freedom/degrees of freedom of the t statistic.
#' @param n Number of independent samples.
#' @param unbiased Logical, whether to use unbiased or alternative estimator. See details.
#' @details This function computes S, the RESI, from a t statistic. The formula for the
#' unbiased estimator (default) is derived by solving the expected value of the
#' t statistic for S. It is unbiased and consistent.
#'
#' The formula for the unbiased conversion is:
#'
#' \eqn{S = (t * \sqrt(2) * \Gamma(rdf/2)) / (\sqrt(n * rdf) * \Gamma((rdf - 1)/2))}
#'
#' The formula for the alternative estimator is derived by squaring the t statistic
#' and using the \code{\link{f2S}} formula. This estimator may be appealing for its
#' intuitive relationship to the F statistic; the absolute value of RESI estimates
#' using this formula will be equal to a RESI estimate using an F statistic for
#' the same model. However, this estimator does have finite sample bias, which is an
#' important consideration for the coverage of the bootstrapping that \code{resi} uses.
#'
#' The formula for the alternative conversion is:
#'
#' \eqn{ \sqrt(max(0, (t^2 * (rdf - 2)/rdf - 1)/rdf))}
#'
#' @return Returns a scalar or vector argument of the robust effect size index estimate.
#' @examples
#' # to obtain t values, first fit a lm
#' mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
#' # run lmtest::coeftest to get t values, using a robust variance-covariance formula
#' ts = lmtest::coeftest(mod, vcov. = sandwich::vcovHC)[,'t value']
#'
#' # get RESI estimates using unbiased estimator
#' t2S(ts, n = nrow(RESI::insurance), rdf = mod$df.residual)
#'
#' # get RESI estimates using alternative estimator
#' t2S(ts, n = nrow(RESI::insurance), rdf = mod$df.residual, unbiased = FALSE)
#' @export
t2S = function(t, rdf, n, unbiased = TRUE){
  if (unbiased){
    S = (t*sqrt(2))/(sqrt(n*rdf))*exp(lgamma(rdf/2) - lgamma((rdf-1)/2))
  } else{
    Ssq = (t^2*(rdf-2)/rdf - 1)/n
    S = sqrt(ifelse(Ssq<0, 0, Ssq))*(sqrt(t^2)/t)
  }
  return(S)
}

#' Covert Cohen's \emph{f}^2 to S
#'
#' Converts Cohen's \emph{f}^2 to robust effect size index (S)
#' using the formula from Vandekar, Tao, & Blume (2020).
#' @param fsq Numeric, value of Cohen's \emph{f}^2.
#' @return Returns an estimate the robust effect size index
#' @details The formula for the conversion is:
#'
#' \eqn{S =  \sqrt(f^2)}
#' @examples
#' # consider a moderate effect size of f^2 = 0.3
#' fsq2S(0.3)
#' # This corresponds to a RESI of 0.5477226
#' @export
fsq2S <- function(fsq){
  if (any(fsq < 0)){
    stop("\nfsq must be non-negative")
  }
  sqrt(fsq)
}

#' @export
print.resi <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  cat("\nAnalysis of effect sizes based on RESI:")
  cat("\nConfidence level = ", x$alpha)
  if (is.null(x$model.reduced$formula)){
    cat("\nCall:  ", paste(deparse(x$model.full$call), sep = "\n", collapse = "\n"),  "\n",sep = "")
  }
  else{
    cat(ifelse(x$model.reduced$formula == as.formula(paste(format(formula(x$model.full)[[2]]), "~ 1")), "\nCall:  ", "\nFull Model:"),
        paste(deparse(x$model.full$call), sep = "\n", collapse = "\n"), "\n",sep = "")
    if (!(x$model.reduced$formula == as.formula(paste(format(formula(x$model.full)[[2]]), "~ 1")))) cat("Reduced Model:", paste(deparse(x$model.reduced$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
  }

  # coefficients table
  if (!is.null(x$coefficients)){
    cat("\nCoefficient Table \n")
    print(round(x$coefficients, digits = digits))
  }

  # anova table
  if (!is.null(x$anova)){
    cat("\n\n")
    print(round(x$anova, digits = digits))
  }

  # overall
  if (!(is.null(x$overall))){
    if (is.null(x$model.reduced$formula)){
      cat("\nOverall RESI comparing model to intercept-only model:\n\n")
      print(round(x$overall, digits = digits))
    } else{
      if ((x$model.reduced$formula == as.formula(paste(format(formula(x$model.full)[[2]]), "~ 1")))){
        cat("\nOverall RESI comparing model to intercept-only model:\n\n")
      }
      else{
        cat("\nOverall RESI comparing full model to reduced model:\n\n")}
      overall = as.data.frame(x$overall)
      overall = overall[nrow(overall),]
      rownames(overall) = NULL
      print(round(overall, digits = digits))
      }
    }

  cat("\nNotes:")
  if (x$naive.var) cat("\n1. The RESI was calculated using the naive covariance estimator.")
  else  cat("\n1. The RESI was calculated using a robust covariance estimator.")
  if (!(is.null(x$nboot))){
    if (x$boot.method == "nonparam") cat("\n2. Confidence intervals (CIs) constructed using", x$nboot,"non-parametric bootstraps. \n")
    if (x$boot.method == "bayes") cat("\n2. Credible intervals constructed using", x$nboot,"Bayesian bootstraps. \n")}
  # if(nzchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep = "")
  # report number of failed bootstraps for nls model
  if (!(is.null(x$nfail))) cat("3. The bootstrap was successful in", x$nboot - x$nfail, "out of", x$nboot, "attempts. \n" )
  cat('\n')
  invisible(x)
}

#' @export
print.summary_resi <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  cat("\nAnalysis of effect sizes based on RESI:")
  cat("\nConfidence level = ", x$alpha)
  cat("\nCall:  ", paste(deparse(x$model.full$call), sep = "\n", collapse = "\n"),  "\n",sep = "")
  cat("\nCoefficient Table \n")
  print(round(x$coefficients, digits = digits))
  invisible(x)
}

#' @export
print.omnibus_resi <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  cat("\nAnalysis of effect sizes based on RESI:")
  cat("\nConfidence level = ", x$alpha, "\n")
  print(round(x$overall, digits = digits))
  invisible(x)
}

#' Robust Effect Size Index (RESI) point and interval estimation for models
#'
#' This function will estimate the robust effect size (RESI) from Vandekar, Tao, & Blume (2020) and its confidence interval in various ways for a fitted model object. The overall RESI is estimated via a Wald test. RESI is (optionally) estimated for each factor in coefficients-style table. RESI is (optionally) estimated for each variable/interaction in an Anova-style table for models with existing Anova methods. CIs can be calculated using either non-parametric or Bayesian bootstrapping.
#' @param model.full \code{lm, glm, nls, survreg, coxph, hurdle, zeroinfl, gee, geeglm} or \code{lme} model object.
#' @param model.reduced Fitted model object of same type as model.full. By default `NULL`; the same model as the full model but only having intercept.
#' @param data Data.frame or object coercible to data.frame of model.full data (required for some model types).
#' @param vcovfunc The variance estimator function for constructing the Wald test statistic. By default, \link[sandwich]{vcovHC} (the robust (sandwich) variance estimator).
#' @param coefficients Logical, whether to produce a coefficients (summary) table with the RESI columns added. By default = `TRUE`.
#' @param anova Logical, whether to produce an Anova table with the RESI columns added. By default = `TRUE`.
#' @param overall Logical, whether to produce an overall Wald test comparing full to reduced model with RESI columns added. By default = `TRUE`.
#' @param ci.method Character, the method used to compute confidence intervals.
#'   One of `"boot"` (bootstrap), `"qf"` (quadratic-form inversion, default for lm and glm),
#'   `"normal"` (normal approximation), or `"cf"` (Cornish-Fisher inversion).
#'   See `resi_pe_asymptotic` for details on the asymptotic methods.
#' @param nboot Numeric, the number of bootstrap replicates. Used only when
#'   `ci.method = "boot"`. By default, 1000.
#' @param boot.method String, which type of bootstrap to use: `nonparam` = non-parametric bootstrap (default); `bayes` = Bayesian bootstrap.
#' @param alpha Numeric, significance level of the constructed CIs. By default, 0.05.
#' @param store.boot Logical, whether to store all the bootstrapped estimates. By default, `FALSE`.
#' @param Anova.args List, additional arguments to be passed to \link[car]{Anova} function.
#' @param vcov.args List, additional arguments to be passed to vcovfunc.
#' @param unbiased Logical, whether to use the unbiased or alternative T/Z statistic to RESI conversion. By default, `TRUE`. See details.
#' @param parallel See documentation for \link[boot]{boot}.
#' @param ncpus See documentation for \link[boot]{boot}.
#' @param long Logical, whether the data is longitudinal/clustered. By default, `FALSE`.
#' @param clvar Character, the name of the cluster/id variable if data is clustered. By default, `NULL`.
#' @param ... Ignored.
#' @importFrom aod wald.test
#' @importFrom boot boot
#' @importFrom car Anova
#' @importFrom lmtest waldtest
#' @importFrom sandwich vcovHC
#' @importFrom stats anova as.formula coef complete.cases formula glm hatvalues lm nobs pchisq pf predict quantile rbinom residuals rnorm runif update vcov
#' @importFrom utils capture.output
#' @export
#' @details The RESI, denoted as S, is applicable across many model types. It is a unitless
#' index and can be easily be compared across models. The RESI can also be
#' converted to Cohen's \emph{d} (\code{\link{S2d}}) under  model homoskedasticity.
#'
#' This function computes the RESI point estimates and bootstrapped confidence
#' intervals based on Chi-square, F, T, or Z statistics. The robust (sandwich)
#' variance is used by default, allowing for consistency under
#' model-misspecification. The RESI is related to the non-centrality parameter
#' of the test statistic. The RESI estimate is consistent for all four
#' (Chi-square, F, T, and Z) types of statistics used. The Chi-square and F-based
#' calculations rely on asymptotic theory, so they may be biased in small samples.
#' When possible, the T and Z statistics are used. There are two formulas for both
#' the T and Z statistic conversion. The first (default, unbiased = TRUE)
#' are based on solving the expected value of the T or Z statistic for the RESI.
#' The alternative is based on squaring the T or Z statistic and using the
#' F or Chi-square statistic conversion. Both of these methods are consistent, but
#' the alternative exhibits a notable amount of finite sample bias. The alternative
#' may be appealing because its absolute value will be equal to the RESI based on
#' the F or Chi-square statistic. The RESI based on the Chi-Square and F statistics
#' is always greater than or equal to 0. The type of statistic
#' used is listed with the output. See \code{\link{f2S}}, \code{\link{chisq2S}},
#' \code{\link{t2S}}, and \code{\link{z2S}} for more details on the formulas.
#'
#' For GEE (\link[geepack]{geeglm}, \link[glmtoolbox]{glmgee}) models, a longitudinal RESI (L-RESI) and a cross-sectional,
#' per-measurement RESI (CS-RESI) is estimated. The longitudinal RESI takes the
#' specified clustering into account, while the cross-sectional RESI is designed to
#' estimate the effect size if a random observation for each participant were collected cross-sectionally.
#'
#' For most \code{lm} and \code{nls} model types, there is a Bayesian bootstrap
#' option available as an alternative to the default, standard non-parametric
#' bootstrap. The interpretation of a Bayesian bootstrapped interval is similar to
#' that of a credible interval.
#'
#' Certain model types require the data used for the model be entered as an argument.
#' These are: \code{nls, survreg,} and \code{coxph}. Additionally, if a model
#' includes certain functions (splines, factor, I), the data needs to be provided.
#'
#' If running into convergence issues with nls models, it is advised to refit the
#' nls model with starting values equal to the estimates provided by the model
#' and then try rerunning \code{resi}.
#'
#' @return Returns a list that includes function arguments, RESI point estimates,
#' and confidence intervals in coefficients/anova-style tables
#' @family RESI functions
#' @seealso \code{\link{resi_pe}}, \link[sandwich]{vcovHC},
#' \code{\link{f2S}}, \code{\link{chisq2S}}, \code{\link{z2S}}, \code{\link{t2S}}
#'
#' @examples
#' ## for timing purposes, a small number of bootstrap replicates is used in the
#' ## examples. Run them with a higher or default `nboot` argument for better performance
#'
#' ## RESI on a linear model
#' # fit linear model
#' mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
#'
#' # run resi on fitted model with desired number of bootstrap replicates
#' # store bootstrap results for calculating different CIs later
#' resi_obj = resi(mod, nboot = 50, store.boot = TRUE)
#' # print output
#' resi_obj
#'
#' # fit a reduced model for comparison
#' mod_red = lm(charges ~ bmi, data = RESI::insurance)
#'
#' # running resi and including the reduced model will provide almost the exact same
#' # output as not including a reduced model. The difference is that the "overall"
#' # portion of the output will compare the full model to the reduced model.
#' # The "summary" and "anova" RESI estimates will be the same. (The bootstrapped
#' # confidence intervals may differ.)
#' resi(model.full = mod, model.reduced = mod_red, nboot = 10)
#'
#' # used stored bootstrap results to get a different alpha-level confidence interval
#' summary(resi_obj, alpha = c(0.01, 0.1))
#' car::Anova(resi_obj, alpha = c(0.01, 0.1))
#'
#' # the result of resi, as well as the summary or Anova of a `resi` object can be plotted
#' # if the resi object was created with the store.boot = `TRUE` option, any alpha
#' # can be specified
#' plot(resi_obj, alpha = 0.01)
#' # if the variable names on the y-axis are too long, you can reduce their size with
#' # the ycex.axis argument (or use regular common solutions like changing the margins)
#' plot(resi_obj, alpha = 0.01, ycex.axis = 0.5)
#'
#' # for some model types and formula structures, data argument is required
#' if(requireNamespace("splines")){
#'   # fit logistic regression model with splines
#'   mod = glm(smoker ~ splines::ns(age, df = 3) + region, data = RESI::insurance,
#'     family = "binomial")
#'
#'   # specify additional arguments to the variance-covariance function via vcov.args
#'   resi_obj = resi(mod, data = RESI::insurance, alpha = 0.01,
#'     vcov.args = list(type = "HC0"), nboot = 25)
#'   summary(resi_obj)
#'   car::Anova(resi_obj)}
#'
#'
#' ## RESI on a survival model with alternate Z2S
#' if(requireNamespace("survival")){
#'   # fit coxph model on example data from survival package
#'   # Note: for survival models, you need to specify robust variance in the model
#'   # creation. resi will ignore the vcovfunc argument for this reason.
#'   mod.coxph =  survival::coxph(survival::Surv(time, status) ~ age + sex + wt.loss,
#'    data=survival::lung, robust = TRUE)
#'
#'   # run resi on the model
#'   # to use the alternative Z to RESI formula (which is equal in absolute value to the
#'   # chi-square to RESI (S) formula), specify unbiased = FALSE.
#'   resi(mod.coxph, data = survival::lung, unbiased = FALSE, nboot = 10)}
#'
#' @references Vandekar S, Tao R, Blume J. A Robust Effect Size Index. \emph{Psychometrika}. 2020 Mar;85(1):232-246. doi: 10.1007/s11336-020-09698-2.
#'
#' Kang, K., Armstrong, K., Avery, S., McHugo, M., Heckers, S., & Vandekar, S. (2021). Accurate confidence interval estimation for non-centrality parameters and effect size indices. \emph{arXiv preprint arXiv:2111.05966}.
#'
#' Jones, M., Kang, K., Vandekar, S. (2025). \emph{Journal of Statistical Software}. RESI: An R Package for Robust Effect Sizes.<doi:10.18637/jss.v112.i03>

resi = function(model.full, ...){
  UseMethod("resi")
}

#' @describeIn resi RESI point and interval estimation for models
#' @export
resi.default = function(model.full, model.reduced = NULL, data, anova = TRUE,
                         coefficients = TRUE, overall = TRUE, nboot = 1000,
                         boot.method = "nonparam", vcovfunc = sandwich::vcovHC,
                         alpha = 0.05, store.boot = FALSE, Anova.args = list(),
                         vcov.args = list(), unbiased = TRUE,
                         parallel = c("no", "multicore", "snow"),
                         ncpus = getOption("boot.ncpus", 1L), long = FALSE,
                         clvar = NULL,
                         ci.method = c("boot", "qf", "cf", "normal"),
                         ...){

  ci.method <- match.arg(ci.method)

  # check for supported model type
  if(!(any(class(model.full) %in%
    sapply(as.character(utils::methods(RESI::resi)), function(x) substr(x, 6, nchar(x)))))){
    warning("Model type not implemented in RESI package, attempting default")
  }
  dots = list(...)

  data_from_model_frame <- missing(data)
  if (data_from_model_frame){
    data = model.full$model
    if (ci.method == "boot") {
      tryCatch(update(model.full, data = data), error = function(e){
        message("Updating model fit failed. Try running with providing data argument")})
    }
  }
  else{
    if (!(is.null(model.full$na.action))){
      data = data[which(!(1:nrow(data)%in% model.full$na.action)),]
    }
    data = as.data.frame(data)
  }

  # For clustered/GEE models, strip rows with NA in any model variable from data
  # before constructing model.reduced or bootstrapping. geeglm may not populate
  # na.action when the model was fitted on already-complete data but the user
  # passes a data frame with NAs. Use only the columns that appear in model.frame()
  # to avoid dropping rows with NAs in unrelated columns.
  if (long) {
    model_vars <- intersect(names(model.frame(model.full)), names(data))
    data <- data[complete.cases(data[, model_vars, drop = FALSE]), , drop = FALSE]
  }

  if (is.null(model.reduced) & overall){
    if(!("skip.red" %in% names(dots))){
      form.reduced = as.formula(paste(format(formula(model.full)[[2]]), "~ 1"))
      if (!(form.reduced == formula(model.full))){
        # How we fit the intercept-only reduced model depends on the data source:
        #
        # (a) data was supplied by the user (raw columns, e.g. 'charges'): use the
        #     expression formula (e.g. log10(charges) ~ 1) so that bootstrap
        #     replicates (which also have raw columns) can update() successfully.
        #
        # (b) data came from model.full$model (the model frame, e.g. column named
        #     'log10(charges)'): use the backtick-quoted pre-computed column name
        #     so that update() finds the response without re-evaluating the
        #     expression on columns that don't exist in the model frame.
        #     This path is only reached for asymptotic CIs (ci.method != "boot"),
        #     since bootstrap with a model-frame data warns and is expected to fail.
        if (data_from_model_frame && !is.null(model.full$model) && !long) {
          resp_name <- colnames(model.full$model)[1]
          form.reduced_fit <- as.formula(paste0("`", resp_name, "` ~ 1"))
          model.reduced = try(update(model.full, formula = form.reduced_fit,
                                      data = data), silent = T)
        } else {
          model.reduced = try(update(model.full, formula = form.reduced,
                                      data = data), silent = T)
        }
        if(inherits(model.reduced, "try-error")){
          warning("Fitting intercept-only model failed. No overall test computed")
          overall = FALSE
        }
    }}}

  # point estimation
  output = list(alpha = alpha, nboot = nboot, boot.method = boot.method)
  output = c(output, resi_pe(model.full = model.full, model.reduced = model.reduced,
                             data = data, anova = anova, coefficients = coefficients,
                             overall = overall, vcovfunc = vcovfunc,
                             Anova.args = Anova.args, vcov.args = vcov.args,
                             unbiased = unbiased, ...))

  if (long){
    data = unique(data[, clvar])
  }

  # ---- asymptotic CIs (skip bootstrap) ----
  if (ci.method != "boot") {
    asym_out <- tryCatch(
      resi_pe_asymptotic(model.full   = model.full,
                         data         = data,
                         vcovfunc     = vcovfunc,
                         coefficients = coefficients,
                         anova        = anova,
                         alpha        = alpha,
                         ci.method    = ci.method,
                         unbiased     = unbiased,
                         Anova.args   = Anova.args,
                         vcov.args    = vcov.args),
      error = function(e) {
        warning("Asymptotic CI computation failed: ", conditionMessage(e),
                "\nFalling back to bootstrap.")
        NULL
      }
    )
    if (!is.null(asym_out)) {
      alpha.order <- sort(c(alpha / 2, 1 - alpha / 2))
      ci_cols     <- paste0(alpha.order * 100, "%")
      if (coefficients && !is.null(output$coefficients) && !is.null(asym_out$coefficients)) {
        # Match row names and add CI columns
        rn <- rownames(output$coefficients)
        rn_asym <- rownames(asym_out$coefficients)
        for (col in ci_cols) {
          if (col %in% colnames(asym_out$coefficients)) {
            output$coefficients[rn %in% rn_asym, col] <-
              asym_out$coefficients[rn_asym %in% rn, col]
          }
        }
      }
      if (anova && !is.null(output$anova) && !is.null(asym_out$anova)) {
        rn <- rownames(output$anova)
        rn_asym <- rownames(asym_out$anova)
        for (col in ci_cols) {
          if (col %in% colnames(asym_out$anova)) {
            output$anova[rn %in% rn_asym, col] <-
              asym_out$anova[rn_asym %in% rn, col]
          }
        }
      }
      output$ci.method  <- ci.method
      # Store model and vcov settings so S3 methods can recompute CIs at a
      # different alpha without re-running resi().
      output$model.full <- model.full
      output$vcovfunc   <- vcovfunc
      output$vcov.args  <- vcov.args
      output$Anova.args <- Anova.args
      output$unbiased   <- unbiased
      class(output) <- "resi"
      return(output)
    }
    # if we fell through (error), continue to bootstrap
    ci.method <- "boot"
  }

  # bootstrapping
  suppressMessages(capture.output(boot_out <- boot::boot(data = data, R = nboot, statistic = resi_stat, parallel = parallel,
                  ncpus = ncpus, mod.full = model.full, mod.reduced = model.reduced,
                  anova = anova, coefficients = coefficients, overall = overall, vcovfunc = vcovfunc,
                  Anova.args = Anova.args, vcov.args = vcov.args, unbiased = unbiased,
                  boot.method = boot.method, clvar = clvar, nest = length(output$estimates), ...), file = nullfile()))

  # bootstrapped estimates
  boot.results = boot_out$t
  colnames(boot.results) = names(boot_out$t0)

  if (all(is.na(boot.results))){
    stop("\nBootstrapping failed. Run resi_pe for point estimates")
  }

  alpha.order = sort(c(alpha/2, 1-alpha/2))

  if (!long){
    if (overall & !is.null(output$overall)){
      if(nrow(output$overall) == 1){
        names.overall = colnames(output$overall)
        for (i in 1:length(alpha.order)){
          output$overall = cbind(output$overall, quantile(boot.results[,1],
                                                          probs = alpha.order[i], na.rm = TRUE))
        }
        colnames(output$overall) = c(names.overall, paste(alpha.order*100, "%", sep=""))
      } else{
        output$overall[nrow(output$overall),paste(alpha.order*100, "%", sep="")] =
          quantile(boot.results[,1], probs = alpha.order, na.rm = TRUE)}
    }

    if (coefficients){
      co = boot.results[,(1+!(is.null(output$overall))):((!is.null(output$overall))+nrow(output$coefficients))]
      if (is.null(dim(co))){
        CIs = quantile(co, probs = alpha.order, na.rm = TRUE)
      } else{
        CIs = apply(co, 2,  quantile, probs = alpha.order, na.rm = TRUE)
      }
      CIs = t(CIs)
      output$coefficients[1:nrow(CIs), paste(alpha.order*100, "%", sep="")] = CIs
    }

    if (anova){
      an = boot.results[,(ncol(boot.results)-length(which(rownames(output$anova) != "Residuals"))+1):ncol(boot.results)]
      if (is.null(dim(an))){
        CIs = quantile(an, probs = alpha.order, na.rm = TRUE)
      } else{
        CIs = apply(an, 2,  quantile, probs = alpha.order, na.rm = TRUE)
      }
      CIs = t(CIs)
      output$anova[1:nrow(CIs), paste(alpha.order*100, "%", sep="")] = CIs
    }
  } else {
    r = 1
    if (!is.null(output$overall)){
      # L-RESI for geeglm model overall Wald test
      output$overall[nrow(output$overall),paste("L ",alpha.order*100, "%", sep="")] =
        quantile(boot.results[,1], probs = alpha.order, na.rm = TRUE)
      r = 2
    }
    # L-RESI and CS-RESI for longitudinal models
    if (coefficients){
      lco = boot.results[,r:(nrow(output$coefficients) + r -1)]
      if (is.null(dim(lco))){
        lCIs = quantile(lco, probs = alpha.order, na.rm = TRUE)
      } else{
        lCIs = apply(lco, 2,  quantile,
                     probs = alpha.order, na.rm = TRUE)
      }
      lCIs = t(lCIs)
      output$coefficients[1:nrow(lCIs), paste("L ",alpha.order*100, "%", sep="")] = lCIs
      cco = boot.results[,(nrow(output$coefficients)+r):(2*nrow(output$coefficients) + r - 1)]
      if (is.null(dim(cco))){
        cCIs = quantile(cco, probs = alpha.order, na.rm = TRUE)
      } else{
        cCIs = apply(cco, 2,  quantile, probs = alpha.order, na.rm = TRUE)
      }
      cCIs = t(cCIs)
      output$coefficients[1:nrow(cCIs), paste("CS ",alpha.order*100, "%", sep="")] = cCIs
    }

    if (anova){
      lan = boot.results[,(ncol(boot.results)-
                             2*length(rownames(output$anova))+1):
                           (ncol(boot.results)-length(rownames(output$anova)))]
      if (is.null(dim(lan))){
        lCIs = quantile(lan, probs = alpha.order, na.rm = TRUE)
      } else{
        lCIs = apply(lan, 2,  quantile, probs = alpha.order, na.rm = TRUE)
      }
      lCIs = t(lCIs)
      output$anova[1:nrow(lCIs), paste("L ", alpha.order*100, "%", sep="")] = lCIs
      can = boot.results[,(ncol(boot.results)-length(rownames(output$anova))+1):
                           ncol(boot.results)]
      if (is.null(dim(can))){
        cCIs = quantile(can, probs = alpha.order, na.rm = TRUE)
      } else{
        cCIs = apply(can, 2,  quantile, probs = alpha.order, na.rm = TRUE)
      }
      cCIs = t(cCIs)
      output$anova[1:nrow(cCIs), paste("CS ", alpha.order*100, "%", sep="")] = cCIs
    }
  }

  if(store.boot){
    #output$boot.results = boot.results
    output$boot.results = boot_out
  }
  class(output) = "resi"
  return(output)
}

#' @describeIn resi RESI point and interval estimation for models
#' @export
resi.glm = function(model.full, model.reduced = NULL, data, anova = TRUE,
                    coefficients = TRUE, overall = TRUE, nboot = 1000,
                    vcovfunc = sandwich::vcovHC, alpha = 0.05, store.boot = FALSE,
                    Anova.args = list(), vcov.args = list(), unbiased = TRUE,
                    parallel = c("no", "multicore", "snow"), ncpus = getOption("boot.ncpus", 1L),
                    ci.method = "qf",
                    ...){  dots = list(...)
  if ("boot.method" %in% names(dots)){    stop("\nOnly nonparametric bootstrap supported for model type")
  }
  resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
               anova = anova, coefficients = coefficients, overall = overall,
               nboot = nboot, vcovfunc = vcovfunc, store.boot = store.boot,
               Anova.args = Anova.args, vcov.args = vcov.args,
               unbiased = unbiased, alpha = alpha, parallel = parallel,
               ncpus = ncpus, ci.method = ci.method, ...)

}

#' @describeIn resi RESI point and interval estimation for lm models
#' @export
resi.lm = function(model.full, model.reduced = NULL, data, anova = TRUE,
                    coefficients = TRUE, overall = TRUE, nboot = 1000,
                    boot.method = "nonparam", vcovfunc = sandwich::vcovHC,
                    alpha = 0.05, store.boot = FALSE, Anova.args = list(),
                    vcov.args = list(), unbiased = TRUE,
                    parallel = c("no", "multicore", "snow"),
                    ncpus = getOption("boot.ncpus", 1L),
                    ci.method = "qf",
                    ...){  
  boot.method = match.arg(tolower(boot.method), choices = c("nonparam", "bayes"))

  resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
               anova = anova, coefficients = coefficients, overall = overall,
               nboot = nboot, vcovfunc = vcovfunc, store.boot = store.boot,
               Anova.args = Anova.args, vcov.args = vcov.args, unbiased = unbiased,
               alpha = alpha, parallel = parallel, ncpus = ncpus,
               boot.method = boot.method, ci.method = ci.method, ...)

}

#' @describeIn resi RESI point and interval estimation for lmrob models (robustbase)
#' @export
resi.lmrob = function(model.full, model.reduced = NULL, data, anova = TRUE,
                      coefficients = TRUE, overall = TRUE, nboot = 1000,
                      boot.method = "nonparam", vcovfunc = stats::vcov,
                      alpha = 0.05, store.boot = FALSE, Anova.args = list(),
                      vcov.args = list(), unbiased = TRUE,
                      parallel = c("no", "multicore", "snow"),
                      ncpus = getOption("boot.ncpus", 1L),
                      ci.method = "boot", ...) {
  # lmrob has a built-in robust sandwich variance; stats::vcov is the natural
  # default. sandwich::vcovHC also works but is redundant and can be unstable
  # on bootstrap samples with near-degenerate design matrices.
  boot.method = match.arg(tolower(boot.method), choices = c("nonparam", "bayes"))
  resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
               anova = anova, coefficients = coefficients, overall = overall,
               nboot = nboot, vcovfunc = vcovfunc, store.boot = store.boot,
               Anova.args = Anova.args, vcov.args = vcov.args, unbiased = unbiased,
               alpha = alpha, parallel = parallel, ncpus = ncpus,
               boot.method = boot.method, ci.method = ci.method, ...)
}

#' @describeIn resi RESI point and interval estimation for glmrob models (robustbase)
#' @export
resi.glmrob = function(model.full, model.reduced = NULL, data, anova = TRUE,
                       coefficients = TRUE, overall = TRUE, nboot = 1000,
                       vcovfunc = stats::vcov, alpha = 0.05, store.boot = FALSE,
                       Anova.args = list(), vcov.args = list(), unbiased = TRUE,
                       parallel = c("no", "multicore", "snow"),
                       ncpus = getOption("boot.ncpus", 1L),
                       ci.method = "boot", ...) {
  dots = list(...)
  if ("boot.method" %in% names(dots)){
    stop("\nOnly nonparametric bootstrap supported for model type")
  }
  if (identical(vcovfunc, sandwich::vcovHC)) {
    vcovfunc <- stats::vcov
    warning("sandwich::vcovHC is not supported for glmrob objects. ",
            "Using the model's built-in robust variance (stats::vcov) instead.")
  }
  resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
               anova = anova, coefficients = coefficients, overall = overall,
               nboot = nboot, vcovfunc = vcovfunc, store.boot = store.boot,
               Anova.args = Anova.args, vcov.args = vcov.args,
               unbiased = unbiased, alpha = alpha, parallel = parallel,
               ncpus = ncpus, ci.method = ci.method, ...)
}

#' @describeIn resi RESI point and interval estimation for nls models
#' @export
resi.nls = function(model.full, model.reduced = NULL, data, coefficients = TRUE,
                     overall = TRUE, nboot = 1000, boot.method = "nonparam",
                     anova = FALSE, vcovfunc = r_nlshc, alpha = 0.05,
                     store.boot = FALSE, vcov.args = list(), unbiased = TRUE,
                     parallel = c("no", "multicore", "snow"), ncpus = getOption("boot.ncpus", 1L),
                     ...){

  boot.method = match.arg(tolower(boot.method), choices = c("nonparam", "bayes"))

  if (missing(data)){
    stop("\nData argument is required for nls model")
  }
  if (identical(vcovfunc, sandwich::vcovHC)){
    vcovfunc = r_nlshc
    warning("Sandwich vcov function not applicable for nls model type, vcovfunc set to regtools::nlshc")
  }

  output = resi.default(model.full = model.full, model.reduced = model.reduced,
                        data = data, anova = FALSE, coefficients = coefficients,
                        overall = overall, nboot = nboot, vcovfunc = vcovfunc,
                        store.boot = TRUE, unbiased = unbiased, alpha = alpha,
                        vcov.args = vcov.args, parallel = parallel,
                        ncpus = ncpus, boot.method = boot.method, skip.red = TRUE, ...)


  # number of bootstrap replicates with failed updating
  output$nfail = length(which(is.na(output$boot.results$t[,1])))

  if (!store.boot){
    output = output[names(output) != "boot.results"]
  }
  class(output) = "resi"
  return(output)
}

#' @describeIn resi RESI point and interval estimation for survreg models
#' @export
resi.survreg = function(model.full, model.reduced = NULL, data, anova = TRUE,
                         coefficients = TRUE, overall = TRUE, nboot = 1000,
                         vcovfunc = vcov, alpha = 0.05, store.boot = FALSE,
                         Anova.args = list(), unbiased = TRUE,
                         parallel = c("no", "multicore", "snow"),
                         ncpus = getOption("boot.ncpus", 1L),...){
  dots = list(...)
  if (missing(data)){
    stop("\nData argument is required for model type")
  }
  if (!identical(vcovfunc, vcov)){
    warning("vcovfunc argument ignored for survreg objects")
  }
  if ("vcov.args" %in% names(dots)){
    warning("vcov.args ignored for model type")
  }
  if ("boot.method" %in% names(dots)){
    stop("\nOnly nonparametric bootstrap supported for model type")
  }

  resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
               anova = anova, coefficients = coefficients, overall = overall,
               nboot = nboot, vcovfunc = vcov, store.boot = store.boot,
               Anova.args = Anova.args, unbiased = unbiased, alpha = alpha,
               boot.method = "nonparam", parallel = parallel, ncpus = ncpus, ...)
}

#' @describeIn resi RESI point and interval estimation for coxph models
#' @export
resi.coxph = function(model.full, model.reduced = NULL, data, anova = TRUE,
                       coefficients = TRUE, overall = TRUE, nboot = 1000,
                       vcovfunc = vcov, alpha = 0.05, store.boot = FALSE,
                       Anova.args = list(), unbiased = TRUE,
                       parallel = c("no", "multicore", "snow"),
                       ncpus = getOption("boot.ncpus", 1L),...){
  dots = list(...)
  if (missing(data)){
    stop("\nData argument is required for model type")
  }
  if (!identical(vcovfunc, vcov)){
    warning("vcovfunc argument ignored for coxph objects")
  }
  if ("vcov.args" %in% names(dots)){
    warning("vcov.args ignored for model type")
  }
  if ("boot.method" %in% names(dots)){
    stop("\nOnly nonparametric bootstrap supported for model type")
  }
  if (!is.null(model.reduced)){
    warning("Reduced model argument ignored for coxph model")
  }
  resi.default(model.full = model.full, model.reduced = NULL, data = data,
               anova = anova, coefficients = coefficients, overall = overall,
               nboot = nboot, vcovfunc = vcov, store.boot = store.boot,
               Anova.args = Anova.args, unbiased = unbiased, alpha = alpha,
               boot.method = "nonparam", parallel = parallel, ncpus = ncpus,
               skip.red = TRUE, ...)
}


#' @describeIn resi RESI point and interval estimation for hurdle models
#' @export
resi.hurdle = function(model.full, model.reduced = NULL, data, coefficients = TRUE,
                        overall = TRUE, nboot = 1000, vcovfunc = sandwich::sandwich,
                        anova = FALSE, alpha = 0.05, store.boot = FALSE,
                        vcov.args = list(), unbiased = TRUE,
                        parallel = c("no", "multicore", "snow"),
                        ncpus = getOption("boot.ncpus", 1L), ...){

  dots = list(...)
  if ("boot.method" %in% names(dots)){
    stop("\nOnly nonparametric bootstrap supported for model type")
  }

  resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
               coefficients = coefficients, overall = overall, nboot = nboot,
               anova = FALSE, vcovfunc = vcovfunc, store.boot = store.boot,
               vcov.args = vcov.args, unbiased = unbiased, alpha = alpha,
               boot.method = "nonparam", parallel = parallel, ncpus = ncpus, ...)

}

#' @describeIn resi RESI point and interval estimation for zeroinfl models
#' @export
resi.zeroinfl = resi.hurdle

#' @describeIn resi RESI point and interval estimation for GEE models
#' @export
resi.geeglm = function(model.full, model.reduced = NULL, data, anova = TRUE,
                        coefficients = TRUE, overall = TRUE, nboot = 1000,
                        alpha = 0.05, store.boot = FALSE,
                        unbiased = TRUE, parallel = c("no", "multicore", "snow"),
                        ncpus = getOption("boot.ncpus", 1L), ...){
  dots = list(...)
  if ("boot.method" %in% names(dots)){
    stop("\nOnly nonparametric bootstrap supported for model type")
  }

  if (missing(data)){
    data = model.full$data
    tryCatch(update(model.full, data = data), error = function(e){
      message("Updating model fit failed. Try rerunning with providing data argument")})
  }
  else{
    data = as.data.frame(data)
  }

  # Strip rows with NA in any model variable so that mod.dat passed to the
  # bootstrap contains only the complete cases used in model fitting.
  # This prevents cluster-size mismatches when geeglm is re-fitted in each
  # bootstrap replicate.
  model_vars_gee <- intersect(names(model.frame(model.full)), names(data))
  data <- data[complete.cases(data[, model_vars_gee, drop = FALSE]), , drop = FALSE]

  # id variable name
  id_var = as.character(model.full$call$id)

  # bootstrapping
  output = resi.default(model.full = model.full, model.reduced = model.reduced,
                        data = data, anova = anova, coefficients = coefficients,
                        overall = overall, nboot = nboot,
                        store.boot = TRUE,
                        unbiased = unbiased, alpha = alpha,
                        parallel = parallel, ncpus = ncpus, boot.method = "nonparam",
                        cluster = TRUE, clvar = id_var, mod.dat = data, long = TRUE, ...)

  # number of bootstrap replicates with failed updating
  output$nfail = length(which(is.na(output$boot.results$t[,1])))


  if (!store.boot){
    output = output[names(output) != "boot.results"]
  }

  class(output) = "resi"

  return(output)
}

#' @describeIn resi RESI point and interval estimation for GEE models in glmtoolbox
#' @export
resi.glmgee = function(model.full, model.reduced = NULL, data, anova = FALSE,
                                     coefficients = TRUE, overall = TRUE, nboot = 1000,
                                     alpha = 0.05, store.boot = FALSE,
                                     unbiased = TRUE, parallel = c("no", "multicore", "snow"),
                                     ncpus = getOption("boot.ncpus", 1L), ...){
  # Capture the call to my_function
  call <- match.call()

  # Get the argument names and their default values
  formal_args <- formals(resi.glmgee)

  # Create a list of all arguments including defaults
  args_list <- as.list(formal_args)
  # remove ellipses
  args_list = args_list[-length(args_list)]

  # Update the list with the user's inputs from the call (if provided)
  for (name in names(call)[-1]) {  # Exclude the function name
    args_list[[name]] <- call[[name]]
  }
  do.call(resi.geeglm, args_list)
}

 #' @describeIn resi RESI point and interval estimation for GEE models
#' @export
resi.gee = function(model.full, data, nboot = 1000, alpha = 0.05,
                     store.boot = FALSE, unbiased = TRUE,
                     parallel = c("no", "multicore", "snow"),
                     ncpus = getOption("boot.ncpus", 1L), ...){
  dots = list(...)
  if ("boot.method" %in% names(dots)){
    stop("\nOnly nonparametric bootstrap supported for model type")
  }
  if (missing(data)){
    stop("\nData argument is required for GEE models from gee package")
  }
  else{
    data = as.data.frame(data)
  }

  id_var = as.character(model.full$call$id)

  # bootstrapping
  output = resi.default(model.full = model.full, model.reduced = NULL, data = data,
                        coefficients = TRUE, nboot = nboot,
                        store.boot = TRUE, anova = FALSE,
                        unbiased = unbiased, alpha = alpha, parallel = parallel,
                        ncpus = ncpus, boot.method = "nonparam", cluster = TRUE,
                        clvar = id_var, mod.dat = data, skip.red = TRUE, long = TRUE,
                        overall = FALSE, ...)

  # number of bootstrap replicates with failed updating
  output$nfail = length(which(is.na(output$boot.results$t[,1])))

  if (!store.boot){
    output = output[names(output) != "boot.results"]
  }

  class(output) = "resi"

  return(output)
}

#' @describeIn resi RESI point and interval estimation for LME (nlme) models
#' @importFrom nlme getGroups
#' @export
resi.lme = function(model.full, alpha = 0.05, nboot = 1000, anova = TRUE, vcovfunc = clubSandwich::vcovCR,
                     vcov.args = list(), ...){
  warning("\nConfidence Interval procedure not developed for lme, returning point estimates only")
  output = list(alpha = alpha, nboot = 0)
  # RESI point estimates
  output = c(output, resi_pe(model.full = model.full, vcovfunc = vcovfunc, vcov.args = vcov.args,
                             anova = anova, ...))
  output$boot.method = "nonparam"
  class(output) = "resi"
  return(output)
}

#' @describeIn resi RESI point and interval estimation for lmerMod models
#' @export
resi.lmerMod = function(model.full, alpha = 0.05, nboot = 1000, anova = TRUE,
                         vcovfunc = clubSandwich::vcovCR, vcov.args = list(), ...){
  warning("\nConfidence Interval procedure not developed for lmerMod, returning point estimates only")
  output = list(alpha = alpha, nboot = 0)
  output = c(output, resi_pe(model.full, vcovfunc = vcovfunc, vcov.args = vcov.args, anova = anova, ...)) # RESI point estimates
  output$boot.method = "nonparam"
  class(output) = "resi"
  return(output)
}


#' @describeIn resi RESI point and interval estimation for glmmTMB models - Gaussian only
#' @export
resi.glmmTMB = function(model.full, alpha = 0.05, nboot = 1000, anova = TRUE,
                        vcovfunc = clubSandwich::vcovCR, vcov.args = list(), ...){
  warning("\nConfidence Interval procedure not developed for glmmTMB, returning point estimates only")
  output = list(alpha = alpha, nboot = 0)
  output = c(output, resi_pe(model.full, vcovfunc = vcovfunc, vcov.args = vcov.args, anova = anova, ...)) # RESI point estimates
  output$boot.method = "nonparam"
  class(output) = "resi"
  return(output)
}

library(car)
#library(splines)
library(sandwich)
#library(survival)
library(nlme)
#library(lme4)
library(tibble)
#library(geepack)

data = RESI::insurance
mod = glm(charges ~ region * age + bmi + sex, data = data)
mod.r = glm(charges ~ bmi +sex, data = data)
mod.lm = lm(charges ~ region * age + bmi + sex, data = data)
mod.log = glm(smoker ~ age + region, data = data, family = "binomial")
mod.glm.int = glm(smoker ~ 1, data = data, family = "binomial")
mod.lm.int = lm(charges ~ 1, data = data)
# splines
if(requireNamespace("splines")){
  mod.s = glm(charges ~ region * splines::ns(age, df=3) + bmi + sex, data = data)
  mod.s.r = glm(charges ~ region * splines::ns(age, df=3), data = data)
  mod.lm.s = lm(charges ~ region*splines::ns(age, df = 3) + bmi + sex, data = data)
  mod.lm.s.r = lm(charges ~ region * splines::ns(age, df=3), data = data)
}

# missing data models
data.nas = data
data.nas[12:17, 'bmi'] = NA
mod.na = lm(charges ~ region * age + bmi + sex, data = data.nas)
mod.log.na = glm(smoker ~ age + region + bmi, data = data.nas, family = "binomial")

#nls
x <- seq(0, 100, 1)
y<-((runif(1, 10, 20)*x)/(runif(1, 0, 10) + x)) +
  rnorm(101, 0, 1)
data.nls = data.frame(X = x, Y = y)
mod.nls = nls(Y~a * X/(b + X), data = data.nls, start = list(a = 1, b = 2))

#survival
if(requireNamespace("survival")){
  data.surv = survival::lung
  mod.surv = survival::survreg(survival::Surv(time, status) ~ age + sex + ph.karno, data=data.surv,
                     dist='weibull', robust = TRUE)
  mod.surv.red = survival::survreg(survival::Surv(time, status) ~ age, data=data.surv[which(!(1:nrow(data.surv)%in% mod.surv$na.action)),],
                         dist='weibull', robust = TRUE)
  mod.surv.nr = survival::survreg(survival::Surv(time, status) ~ age + sex + ph.karno, data=data.surv,
                        dist='weibull', robust = FALSE)
  mod.surv.int = survival::survreg(survival::Surv(time, status) ~ 1, data=data.surv,
                                   dist='weibull', robust = TRUE)
  mod.coxph =  survival::coxph(survival::Surv(time, status) ~ age + sex + wt.loss, data=survival::lung, robust = TRUE)
  mod.coxph.red =  survival::coxph(survival::Surv(time, status) ~ age, data=survival::lung, robust = TRUE)
}


# hurdle
if(requireNamespace("pscl")){
  data.hurdle = pscl::bioChemists
  mod.hurdle = pscl::hurdle(art ~ fem + mar + kid5 + phd + ment | fem + mar + kid5 + phd + ment, data = data.hurdle)
  mod.hurdle.r = pscl::hurdle(art ~ kid5 + phd | kid5 + phd, data = data.hurdle)
  mod.hurdle.int = pscl::hurdle(art ~ 1, data = data.hurdle)

  # zeroinfl
  mod.zinf = pscl::zeroinfl(art ~ fem + mar + kid5 + phd + ment | fem + mar + kid5 + phd + ment, data = data.hurdle)
  mod.zinf.r = pscl::zeroinfl(art ~ kid5 + phd | kid5 + phd, data = data.hurdle)
  mod.zinf.int = pscl::zeroinfl(art ~ 1, data = data.hurdle)
}

data.gee = RESI::depression
# gee and lme
if(requireNamespace("gee")){
  mod.gee = gee::gee(depression ~ diagnose + drug*time, data = data.gee,
                     id = id, family = binomial, corstr = "independence")
  mod.gee.int = gee::gee(depression ~ 1, data = data.gee,
                         id = id, family = binomial, corstr = "independence")
}

if(requireNamespace("geepack")){
  library(geepack)
  mod.geeglm = geepack::geeglm(depression ~ diagnose + drug*time, data = data.gee,
                               id = id, family = binomial, corstr = "independence")
  mod.geeglm.r = geepack::geeglm(depression ~ diagnose, data = data.gee, id = id,
                                 family = binomial, corstr = "independence")
  mod.geeglm.int = geepack::geeglm(depression ~ 1, data = data.gee, id = id,
                                 family = binomial, corstr = "independence")
}


if(requireNamespace("glmtoolbox")){
  #library(glmtoolbox)
  mod.glmgee = glmtoolbox::glmgee(depression ~ diagnose + drug*time, data = data.gee,
                               id = id, family = binomial, corstr = "independence")
  mod.glmgee.r = glmtoolbox::glmgee(depression ~ diagnose, data = data.gee, id = id,
                                 family = binomial, corstr = "independence")
  mod.glmgee.int = glmtoolbox::glmgee(depression ~ 1, data = data.gee, id = id,
                                   family = binomial, corstr = "independence")
}

mod.lme = nlme::lme(distance ~ age + Sex, data = nlme::Orthodont, random = ~ 1)
mod.lme.int = nlme::lme(distance ~ 1, data = nlme::Orthodont, random = ~ 1)

if(requireNamespace("lme4")){
  mod.lmerMod = lme4::lmer(Reaction ~ Days + (Days | Subject), lme4::sleepstudy)
  data.lmer = nlme::Orthodont
  data.lmer$nsex <- as.numeric(data.lmer$Sex=="Male")
  data.lmer$nsexage <- with(data.lmer, nsex*age)
  mod.lmerMod2 = lme4::lmer(distance ~ age + (age|Subject) + (0+nsex|Subject) +
                        (0 + nsexage|Subject), data=data.lmer)
  mod.lmerMod.int = lme4::lmer(Reaction ~ 1 + (1 | Subject), lme4::sleepstudy)
}

# tibbles
if(requireNamespace("tibble")){
  tib.dat <- tibble::as_tibble(data)
  mod.glm.tib <- glm(charges ~ region * age + bmi + sex, data = tib.dat)
  mod.lm.tib <- lm(charges ~ region * age + bmi + sex, data = tib.dat)
  tib.nls <- tibble::as_tibble(data.nls)
  mod.nls.tib <- nls(Y~a * X/(b + X), data = tib.nls, start = list(a = 1, b = 2))
  if(requireNamespace("survival")){
    tib.surv <- tibble::as_tibble(data.surv)
    mod.surv.tib <- survival::survreg(survival::Surv(time, status) ~ age + sex + ph.karno, data=tib.surv,
                            dist='weibull', robust = TRUE)
    mod.coxph.tib <-  survival::coxph(survival::Surv(time, status) ~ age + sex + wt.loss,
                            data=tibble::as_tibble(survival::lung), robust = TRUE)
  }
  if(requireNamespace("pscl")){
    tib.hurdle <- tibble::as_tibble(data.hurdle)
    mod.hurdle.tib <- pscl::hurdle(art ~ fem + mar + kid5 + phd + ment | fem + mar +
                                     kid5 + phd + ment, data = tib.hurdle)
    mod.zinf.tib <- pscl::zeroinfl(art ~ fem + mar + kid5 + phd + ment | fem + mar +
                                     kid5 + phd + ment, data = tib.hurdle)
  }
  tib.gee <- tibble::as_tibble(data.gee)
  if(requireNamespace("gee")){
    mod.gee.tib <- gee::gee(depression ~ diagnose + drug*time, data = tib.gee,
                            id = id, family = binomial, corstr = "independence")
  }
  if(requireNamespace("geepack")){
    mod.geeglm.tib <- geepack::geeglm(depression ~ diagnose + drug*time,
                                      data = tib.gee, id = id, family = binomial,
                                      corstr = "independence")
  }

  if(requireNamespace("glmtoolbox")){
    mod.glmgee.tib <- glmtoolbox::glmgee(depression ~ diagnose + drug*time,
                                      data = tib.gee, id = id, family = binomial,
                                      corstr = "independence")
  }

  if(requireNamespace("lme4")){
    lmerModtibdat = tibble::as_tibble(lme4::sleepstudy)
    mod.lmerMod.tib <- lme4::lmer(Reaction ~ Days + (Days | Subject),
                                  lmerModtibdat)
  }}

# glmmTMB
if(requireNamespace("glmmTMB")){
  model_glmmTMB <- glmmTMB::glmmTMB(Reaction ~ Days + (1 | Subject), data = lme4::sleepstudy, family = gaussian())
}

# Models with transformed responses -- used to test that bootstrap (resi_stat)
# can update() the reduced model on bootstrap data that has the raw columns
# (e.g. 'charges') but NOT the pre-computed transformed column ('log10(charges)').
mod.lm.log10  <- lm(log10(charges) ~ region + age + bmi + sex, data = data)
mod.glm.I     <- glm(I(charges > 15000) ~ region + age + bmi + sex,
                     data = data, family = binomial())

## tests
test_that("Specifying non-allowed vcov produces warning",{
  expect_warning(resi(mod.nls, data = data.nls, nboot = 1, vcovfunc = sandwich::vcovHC),
                 "Sandwich vcov function not applicable for nls model type, vcovfunc set to regtools::nlshc")
  if(requireNamespace("survival")){
  expect_warning(resi(mod.surv, data = data.surv, nboot = 1, vcovfunc = sandwich::vcovHC),
                 "vcovfunc argument ignored for survreg objects")
  expect_warning(resi(mod.coxph, data = data.surv, nboot = 1, vcovfunc = sandwich::vcovHC),
                 "vcovfunc argument ignored for coxph objects")}
})


test_that("boot.method = 'bayes' only works for lm and nls", {
  expect_true(resi(mod.lm, nboot = 1, boot.method = "bayes")$boot.method == "bayes")
  expect_true(resi(mod.nls, data = data.nls, nboot = 1, boot.method = "bayes")$boot.method == "bayes")
  expect_error(resi(mod, nboot = 1, boot.method = "bayes"))
  if(requireNamespace("survival")){
    expect_error(resi(mod.surv, data = data.surv, nboot = 1, boot.method = "bayes"))
    expect_error(resi(mod.coxph, data = data.surv, nboot = 1, boot.method = "bayes"))}
  if(requireNamespace("pscl")){
    expect_error(resi(mod.hurdle, nboot = 1, boot.method = "bayes"))
    expect_error(resi(mod.zinf, nboot = 1, boot.method = "bayes"))}
})

test_that("data is needed for certain models", {
  # mod.s now uses ci.method='qf' by default, which succeeds without data;
  # use ci.method='boot' to test that bootstrapping fails without data
  expect_error(resi(mod.s, nboot = 1, ci.method = "boot"))
  # mod.lm.s now uses ci.method='qf' by default, which succeeds without data;
  # use ci.method='boot' to test that bootstrapping fails without data
  expect_error(resi(mod.lm.s, nboot = 1, ci.method = "boot"))
  expect_error(resi(mod.nls, nboot = 1))
  if(requireNamespace("survival")){
  expect_error(resi(mod.surv, nboot = 1))
  expect_error(resi(mod.coxph, nboot = 1))}
  if(requireNamespace("gee")){
  expect_error(resi(mod.gee))}
})

# using different boot.results storage now
# test_that("boot.results stores correctly",{
#   expect_equal(colnames(resi(mod.log, nboot = 2, store.boot = TRUE)$boot.results$t), c("Overall", "(Intercept)", "age", "regionnorthwest", "regionsoutheast", "regionsouthwest", "age", "region"))
# })

test_that("resi produces the correct estimates", {
  expect_equal(unname(resi(mod, nboot = 1)$estimates), c(0.35982590, -0.06733537, -0.02670248, -0.03341748, -0.00246893, 0.14980504, 0.15238719,
                                                         0.05839381, 0.01667149, 0.03610640, -0.01497171, 0.03655364, 0.29533571, 0.14991488,
                                                         0.05159896, 0.01617303), tolerance = 1e-07)
  expect_equal(unname(resi(mod.lm, nboot = 1)$estimates), c(0.3595407548, -0.0672973369,
                                                            -0.0266873974, -0.0333986021,
                                                            -0.0024675352, 0.1497204142,
                                                            0.1523011075, 0.0583608197,
                                                            0.0166620711, 0.0360860037,
                                                            -0.0149632506, 0.0364798507,
                                                            0.2951113263, 0.1497981921,
                                                            0.0515491712, 0.0160560332), tolerance = 1e-07)
  expect_equal(unname(resi(mod.s, nboot = 1, data = data)$estimates[c(1, 6, 11, 24)]),
               c(0.35634034, 0.06143486, 0.00748577, 0.05339006), tolerance = 1e-07)
  if(requireNamespace("survival")){
  expect_equal(unname(resi(mod.surv, nboot = 1, data = data.surv)$estimates),
               c(0.194002944, 0.515785774, -0.080800275, 0.200549139, 0.105353954, -0.276392759,
                 0.046080344, 0.189247644, 0.081817903), tolerance = 1e-07)
  expect_equal(unname(resi(mod.coxph, nboot = 1, data = data.surv)$estimates),
               c(0.224354226, 0.136138740, -0.213293162, 0.008641209, 0.117732151,
                 0.202042262, 0.000000000))
  }
  if(requireNamespace("pscl")){
  expect_equal(unname(resi(mod.hurdle, nboot = 1)$estimates[c(1, 2, 5, 8, 10)]),
               c(0.28063439, 0.12770489, -0.06898884, 0.02686305, 0.06039791),
               tolerance = 1e-07)
  expect_equal(unname(resi(mod.zinf, nboot = 1)$estimates[c(1, 2, 5, 8, 10)]),
               c(0.23725614, 0.11892936, -0.07014934, -0.03481673, -0.03461235),
               tolerance = 1e-05)
  }
  if(requireNamespace("gee")){
  expect_equal(unname(resi(mod.gee, nboot = 10, data = data.gee)$coefficients[,'L-RESI']),
               c(-0.02585617, -0.48811210, 0.01414410, 0.56177861, -0.29398258),
               tolerance = 1e-07)
  expect_equal(unname(resi(mod.gee, nboot = 10, data = data.gee)$coefficients[,'CS-RESI']),
               c(-0.007006305,-0.121182453,0.003769394,0.119402133,-0.069223594),
               tolerance = 1e-07)
  }
  if(requireNamespace("geepack")){
  expect_equal(unname(resi(mod.geeglm, nboot = 10, data = data.gee)$coefficients[,'L-RESI']),
               c(-0.02585617, -0.48811210, 0.01414410, 0.56177861, -0.29398258),
               tolerance = 1e-07)
  expect_equal(unname(resi(mod.geeglm, nboot = 10, data = data.gee)$coefficients[,'CS-RESI']),
               c(-0.007006305,-0.121182453,0.003769394,0.119402133,-0.069223594),
               tolerance = 1e-07)
  }

  # if(requireNamespace("glmtoolbox")){
  # for some reason test is saying mod.glmgee not found, can't resolve. manually tested
  #   # tests don't seem to like passing data argument. I think glmtoolbox deals with environments strangely
  #   mod.glmgee = glmtoolbox::glmgee(depression ~ diagnose + drug*time, data = data.gee,
  #                                   id = id, family = binomial, corstr = "independence")
  #   expect_equal(unname(resi(mod.glmgee, nboot = 10)$coefficients[,'L-RESI']),
  #                c(-0.02585617, -0.48811210, 0.01414410, 0.56177861, -0.29398258),
  #                tolerance = 1e-07)
  #   expect_equal(unname(resi(mod.glmgee, nboot = 10)$coefficients[,'CS-RESI']),
  #                c(-0.007006305,-0.121182453,0.003769394,0.119402133,-0.069223594),
  #                tolerance = 1e-07)
  #
  #   # # SNV: temporary pasted in for now
  #   # library(glmtoolbox)
  #   # data(spruce)
  #   # m1 =  glmgee(size ~ poly(days,4) + treat, id=tree, family=Gamma(log), data=spruces)
  #   # debug(resi.glmgee)
  #   # glmtoolboxRESI = resi(m1, nboot=10, data=spruces)
  #   # glmtoolboxRESI = resi_pe(m1, nboot=10, data=spruces)
  #   # ggmod = geeglm(size ~ poly(days,4) + treat, id=tree, family=Gamma(log), data=spruces)
  #   # geeglmRESI = resi_pe(ggmod)
  # }
  expect_equal(unname(suppressWarnings(resi(mod.lme, nboot = 10)$coefficients[,'RESI'])),
               c(3.659090, 1.739166, 0.512371), tolerance = 1e-07)
  if(requireNamespace("lme4")){
  expect_equal(unname(suppressWarnings(resi(mod.lmerMod, nboot = 10)$coefficients[,'L-RESI'])),
               c(8.434942, 1.533073), tolerance = 1e-07)}
  if(requireNamespace("glmmTMB")){
    expect_equal(unname(suppressWarnings(resi(model_glmmTMB, nboot = 10)$coefficients[,'RESI'])),
                 c( 6.22903, 3.06823), tolerance = 1e-05)
  }
})

test_that("RESI estimates are in between the confidence limits", {
  resi.obj = resi(mod, nboot = 500, store.boot = TRUE)
  expect_true(all(resi.obj$coefficients$RESI >= resi.obj$coefficients$`2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`97.5%`))
  expect_true(all(resi.obj$anova$RESI >= resi.obj$anova$`2.5%`) & all(resi.obj$anova$RESI <= resi.obj$anova$`97.5%`))
  An.obj = car::Anova(resi.obj, alpha = 0.01)
  expect_true(all(An.obj$RESI >= An.obj$`0.05%`) & all(An.obj$RESI <= An.obj$`99.5%`))
  # ci.method='boot' needed: resi.lm now defaults to 'qf' which skips bootstrap
  # and store.boot=TRUE would have no effect, breaking Anova.resi at a new alpha
  resi.obj = resi(mod.lm, nboot = 500, store.boot = TRUE, ci.method = "boot")
  expect_true(all(resi.obj$coefficients$RESI >= resi.obj$coefficients$`2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`97.5%`))
  expect_true(all(resi.obj$anova[1:5, "RESI"] >= resi.obj$anova[1:5, "2.5%"]) & all(resi.obj$anova[1:5, "RESI"] <= resi.obj$anova[1:5, "RESI"]))
  An.obj = car::Anova(resi.obj, alpha = 0.01)
  expect_true(all(An.obj$RESI[1:5] >= An.obj$`0.05%`[1:5]) & all(An.obj$RESI[1:5] <= An.obj$`99.5%`[1:5]))
  resi.obj = resi(mod.nls, data = data.nls)
  expect_true(all(resi.obj$coefficients$RESI >= resi.obj$coefficients$`2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`97.5%`))
  if(requireNamespace("survival")){
  resi.obj = resi(mod.coxph, data = data.surv, nboot = 500)
  expect_true(all(resi.obj$coefficients$RESI >= resi.obj$coefficients$`2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`97.5%`))
  expect_true(all(resi.obj$anova$RESI >= resi.obj$anova$`2.5%`) & all(resi.obj$anova$RESI <= resi.obj$anova$`97.5%`))}
  # commented for time
  # resi.obj = resi(mod.hurdle, nboot = 500)
  # expect_true(all(resi.obj$coefficients$RESI >= resi.obj$coefficients$`2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`97.5%`))
  if(requireNamespace("geepack")){
  resi.obj = resi(mod.geeglm, nboot = 500)
  expect_true(all(resi.obj$coefficients$`L-RESI` >= resi.obj$coefficients$`L 2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`L 97.5%`))
  expect_true(all(resi.obj$coefficients$`CS-RESI` >= resi.obj$coefficients$`CS 2.5%`) & all(resi.obj$coefficients$RESI <= resi.obj$coefficients$`CS 97.5%`))
  expect_true(all(resi.obj$anova$`L-RESI` >= resi.obj$anova$`L 2.5%`) & all(resi.obj$anova$RESI <= resi.obj$anova$`L 97.5%`))
  expect_true(all(resi.obj$anova$`CS-RESI` >= resi.obj$anova$`CS 2.5%`) & all(resi.obj$anova$RESI <= resi.obj$anova$`CS 97.5%`))}
})

# Regression test: bootstrap must succeed for models with transformed responses.
# Bug: resi.default previously fit the intercept-only reduced model using a
# backtick-quoted column name (e.g. `log10(charges)` ~ 1) from model.frame,
# which broke resi_stat's update(model.reduced, data = boot.data) because
# boot.data has 'charges' but not a pre-computed 'log10(charges)' column.
# Every replicate returned NA -> "Bootstrapping failed" error.
test_that("bootstrap succeeds for models with transformed responses", {
  set.seed(1)
  # lm with log-transformed response
  r.lm <- resi(mod.lm.log10, data = data, nboot = 20, ci.method = "boot")
  expect_s3_class(r.lm, "resi")
  expect_false(anyNA(r.lm$coefficients$RESI))
  expect_true(all(r.lm$coefficients$RESI >= r.lm$coefficients$`2.5%`))
  expect_true(all(r.lm$coefficients$RESI <= r.lm$coefficients$`97.5%`))

  # glm with I() binary response
  r.glm <- suppressWarnings(
    resi(mod.glm.I, data = data, nboot = 20, ci.method = "boot")
  )
  expect_s3_class(r.glm, "resi")
  expect_false(anyNA(r.glm$coefficients$RESI))
  expect_true(all(r.glm$coefficients$RESI >= r.glm$coefficients$`2.5%`))
  expect_true(all(r.glm$coefficients$RESI <= r.glm$coefficients$`97.5%`))
})

if(requireNamespace("lme4")){
test_that("lmerMod: L-RESI and CS-RESI point estimates are correct", {
  pe = suppressWarnings(resi_pe(mod.lmerMod))
  expect_equal(unname(pe$coefficients[,'L-RESI']),
               c(8.434942, 1.533073), tolerance = 1e-07)
  expect_equal(unname(pe$coefficients[,'CS-RESI']),
               c(3.6680844, 0.6185898), tolerance = 1e-06)
  expect_equal(unname(pe$anova[,'L-RESI']),  1.533073, tolerance = 1e-06)
  expect_equal(unname(pe$anova[,'CS-RESI']), 0.5719247, tolerance = 1e-06)
})

if(requireNamespace("geepack")){
test_that("geeglm (exchangeable, positive rho) L-RESI matches lmerMod (CR0) L-RESI", {
  sdata = lme4::sleepstudy
  sdata$subjid = as.integer(sdata$Subject)
  mod_lmer_ri  = lme4::lmer(Reaction ~ Days + (1 | Subject), sdata)
  mod_gee_exch = geepack::geeglm(Reaction ~ Days, data = sdata, id = subjid,
                                  family = gaussian, corstr = "exchangeable")
  pe_lmer = suppressWarnings(resi_pe(mod_lmer_ri, vcov.args = list(type = "CR0")))
  pe_gee  = resi_pe(mod_gee_exch)
  # for positive exchangeable correlation (alpha ~ 0.58), GEE and CR0 sandwich
  # give the same L-RESI up to finite-sample differences
  expect_equal(unname(pe_lmer$coefficients[,'L-RESI']),
               unname(pe_gee$coefficients[,'L-RESI']), tolerance = 0.05)
  # CS-RESI is identical: both compute the same weighted independence lm + HC0
  expect_equal(unname(pe_lmer$coefficients[,'CS-RESI']),
               unname(pe_gee$coefficients[,'CS-RESI']), tolerance = 1e-07)
})}}

if(requireNamespace("gee") & requireNamespace("geepack")){
test_that("vcov methods same for gee and geeglm",{
  expect_equal(unname(mod.gee$robust.variance), mod.geeglm$geese$vbeta)
  modg1 = glm(formula = formula(mod.gee), family = mod.gee$family, data = data.gee,
              contrasts = mod.gee$contrasts)
  modg2 = glm(formula = formula(mod.geeglm), family = mod.geeglm$family, data = data.gee,
              contrasts = mod.geeglm$contrasts)
  expect_equal(sandwich::vcovHC(modg1, type = "HC0"), sandwich::vcovHC(modg2, type = "HC0"))
})

test_that("boot.results same (approx) for gee and geeglm",{
  set.seed(123)
  resi.obj = resi(mod.geeglm, nboot = 5, store.boot = T, anova = F)
  set.seed(123)
  resi.obj2 = resi(mod.gee, data = data.gee, nboot = 5, store.boot = T)
  expect_equal(resi.obj$boot.results$t[,-1], resi.obj2$boot.results$t, tolerance = 1e-09)
})}

if(requireNamespace("glmtoolbox") & requireNamespace("geepack")){
  test_that("vcov and estimates are the same for glmgee and geeglm",{
    expect_equal(unname(mod.glmgee$R), mod.geeglm$geese$vbeta)
  })

  test_that("boot.results same (approx) for gee and geeglm",{
    set.seed(123)
    resi.obj = resi(mod.geeglm, nboot = 5, store.boot = T, anova = F)
    set.seed(123)
    resi.obj2 = resi(mod.gee, data = data.gee, nboot = 5, store.boot = T)
    expect_equal(resi.obj$boot.results$t[,-1], resi.obj2$boot.results$t, tolerance = 1e-09)
  })}

test_that("unbiased = FALSE returns same abs. RESI as Chi-sq/F",{
  resi.obj = resi(mod, nboot = 1, unbiased = FALSE)
  expect_equal(abs(resi.obj$coefficients[6:7, 'RESI']), resi.obj$anova[3:4, 'RESI'], tolerance = 1e-03)
  if(requireNamespace("survival")){
  resi.obj = resi(mod.surv, data = data.surv, nboot = 1, unbiased = FALSE)
  expect_equal(abs(resi.obj$coefficients[2:4, 'RESI']), resi.obj$anova[1:3, 'RESI'], tolerance = 1e-07)
  resi.obj = resi(mod.coxph, data = data.surv, nboot = 1, unbiased = FALSE)
  expect_equal(abs(resi.obj$coefficients[1:3, 'RESI']), resi.obj$anova[1:3, 'RESI'], tolerance = 1e-07)}
  resi.obj = resi(mod.lm, nboot = 1, unbiased = FALSE)
  expect_equal(abs(resi.obj$coefficients[6:7,'RESI']), resi.obj$anova[3:4,'RESI'], tolerance = 1e-07)
})

test_that("Using a matrix for data works", {
  expect_true(class(resi(mod.nls, nboot = 10, data = as.matrix(data.nls))) == "resi")
})

test_that("Specifying a reduced model only changes overall output", {
  resi.obj = resi(mod, nboot = 1)
  resi.objr = resi(mod, model.reduced = mod.r, nboot = 1)
  expect_equal(resi.obj$coefficients$RESI, resi.objr$coefficients$RESI)
  expect_equal(resi.obj$anova$RESI, resi.objr$anova$RESI)
  expect_false(resi.obj$overall$RESI[2] == resi.objr$overall$RESI[2])
  resi.obj = resi(mod.lm.s, data = data, nboot = 1)
  resi.objr = resi(mod.lm.s, model.reduced = mod.lm.s.r, data = data, nboot = 1)
  expect_equal(resi.obj$coefficients$RESI, resi.objr$coefficients$RESI)
  expect_equal(resi.obj$anova$RESI, resi.objr$anova$RESI)
  expect_false(resi.obj$overall$RESI[2] == resi.objr$overall$RESI[2])
  if(requireNamespace("survival")){
  resi.obj = resi(mod.surv, data = data.surv, nboot = 1)
  resi.objr = resi(mod.surv, model.reduced = mod.surv.red, data = data.surv[which(!(1:nrow(data.surv)%in% mod.surv$na.action)),], nboot = 10)
  expect_false(resi.obj$overall$RESI[2] == resi.objr$overall$RESI[2])}
  if(requireNamespace("pscl")){
  resi.obj = resi(mod.hurdle, nboot = 1)
  resi.objr = resi(mod.hurdle, model.reduced = mod.hurdle.r, nboot = 1)
  expect_equal(resi.obj$coefficients$RESI, resi.objr$coefficients$RESI)
  expect_equal(resi.obj$anova$RESI, resi.objr$anova$RESI)
  expect_false(resi.obj$overall$RESI[2] == resi.objr$overall$RESI[2])}
})

test_that("specifying additional vcov args works",{
  expect_false(any(resi(mod, nboot = 1, vcov.args = list(type = "HC0"))$coefficients[,'RESI'] == resi(mod, nboot = 1)$coefficients[,'RESI']))
})

test_that("specifying additional Anova args works",{
  expect_false(all(resi(mod, nboot = 1, Anova.args = list(type = "3"))$anova[2:6,'RESI'] == resi(mod, nboot = 1)$anova[,'RESI']))
  if(requireNamespace("survival")){
  expect_false(length(resi(mod.surv, data = data.surv, nboot = 1, Anova.args = list(type = "3"))$anova$RESI) == length(resi(mod.surv, data = data.surv, nboot = 1)$anova$RESI))}
})

test_that("vcovfunc = vcov changes naive.var to TRUE",{
  expect_true(resi(mod, vcovfunc = vcov, nboot = 10)$naive.var == TRUE)
})

if(requireNamespace("tibble")){
test_that("tibbles work", {
  expect_equal(unname(resi(mod.glm.tib, nboot = 1)$estimates), c(0.35982590, -0.06733537, -0.02670248, -0.03341748, -0.00246893, 0.14980504, 0.15238719,
                                                                 0.05839381, 0.01667149, 0.03610640, -0.01497171, 0.03655364, 0.29533571, 0.14991488,
                                                                 0.05159896, 0.01617303), tolerance = 1e-07)
  expect_equal(unname(resi(mod.lm.tib, nboot = 1)$estimates), c(0.3595407548, -0.0672973369,
                                                                -0.0266873974, -0.0333986021,
                                                                -0.0024675352, 0.1497204142,
                                                                0.1523011075, 0.0583608197,
                                                                0.0166620711, 0.0360860037,
                                                                -0.0149632506, 0.0364798507,
                                                                0.2951113263, 0.1497981921,
                                                                0.0515491712, 0.0160560332))
  if(requireNamespace("survival")){
  expect_equal(unname(resi(mod.surv.tib, nboot = 1, data = data.surv)$estimates), c(0.194002944, 0.515785774, -0.080800275, 0.200549139, 0.105353954, -0.276392759,
                                                                                    0.046080344, 0.189247644, 0.081817903), tolerance = 1e-07)
  expect_equal(unname(resi(mod.coxph.tib, nboot = 1, data = data.surv)$estimates), c(0.224354226, 0.136138740, -0.213293162, 0.008641209, 0.117732151, 0.202042262, 0.000000000))
  }
  if(requireNamespace("pscl")){
  expect_equal(unname(resi(mod.hurdle.tib, nboot = 1)$estimates[c(1, 2, 5, 8, 10)]), c(0.28063439, 0.12770489, -0.06898884, 0.02686305, 0.06039791), tolerance = 1e-07)
  expect_equal(unname(resi(mod.zinf.tib, nboot = 1)$estimates[c(1, 2, 5, 8, 10)]), c(0.23725614, 0.11892936, -0.07014934, -0.03481673, -0.03461235), tolerance = 1e-05)
  }
  if(requireNamespace("gee")){
  expect_equal(unname(resi(mod.gee.tib, nboot = 10, data = data.gee)$coefficients[,'L-RESI']), c(-0.02585617, -0.48811210, 0.01414410, 0.56177861, -0.29398258), tolerance = 1e-07)
  }
  if(requireNamespace("geepack")){
  expect_equal(unname(resi(mod.geeglm.tib, nboot = 10, data = data.gee)$coefficients[,'L-RESI']), c(-0.02585617, -0.48811210, 0.01414410, 0.56177861, -0.29398258), tolerance = 1e-07)
  }
  # if(requireNamespace("glmtoolbox")){
  #   expect_equal(unname(resi(mod.glmgee.tib, nboot = 10)$coefficients[,'L-RESI']), c(-0.02585617, -0.48811210, 0.01414410, 0.56177861, -0.29398258), tolerance = 1e-07)
  # }
  if(requireNamespace("lme4")){
  expect_equal(unname(suppressWarnings(resi(mod.lmerMod.tib, nboot = 10))$coefficients[,'L-RESI']),c(8.434942, 1.533073), tolerance = 1e-07)
  }
})}

test_that("intercept-only full model works and doesn't have overall element", {
  expect_true(is.null(resi(mod.glm.int, nboot = 1, Anova.args = list(type = 3))$overall))
  expect_true(is.null(resi(mod.lm.int, nboot = 1, Anova.args = list(type = 3))$overall))
  if(requireNamespace("survival")){
    expect_true(is.null(resi(mod.surv.int, data = data.surv, nboot = 1, Anova.args = list(type = 3))$overall))
  }
  if(requireNamespace("pscl")){
    expect_true(is.null(resi(mod.hurdle.int, nboot = 1, Anova.args = list(type = 3))$overall))
    expect_true(is.null(resi(mod.zinf.int, nboot = 1, Anova.args = list(type = 3))$overall))
  }
  # to be implemented
  # if(requireNamespace("geepack")){
  #   expect_true(is.null(resi(mod.geeglm.int, nboot = 1)$overall))
  # }

})

test_that("same model.full and model.reduced produces error",{
  expect_error(resi(model.full = mod, model.reduced = mod, nboot = 1))
})

# important because some resi_pe methods use waldtest and some use wald.test
test_that("wald.test and waldtest still consistent",{
  expect_equal(unname(lmtest::waldtest(mod.lm, vcov = vcovHC, test = "Chisq")$Chisq[2]),
               unname(aod::wald.test(vcovHC(mod.lm), coef(mod.lm),
                                     Terms = 2:nrow(vcovHC(mod.lm)))$result$chi2["chi2"]))
})

# issue from github
test_that("overall = FALSE doesn't produce an error", {
  expect_equal(unname(resi(mod.lm, nboot = 1, overall = F, coefficients = F)$estimates),
               c(0.03647985, 0.29511133, 0.14979819, 0.05154917, 0.01605603), tolerance = 1e-05)
})

# robustbase: lmrob and glmrob
if (requireNamespace("robustbase", quietly = TRUE)) {
  set.seed(7)
  n_rob <- 150
  df_rob <- data.frame(
    x1 = rnorm(n_rob),
    x2 = factor(sample(c("a", "b", "c"), n_rob, replace = TRUE)),
    y  = rnorm(n_rob)
  )
  df_rob$y <- df_rob$y + 2 * df_rob$x1 +
    ifelse(df_rob$x2 == "b", 1.5, ifelse(df_rob$x2 == "c", -1, 0))
  df_rob$y[1:5] <- df_rob$y[1:5] + 15   # outliers

  mod.lmrob     <- robustbase::lmrob(y ~ x1 + x2, data = df_rob)
  mod.lmrob.r   <- robustbase::lmrob(y ~ x1,      data = df_rob)
  mod.lmrob.int <- robustbase::lmrob(y ~ 1,        data = df_rob)

  set.seed(7)
  n_grob <- 150
  df_glmrob <- data.frame(
    x1 = rnorm(n_grob),
    x2 = factor(sample(c("a", "b", "c"), n_grob, replace = TRUE))
  )
  lp_grob <- 1.5 * df_glmrob$x1 +
    ifelse(df_glmrob$x2 == "b", 1, -0.5)
  df_glmrob$y <- rbinom(n_grob, 1, plogis(lp_grob))

  mod.glmrob     <- suppressWarnings(robustbase::glmrob(y ~ x1 + x2, family = binomial, data = df_glmrob))
  mod.glmrob.r   <- suppressWarnings(robustbase::glmrob(y ~ x1,      family = binomial, data = df_glmrob))
  mod.glmrob.int <- suppressWarnings(robustbase::glmrob(y ~ 1,        family = binomial, data = df_glmrob))

  test_that("lmrob: resi_pe produces correct estimates", {
    pe <- suppressWarnings(resi_pe(mod.lmrob))
    expect_equal(unname(pe$estimates),
                 c(2.1317677, 0.0321117, 1.7228418, 0.5869813, -0.2935287,
                   1.7298297, 0.9128559),
                 tolerance = 1e-06)
    expect_equal(unname(pe$anova$RESI),  c(1.7298297, 0.9128559), tolerance = 1e-06)
    expect_equal(unname(pe$coefficients$RESI),
                 c(0.0321117, 1.7228418, 0.5869813, -0.2935287), tolerance = 1e-06)
  })

  test_that("lmrob: resi() returns RESI within CI bounds", {
    set.seed(1)
    ro <- suppressWarnings(resi(mod.lmrob, nboot = 100))
    expect_true(all(ro$coefficients$RESI >= ro$coefficients$`2.5%`))
    expect_true(all(ro$coefficients$RESI <= ro$coefficients$`97.5%`))
    expect_true(all(ro$anova$RESI >= ro$anova$`2.5%`))
    expect_true(all(ro$anova$RESI <= ro$anova$`97.5%`))
  })

  test_that("lmrob: specifying a reduced model only changes overall output", {
    ro  <- suppressWarnings(resi(mod.lmrob,   nboot = 1))
    ror <- suppressWarnings(resi(mod.lmrob, model.reduced = mod.lmrob.r, nboot = 1))
    expect_equal(ro$coefficients$RESI, ror$coefficients$RESI)
    expect_equal(ro$anova$RESI,        ror$anova$RESI)
    expect_false(ro$overall$RESI[2]   == ror$overall$RESI[2])
  })

  test_that("lmrob: intercept-only model has no overall element", {
    expect_true(is.null(suppressWarnings(resi(mod.lmrob.int, nboot = 1, anova = FALSE))$overall))
  })

  test_that("glmrob: resi_pe produces correct estimates", {
    pe2 <- suppressWarnings(resi_pe(mod.glmrob))
    expect_equal(unname(pe2$estimates),
                 c(0.4340252, -0.1257804, 0.4254354, 0.2549712, 0.0386988,
                   0.4175268, 0.2551174),
                 tolerance = 1e-06)
    expect_equal(unname(pe2$anova$RESI), c(0.4175268, 0.2551174), tolerance = 1e-06)
  })

  test_that("glmrob: resi() returns RESI within CI bounds", {
    set.seed(1)
    ro2 <- suppressWarnings(resi(mod.glmrob, data = df_glmrob, nboot = 100))
    expect_true(all(ro2$coefficients$RESI >= ro2$coefficients$`2.5%`))
    expect_true(all(ro2$coefficients$RESI <= ro2$coefficients$`97.5%`))
    expect_true(all(ro2$anova$RESI >= ro2$anova$`2.5%`))
    expect_true(all(ro2$anova$RESI <= ro2$anova$`97.5%`))
  })

  test_that("glmrob: vcovHC redirects to stats::vcov with warning", {
    expect_warning(
      resi_pe(mod.glmrob, vcovfunc = sandwich::vcovHC),
      "sandwich::vcovHC is not supported for glmrob"
    )
  })

  test_that("glmrob: intercept-only model has no overall element", {
    expect_true(is.null(suppressWarnings(resi(mod.glmrob.int, data = df_glmrob, nboot = 1, anova = FALSE))$overall))
  })
}

#' Compute the robust effect size index estimate from Z statistic
#'
#' This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
#' Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
#' @param z The Z statistic for the parameter of interest.
#' @param n Number of independent samples.
#' @param unbiased Logical, whether to use unbiased or alternative estimator. See details.
#' @details This function computes S, the RESI, from a Z statistic. The formula for the
#' unbiased estimator (default) is derived by solving the expected value of the
#' Z statistic for S. It is unbiased and consistent.
#'
#' The formula for the unbiased conversion is:
#'
#' \eqn{S = Z/\sqrt(n)}
#'
#' The formula for the alternative estimator is derived by squaring the Z statistic
#' and using the \code{\link{chisq2S}} formula. This estimator may be appealing for its
#' intuitive relationship to the Chi-square statistic; the absolute value of RESI estimates
#' using this formula will be equal to a RESI estimate using a Chi-square statistic for
#' the same model. However, this estimator does have finite sample bias, which is an
#' important consideration for the coverage of the bootstrapping that \code{resi} uses.
#'
#' The formula for the alternative conversion is:
#'
#' \eqn{ \sqrt(max(0, (Z^2 - 1)/n)) * sign(Z)}
#' @return Returns a scalar or vector argument of the robust effect size index estimate.
#' @examples
#' # to obtain example z values, first fit a glm
#' mod = glm(charges ~ region * age + bmi + sex, data = RESI::insurance)
#' # run coeftest to get z values using a robust variance-covariance function
#' zs = lmtest::coeftest(mod, vcov. = sandwich::vcovHC)[,'z value']
#'
#' # get RESI estimates using unbiased estimator
#' z2S(zs, n = nrow(RESI::insurance))
#'
#' # get RESI estimates usng alternative estimator
#' z2S(zs, n = nrow(RESI::insurance), unbiased = FALSE)
#' @export
z2S = function(z, n, unbiased = TRUE){
  if (unbiased){
    S = z/sqrt(n)
  } else{
    S = ifelse((z^2-1)<0, 0, sqrt((z^2-1)/n)*(sqrt(z^2)/z))
  }
  return(S)
}

#' Covert S to Cohen's \emph{f}^2
#'
#' Converts robust effect size index (S) to Cohen's \emph{f}^2
#' (effect size for multiple regression) using the formula from Vandekar, Tao, & Blume (2020).
#' @param S Numeric,the robust effect size index.
#' @details The formula for the conversion is:
#'
#' \eqn{ f^2 = S^2}
#' @return Returns an estimate of Cohen's \emph{f}^2 based on the RESI
#' @examples
#' # fit a linear regression model with continuous outcome and predictor
#' mod = lm(charges ~ age, data = RESI::insurance)
#'
#' # obtain t value for calculating RESI
#' t = summary(mod)$coefficients[2, "t value"]
#'
#' # calculate RESI
#' S = t2S(t, n = 1338, rdf = 1336)
#'
#' # convert to f^2
#' S2fsq(S)
#'
#' @export
S2fsq <- function(S){
  S^2
}

#' Omnibus (Overall) Wald Test for resi objects
#'
#' After running the \code{\link{resi}} function on a fitted model, this function
#' can be used to print the overall Wald test component. If the resi function was run
#' with the `store.boot = TRUE` option to store the full matrix of bootstrapped estimates,
#' the user can specify a different alpha level for this function's confidence intervals.
#' @param object an object resulting from resi function
#' @param alpha an optional new specification for the confidence level. Can be vector-valued
#' @param ... ignored
#' @return Returns a `omnibus_resi` object containing the computed omnibus Wald test
#' @examples
#' # fit a model
#' mod = lm(charges ~ bmi + sex, data = RESI::insurance)
#'
#' # run resi with the store.boot = TRUE option (ci.method must be "boot")
#' resi_obj = resi(mod, nboot = 100, store.boot = TRUE, alpha = 0.01, ci.method = "boot")
#'
#' # run summary, specifying a different alpha level if desired
#' omnibus(resi_obj, alpha = 0.05)
#' @export
omnibus <- function(object, alpha = NULL, ...){
  if(!"resi" %in% class(object)){
    stop("\nThis function only works on resi objects")
  }
  if(is.null(object$overall)){
    stop("\nThe resi object did not compute an omnibus test")
  }

  output = list(alpha = alpha, model.full = object$model.full)
  if (is.null(alpha)){
    output$alpha = object$alpha
    output$overall = object$overall
  }
  else{
    if (!(all(alpha %in% object$alpha))){
      if (is.null(object$boot.results)){
        stop("\nresi function was not run with store.boot = TRUE option")}
    }
    if(is.null(object$boot.results)){
      output$overall = object$overall[,c(1:(which(colnames(object$overall)
                                                           == "RESI")),
                                                  which(colnames(object$overall)%in%
                                                          c(paste(alpha/2*100, "%", sep=""),
                                                            paste((1-alpha/2)*100, "%", sep=""))))]
    }
    else{
      output$overall = object$overall[,1:(which(colnames(object$overall) == "RESI"))]
      boot.results = object$boot.results$t
      CIs = quantile(boot.results[,1], probs = sort(c(alpha/2, 1-alpha/2)),
                     na.rm = TRUE)
      output$overall[nrow(object$overall), c(paste(alpha/2*100, "%", sep=""),
                                         paste((1-rev(alpha)/2)*100, "%", sep=""))] = CIs
    }
  }
  class(output) = c("omnibus_resi")
  output
}

utils::globalVariables("w")
utils::globalVariables("w_cs")


#' Robust Effect Size Index (RESI) Point Estimation
#'
#' This function will estimate the robust effect size (RESI) from Vandekar, Tao, & Blume (2020).
#' The overall RESI is estimated via a Wald test. RESI is (optionally) estimated for each factor in coefficients-style table.
#' RESI is (optionally) estimated for each variable/interaction in an Anova-style table
#' for models with existing Anova methods. This function is the building block for the \code{\link{resi}} function.
#' @param model.full \code{lm, glm, nls, survreg, coxph, hurdle, zeroinfl, gee, geeglm} or \code{lme} model object.
#' @param model.reduced Fitted model object of same type as model.full. By default `NULL`; the same model as the full model but only having intercept.
#' @param data Data.frame or object coercible to data.frame of model.full data (required for some model types).
#' @param vcovfunc The variance estimator function for constructing the Wald test statistic. By default, sandwich::vcovHC (the robust (sandwich) variance estimator).
#' @param coefficients Logical, whether to produce a coefficients (summary) table with the RESI columns added. By default = `TRUE`.
#' @param anova Logical, whether to produce an Anova table with the RESI columns added. By default = `TRUE`.
#' @param overall Logical, whether to produce an overall Wald test comparing full to reduced model with RESI columns added. By default = `TRUE`.
#' @param Anova.args List, additional arguments to be passed to Anova function.
#' @param vcov.args List, additional arguments to be passed to vcovfunc.
#' @param unbiased Logical, whether to use the unbiased or alternative T/Z statistic to RESI conversion. By default, `TRUE`. See details.
#' @param waldtype Numeric, indicates which function to use for overall Wald test. 0 (default) = lmtest::waldtest Chi-square, 1 = lmtest::waldtest F, 2 = aod::wald.test
#' @param ... Ignored.
#' @importFrom aod wald.test
#' @importFrom car Anova
#' @importFrom lmtest waldtest
#' @importFrom sandwich vcovHC
#' @importFrom stats coef df.residual formula glm hatvalues model.frame pf predict quantile residuals update vcov
#' @importFrom utils methods
#' @export
#' @details The Robust Effect Size Index (RESI) is an effect size measure based on M-estimators.
#' This function is called by \code{\link{resi}} a specified number of times to
#' form bootstrapped confidence intervals. Called by itself, this function will
#' only calculate point estimates.
#'
#' The RESI, denoted as S, is applicable across many model types. It is a unitless
#' index and can be easily be compared across models. The RESI can also be
#' converted to Cohen's \emph{d} (\code{\link{S2d}}) under model homoskedasticity.
#'
#' The RESI is related to the non-centrality parameter
#' of the test statistic. The RESI estimate is consistent for all four
#' (Chi-square, F, T, and Z) types of statistics used. The Chi-square and F-based
#' calculations rely on asymptotic theory, so they may be biased in small samples.
#' When possible, the T and Z statistics are used. There are two formulas for both
#' the T and Z statistic conversion. The first (default, unbiased = TRUE)
#' are based on solving the expected value of the T or Z statistic for the RESI.
#' The alternative is based on squaring the T or Z statistic and using the
#' F or Chi-square statistic conversion. Both of these methods are consistent, but
#' the alternative exhibits a notable amount of finite sample bias. The alternative
#' may be appealing because its absolute value will be equal to the RESI based on
#' the F or Chi-square statistic. The RESI based on the Chi-Square and F statistics
#' is always greater than or equal to 0. The type of statistic
#' used is listed with the output. See \code{\link{f2S}}, \code{\link{chisq2S}},
#' \code{\link{t2S}}, and \code{\link{z2S}} for more details on the formulas.
#'
#' For GEE (\code{geeglm}) models, a longitudinal RESI (L-RESI) and a cross-sectional,
#' per-measurement RESI (CS-RESI) is estimated. The longitudinal RESI takes the
#' specified clustering into account, while the cross-sectional RESI is estimated
#' using a model where each measurement is its own cluster.
#'
#' @return Returns a list containing RESI point estimates
#' @examples
#' # This function produces point estimates for the RESI. The resi function will
#' # provide the same point estimates but adds confidence intervals. See resi for
#' # more detailed examples.
#'
#' ## resi_pe for a linear model
#' # fit linear model
#' mod <- lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
#' # run resi_pe on the model
#' resi_pe(mod)
#'
#' # if you want to have RESI estimates in the coefficient table that are equal in absolute
#' # value to those in the Anova table (except for those with >1 df and/or included in other
#' # interaction terms), you can specify unbiased = FALSE to use the alternate conversion.
#' resi_pe(mod, unbiased = FALSE)
#' @references Vandekar S, Tao R, Blume J. A Robust Effect Size Index. \emph{Psychometrika}. 2020 Mar;85(1):232-246. doi: 10.1007/s11336-020-09698-2.

resi_pe = function(...){
  UseMethod("resi_pe")
}

#' @describeIn resi_pe RESI point estimation
#' @export
resi_pe.default = function(model.full, model.reduced = NULL, data, anova = TRUE,
                           coefficients = TRUE, overall = TRUE, vcovfunc = sandwich::vcovHC,
                           Anova.args = list(), vcov.args = list(), unbiased = TRUE,
                           waldtype = 0, ...){

  # check for supported model type
  if(!(any(class(model.full) %in%
           sapply(as.character(utils::methods(RESI::resi_pe)), function(x) substr(x, 9, nchar(x)))))){
    warning("Model type not implemented in RESI package, attempting default")
  }

  dots = list(...)
  if (missing(data)){
    if (!is.null(model.full$model)){
      data = model.full$model
    } else{
      stop("\nData argument needed for model type")
    }
  }
  else{
    data = as.data.frame(data)
  }

  if (is.null(model.reduced) & overall){
    if(!("skip.red.pe" %in% names(dots))){
      form.reduced = as.formula(paste(format(formula(model.full)[[2]]), "~ 1"))
      if(!all(all.vars(form.reduced) %in% names(data)) ){
        form.reduced = as.formula( paste0('`', format(formula(model.full)[[2]]), '`', "~1"))
        if (length(attr(terms(formula(model.full)), "term.labels")) == 0) overall = FALSE
      }
      if (overall & (form.reduced == formula(model.full))){
        overall = FALSE} else{
          if (is.null(model.full$na.action)){
            model.reduced <- try(update(model.full, formula = form.reduced,
                                      data = data), silent = T)
          }
          else{
            if (!is.null(model.full$model)){
              model.reduced <- try(update(model.full, formula = form.reduced,
                                      data = model.full$model), silent = T)
            } else{
              model.reduced <- try(update(model.full, formula = form.reduced,
                                      data = data[which(!(1:nrow(data)%in% model.full$na.action)),]),
                                   silent = T)
            }}
          if(inherits(model.reduced, "try-error")){
            warning("Fitting intercept-only model failed. No overall test computed")
            overall = FALSE}
          }}}

  # dealing with additional vcov args
  # Guard: if vcovfunc is sandwich::vcovHC and vcov.args contains type = "const",
  # the user likely intends the parametric (model-based) variance. "const" is not
  # a valid HC type and will either error or silently give incorrect results.
  # Users should pass vcovfunc = stats::vcov for parametric variance instead.
  if (identical(vcovfunc, sandwich::vcovHC) && isTRUE(vcov.args[["type"]] == "const")) {
    stop(paste0(
      "\nvcov.args = list(type = \"const\") is not valid for sandwich::vcovHC.\n",
      "For parametric (model-based) variance, use vcovfunc = stats::vcov instead.\n",
      "For a heteroskedasticity-consistent estimator, use type = \"HC3\" (default),\n",
      "  \"HC0\", \"HC1\", \"HC2\", \"HC4\", etc."
    ))
  }
  if (length(vcov.args) == 0){
    vcovfunc2 <- vcovfunc
  } else{
    vcov.args = c(formals(vcovfunc)[1], vcov.args)
    vcovfunc2 <- function(x) {
      vcov.args[[1]] = x
      do.call(vcovfunc, vcov.args)}
  }

  # covariance matrix
  var.names = names(coef(model.full))
  tryCatch(vcovmat <- vcovfunc2(model.full), error = function(e){
    stop("\nComputing covariance failed. Trying running with different function for vcovfunc")
  })
  tryCatch(rownames(vcovmat) <- colnames(vcovmat) <- var.names, error = function(e){})


  # set up output
  if ((is.null(model.reduced) & !("skip.red.pe" %in% names(dots))) |
      inherits(model.reduced, "try-error")){
    output = list(model.full = list(call = model.full$call, formula = formula(model.full)),
                  model.reduced = NULL)
    overall = FALSE
  } else{
  output = list(model.full = list(call = model.full$call, formula = formula(model.full)),
                 model.reduced = list(call = model.reduced$call, formula = unlist(formula(model.reduced))))
  }
  names.est = c()
  # overall
  if (overall){
    # check if full and reduced model are the same
    if (!(is.null(model.reduced))){
      if (model.full$call == model.reduced$call){
        stop("\nFull and reduced model are identical")
      }}

    if (waldtype %in% c(0,1)){
      # 0 is Chisq with lmtest
      # 1 is F with lmtest
      wtype = ifelse(waldtype == 0, "Chisq", "F")
      tryCatch(overall.tab <- lmtest::waldtest(model.reduced, model.full, vcov = vcovfunc2,
                                               test = wtype), error = function(e){
                                                 stop("Overall Wald test failed, try running with overall = FALSE")})

      stats = overall.tab[,wtype][2]
      overall.df = overall.tab$Df[2]
      res.df = overall.tab$Res.Df[2]
    }

    if (waldtype == 2){
      # 2 is aod (nls, coxph)
      tryCatch(overall.tab <- aod::wald.test(vcovmat, coef(model.full),
                                             Terms = 1:length(coef(model.full)))$result$chi2,
               error = function(e){
                 stop("Overall Wald test failed, try running with overall = FALSE")})
      stats = overall.tab["chi2"]
      overall.df = overall.tab["df"]
      res.df = nrow(data) - overall.df
      overall.tab = t(as.data.frame(overall.tab))
    }
    overall.resi.hat = ifelse(waldtype == 1, f2S(stats, overall.df, res.df, nrow(data)),
                              chisq2S(stats, overall.df, nrow(data)))
    if (nrow(overall.tab) == 1){
      overall.tab = cbind(overall.tab, RESI = overall.resi.hat)
    } else{
      overall.tab[nrow(overall.tab), "RESI"] = overall.resi.hat
    }
    output$estimates = overall.resi.hat
    output$overall = overall.tab
    names.est = "Overall"
    names(output$estimates) = names.est
  } else{
    # get residual df if not computing overall test
    res.df = model.full$df.residual
  }

  # coefficients table (z or t statistics)
  if (coefficients){
    if ("coeftest_df" %in% names(dots)){
      tryCatch(coefficients.tab <- lmtest::coeftest(model.full, vcov. = vcovmat, df = dots$coeftest_df),
               error = function(e){
                 stop("Coefficients table failed, try running with coefficients = FALSE")})
      } else{
    tryCatch(coefficients.tab <- lmtest::coeftest(model.full, vcov. = vcovmat), error = function(e){
      stop("Overall Wald test failed, try running with overall = FALSE")})}

    torZ = ifelse("z value" %in% colnames(coefficients.tab), "z", "t")
    coefficients.df = data.frame(coefficients.tab[,"Estimate"], coefficients.tab[,"Std. Error"],
                                 coefficients.tab[,paste(torZ, "value")],
                                 coefficients.tab[,paste("Pr(>|", torZ, "|)", sep = "")],
                                 row.names = rownames(coefficients.tab))
    colnames(coefficients.df) = colnames(coefficients.tab)

    if (torZ == "z"){
      coefficients.df[,"RESI"] = suppressWarnings(z2S(coefficients.df[,3], nrow(data), unbiased))
    } else{
      coefficients.df[,"RESI"] = suppressWarnings(t2S(coefficients.df[,"t value"],
                                                      res.df, nrow(data), unbiased))
    }
    output$coefficients = coefficients.df
    output$estimates = c(output$estimates, coefficients.df$RESI)
    names.est = c(names.est, rownames(coefficients.df))
    names(output$estimates) = names.est
  }

  if (anova){
    if ("anovaF" %in% names(dots)){
      tryCatch(suppressMessages(anova.tab <- do.call(car::Anova, c(list(mod = model.full,
                                              vcov. = vcovmat), Anova.args))),
               error = function(e){
                 stop("car::Anova failed. Try rerunning with anova = FALSE")})
      anova.tab = anova.tab[which(rownames(anova.tab) != "Residuals"),]
      anova.tab[,"RESI"] = f2S(anova.tab[,"F"], anova.tab[,"Df"], res.df, nrow(data))
    } else {
      # For most model types, pass test.statistic = "Wald" explicitly.
      # For model types where car::Anova uses chi-square by default and does not
      # accept test.statistic = "Wald" (e.g. lmrob), the caller can pass
      # anovaChisq = TRUE via dots to skip that override.
      test_stat_arg <- if ("anovaChisq" %in% names(dots)) list() else list(test.statistic = "Wald")
      tryCatch(suppressMessages(anova.tab <- do.call(car::Anova, c(list(mod = model.full,
                                             vcov. = vcovmat), test_stat_arg, Anova.args))),
               error = function(e){
          stop("car::Anova failed. Try rerunning with anova = FALSE")})
      anova.tab = anova.tab[which(rownames(anova.tab) != "Residuals"),]
      anova.tab[,"RESI"] = chisq2S(anova.tab[,"Chisq"], anova.tab[,"Df"], nrow(data))
    }

    output$anova = anova.tab
    names.est = names(output$estimates)
    output$estimates = c(output$estimates, anova.tab$RESI)
    names.est = c(names.est, rownames(anova.tab))
    names(output$estimates) = names.est
    class(output$anova) = c("anova_resi", class(output$anova))
  }

  if(identical(vcovfunc, stats::vcov)){
    output$naive.var = TRUE
  }
  else{
    output$naive.var = FALSE
  }

  class(output) = c("resi", "list")

  return(output)
}


#' @describeIn resi_pe RESI point estimation for generalized linear models
#' @export
resi_pe.glm = resi_pe.default

#' @describeIn resi_pe RESI point estimation for linear models
#' @export
resi_pe.lm = function(model.full, model.reduced = NULL, data, anova = TRUE,
                      coefficients = TRUE, vcovfunc = sandwich::vcovHC, Anova.args = list(),
                      vcov.args = list(), unbiased = TRUE,
                      overall = TRUE, ...){

  output = resi_pe.default(model.full = model.full, model.reduced = model.reduced,
                           data = data, anova = anova, coefficients = coefficients,
                           vcovfunc = vcovfunc, Anova.args = Anova.args,
                           vcov.args = vcov.args, unbiased = unbiased,
                           overall = overall, waldtype = 1, anovaF = TRUE, ...)
  return(output)
}



#' @describeIn resi_pe RESI point estimation for nonlinear least squares models
#' @export
resi_pe.nls = function(model.full, model.reduced = NULL, data, coefficients = TRUE,
                       anova = FALSE, vcovfunc = r_nlshc, vcov.args = list(),
                       unbiased = TRUE, overall = TRUE, ...){
  if (missing(data) | is.null(data)){
    stop("\nData argument is required for nls model")
  }
  data = as.data.frame(data)

  # currently not accepting other reduced models for nls
  if (!is.null(model.reduced)){
    warning("Reduced model argument ignored for nls model")
  }

  if (identical(vcovfunc, sandwich::vcovHC)){
    vcovfunc = r_nlshc
    warning("Sandwich vcov function not applicable for nls model type, vcovfunc set to regtools::nlshc")
  }

  output = resi_pe.default(model.full = model.full, model.reduced = NULL, data = data,
                           anova = FALSE, coefficients = coefficients, vcovfunc = vcovfunc,
                           vcov.args = vcov.args, unbiased = unbiased, overall = overall,
                           waldtype = 2, skip.red.pe = TRUE, ...)

  return(output)
}

#' @describeIn resi_pe RESI point estimation for survreg
#' @export
resi_pe.survreg = function(model.full, model.reduced = NULL, data, anova = TRUE,
                           coefficients = TRUE, vcovfunc = vcov, Anova.args = list(),
                           unbiased = TRUE, overall = TRUE, ...){

  if (missing(data)){
    stop("\nData argument is required for survreg model")
  }
  else{
    data = as.data.frame(data)
  }

  if (!identical(vcovfunc, vcov)){
    warning("vcovfunc argument ignored for survreg objects")
  }

  output = resi_pe.default(model.full = model.full, model.reduced = model.reduced,
                           data = data, anova = anova, coefficients = coefficients,
                           vcovfunc = vcov, Anova.args = Anova.args, unbiased = unbiased,
                           overall = overall, waldtype = 0, ...)

  if(is.null(model.full$naive.var)){
    output$naive.var = TRUE
  }
  else{
    output$naive.var = FALSE
  }

  return(output)
}

#' @describeIn resi_pe RESI point estimation for coxph models
#' @export
resi_pe.coxph = function(model.full, model.reduced = NULL, data, anova = TRUE,
                         coefficients = TRUE, vcovfunc = vcov, Anova.args = list(),
                         unbiased = TRUE, overall = TRUE, ...){
  if (missing(data)){
    stop("\nData argument is required for coxph model")
  }
  else{
    data = as.data.frame(data)
  }

  if (!identical(vcovfunc, vcov)){
    warning("vcovfunc argument ignored for coxph objects")
  }

  # currently not accepting other reduced models for coxph
  if (!is.null(model.reduced)){
    warning("Reduced model argument ignored for coxph model")
  }

  output = resi_pe.default(model.full = model.full, model.reduced = NULL,
                           data = data, anova = anova, coefficients = coefficients,
                           vcovfunc = vcov, Anova.args = Anova.args, unbiased = unbiased,
                           overall = overall, waldtype = 2, ...)

  if(is.null(model.full$naive.var)){
    output$naive.var = TRUE
  }
  else{
    output$naive.var = FALSE
  }

  return(output)
}

#' @describeIn resi_pe RESI point estimation for hurdle models
#' @export
resi_pe.hurdle = function(model.full, model.reduced = NULL, data, coefficients = TRUE,
                          anova = TRUE, vcovfunc = sandwich::sandwich, vcov.args = list(),
                          unbiased = TRUE, overall = TRUE, ...){

  if (identical(vcovfunc, sandwich::vcovHC)){
    vcovfunc = sandwich::sandwich
  }

  output = resi_pe.default(model.full = model.full, model.reduced = model.reduced,
                           data = data, anova = FALSE, coefficients = coefficients,
                           vcovfunc = vcovfunc, vcov.args = vcov.args, unbiased = unbiased,
                           overall = overall, waldtype = 0,
                           coeftest_df = 0, ...)

  return(output)
}

#' @describeIn resi_pe RESI point estimation for zeroinfl models
#' @export
resi_pe.zeroinfl = resi_pe.hurdle

#' @describeIn resi_pe RESI point estimation for lmrob objects (robustbase package)
#' @export
resi_pe.lmrob <- function(model.full, model.reduced = NULL, data, anova = TRUE,
                          coefficients = TRUE, vcovfunc = stats::vcov,
                          Anova.args = list(), vcov.args = list(),
                          unbiased = TRUE, overall = TRUE, ...) {
  # lmrob (robustbase) has a built-in robust sandwich variance estimator
  # returned by stats::vcov(), so that is the natural default. sandwich::vcovHC
  # also works but is redundant. car::Anova() for lmrob returns chi-square
  # statistics, so we use waldtype = 0 (chi-square Wald test).
  output <- resi_pe.default(
    model.full   = model.full,
    model.reduced = model.reduced,
    data         = data,
    anova        = anova,
    coefficients = coefficients,
    vcovfunc     = vcovfunc,
    Anova.args   = Anova.args,
    vcov.args    = vcov.args,
    unbiased     = unbiased,
    overall      = overall,
    waldtype     = 0, anovaChisq = TRUE, ...)
  return(output)
}

#' @describeIn resi_pe RESI point estimation for glmrob objects (robustbase package)
#' @export
resi_pe.glmrob <- function(model.full, model.reduced = NULL, data, anova = TRUE,
                           coefficients = TRUE, vcovfunc = stats::vcov,
                           Anova.args = list(), vcov.args = list(),
                           unbiased = TRUE, overall = TRUE, ...) {
  # glmrob (robustbase) has a built-in robust sandwich variance estimator.
  # sandwich::vcovHC does not support glmrob; redirect to stats::vcov.
  if (identical(vcovfunc, sandwich::vcovHC)) {
    vcovfunc <- stats::vcov
    warning("sandwich::vcovHC is not supported for glmrob objects. ",
            "Using the model's built-in robust variance (stats::vcov) instead.")
  }
  output <- resi_pe.default(
    model.full   = model.full,
    model.reduced = model.reduced,
    data         = data,
    anova        = anova,
    coefficients = coefficients,
    vcovfunc     = vcovfunc,
    Anova.args   = Anova.args,
    vcov.args    = vcov.args,
    unbiased     = unbiased,
    overall      = overall,
    waldtype     = 0, ...)
  return(output)
}

#' @describeIn resi_pe RESI point estimation for geeglm object
#' @export
resi_pe.geeglm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
                           coefficients = TRUE, overall = TRUE, unbiased = TRUE, ...){

  if (missing(data)){
    data = model.full$data
  }

  # sample size
  N = length(summary(model.full)$clusz)
  # total num of observations
  tot_obs = nrow(data)
  # Get the rownames of observations actually used in model fitting
  # This excludes any rows with missing values automatically dropped by model.frame()
  used_rows <- rownames(model.frame(model.full))

  # num of observations from each subject
  n_i <- table(model.full$id)
  n_i = rep(n_i, times = n_i)

  # Assign weights: for each row, if ID is present, assign weights from model
  # If ID is NA or not used in model fitting, weight is set to NA
  # MJ: changed back to 1/n_i because weights from model need to be divided by number of measurement points for CS model
  data[used_rows, "w"] <- 1/n_i

  # sample size
  N = length(unique(model.full$id))
  # total num of observations
  tot_obs = nrow(data)

  # model form
  form = formula(model.full)
  # independence model
  # non-integer #successes in a binomial glm! occurs due to weights
  mod_indg = suppressWarnings(glm(formula = form, family = model.full$family, data = data,
                                  weights = w, contrasts = model.full$contrasts))
  mod_indg$residuals = mod_indg$residuals / sqrt(mod_indg$weights)
  # the var-cov matrix estimate from the independence model
  # Note: this is the estimate for Cov[(\hat{\beta}_ind - \beta_0)]
  cov_ind = sandwich::vcovHC(mod_indg, type = "HC0")

  # make copy of model.full, replace vbeta with independence
  mod_ind = model.full
  mod_ind$geese$vbeta = cov_ind

  output = list(model.full = list(call = model.full$call, formula = form),
                model.reduced = NULL)
  names.est = c()
  if (overall){
  # reduced model
  if (is.null(model.reduced)){
    form.reduced = as.formula(paste(format(formula(model.full)[[2]]), "~ 1"))
    model.reduced = update(model.full, formula = form.reduced, data = data)
  } else{
    form.reduced = formula(model.reduced)
  }

  # overall
  # longitudinal
  overall.tab = anova(model.full, model.reduced)
  overall.tab[,"L-RESI"] = chisq2S(overall.tab[,"X2"], overall.tab[,"Df"], N)

  output$model.reduced = list(call = model.reduced$call, formula = form.reduced)
  output$estimates = c(overall.tab[,"L-RESI"])
  output$overall = overall.tab
  names.est = c("Overall-L")
  names(output$estimates) = names.est
  }

  # longitudinal RESI
  # coefficients (z statistics)
  if (coefficients) {
    # longitudinal resi
    # geeglm uses robust estimate by default, do not need to specify vcov
    coefficients.tab <- lmtest::coeftest(model.full)
    coefficients.df = data.frame(coefficients.tab[,"Estimate"],
                                 coefficients.tab[,"Std. Error"],
                                 coefficients.tab[,"z value"],
                                 coefficients.tab[,"Pr(>|z|)"],
                                 row.names = rownames(coefficients.tab))
    colnames(coefficients.df) = colnames(coefficients.tab)
    coefficients.df[,"L-RESI"] = z2S(coefficients.df[,"z value"], N, unbiased)

    # CS RESI
    # M: use independence mod
    coefficients.tabcs <- lmtest::coeftest(mod_ind, vcov. = cov_ind)
    z_cs = coefficients.tabcs[,"z value"]
    coefficients.df[,"CS-RESI"] = z2S(z_cs, N, unbiased)

    output$coefficients = coefficients.df
    output$estimates = c(output$estimates, coefficients.df$`L-RESI`,
                         coefficients.df[,"CS-RESI"])
    names.est = c(names.est, rep(rownames(coefficients.df), 2))
    names(output$estimates) = names.est
  }

  # anova
  if (anova) {
    # longitudinal RESI
    anova.tab <- anova(model.full)
    anova.tab[,"L-RESI"] = chisq2S(anova.tab[,"X2"], anova.tab[,"Df"], N)

    # CS RESI
    # M: use anova() on new mod with substituted variance
    anova.tabcs <- suppressMessages(car::Anova(mod_indg, vcov. = cov_ind,
                                               test.statistic = "Wald"))
    anova.tab[,"CS-RESI"] = chisq2S(anova.tabcs[,"Chisq"], anova.tabcs[,"Df"], N)

    output$anova = anova.tab
    output$estimates = c(output$estimates, anova.tab$`L-RESI`,
                         anova.tab[,"CS-RESI"])
    names.est = c(names.est, rep(rownames(anova.tab), 2))
    names(output$estimates) = names.est
    class(output$anova) = c("anova_resi", class(output$anova))
  }

  output$naive.var = FALSE
  class(output) = c("resi", "list")
  return(output)
}


#' @describeIn resi_pe RESI point estimation for glmgee object
#' @export
resi_pe.glmgee <- function(model.full, model.reduced = NULL, data, anova = TRUE,
                           coefficients = TRUE, overall = TRUE, unbiased = TRUE, ...){
  if (missing(data)){
    data = model.full$data
  }

  # sample size
  N = model.full$clusters[1]
  # total num of observations
  tot_obs = nrow(data)

  # num of observations from each subject
  n_i = table(model.full$id)
  n_i = rep(n_i, times = n_i)
  # weight in independent model
  data$w = 1 / n_i

  # model form
  form = formula(model.full)

  # the var-cov matrix estimate from the independence model
  # Note: this is the estimate for Cov[(\hat{\beta}_ind - \beta_0)]
  # non-integer #successes in a binomial glm! occurs due to weights
  mod_indg = suppressWarnings(glm(formula = form, family = model.full$family, data = data,
                 weights = w, contrasts = model.full$contrasts))
  mod_indg$residuals = mod_indg$residuals / sqrt(mod_indg$weights)
  cov_ind = sandwich::vcovHC(mod_indg, type = "HC0")


  # make copy of model.full, replace vbeta with independence
  mod_ind = model.full
  mod_ind$R = cov_ind

  output = list(model.full = list(call = model.full$call, formula = form),
                model.reduced = NULL)
  names.est = c()
  if (overall){
    # reduced model
    if (is.null(model.reduced)){
      form.reduced = as.formula(paste(format(formula(model.full)[[2]]), "~ 1"))
      model.reduced = update(model.full, formula = form.reduced, data = data)
    } else{
      form.reduced = formula(model.reduced)
    }

    # overall
    # longitudinal
    overall.tab = as.data.frame(anova(model.reduced, model.full, verbose=FALSE))
    overall.tab[,"L-RESI"] = chisq2S(overall.tab[," Chi  "], overall.tab[," df"], N)

    output$model.reduced = list(call = model.reduced$call, formula = form.reduced)
    output$estimates = c(overall.tab[,"L-RESI"])
    output$overall = overall.tab
    names.est = c("Overall-L")
    names(output$estimates) = names.est
  }

  # longitudinal RESI
  # coefficients (z statistics)
  if (coefficients) {
    # longitudinal resi
    # geeglm uses robust estimate by default, do not need to specify vcov
    coefficients.tab <- lmtest::coeftest(model.full, df=Inf)
    coefficients.df = data.frame(coefficients.tab[,"Estimate"],
                                 coefficients.tab[,"Std. Error"],
                                 coefficients.tab[,"z value"],
                                 coefficients.tab[,"Pr(>|z|)"],
                                 row.names = rownames(coefficients.tab))
    colnames(coefficients.df) = colnames(coefficients.tab)
    # glmgee uses t-statistic and t- reference distribution
    coefficients.df[,"L-RESI"] = z2S(coefficients.df[,"z value"], n=N, unbiased)

    # CS RESI
    # M: use independence mod
    coefficients.tabcs <- lmtest::coeftest(mod_ind, vcov. = cov_ind, df=Inf)
    coefficients.df[,"CS-RESI"] = z2S(coefficients.tabcs[,"z value"], n=N, unbiased)

    output$coefficients = coefficients.df
    output$estimates = c(output$estimates, coefficients.df$`L-RESI`,
                         coefficients.df[,"CS-RESI"])
    names.est = c(names.est, rep(rownames(coefficients.df), 2))
    names(output$estimates) = names.est
  }

  # anova
  if (anova) {
    warning('anova not currently supported for glmgee class.')
  #   # longitudinal RESI
  #   anova.tab <- as.data.frame(anova(model.full, verbose=FALSE))
  #   anova.tab[,"L-RESI"] = chisq2S(anova.tab[," Chi  "], anova.tab[," df"], N)
  #
  #   # CS RESI
  #   # M: use anova() on new mod with substituted variance
  #   # anova.tabcs <- as.data.frame(anova(mod_ind, verbose=FALSE))
  #   # anova.tab[,"CS-RESI"] = chisq2S(anova.tabcs[," Chi  "], anova.tabcs[," df"], N)
  #   #
  #   output$anova = anova.tab
  #   output$estimates = c(output$estimates, anova.tab$`L-RESI`)
  #   names.est = c(names.est, rep(rownames(anova.tab), 1))
  #   # output$estimates = c(output$estimates, anova.tab$`L-RESI`,
  #   #                      anova.tab[,"CS-RESI"])
  #   # names.est = c(names.est, rep(rownames(anova.tab), 2))
  #   names(output$estimates) = names.est
  #   class(output$anova) = c("anova_resi", class(output$anova))
  }

  output$naive.var = FALSE
  class(output) = c("resi", "list")
  return(output)
}

#' @describeIn resi_pe RESI point estimation for gee object
#' @export
resi_pe.gee <- function(model.full, data, unbiased = TRUE, ...){
  # sample size
  N = length(unique(model.full$id))
  # total num of observations
  tot_obs = nrow(data)

  # num of observations from each subject
  n_i = table(model.full$id)
  n_i = rep(n_i, times = n_i)
  # weight in independent model
  w = 1 / n_i
  data$w = w

  mod_indg = suppressWarnings(glm(formula = formula(model.full), family = model.full$family, data = data,
                                  contrasts = model.full$contrasts, weights = w))
  mod_indg$residuals = mod_indg$residuals / sqrt(mod_indg$weights)
  # the var-cov matrix estimate from the independence model
  # Note: this is the estimate for Cov[(\hat{\beta}_ind - \beta_0)]
  cov_ind = sandwich::vcovHC(mod_indg, type = "HC0")
  # cov_ind = cov_ind * tot_obs/N
  # make copy of full model and substitute independence variance
  mod_ind = model.full
  mod_ind$robust.variance = cov_ind

  output <- list(model.full = list(call = model.full$call, formula = formula(model.full)),
                 estimates = c())
  names.est = c()



  # longitudinal RESI
  # coefficients (z statistics)
  # uses different function to calculate than other classes
  coefficients.tab <- summary(model.full)$coefficients
  coefficients.df = data.frame(coefficients.tab[,'Estimate'],
                               coefficients.tab[,'Robust S.E.'],
                               coefficients.tab[,'Robust z'],
                               row.names = rownames(coefficients.tab))
  colnames(coefficients.df) = c('Estimate', 'Std. Error', 'z value')
  coefficients.df[,'L-RESI'] = z2S(coefficients.df[,'z value'], N, unbiased)

  # CS RESI
  coefficients.tabcs = summary(mod_ind)$coefficients
  z_cs = coefficients.tabcs[,'Robust z']
  coefficients.df[,'CS-RESI'] = z2S(z_cs, N, unbiased)

  output$coefficients = coefficients.df
  output$estimates = c(coefficients.df$`L-RESI`, coefficients.df[,"CS-RESI"])
  names.est = rep(rownames(coefficients.df),2)
  names(output$estimates) = names.est

  output$naive.var = FALSE
  class(output) = c("resi", "list")
  return(output)
}

#' @describeIn resi_pe RESI point estimation for lme object
#' @importFrom clubSandwich vcovCR
#' @export
resi_pe.lme <- function(model.full, anova = TRUE, vcovfunc = clubSandwich::vcovCR,
                        Anova.args = list(), vcov.args = list(), ...){
  x = as.matrix(summary(model.full)$tTable)
  #sample size
  N = summary(model.full)$dims$ngrps[1]
  # robust se
  if (length(vcov.args) == 0){
    vcov.args = c(formals(vcovfunc)[1], list(type = "CR3"))
  } else{
    vcov.args = c(formals(vcovfunc)[1], vcov.args)
    if (!("type" %in% names(vcov.args))){
      vcov.args = c(vcov.args, list(type = "CR3"))
    }}
  vcovfunc2 <- function(x) {
    vcov.args[[1]] = x
    do.call(vcovfunc, vcov.args)}

  robust.var = diag(vcovfunc2(model.full))
  robust.se = sqrt(robust.var)
  coefficients.df = as.data.frame(cbind(x, 'Robust.SE' = robust.se,
                                        'Robust Wald' = (x[, 'Value']^2/robust.var),
                                        RESI = RESI::chisq2S(x[, 'Value']^2/robust.var, 1, N)))
  output = list(model.full = list(call = model.full$call, formula = formula(model.full)),
                estimates = coefficients.df$RESI, coefficients = coefficients.df)
  names(output$estimates) = rownames(coefficients.df)

  # Anova table (Chi sq statistics)
  if (anova){
    suppressMessages(anova.tab <- do.call(car::Anova, c(list(mod = model.full, vcov. = vcovfunc2), Anova.args)))
    anova.tab[,'RESI'] = chisq2S(anova.tab[,'Chisq'], anova.tab[,'Df'], N)
    names.est = names(output$estimates)
    output$estimates = c(output$estimates, anova.tab$RESI)
    names.est = c(names.est, rownames(anova.tab))
    names(output$estimates) = names.est
    output$anova = anova.tab
    class(output$anova) = c("anova_resi", class(output$anova))
  }
  if(identical(vcovfunc, stats::vcov)){
    output$naive.var = TRUE
  }
  else{
    output$naive.var = FALSE
  }
  class(output) = c("resi", "list")
  return(output)
}

#' @describeIn resi_pe RESI point estimation for lmerMod object
#' @export
resi_pe.lmerMod <- function(model.full, anova = TRUE, vcovfunc = clubSandwich::vcovCR,
                            Anova.args = list(), vcov.args = list(), unbiased = TRUE, ...){
  x = as.matrix(summary(model.full)$coefficients)
  # sample size (number of clusters from primary grouping factor)
  N = summary(model.full)$ngrps[[1]]

  # CS-RESI: extract primary cluster variable and compute per-observation weights
  id_var <- names(lme4::ranef(model.full))[[1]]
  data_mf <- as.data.frame(model.frame(model.full))
  n_i <- table(data_mf[[id_var]])
  data_mf$w_cs <- 1 / as.numeric(n_i[match(as.character(data_mf[[id_var]]), names(n_i))])
  # weighted independence lm (fixed effects only, random effects stripped)
  fix_form <- lme4::nobars(formula(model.full))
  mod_ind_lm <- lm(fix_form, data = data_mf, weights = w_cs)
  # rescale residuals to undo the weight transformation (mirrors geeglm CS approach)
  mod_ind_lm$residuals <- mod_ind_lm$residuals / sqrt(mod_ind_lm$weights)
  # HC0 sandwich variance for CS model
  cov_ind <- sandwich::vcovHC(mod_ind_lm, type = "HC0")
  cs_coef_tab <- lmtest::coeftest(mod_ind_lm, vcov. = cov_ind)

  # robust se (L-RESI)
  if (identical(vcovfunc, stats::vcov)) {
    coefficients.tab = cbind(x, 'Wald' = x[, 't value']^2,
                             'L-RESI' = RESI::chisq2S(x[, 't value']^2, 1, N),
                             'CS-RESI' = z2S(cs_coef_tab[, 't value'], N, unbiased))
    output = list(estimates = coefficients.tab[,"L-RESI"], coefficients = coefficients.tab, naive.var = TRUE)
    vcovfunc2 = vcovfunc
  } else {
    if (length(vcov.args) == 0){
      if (identical(vcovfunc, clubSandwich::vcovCR)){
        vcov.args = c(formals(vcovfunc)[1], list(type = "CR3"))
      }} else{
        if (identical(vcovfunc, clubSandwich::vcovCR)){
          if (!("type" %in% names(vcov.args))){
            vcov.args = c(vcov.args, list(type = "CR3"))}}
        vcov.args = c(formals(vcovfunc)[1], vcov.args)}

    vcovfunc2 <- function(x) {
      vcov.args[[1]] = x
      do.call(vcovfunc, vcov.args)}
    robust_var = diag(vcovfunc2(model.full))
    robust_se = sqrt(robust_var)
    coefficients.tab = cbind(x, 'Robust.SE' = robust_se,
                             'Robust Wald' = (x[, 'Estimate']^2/robust_var),
                             'L-RESI' = RESI::chisq2S(x[, 'Estimate']^2/robust_var, 1, N),
                             'CS-RESI' = z2S(cs_coef_tab[, 't value'], N, unbiased))
    output = list(estimates = coefficients.tab[,"L-RESI"], coefficients = coefficients.tab,
                  naive.var = FALSE)
  }
  names(output$estimates) = rownames(coefficients.tab)
  # Anova table (Chi sq statistics)
  if (anova){
    suppressMessages(anova.tab <- do.call(car::Anova,
                                          c(list(mod = model.full, vcov. = vcovfunc2), Anova.args)))
    anova.tab[,'L-RESI'] = chisq2S(anova.tab[,'Chisq'], anova.tab[,'Df'], N)
    # CS-RESI anova table: car::Anova on lm returns F-stats; F * Df = Wald Chisq
    anova.tabcs <- suppressMessages(car::Anova(mod_ind_lm, vcov. = cov_ind))
    anova.tabcs <- anova.tabcs[!is.na(anova.tabcs[, "F"]), , drop = FALSE]
    anova.tab[,'CS-RESI'] = chisq2S(anova.tabcs[,'F'] * anova.tabcs[,'Df'], anova.tabcs[,'Df'], N)
    names.est = names(output$estimates)
    output$estimates = c(output$estimates, anova.tab$`L-RESI`, anova.tab$`CS-RESI`)
    names.est = c(names.est, rownames(anova.tab), rownames(anova.tab))
    names(output$estimates) = names.est
    output$anova = anova.tab
    class(output$anova) = c("anova_resi", class(output$anova))
  }
  output$model.full = list(call = model.full@call, formula = formula(model.full))
  class(output) = c("resi", "list")
  return(output)
}


#' @describeIn resi_pe RESI point estimation for glmmTMB object - Gaussian only
#' @export
resi_pe.glmmTMB <- function(model.full, anova = TRUE, vcovfunc = clubSandwich::vcovCR,
                            Anova.args = list(), vcov.args = list(), ...){
  # only support Gaussian family at the moment (similar to lmer but not glmer)
  fam <- model.full$modelInfo$family$family
    if (fam != "gaussian") {
      stop("Only Gaussian GLMM supported at this time.")
    }

  x = as.matrix(summary(model.full)$coefficients$cond)
  #sample size
  N = summary(model.full)$ngrps$cond

  # robust se not currently available for GLMM
  if(!identical(vcovfunc, stats::vcov)){
    warning("Robust standard errors not implemented for glmmTMB. Consider using lme4 or nlme if needed.")
  }

  coefficients.tab = cbind(x, 'Wald' = x[, 'z value']^2, RESI = RESI::chisq2S(x[, 'z value']^2, 1, N))
  output = list(estimates = coefficients.tab[,"RESI"], coefficients = coefficients.tab, naive.var = TRUE)

  names(output$estimates) = rownames(coefficients.tab)
  # Anova table (Chi sq statistics)
  if (anova){
    suppressMessages(anova.tab <- do.call(car::Anova,
                                          c(list(mod = model.full), Anova.args)))
    anova.tab[,'RESI'] = chisq2S(anova.tab[,'Chisq'], anova.tab[,'Df'], N)
    names.est = names(output$estimates)
    output$estimates = c(output$estimates, anova.tab$RESI)
    names.est = c(names.est, rownames(anova.tab))
    names(output$estimates) = names.est
    output$anova = anova.tab
    class(output$anova) = c("anova_resi", class(output$anova))
  }
  output$model.full = list(call = model.full$call, formula = formula(model.full))
  class(output) = c("resi", "list")
  return(output)
}

#' @param object emmGrid object
#' @param model model used to generate emmeans
#' @param N sample size
#' @describeIn resi_pe RESI point estimation for emmeans object
#' @export
resi_pe.emmGrid <- function(object, model, N = NULL, unbiased = TRUE, ...) {

  if (!inherits(object, "emmGrid")) stop("object must be an emmGrid from emmeans")
  if (missing(model)) stop("You must supply the model used to generate emmeans")

  # Extract info
  beta_hat <- object@bhat
  Sigma_hat <- object@V
  linfct <- object@linfct


  # Use sample size if not given
  if (is.null(N)) {
    if (!is.null(model$model)) {
      N <- length(unique(model$model[[1]]))  # fallback to number of clusters or subjects
    } else {
      N <- nobs(model)  # for lm
    }
  }

  # Compute RESI for each linear combination
  resi_values <- apply(linfct, 1, function(l) {
    est <- as.numeric(t(l) %*% beta_hat)
    var <- as.numeric(t(l) %*% Sigma_hat %*% l)
    z <- est / sqrt(var)
    RESI::z2S(z, n = N, unbiased = unbiased)
  })

  # Attach to emmeans result
  result_df <- as.data.frame(object)
  result_df$RESI <- resi_values

  return(result_df)
}

#' Covert R^2 to S
#'
#' Converts R^2, the partial coefficient of determination, to
#' robust effect size index (S) using the formula from Vandekar, Tao, & Blume (2020).
#' @param Rsq Numeric, R^2
#' @return Returns an estimate of R^2 based on the RESI
#' @details The formula for the conversion is:
#'
#' \eqn{S = \sqrt((-R^2)/(R^2 - 1))}
#' @examples
#' # consider a moderate effect size of R^2 = 0.1
#' Rsq2S(0.1)
#' # this corresponds to a RESI of 0.333
#' @export
Rsq2S <- function(Rsq){
  if (any(Rsq < 0 | Rsq > 1)){
    stop("\nRsq must be between 0 and 1")
  }
  S = c()
  for (i in 1:length(Rsq)){
    if (Rsq[i] == 1){
      S[i] = Inf
    } else{
      S[i] = sqrt((-Rsq[i])/(Rsq[i]-1))
    }
  }
  return(S)
}

#' Summary method for resi objects
#'
#' After running the \code{\link{resi}} function on a fitted model, this function can
#' be used to print the coefficients table component. If the resi function was run with
#' the `store.boot = TRUE` option to store the full matrix of bootstrapped estimates,
#' the user can specify a different alpha level for this function's confidence intervals.
#' @param object an object resulting from resi function
#' @param alpha an optional new specification for the confidence level. Can be vector-valued
#' @param ... ignored
#' @return Returns a `summary_resi` object containing the computed coefficients table
#' @examples
#' # fit a model
#' mod = lm(charges ~ bmi + sex, data = RESI::insurance)
#'
#' # run resi with the store.boot = TRUE option
#' resi_obj = resi(mod, nboot = 100, store.boot = TRUE, alpha = 0.01)
#'
#' # run summary, specifying a different alpha level if desired
#' summary(resi_obj, alpha = 0.05)
#' @export
summary.resi <- function(object, alpha = NULL, ...){
  if(is.null(object$coefficients)){
    stop("\nresi function was not run with coefficients = TRUE option")
  }

  output = list(alpha = alpha, model.full = object$model.full)
  if (is.null(alpha)){
    output$alpha = object$alpha
    output$coefficients = object$coefficients
  }
  else{
    if (!(all(alpha %in% object$alpha))){
      if (!is.null(object$ci.method) && object$ci.method != "boot" &&
          !is.null(object$model.full)) {
        # Asymptotic (qf/normal): recompute CIs analytically at the new alpha(s).
        output$coefficients <- object$coefficients[, 1:(which(colnames(object$coefficients) == "RESI")),
                                                   drop = FALSE]
        for (a in alpha) {
          asym_out <- tryCatch(
            resi_pe_asymptotic(
              model.full   = object$model.full,
              alpha        = a,
              ci.method    = object$ci.method,
              vcovfunc     = if (!is.null(object$vcovfunc)) object$vcovfunc else sandwich::vcovHC,
              vcov.args    = if (!is.null(object$vcov.args)) object$vcov.args else list(),
              Anova.args   = if (!is.null(object$Anova.args)) object$Anova.args else list(),
              unbiased     = if (!is.null(object$unbiased)) object$unbiased else TRUE,
              coefficients = TRUE,
              anova        = FALSE
            ),
            error = function(e) {
              stop(paste0("\nFailed to recompute asymptotic CIs at alpha = ", a,
                          ":\n", conditionMessage(e)))
            }
          )
          ci_cols <- paste0(c(a / 2, 1 - a / 2) * 100, "%")
          rn_match <- intersect(rownames(output$coefficients), rownames(asym_out$coefficients))
          output$coefficients[rn_match, ci_cols] <- asym_out$coefficients[rn_match, ci_cols]
        }
        class(output) <- "summary_resi"
        return(output)
      } else if (is.null(object$boot.results)) {
        # No asymptotic recomputation possible and no stored bootstrap results.
        stop(paste0(
          "\nCannot compute CIs at a different alpha level. Either:\n",
          "  (a) bootstrapping is not yet supported for this model type, or\n",
          "  (b) resi() was not run with store.boot = TRUE.\n",
          "Re-run resi() with the desired alpha value directly."
        ))
      }
    }
    if(is.null(object$boot.results)){
      output$coefficients = object$coefficients[c(1:(which(colnames(object$coefficients)
                                                           == "RESI")),
                                                  which(colnames(object$coefficients)%in%
                                                          c(paste(alpha/2*100, "%", sep=""),
                                                            paste((1-alpha/2)*100, "%", sep=""))))]
    }
    else{
      output$coefficients = object$coefficients[,1:(which(colnames(object$coefficients) == "RESI"))]
      boot.results = object$boot.results$t
      CIs = apply(boot.results[,2:(1+nrow(object$coefficients))], 2,
                  quantile, probs = sort(c(alpha/2, 1-alpha/2)), na.rm = TRUE)
      CIs = t(CIs)
      output$coefficients[1:nrow(CIs), c(paste(alpha/2*100, "%", sep=""),
                                         paste((1-rev(alpha)/2)*100, "%", sep=""))] = CIs
    }
  }
  class(output) = c("summary_resi")
  output
}

# Internal functions

#' Transformation from Wald test statistics to squared RESI
#' @param chisq The chi-square statistic for the parameter(s) of interest.
#' @param df Number of degrees of freedom of the chi-square statistic.
#' @param n Number of independent samples.
#' @return Returns a scalar or vector argument of the squared robust effect size index estimate.
#' @noRd
chisq2Ssq = function(chisq, df, n){
  S = (chisq - df)/n
}

#' Statistic for bootstrapping RESI estimates with boot package
#'
#' @param dat data.frame; original dataset
#' @param inds bootstrapped indices
#' @param mod.full full model passed to resi
#' @param mod.reduced reduced model passed to resi
#' @param boot.method non-parametric (default) or Bayesian bootstrap
#' @return Returns a vector of RESI estimates for a single bootstrap replicate
#' @noRd
resi_stat = function(dat, inds, mod.full, mod.reduced, boot.method = "nonparam",
                     cluster = FALSE, clvar = NULL, mod.dat = NULL, overall = TRUE,
                     nest = NULL, ...){
  # nonparametric bootstrap
  if (boot.method == "nonparam"){
    if(!cluster){
      boot.data = as.data.frame(dat[inds,])
      colnames(boot.data) = colnames(dat)
      mod.full = try(update(mod.full, data = boot.data), silent = T)
      if (!(is.null(mod.reduced)) & overall){
        mod.reduced = try(update(mod.reduced, data = boot.data), silent = T)}
    }
     else{
      # for clustered data, dat is unique ids and inds is which of the ids
      boot.data = mod.dat[unlist(lapply(dat[inds], function(x) which(x == mod.dat[, clvar]))), ]
      bootid = rep(1:length(unique(unlist(mod.dat[, clvar]))),
                    unlist(lapply(dat[inds], function(x) length(which(x==mod.dat[,clvar])))))
      boot.data$bootid = bootid

      mod.full = try(update(mod.full, data = boot.data, id = bootid), silent = T)
      if (!(is.null(mod.reduced)) & overall){
        mod.reduced = try(update(mod.reduced, data = boot.data, id = bootid), silent = T)}
    }}

  else{
    # Bayesian bootstrap
    n = nrow(dat)
    repeat{
      # Generate the random numbers from unif(0, 1)
      u = runif(n-1)
      u.sort = sort(u)
      g = c(u.sort, 1) - c(0, u.sort)
      if (sum(g == 0) == 0) break
    } # end `repeat`
    boot.data = cbind(dat, g)
    mod.full = try(update(mod.full, data = boot.data, weights = g), silent = T)
    if (!(is.null(mod.reduced)) & overall){
      mod.reduced = try(update(mod.reduced, data = boot.data, weights = g), silent = T)}
  }

  if(!(inherits(mod.full, "try-error") | (inherits(mod.reduced, "try-error") & overall))){
    out = try(resi_pe(mod.full, model.reduced = mod.reduced, data = boot.data, overall = overall, ...)$estimates, silent = T)
    if(inherits(out, "try-error")){
      out = NA
    } else{
      # for factored variables, it's possible the model will not contain all factor levels and not represent the original model
      if(length(out) != nest){
        out = NA
      }
    }
  } else{
    out = NA
  }

  # for models with fail checking, count nboot-nrow(t0) for number of failures
  return(out)
}

# function taken from regtools::nlshc (removing dependency that was causing error)
#' Statistic for bootstrapping RESI estimates with boot package
#'
#' @param nlsout nls model
#' @param type for sandwich variance
#' @return Returns robust covariance matrix for nls model
#' @noRd
r_nlshc <- function (nlsout, type = "HC")
{
  b <- coef(nlsout)
  m <- nlsout$m
  resid <- m$resid()
  hmat <- m$gradient()
  xhm <- hmat
  yresidhm <- resid + hmat %*% b
  lmout <- lm(yresidhm ~ xhm - 1)
  sandwich::vcovHC(lmout, type)
}

#' Anova method for resi objects
#'
#' After running the \code{\link{resi}} function on a fitted model, this function can be used to print the Anova-style table component. If the resi function was run with the `store.boot = TRUE` option to store the full matrix of bootstrapped estimates, the user can specify a different alpha level for this function's confidence intervals.
#' @param mod an object resulting from resi function
#' @param alpha an optional new specification for the confidence level. Can be vector-valued
#' @param ... ignored
#' @return Returns an `anova` object containing the computed Anova-style table
#' @examples
#' # fit a model
#' mod = lm(charges ~ bmi + sex, data = RESI::insurance)
#'
#' # run resi with the store.boot = TRUE option
#' resi.obj = resi(mod, nboot = 100, store.boot = TRUE, alpha = 0.01)
#'
#' # run Anova, specifying a different alpha level if desired
#' car::Anova(resi.obj, alpha = 0.05)
#' @noRd
#' @export
Anova.resi <- function(mod, alpha = NULL, ...){
  if(is.null(mod$anova)){
    stop('\nresi function was not run with anova = TRUE option')
  }

  if (is.null(alpha)){
    output = mod$anova
  }
  else{
    if (!(all(alpha %in% mod$alpha))){
      if (!is.null(mod$ci.method) && mod$ci.method != "boot" &&
          !is.null(mod$model.full)) {
        # Asymptotic (qf/normal): recompute anova CIs analytically.
        output <- mod$anova[, 1:(which(colnames(mod$anova) == 'RESI')), drop = FALSE]
        for (a in alpha) {
          asym_out <- tryCatch(
            resi_pe_asymptotic(
              model.full   = mod$model.full,
              alpha        = a,
              ci.method    = mod$ci.method,
              vcovfunc     = if (!is.null(mod$vcovfunc)) mod$vcovfunc else sandwich::vcovHC,
              vcov.args    = if (!is.null(mod$vcov.args)) mod$vcov.args else list(),
              Anova.args   = if (!is.null(mod$Anova.args)) mod$Anova.args else list(),
              unbiased     = if (!is.null(mod$unbiased)) mod$unbiased else TRUE,
              coefficients = FALSE,
              anova        = TRUE
            ),
            error = function(e) {
              stop(paste0("\nFailed to recompute asymptotic CIs at alpha = ", a,
                          ":\n", conditionMessage(e)))
            }
          )
          ci_cols <- paste0(c(a / 2, 1 - a / 2) * 100, '%')
          rn_match <- intersect(rownames(output), rownames(asym_out$anova))
          output[rn_match, ci_cols] <- asym_out$anova[rn_match, ci_cols]
        }
        return(output)
      } else if (is.null(mod$boot.results)) {
        stop('\nresi function was not run with store.boot = TRUE option')
      }
    }
    if(is.null(mod$boot.results)){
      output = mod$anova[c(1:(which(colnames(mod$anova) == 'RESI')),
                              which(colnames(mod$anova)%in%
                                      c(paste(alpha/2*100, '%', sep=''),
                                        paste((1-alpha/2)*100, '%', sep=''))))]
    }
    else{
      output = mod$anova[,1:(which(colnames(mod$anova) == 'RESI'))]
      boot.results = mod$boot.results$t
      CIs = apply(boot.results[,(ncol(boot.results)-
                                          nrow(mod$anova[which(rownames(mod$anova)
                                                                  != "Residuals"),])+1):
                                        ncol(boot.results)], 2,  quantile,
                  probs = sort(c(alpha/2, 1-alpha/2)), na.rm = TRUE)
      CIs = t(CIs)
      output[1:nrow(CIs), c(paste(alpha/2*100, '%', sep=''),
                            paste((1-rev(alpha)/2)*100, '%', sep=''))] = CIs
    }
  }
  return(output)
}

#' Anova method for resi objects
#'
#' After running the \code{\link{resi}} function on a fitted model, this function can be used to print the Anova-style table component. If the resi function was run with the `store.boot = TRUE` option to store the full matrix of bootstrapped estimates, the user can specify a different alpha level for this function's confidence intervals.
#' @param object an object resulting from resi function
#' @param alpha an optional new specification for the confidence level. Can be vector-valued
#' @param ... ignored
#' @return Returns an `anova` object containing the computed Anova-style table
#' @details The resi function uses the car::Anova function to compute the Anova table.
#'@examples
#' # fit a model
#' mod = lm(charges ~ bmi + sex, data = RESI::insurance)
#'
#' # run resi with the store.boot = TRUE option
#' resi.obj = resi(mod, nboot = 100, store.boot = TRUE, alpha = 0.01)
#'
#' # run anova, specifying a different alpha level if desired
#' anova(resi.obj, alpha = 0.05)
#' @export
anova.resi <- function(object, alpha = NULL, ...){
  if(is.null(object$anova)){
    stop("\nresi function was not run with anova = TRUE option")
  }

  if (is.null(alpha)){
    output = object$anova
  }
  else{
    if (!(all(alpha %in% object$alpha))){
      if (!is.null(object$ci.method) && object$ci.method != "boot" &&
          !is.null(object$model.full)) {
        # Asymptotic (qf/normal): recompute anova CIs analytically.
        output <- object$anova[, 1:(which(colnames(object$anova) == "RESI")), drop = FALSE]
        for (a in alpha) {
          asym_out <- tryCatch(
            resi_pe_asymptotic(
              model.full   = object$model.full,
              alpha        = a,
              ci.method    = object$ci.method,
              vcovfunc     = if (!is.null(object$vcovfunc)) object$vcovfunc else sandwich::vcovHC,
              vcov.args    = if (!is.null(object$vcov.args)) object$vcov.args else list(),
              Anova.args   = if (!is.null(object$Anova.args)) object$Anova.args else list(),
              unbiased     = if (!is.null(object$unbiased)) object$unbiased else TRUE,
              coefficients = FALSE,
              anova        = TRUE
            ),
            error = function(e) {
              stop(paste0("\nFailed to recompute asymptotic CIs at alpha = ", a,
                          ":\n", conditionMessage(e)))
            }
          )
          ci_cols <- paste0(c(a / 2, 1 - a / 2) * 100, "%")
          rn_match <- intersect(rownames(output), rownames(asym_out$anova))
          output[rn_match, ci_cols] <- asym_out$anova[rn_match, ci_cols]
        }
        return(output)
      } else if (is.null(object$boot.results)) {
        stop("\nresi function was not run with store.boot = TRUE option")
      }
    }
    if(is.null(object$boot.results)){
      output = object$anova[c(1:(which(colnames(object$anova) == "RESI")),
                              which(colnames(object$anova)%in%
                                      c(paste(alpha/2*100, "%", sep=""),
                                        paste((1-alpha/2)*100, "%", sep=""))))]
    }
    else{
      output = object$anova[,1:(which(colnames(object$anova) == "RESI"))]
      boot.results = object$boot.results$t
      CIs = apply(boot.results[,(ncol(boot.results)-
                                          nrow(object$anova)+1):
                                        ncol(boot.results)], 2,
                  quantile, probs = sort(c(alpha/2, 1-alpha/2)), na.rm = TRUE)
      CIs = t(CIs)
      output[1:nrow(CIs), c(paste(alpha/2*100, "%", sep=""),
                            paste((1-rev(alpha)/2)*100, "%", sep=""))] = CIs
    }
  }
  return(output)
}

#' Compute the robust effect size index estimate from chi-squared statistic.
#'
#' This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
#' Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
#' For mixed effects models, RESI is conditional on the average correlation
#' structure within subjects.
#' @param chisq The chi-square statistic for the parameter of interest.
#' @param df Number of degrees of freedom of the chi-square statistic.
#' @param n Number of independent samples.
#' @return Returns a scalar or vector argument of the robust effect size index estimate.
#' @details The formula for converting a Chi-square statistic to RESI is:
#'
#' \eqn{ S = \sqrt(max( 0, (chisq - df)/n))}
#' @examples
#' # obtain Chi-sq value by fitting an lm and running a Wald test
#' mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
#'
#' # run a Wald test with robust variance
#' wt = lmtest::waldtest(mod, vcov = sandwich::vcovHC, test = "Chisq")
#'
#' # get Chi-sq value and degrees of freedom
#' chisq = wt$Chisq[2]
#' df = abs(wt$Df[2])
#'
#' # run chisq2S to convert to RESI
#' chisq2S(chisq, df = df, n = nrow(mod$model))
#'
#' @export
chisq2S <- function(chisq, df, n){
  if (any(chisq < 0)){
    stop("\nChi-square statistic must be non-negative")
  }
  S = (chisq - df)/n
  sqrt(ifelse(S<0, 0, S))
}

#' US Health Insurance Data
#'
#' A dataset with 1338 observations on health insurance charges and demographic factors.
#'
#' @format A data frame with 1338 rows and 7 variables:
#' \describe{
#' \item{age}{age of primary beneficiary in years}
#' \item{sex}{insurance contractor sex, male/female}
#' \item{bmi}{body mass index}
#' \item{children}{number of dependents}
#' \item{smoker}{smoker/non-smoker}
#' \item{region}{beneficiary's region of US}
#' \item{charges}{individual medical costs billed by health insurance}
#' }
#'
#' @source Kaggle dataset teertha ushealthinsurancedataset. url checker thinks the link is broken.
"insurance"


#' Depression Treatment Data
#'
#' A longitudinal dataset comparing two treatments for depression.
#'
#' @format A data frame with 1020 rows and 5 variables:
#' \describe{
#' \item{diagnose}{diagnosed depression severity}
#' \item{drug}{treatment; standard or new}
#' \item{id}{patient id}
#' \item{time}{time point of treatment}
#' \item{depression}{depression response at time of treatment. 1 = Normal, 0 = Abnormal}
#' }
#'
#' @source \url{http://static.lib.virginia.edu/statlab/materials/data/depression.csv}
#' @references Agresti, A. (2002). Categorical Data Analysis. Wiley, 2nd Edition.
"depression"

#' Convert non-zero S to Chi-square statistic
#'
#' Converts the robust effect size index (S) to Chi-square statistic, given that S is greater than 0.
#' For an S value of 0, only an upper bound on the Chi-square statistic can be computed.
#' Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
#' @param S The value of the RESI estimate.
#' @param df Number of degrees of freedom of the chi-square statistic.
#' @param n Number of independent samples.
#' @return Returns a scalar or vector argument of the Chi-square statistic.
#' @details The formula for converting a RESI estimate above 0 to Chi-square statistic is:
#'
#' \eqn{ chisq = n*S^2 + df}
#'
#' If the RESI estimate is 0, all that is known is that the Chi-square statistic is less than or equal to the degrees of freedom.
#' @examples
#' # convert S estimates with corresponding degrees of freedom to Chi-square estimates
#' S_ests = c(0.2, 0.4, 0.6)
#' dfs = c(2, 1, 3)
#' S2chisq(S = S_ests, df = dfs, n = 300)
#'
#' @export
S2chisq = function(S, df, n){
  if (any(S < 0)){
    stop("RESI estimates must be non-negative for Chi-square statistics")
  }
  chisq = n*S^2 + df
  if (any(S == 0)){
    if(length(S) == 1){
      stop(paste0("Function is not 1-1 for S = 0, Chi-square is less than or equal to df (", df, ")"))
    } else{
      chisq[S == 0]= NA
      warning("Function is not 1-1 for S = 0, Chi-square is less than or equal to corresponding df")
    }
  }
  chisq
}

#' Covert S to R^2
#'
#' Converts robust effect size index (S) to R^2, the partial
#' coefficient of determination, using the formula from Vandekar, Tao, & Blume (2020).
#' @param S Numeric, the robust effect size index.
#' @details The formula for the conversion is:
#'
#' \eqn{ R^2 = S^2 / (1 + S^2)}
#' @return Returns an estimate of R^2 based on the RESI
#' @examples
#' # fit a simple linear regression with a binary predictor
#' mod = lm(charges ~ sex, data = RESI::insurance)
#'
#' # calculate t-value
#' t = summary(mod)$coefficients[2, "t value"]
#'
#' # calculate RESI (S)
#' S = t2S(t, n = 1338, rdf = 1336)
#'
#' # convert S to R^2
#' S2Rsq(S)
#'
#' @export
S2Rsq <- function(S){
  S^2/(1+S^2)
}

#' Plotting RESI Estimates and CIs
#'
#' This function uses base graphics to plot robust effect size (RESI) estimates and confidence intervals
#' from `resi`, `summary_resi`, and `anova_resi` objects.
#' @param x Object of `resi`, `summary_resi`, or `anova_resi` class
#' @param alpha Numeric, desired alpha level for confidence intervals
#' @param ycex.axis Numeric, scale specifically for the variable name labels
#' @param yaxis.args List, other arguments to be passed to \code{\link{axis}} for the y-axis
#' @param automar Logical, whether to automatically adjust the plotting margins to accommodate variable names. Default = `TRUE`
#' @param ... Other graphical parameters passed to \code{\link{plot}} and \code{\link{lines}}
#' @details This function creates a forest-like plot with RESI estimates for each variable or factor.
#' The size of the left margin will be automatically adjusted (and returned to original after plotting)
#' unless `automar = FALSE`. Additional graphics parameters will be passed to the main
#' plot function, the confidence intervals. Arguments specifically for the y-axis (variable names)
#' can be specified using `yaxis.args`. To manually adjust the size of the y-axis labels without
#' affecting the x-axis, the user can specify a value for `ycex.axis`.
#' @return Returns a plot of RESI point estimates
#' @examples
#' # create a resi object
#' resi_obj <- resi(lm(charges ~ region * age + bmi + sex, data = RESI::insurance),
#' nboot = 10)
#'
#' # plot coefficients table, changing size of labels for both axes in the usual way
#' plot(resi_obj, cex.axis = 0.7)
#'
#' # plot ANOVA table, changing the size of just the y-axis
#' plot(resi_obj, ycex.axis = 0.8)
#' @importFrom graphics abline axis lines par strwidth
#' @export
plot.resi = function(x, alpha = NULL, ycex.axis = NULL, yaxis.args = list(),
                     automar = TRUE, ...){
  dots = list(...)

  if (!(is.null(x$alpha))){
    if (is.null(alpha)){
      alpha = x$alpha[1]
    }
    else{
      if (length(alpha) > 1){
        warning("\nOnly first alpha will be plotted")
        alpha = alpha[1]
      }
      if (!(alpha %in% x$alpha)){
        if (!is.null(x$ci.method) && x$ci.method != "boot" && !is.null(x$model.full)) {
          # Asymptotic (qf/normal): recompute coefficient CIs at the new alpha.
          asym_out <- tryCatch(
            resi_pe_asymptotic(
              model.full   = x$model.full,
              alpha        = alpha,
              ci.method    = x$ci.method,
              vcovfunc     = if (!is.null(x$vcovfunc)) x$vcovfunc else sandwich::vcovHC,
              vcov.args    = if (!is.null(x$vcov.args)) x$vcov.args else list(),
              Anova.args   = if (!is.null(x$Anova.args)) x$Anova.args else list(),
              unbiased     = if (!is.null(x$unbiased)) x$unbiased else TRUE,
              coefficients = TRUE,
              anova        = FALSE
            ),
            error = function(e) {
              stop(paste0("\nFailed to recompute asymptotic CIs at alpha = ", alpha,
                          ":\n", conditionMessage(e)))
            }
          )
          ci_cols <- paste0(c(alpha / 2, 1 - alpha / 2) * 100, "%")
          rn_match <- intersect(rownames(x$coefficients), rownames(asym_out$coefficients))
          x$coefficients[rn_match, ci_cols] <- asym_out$coefficients[rn_match, ci_cols]
        } else if (is.null(x$boot.results)) {
          stop("\nSpecified alpha not found in the resi object")
        } else {
          CIs = apply(x$boot.results$t[,2:(1+nrow(x$coefficients))], 2,  quantile, probs = c(alpha/2, 1-alpha/2), na.rm = TRUE)
          CIs = t(CIs)
          x$coefficients[1:nrow(CIs), c(paste(alpha/2*100, "%", sep=""), paste((1-rev(alpha)/2)*100, "%", sep=""))] = CIs
        }
      }
    }
    ll = paste(alpha/2*100, "%", sep = "")
    ul = paste((1-alpha/2)*100, "%", sep = "")
  } else{
    ll = ul = "RESI"
  }

  vnames = rownames(x$coefficients)
  # width of labels
  if ((!(is.null(ycex.axis))) | "cex.axis" %in% names(dots)){
    if (!(is.null(ycex.axis))){
      ycex = ycex.axis
    } else{
      ycex = dots$cex.axis
    }
    w = strwidth(vnames, "inches", cex = ycex)
  } else{
    w = strwidth(vnames, "inches")
    ycex = 1
  }

  # save original margins
  omar = par("mai")
  if (automar){
    # set margins to accomodate labels
    par("mai" = omar + c(0, max(w) - omar[2] + 0.25, 0, 0))
  }

  er = 0.2 * (length(vnames)/(length(vnames) + 1))

  plot(x = x$coefficients[,"RESI"], y = length(vnames):1,
       xlim = c(min(0, min(x$coefficients[,ll])), max(x$coefficients[,ul])),
       xlab = "RESI Estimate", yaxt = "n", ylab = "",
       main = paste("Coefficient RESI Estimates and ", (1-alpha)*100, "%", " CIs", sep=""),...)
  if (!(is.null(x$alpha))){
    for (i in length(vnames):1){
      lines(x = c(x$coefficients[-1*(i-length(vnames)) + 1,ll],
                  x$coefficients[-1*(i-length(vnames)) + 1,ul]), y = c(i,i),
            ...)
      lines(x = c(x$coefficients[-1*(i-length(vnames)) + 1,ll],
                  x$coefficients[-1*(i-length(vnames)) + 1,ll]),
            y = c(i-er,i+er),
            ...)
      lines(x = c(x$coefficients[-1*(i-length(vnames)) + 1,ul],
                  x$coefficients[-1*(i-length(vnames)) + 1,ul]),
            y = c(i-er,i+er),
            ...)}

  }
  do.call(axis, c(list(side = 2, at = length(vnames):1,
                       labels = vnames, las = 1,
                       cex.axis = ycex), yaxis.args))
  abline(v = 0, lty = 2)
  # return to original margins
  par("mai" = omar)
}

#' @export
plot.summary_resi = plot.resi

#' @export
plot.anova_resi = function(x, alpha = NULL, ycex.axis = NULL, yaxis.args = list(),
                           automar = TRUE, ...){
  dots = list(...)

  cols = grep("%", colnames(x))
  if (length(cols) != 0){
    if (is.null(alpha)){
      alpha = gsub("%", "", colnames(x)[cols[1]])
      alpha = as.numeric(alpha)*2/100
    }
    else{
      if (length(alpha) > 1){
        warning("\nOnly first alpha will be plotted")
        alpha = alpha[1]
      }
    }
    ll = paste(alpha/2*100, "%", sep="")
    ul = paste((1-alpha/2)*100, "%", sep="")
    if (!(ll %in% colnames(x))){
      stop("\nSpecified alpha not found in the resi object")
    }} else{
      ll = ul = "RESI"
    }

  vnames = rownames(x)[which(rownames(x) != "Residuals")]
  # width of labels
  if ((!(is.null(ycex.axis))) | "cex.axis" %in% names(dots)){
    if (!(is.null(ycex.axis))){
      ycex = ycex.axis
    } else{
      ycex = dots$cex.axis
    }
    w = strwidth(vnames, "inches", cex = ycex)
  } else{
    w = strwidth(vnames, "inches")
    ycex = 1
  }

  # save original margins
  omar = par("mai")
  if (automar){
    # set margins to accomodate labels
    par("mai" = omar + c(0, max(w) - omar[2] + 0.25, 0, 0))
  }

  er = 0.2 * (length(vnames)/(length(vnames) + 1))

  plot(x = x[which(rownames(x) != "Residuals"), "RESI"], y = length(vnames):1,
       xlim = c(0, max(x[,ul], na.rm = TRUE)), xlab = "RESI Estimate",
       yaxt = "n", ylab = "", main = paste("Anova RESI Estimates and ",
                                           (1-alpha)*100, "%", " CIs", sep=""),...)
  if (length(cols) != 0){
    for (i in length(vnames):1){
      lines(x = c(x[-1*(i-length(vnames)) + 1,ll],
                  x[-1*(i-length(vnames)) + 1,ul]), y = c(i,i), ...)
      lines(x = c(x[-1*(i-length(vnames)) + 1,ll],
                  x[-1*(i-length(vnames)) + 1,ll]),
            y = c(i- er,i+er), ...)
      lines(x = c(x[-1*(i-length(vnames)) + 1,ul],
                  x[-1*(i-length(vnames)) + 1,ul]),
            y = c(i-er,i+er), ...)

    }}
  do.call(axis, c(list(side = 2, at = length(vnames):1,
                       labels = vnames, las = 1,
                       cex.axis = ycex), yaxis.args))
  abline(v = 0, lty = 2)
  # return to original margins
  par("mai" = omar)
}

#' Compute the robust effect size index estimate from F-statistic
#'
#' This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
#' Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
#' @param f The F statistic for the parameter of interest.
#' @param df Number of degrees of freedom of the F statistic.
#' @param rdf Model residual degrees of freedom.
#' @param n Number of independent samples.
#' @details The formula for converting an F statistic to S is:
#'
#' \eqn{ S = \sqrt(max(0, (f * df * (rdf - 2)/rdf - df)/n))}
#'
#' The estimator is derived by setting the statistic equal to the expected value of
#' the test statistic and solving for S.
#'
#' @return Returns a scalar or vector argument of the robust effect size index estimate.
#'
#' @examples
#'
#' # to obtain example F values, first fit a lm
#' mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
#'
#' # run Anova, using a robust variance-covariance function
#' # get the F values and Df values
#' fs = car::Anova(mod, vcov. = sandwich::vcovHC)[1:5, "F"]
#' dfs = car::Anova(mod, vcov. = sandwich::vcovHC)[1:5, "Df"]
#'
#' # get RESI estimates
#' f2S(fs, df = dfs, rdf = mod$df.residual, n = nrow(RESI::insurance))
#' @export
f2S <- function(f, df, rdf, n){
  if (any(f < 0)){
    stop("\nF statistic must be non-negative")
  }
  S = (f*df*(rdf-2)/rdf - df)/n
  sqrt(ifelse(S<0, 0, S))
}

#' Plotting RESI Estimates and CIs
#'
#' This function uses ggplot2 graphics to plot robust effect size (RESI) estimates and confidence intervals
#' from `resi`, `summary_resi`, and `anova_resi` objects.
#' @param data Object of `resi`, `summary_resi`, or `anova_resi` class
#' @param mapping Ignored, included for consistency with `ggplot` generic
#' @param alpha Numeric, desired alpha level for confidence intervals
#' @param error.bars Logical, whether to include end caps on the confidence intervals. Default = `TRUE`
#' @param ... Ignored
#' @param environment Ignored, included for consistency with `ggplot` generic
#' @return Returns a ggplot of RESI point estimates
#' @examples
#' # create a resi object
#' resi_obj <- resi(lm(charges ~ region * age + bmi + sex, data = RESI::insurance),
#' nboot = 10)
#'
#' # plot ANOVA table
#' ggplot2::ggplot(anova(resi_obj))
#' @import ggplot2
#' @export
ggplot.resi = function(data, mapping, alpha = NULL, error.bars = TRUE, ...,
                       environment){
  dots = list(...)

  if (!is.null(data$alpha)){
    if (is.null(alpha)){
      alpha = data$alpha[1]
    }
    else{
      if (length(alpha) > 1){
        warning("\nOnly first alpha will be plotted")
        alpha = alpha[1]
      }
      if (!(alpha %in% data$alpha)){
        if (!is.null(data$ci.method) && data$ci.method != "boot" && !is.null(data$model.full)) {
          # Asymptotic (qf/normal): recompute coefficient CIs at the new alpha.
          asym_out <- tryCatch(
            resi_pe_asymptotic(
              model.full   = data$model.full,
              alpha        = alpha,
              ci.method    = data$ci.method,
              vcovfunc     = if (!is.null(data$vcovfunc)) data$vcovfunc else sandwich::vcovHC,
              vcov.args    = if (!is.null(data$vcov.args)) data$vcov.args else list(),
              Anova.args   = if (!is.null(data$Anova.args)) data$Anova.args else list(),
              unbiased     = if (!is.null(data$unbiased)) data$unbiased else TRUE,
              coefficients = TRUE,
              anova        = FALSE
            ),
            error = function(e) {
              stop(paste0("\nFailed to recompute asymptotic CIs at alpha = ", alpha,
                          ":\n", conditionMessage(e)))
            }
          )
          ci_cols <- paste0(c(alpha / 2, 1 - alpha / 2) * 100, "%")
          rn_match <- intersect(rownames(data$coefficients), rownames(asym_out$coefficients))
          data$coefficients[rn_match, ci_cols] <- asym_out$coefficients[rn_match, ci_cols]
        } else if (is.null(data$boot.results)) {
          stop("\nSpecified alpha not found in the resi object")
        } else {
          CIs = apply(data$boot.results$t[,2:(1+nrow(data$coefficients))], 2,  quantile, probs = c(alpha/2, 1-alpha/2), na.rm = TRUE)
          CIs = t(CIs)
          data$coefficients[1:nrow(CIs), c(paste(alpha/2*100, "%", sep=""), paste((1-rev(alpha)/2)*100, "%", sep=""))] = CIs
        }
      }
    }
    ll = paste(alpha/2*100, "%", sep = "")
    ul = paste((1-alpha/2)*100, "%", sep = "")} else{
      ll = ul = "RESI"
    }

  vnames = rownames(data$coefficients)

  dat = data$coefficients
  ggpos = length(vnames):1
  dat$ggpos = ggpos
  dat = dat[order(dat$ggpos),]
  RESI = dat$RESI
  p = ggplot(dat, aes(x = RESI, y = ggpos)) +
    geom_point() +
    scale_y_continuous(name = NULL, breaks = 1:length(vnames), labels = rev(vnames)) +
    geom_vline(xintercept = 0, linetype="dotted") +
    xlab("RESI Estimate") +
    ggtitle(ifelse(is.null(data$alpha), "Coefficient RESI Estimates",
                   paste("Coefficient RESI Estimates and ", (1-alpha)*100, "%", " CIs", sep="")))
  if (!is.null(data$alpha)){
    p = p + sapply(length(vnames):1, function(i) geom_segment(data = dat[-1*(i-length(vnames)) + 1,],
                                                              aes(x = get(ll),
                                                                  xend = get(ul),
                                                                  y = ggpos, yend = ggpos,
                                                              )))
    if (error.bars) {
      er = 0.2 * (length(vnames)/(length(vnames) + 1))
      p = p + sapply(length(vnames):1, function(i) geom_segment(data = dat[-1*(i-length(vnames)) + 1,],
                                                                aes(x = get(ll),
                                                                    xend = get(ll),
                                                                    y = ggpos - er, yend = ggpos + er,
                                                                ))) +
        sapply(length(vnames):1, function(i) geom_segment(data = dat[-1*(i-length(vnames)) + 1,],
                                                          aes(x = get(ul),
                                                              xend = get(ul),
                                                              y = ggpos - er, yend = ggpos + er,
                                                          )))
    }}
  p
}

#' @export
ggplot.summary_resi = ggplot.resi

#' @export
ggplot.anova_resi = function(data, mapping, alpha = NULL, error.bars = TRUE, ...,
                             environment){
  dots = list(...)
  cols = grep("%", colnames(data))
  if (length(cols) != 0){
    if (is.null(alpha)){
      alpha = gsub("%", "", colnames(data)[cols[1]])
      alpha = as.numeric(alpha)*2/100
    }
    else{
      if (length(alpha) > 1){
        warning("\nOnly first alpha will be plotted")
        alpha = alpha[1]
      }
    }
    ll = paste(alpha/2*100, "%", sep="")
    ul = paste((1-alpha/2)*100, "%", sep="")
    if (!(ll %in% colnames(data))){
      stop("\nSpecified alpha not found in the resi object")
    }} else{
      ll = ul = "RESI"
    }

  vnames = rownames(data)[which(rownames(data) != "Residuals")]

  dat = as.data.frame(data[which(rownames(data) != "Residuals"),])
  ggpos = length(vnames):1
  dat$ggpos = ggpos
  dat = dat[order(dat$ggpos),]
  RESI = dat$RESI
  p = ggplot(dat, aes(x = RESI, y = ggpos)) +
    geom_point()  +
    scale_y_continuous(name = NULL, breaks = 1:length(vnames), labels = rev(vnames)) +
    xlab("RESI Estimate") +
    geom_vline(xintercept = 0, linetype="dotted") +
    ggtitle(ifelse(length(cols) == 0, "ANOVA RESI Estimates",
                   paste("ANOVA RESI Estimates and ", (1-alpha)*100, "%", " CIs", sep="")))

  if (length(cols) != 0){
    p = p + sapply(length(vnames):1, function(i) geom_segment(data = dat[-1*(i-length(vnames)) + 1,],
                                                      aes(x = get(ll),
                                                          xend = get(ul),
                                                          y = ggpos, yend = ggpos,
                                                      )))
    if (error.bars) {
      er = 0.2 * (length(vnames)/(length(vnames) + 1))
      p = p + sapply(length(vnames):1, function(i) geom_segment(data = dat[-1*(i-length(vnames)) + 1,],
                                                                aes(x = get(ll),
                                                                    xend = get(ll),
                                                                    y = ggpos - er, yend = ggpos + er,
                                                                ))) +
        sapply(length(vnames):1, function(i) geom_segment(data = dat[-1*(i-length(vnames)) + 1,],
                                                          aes(x = get(ul),
                                                              xend = get(ul),
                                                              y = ggpos - er, yend = ggpos + er,
                                                          )))
    }}
  p
}

#' Convert S to Cohen's d
#'
#' Converts the robust effect size index (S) to Cohen's d using the formula from
#' Vandekar, Tao, & Blume (2020).
#' @param S Numeric, the robust effect size index.
#' @param pi Numeric, the sampling proportions.
#' @return Returns an estimate of Cohen's \emph{d} based on the RESI.
#' @details The pi parameter comes from the fact that Cohen's d doesn't account
#' for unequal sample proportions in the population, but S does.
#'
#' The default is set to a natural value 1/2, which corresponds to a case
#' control design, for example, where sampling proportions always are
#' controlled by the experimenter.
#'
#' The formula for the conversion is:
#'
#' \eqn{ d = | S * \sqrt(1/\pi + 1/(1 - \pi)) |}
#' @examples
#' # fit a simple linear regression with a binary predictor
#' mod = lm(charges ~ sex, data = RESI::insurance)
#'
#' # calculate t-value
#' t = summary(mod)$coefficients[2, "t value"]
#'
#' # calculate RESI (S)
#' S = t2S(t, n = 1338, rdf = 1336)
#'
#' # determine sample proportions
#' pi = length(which(RESI::insurance[,"sex"]=="male"))/1338
#'
#' # convert S to Cohen's d
#' S2d(S = S, pi = pi)
#'
#' @export
S2d = function(S, pi=0.5){
  if (pi < 0 | pi > 1){
    stop("\npi must be between 0 and 1")
  }
  abs(S * sqrt(1/pi + 1/(1-pi)))
}

#' Covert Cohen's \emph{d} to |S|
#'
#' Converts Cohen's \emph{d} robust effect size index (S) using the formula from
#' Vandekar, Tao, & Blume (2020).
#' @param d Numeric, value of Cohen's \emph{d}.
#' @param pi Numeric, the sampling proportions.
#' @return Returns an estimate the robust effect size index
#' @details The pi parameter comes from the fact that Cohen's d doesn't account
#' for unequal sample proportions in the population, but S does.
#'
#' The default is set to a natural value 1/2, which corresponds to a case
#' control design, for example, where sampling proportions always are
#' controlled by the experimenter.
#'
#' The formula to convert Cohen's \emph{d} to S is:
#'
#' \eqn{S = d/\sqrt( 1/\pi + 1/ (1 - \pi))}
#' @examples
#' # Consider an experiment with equal sampling proportions and a medium effect size
#' # corresponding to a Cohen's d of 0.5.
#' # convert to RESI (S)
#' d2S(d = 0.5)
#'
#' # This corresponds to a RESI of 0.25.
#' @export
d2S <- function(d, pi = 0.5){
  if (pi < 0 | pi > 1){
    stop("\npi must be between 0 and 1")
  }
  abs(d)/sqrt(1/pi + 1/(1-pi))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S2d.R
\name{S2d}
\alias{S2d}
\title{Convert S to Cohen's d}
\usage{
S2d(S, pi = 0.5)
}
\arguments{
\item{S}{Numeric, the robust effect size index.}

\item{pi}{Numeric, the sampling proportions.}
}
\value{
Returns an estimate of Cohen's \emph{d} based on the RESI.
}
\description{
Converts the robust effect size index (S) to Cohen's d using the formula from
Vandekar, Tao, & Blume (2020).
}
\details{
The pi parameter comes from the fact that Cohen's d doesn't account
for unequal sample proportions in the population, but S does.

The default is set to a natural value 1/2, which corresponds to a case
control design, for example, where sampling proportions always are
controlled by the experimenter.

The formula for the conversion is:

\eqn{ d = | S * \sqrt(1/\pi + 1/(1 - \pi)) |}
}
\examples{
# fit a simple linear regression with a binary predictor
mod = lm(charges ~ sex, data = RESI::insurance)

# calculate t-value
t = summary(mod)$coefficients[2, "t value"]

# calculate RESI (S)
S = t2S(t, n = 1338, rdf = 1336)

# determine sample proportions
pi = length(which(RESI::insurance[,"sex"]=="male"))/1338

# convert S to Cohen's d
S2d(S = S, pi = pi)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resi_asymptotic.R
\name{resi_pe_asymptotic}
\alias{resi_pe_asymptotic}
\title{Robust Effect Size Index with Asymptotic Confidence Intervals}
\usage{
resi_pe_asymptotic(
  model.full,
  data,
  vcovfunc = sandwich::vcovHC,
  coefficients = TRUE,
  anova = TRUE,
  alpha = 0.05,
  ci.method = c("qf", "normal", "cf"),
  type = "HC3",
  unbiased = TRUE,
  Anova.args = list(),
  vcov.args = list(),
  ...
)
}
\arguments{
\item{model.full}{Fitted \code{lm} or \code{glm} model object.}

\item{data}{Data frame of model data.}

\item{vcovfunc}{Variance estimator for RESI point estimates. Default:
\code{sandwich::vcovHC}.}

\item{coefficients}{Logical; include coefficient table. Default \code{TRUE}.}

\item{anova}{Logical; include anova table. Default \code{TRUE}.}

\item{alpha}{Numeric; significance level. Default \code{0.05}.}

\item{ci.method}{Character; \code{"qf"} (quadratic-form Imhof, default),
\code{"normal"} (truncated normal),
or \code{"cf"} (Cornish-Fisher test inversion).}

\item{type}{Character; HC type for the sandwich variance used in CI
construction (controls tau_i weights in SigmaXw). Default \code{"HC3"}.
Supported: \code{"HC0"}–\code{"HC5"}, \code{"const"}.}

\item{unbiased}{Logical; use bias-corrected RESI point estimate. Default
\code{TRUE}.}

\item{Anova.args}{List; additional arguments passed to \code{car::Anova}.}

\item{vcov.args}{List; additional arguments passed to \code{vcovfunc}.}

\item{...}{Ignored.}
}
\value{
A list of class \code{"resi"} with \code{coefficients} and/or
  \code{anova} tables containing RESI point estimates and CIs.
}
\description{
Computes RESI point estimates and asymptotic confidence intervals using
either a normal approximation (Zhang et al., 2025) or a quadratic-form
(Imhof/Davies) approach.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{simCalibrationFigures}
\alias{simCalibrationFigures}
\title{Calibration Figures for RESI Variance Estimates}
\usage{
simCalibrationFigures(
  output.dir = "resiCalibrationSim",
  figures.dir = NULL,
  alpha = 0.05,
  deriv_method = c("extended", "corrected", "original", "population", "population2",
    "zeroB", "zero_phi_cross")
)
}
\arguments{
\item{output.dir}{Character, directory containing output from
\code{\link{simCalibrationSim}}. Default \code{"resiCalibrationSim"}.}

\item{figures.dir}{Character, output directory for PDFs.
Default \code{file.path(output.dir, "figures")}.}

\item{alpha}{Numeric, nominal CI level. Default 0.05.}

\item{deriv_method}{Character, type of derivative being evaluated. Package now only includes "extended" in resi implementation.}
}
\value{
Invisibly returns the summary data frame. Saves PDF figures to
  \code{figures.dir}.
}
\description{
Reads output from \code{\link{simCalibrationSim}} and produces one PDF per
model setting showing convergence of point estimates, covariance estimates,
RESI estimates, and RESI variance estimates to their population targets.
}
\seealso{
\code{\link{simCalibrationSim}}, \code{\link{insurancePlasmodeSim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S2z.R
\name{S2z}
\alias{S2z}
\title{Convert RESI (S) estimate to Z statistic}
\usage{
S2z(S, n, unbiased = TRUE)
}
\arguments{
\item{S}{The value of the RESI estimate.}

\item{n}{Number of independent samples.}

\item{unbiased}{Logical, whether the unbiased or alternative estimator was used to compute RESI estimate. Default is TRUE.}
}
\value{
Returns a scalar or vector argument of the Chi-square statistic.
}
\description{
Converts the robust effect size index (S) to Z statistic.
Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
}
\details{
The formula for converting a RESI estimate to a corresponding Z statistic depends on which estimator
was used to compute the RESI estimate (unbiased vs. alternative, see \code{\link{z2S}}). For the unbiased estimator,
the RESI can be positive or negative and there is a 1-1 transformation from S to Z. The formula for converting S (unbiased) to the
Z statistic is:

\eqn{\sqrt(n)*S}

For the alternative formula, if the RESI estimate is 0, the Z statistic is only known within an interval, [-1, 1].
For a non-zero S, the formula is:

\eqn{\sqrt{S^2}/S\sqrt(n*abs(S) + 1)}
}
\examples{
# convert S estimates with corresponding degrees of freedom to
# Z statistics estimates (using unbiased formula)
S_ests = c(-0.2, 0, 0.1)
S2z(S = S_ests, n = 300, unbiased = TRUE)

# convert S estimates with corresponding degrees of freedom to
# Z statistics estimates (using alernative formula)
S_ests = c(-0.2, 0, 0.1)
S2z(S = S_ests, n = 300, unbiased = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{insurancePlasmodeSim}
\alias{insurancePlasmodeSim}
\title{Insurance Plasmode Simulation for RESI Evaluation}
\usage{
insurancePlasmodeSim(
  nsim = 1000L,
  n.vec = c(50, 100, 200, 500, 1000, 2000, 5000),
  nboot = 500L,
  alpha = 0.05,
  ci.method = c("boot", "normal", "qf", "cf"),
  output.dir = NULL,
  fixed.knots = FALSE,
  mc.cores.settings = 1L,
  mc.cores.reps = 1L
)
}
\arguments{
\item{nsim}{Integer, number of simulation replicates per (setting, \code{n}) cell.
Default 1000. Use 10 for initial testing.}

\item{n.vec}{Integer vector of sample sizes. Default
\code{c(50, 100, 200, 500, 1000, 2000, 5000)}.}

\item{nboot}{Integer, bootstrap replicates per internal \code{\link{resi}} call.
Default 500. Use 10 for initial testing. Ignored when \code{ci.method != "boot"}.}

\item{alpha}{Numeric, CI significance level. Default 0.05.}

\item{ci.method}{Character, CI method passed to \code{\link{resi}}. One of
\code{"boot"} (bootstrap, default), \code{"normal"} (asymptotic truncated-normal),
or \code{"qf"} (asymptotic quadratic-form / Imhof). When \code{ci.method != "boot"},
\code{nboot} is ignored.}

\item{output.dir}{Character, path to the directory where all results are saved.
Created if it does not exist. Defaults to \code{"resiBootSim"},
\code{"resiAsympNormalSim"}, or \code{"resiAsympQFSim"} based on
\code{ci.method} when \code{NULL}.}

\item{fixed.knots}{Logical. If \code{TRUE}, spline knots are fixed at the
empirical tertiles of \code{age} in the full insurance dataset rather than
re-selected by \code{df = 3} in each bootstrap sample. Default \code{FALSE}.}

\item{mc.cores.settings}{Integer, cores for the outer
\code{mclapply} over (setting \eqn{\times} sample size) combinations. Default 1.}

\item{mc.cores.reps}{Integer, cores for the inner \code{mclapply} over simulation
replicates within each (setting, \code{n}) cell. Default 1.}
}
\value{
Invisibly returns the summary metrics \code{data.frame}. Side effects:
\itemize{
  \item \code{output.dir/sim_raw/<setting>_n<n>.rds}: list of per-replicate
    \code{anova} and \code{coefficients} tables.
  \item \code{output.dir/summary_table.rds}: combined metrics table with columns
    \code{model, vcov, n, n_success, table, term, bias, mse, coverage, width}.
}
}
\description{
Runs a plasmode simulation study using the \code{\link{insurance}} dataset to
evaluate RESI confidence interval performance. In each replicate, \code{n}
observations are resampled with replacement from the full insurance dataset
(\emph{N} = 1338). The RESI point estimates from \code{\link{resi_pe}} applied
to the full dataset are treated as the true parameter values for computing bias,
MSE, CI coverage, and CI width.
}
\details{
Two models are evaluated:
\itemize{
  \item \strong{lm}: \code{log10(charges) ~ ns(age, df=3) * sex + bmi + smoker + region}
  \item \strong{glm}: \code{I(charges > 10000) ~ ns(age, df=3) * sex + bmi + smoker + region}
    with \code{family = binomial()}
}
Each model is evaluated under both parametric (\code{vcovfunc = stats::vcov}) and
robust (\code{vcovfunc = sandwich::vcovHC}) variance settings, yielding four
simulation conditions.

Parallelization is via \code{\link[parallel]{mclapply}}, which uses forking and is
not supported on Windows (falls back to sequential evaluation on Windows).
}
\seealso{
\code{\link{simFigures}}, \code{\link{simCompareMethodsFigures}},
  \code{\link{resi}}, \code{\link{resi_pe}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/t2S.R
\name{t2S}
\alias{t2S}
\title{Compute the robust effect size index estimate from t statistic (default)}
\usage{
t2S(t, rdf, n, unbiased = TRUE)
}
\arguments{
\item{t}{The t statistic for the parameter of interest.}

\item{rdf}{Model residual degrees of freedom/degrees of freedom of the t statistic.}

\item{n}{Number of independent samples.}

\item{unbiased}{Logical, whether to use unbiased or alternative estimator. See details.}
}
\value{
Returns a scalar or vector argument of the robust effect size index estimate.
}
\description{
This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
}
\details{
This function computes S, the RESI, from a t statistic. The formula for the
unbiased estimator (default) is derived by solving the expected value of the
t statistic for S. It is unbiased and consistent.

The formula for the unbiased conversion is:

\eqn{S = (t * \sqrt(2) * \Gamma(rdf/2)) / (\sqrt(n * rdf) * \Gamma((rdf - 1)/2))}

The formula for the alternative estimator is derived by squaring the t statistic
and using the \code{\link{f2S}} formula. This estimator may be appealing for its
intuitive relationship to the F statistic; the absolute value of RESI estimates
using this formula will be equal to a RESI estimate using an F statistic for
the same model. However, this estimator does have finite sample bias, which is an
important consideration for the coverage of the bootstrapping that \code{resi} uses.

The formula for the alternative conversion is:

\eqn{ \sqrt(max(0, (t^2 * (rdf - 2)/rdf - 1)/rdf))}
}
\examples{
# to obtain t values, first fit a lm
mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
# run lmtest::coeftest to get t values, using a robust variance-covariance formula
ts = lmtest::coeftest(mod, vcov. = sandwich::vcovHC)[,'t value']

# get RESI estimates using unbiased estimator
t2S(ts, n = nrow(RESI::insurance), rdf = mod$df.residual)

# get RESI estimates using alternative estimator
t2S(ts, n = nrow(RESI::insurance), rdf = mod$df.residual, unbiased = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{simFigures}
\alias{simFigures}
\title{Simulation Performance Figures for RESI Evaluation}
\usage{
simFigures(output.dir = "resiBootSim", alpha = 0.05, ci.label = NULL)
}
\arguments{
\item{output.dir}{Character, directory containing simulation output from
\code{\link{insurancePlasmodeSim}}. Default \code{"resiBootSim"}.}

\item{alpha}{Numeric, nominal CI level used for the coverage reference line.
Default 0.05.}

\item{ci.label}{Character, label for the CI method used in figure titles and file
names. Default \code{NULL}, which auto-detects from \code{output.dir}:
\code{"boot"} if the directory name contains "Boot"/"boot",
\code{"normal"} if it contains "Normal"/"normal",
\code{"qf"} if it contains "QF"/"qf", otherwise \code{basename(output.dir)}.}
}
\value{
Invisibly returns the summary metrics \code{data.frame}. Saves PDF figures
  to \code{file.path(output.dir, "figures")}, named
  \code{sim_<model>_<vcov>_<ci.label>.pdf}.
}
\description{
Produces performance figures from the output of \code{\link{insurancePlasmodeSim}}.
Creates one PDF figure per (model type) \eqn{\times} (variance estimator) combination
(4 figures total by default). Each figure is a 2 \eqn{\times} 4 panel grid:
\itemize{
  \item \strong{Top row}: Anova-table RESI metrics (Bias, MSE, CI Coverage, CI Width)
  \item \strong{Bottom row}: Coefficients-table RESI metrics (same metrics; intercept excluded)
  \item \strong{Colors}: one colored line per model term (high-contrast matte palette)
  \item \strong{x-axis}: sample size on a log scale
}
Dashed reference lines are drawn at zero for Bias and at \eqn{1 - \alpha} for CI Coverage.
}
\seealso{
\code{\link{insurancePlasmodeSim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisq2S.R
\name{chisq2S}
\alias{chisq2S}
\title{Compute the robust effect size index estimate from chi-squared statistic.}
\usage{
chisq2S(chisq, df, n)
}
\arguments{
\item{chisq}{The chi-square statistic for the parameter of interest.}

\item{df}{Number of degrees of freedom of the chi-square statistic.}

\item{n}{Number of independent samples.}
}
\value{
Returns a scalar or vector argument of the robust effect size index estimate.
}
\description{
This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
For mixed effects models, RESI is conditional on the average correlation
structure within subjects.
}
\details{
The formula for converting a Chi-square statistic to RESI is:

\eqn{ S = \sqrt(max( 0, (chisq - df)/n))}
}
\examples{
# obtain Chi-sq value by fitting an lm and running a Wald test
mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)

# run a Wald test with robust variance
wt = lmtest::waldtest(mod, vcov = sandwich::vcovHC, test = "Chisq")

# get Chi-sq value and degrees of freedom
chisq = wt$Chisq[2]
df = abs(wt$Df[2])

# run chisq2S to convert to RESI
chisq2S(chisq, df = df, n = nrow(mod$model))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{simCalibrationSim}
\alias{simCalibrationSim}
\title{Asymptotic Calibration Check for RESI Variance Estimates}
\usage{
simCalibrationSim(
  nsim = 500L,
  n.vec = c(100L, 200L, 500L, 1000L, 2000L, 5000L),
  alpha = 0.05,
  output.dir = "resiCalibrationSim",
  fixed.knots = FALSE,
  deriv_method = c("corrected", "original", "population", "population2", "zeroB",
    "zero_phi_cross", "extended"),
  mc.cores.reps = 1L
)
}
\arguments{
\item{nsim}{Integer, replicates per (setting, \eqn{n}) cell. Default 500.}

\item{n.vec}{Integer vector of sample sizes.
Default \code{c(100, 200, 500, 1000, 2000, 5000)}.}

\item{alpha}{Numeric, nominal CI level. Default 0.05.}

\item{output.dir}{Character, directory for raw per-cell RDS files.
Default \code{"resiCalibrationSim"}.}

\item{fixed.knots}{Logical. Fix spline knots at full-dataset quantiles.
Default \code{FALSE}.}

\item{deriv_method}{Character, type of derivative being evaluated. Package now only includes "extended" in resi implementation.}

\item{mc.cores.reps}{Integer, cores for within-cell parallelism. Default 1.}
}
\value{
Invisibly returns the combined metrics \code{data.frame}.
}
\description{
Runs a plasmode simulation to verify that the asymptotic normal CI machinery
is correctly calibrated for all four model settings (lm/glm x
parametric/robust).  For each (setting, sample size) cell the function
checks:
\enumerate{
  \item \strong{Bias(theta)}: \eqn{\hat\theta \to \theta_{\rm true}} --
    raw coefficients (and \eqn{\phi = \hat\sigma^2} for \code{lm}) converge
    to the full-dataset values.
  \item \strong{vcov check A} (estimator consistency):
    \eqn{n \cdot \bar V_{\rm analytic} / (n_{\rm full} \cdot V_{\rm true}) \to 1}.
  \item \strong{vcov check B} (calibration):
    \eqn{n \cdot \widehat{\rm Var}(\hat\beta) / (n_{\rm full} \cdot V_{\rm true}) \to 1}.
  \item \strong{Bias(R)}: RESI point estimates converge to the full-dataset
    values.
  \item \strong{sigma2S check A} (estimator consistency):
    \eqn{\bar{\hat\sigma}^2_S / \sigma^2_{S,\rm true} \to 1}.
  \item \strong{sigma2S check B} (CI calibration):
    \eqn{n \cdot \widehat{\rm Var}(\hat R) / \sigma^2_{S,\rm true} \to 1}.
}
True values are taken from the full \code{\link{insurance}} dataset using the
same definitions as \code{\link{insurancePlasmodeSim}}.  \eqn{\sigma^2_S} is
extracted from the asymptotic-normal CI half-width:
\eqn{\hat\sigma^2_S = n \cdot ({\rm hw}/z_{\alpha/2})^2}.
}
\seealso{
\code{\link{insurancePlasmodeSim}}, \code{\link{simCompareMethodsFigures}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resi.R
\name{resi}
\alias{resi}
\alias{resi.default}
\alias{resi.glm}
\alias{resi.lm}
\alias{resi.lmrob}
\alias{resi.glmrob}
\alias{resi.nls}
\alias{resi.survreg}
\alias{resi.coxph}
\alias{resi.hurdle}
\alias{resi.zeroinfl}
\alias{resi.geeglm}
\alias{resi.glmgee}
\alias{resi.gee}
\alias{resi.lme}
\alias{resi.lmerMod}
\alias{resi.glmmTMB}
\title{Robust Effect Size Index (RESI) point and interval estimation for models}
\usage{
resi(model.full, ...)

\method{resi}{default}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  boot.method = "nonparam",
  vcovfunc = sandwich::vcovHC,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  long = FALSE,
  clvar = NULL,
  ci.method = c("boot", "qf", "cf", "normal"),
  ...
)

\method{resi}{glm}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  vcovfunc = sandwich::vcovHC,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ci.method = "qf",
  ...
)

\method{resi}{lm}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  boot.method = "nonparam",
  vcovfunc = sandwich::vcovHC,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ci.method = "qf",
  ...
)

\method{resi}{lmrob}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  boot.method = "nonparam",
  vcovfunc = stats::vcov,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ci.method = "boot",
  ...
)

\method{resi}{glmrob}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  vcovfunc = stats::vcov,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ci.method = "boot",
  ...
)

\method{resi}{nls}(
  model.full,
  model.reduced = NULL,
  data,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  boot.method = "nonparam",
  anova = FALSE,
  vcovfunc = r_nlshc,
  alpha = 0.05,
  store.boot = FALSE,
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{survreg}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  vcovfunc = vcov,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{coxph}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  vcovfunc = vcov,
  alpha = 0.05,
  store.boot = FALSE,
  Anova.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{hurdle}(
  model.full,
  model.reduced = NULL,
  data,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  vcovfunc = sandwich::sandwich,
  anova = FALSE,
  alpha = 0.05,
  store.boot = FALSE,
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{zeroinfl}(
  model.full,
  model.reduced = NULL,
  data,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  vcovfunc = sandwich::sandwich,
  anova = FALSE,
  alpha = 0.05,
  store.boot = FALSE,
  vcov.args = list(),
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{geeglm}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  alpha = 0.05,
  store.boot = FALSE,
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{glmgee}(
  model.full,
  model.reduced = NULL,
  data,
  anova = FALSE,
  coefficients = TRUE,
  overall = TRUE,
  nboot = 1000,
  alpha = 0.05,
  store.boot = FALSE,
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{gee}(
  model.full,
  data,
  nboot = 1000,
  alpha = 0.05,
  store.boot = FALSE,
  unbiased = TRUE,
  parallel = c("no", "multicore", "snow"),
  ncpus = getOption("boot.ncpus", 1L),
  ...
)

\method{resi}{lme}(
  model.full,
  alpha = 0.05,
  nboot = 1000,
  anova = TRUE,
  vcovfunc = clubSandwich::vcovCR,
  vcov.args = list(),
  ...
)

\method{resi}{lmerMod}(
  model.full,
  alpha = 0.05,
  nboot = 1000,
  anova = TRUE,
  vcovfunc = clubSandwich::vcovCR,
  vcov.args = list(),
  ...
)

\method{resi}{glmmTMB}(
  model.full,
  alpha = 0.05,
  nboot = 1000,
  anova = TRUE,
  vcovfunc = clubSandwich::vcovCR,
  vcov.args = list(),
  ...
)
}
\arguments{
\item{model.full}{\code{lm, glm, nls, survreg, coxph, hurdle, zeroinfl, gee, geeglm} or \code{lme} model object.}

\item{...}{Ignored.}

\item{model.reduced}{Fitted model object of same type as model.full. By default `NULL`; the same model as the full model but only having intercept.}

\item{data}{Data.frame or object coercible to data.frame of model.full data (required for some model types).}

\item{anova}{Logical, whether to produce an Anova table with the RESI columns added. By default = `TRUE`.}

\item{coefficients}{Logical, whether to produce a coefficients (summary) table with the RESI columns added. By default = `TRUE`.}

\item{overall}{Logical, whether to produce an overall Wald test comparing full to reduced model with RESI columns added. By default = `TRUE`.}

\item{nboot}{Numeric, the number of bootstrap replicates. Used only when
`ci.method = "boot"`. By default, 1000.}

\item{boot.method}{String, which type of bootstrap to use: `nonparam` = non-parametric bootstrap (default); `bayes` = Bayesian bootstrap.}

\item{vcovfunc}{The variance estimator function for constructing the Wald test statistic. By default, \link[sandwich]{vcovHC} (the robust (sandwich) variance estimator).}

\item{alpha}{Numeric, significance level of the constructed CIs. By default, 0.05.}

\item{store.boot}{Logical, whether to store all the bootstrapped estimates. By default, `FALSE`.}

\item{Anova.args}{List, additional arguments to be passed to \link[car]{Anova} function.}

\item{vcov.args}{List, additional arguments to be passed to vcovfunc.}

\item{unbiased}{Logical, whether to use the unbiased or alternative T/Z statistic to RESI conversion. By default, `TRUE`. See details.}

\item{parallel}{See documentation for \link[boot]{boot}.}

\item{ncpus}{See documentation for \link[boot]{boot}.}

\item{long}{Logical, whether the data is longitudinal/clustered. By default, `FALSE`.}

\item{clvar}{Character, the name of the cluster/id variable if data is clustered. By default, `NULL`.}

\item{ci.method}{Character, the method used to compute confidence intervals.
One of `"boot"` (bootstrap), `"qf"` (quadratic-form inversion, default for lm and glm),
`"normal"` (normal approximation), or `"cf"` (Cornish-Fisher inversion).
See `resi_pe_asymptotic` for details on the asymptotic methods.}
}
\value{
Returns a list that includes function arguments, RESI point estimates,
and confidence intervals in coefficients/anova-style tables
}
\description{
This function will estimate the robust effect size (RESI) from Vandekar, Tao, & Blume (2020) and its confidence interval in various ways for a fitted model object. The overall RESI is estimated via a Wald test. RESI is (optionally) estimated for each factor in coefficients-style table. RESI is (optionally) estimated for each variable/interaction in an Anova-style table for models with existing Anova methods. CIs can be calculated using either non-parametric or Bayesian bootstrapping.
}
\details{
The RESI, denoted as S, is applicable across many model types. It is a unitless
index and can be easily be compared across models. The RESI can also be
converted to Cohen's \emph{d} (\code{\link{S2d}}) under  model homoskedasticity.

This function computes the RESI point estimates and bootstrapped confidence
intervals based on Chi-square, F, T, or Z statistics. The robust (sandwich)
variance is used by default, allowing for consistency under
model-misspecification. The RESI is related to the non-centrality parameter
of the test statistic. The RESI estimate is consistent for all four
(Chi-square, F, T, and Z) types of statistics used. The Chi-square and F-based
calculations rely on asymptotic theory, so they may be biased in small samples.
When possible, the T and Z statistics are used. There are two formulas for both
the T and Z statistic conversion. The first (default, unbiased = TRUE)
are based on solving the expected value of the T or Z statistic for the RESI.
The alternative is based on squaring the T or Z statistic and using the
F or Chi-square statistic conversion. Both of these methods are consistent, but
the alternative exhibits a notable amount of finite sample bias. The alternative
may be appealing because its absolute value will be equal to the RESI based on
the F or Chi-square statistic. The RESI based on the Chi-Square and F statistics
is always greater than or equal to 0. The type of statistic
used is listed with the output. See \code{\link{f2S}}, \code{\link{chisq2S}},
\code{\link{t2S}}, and \code{\link{z2S}} for more details on the formulas.

For GEE (\link[geepack]{geeglm}, \link[glmtoolbox]{glmgee}) models, a longitudinal RESI (L-RESI) and a cross-sectional,
per-measurement RESI (CS-RESI) is estimated. The longitudinal RESI takes the
specified clustering into account, while the cross-sectional RESI is designed to
estimate the effect size if a random observation for each participant were collected cross-sectionally.

For most \code{lm} and \code{nls} model types, there is a Bayesian bootstrap
option available as an alternative to the default, standard non-parametric
bootstrap. The interpretation of a Bayesian bootstrapped interval is similar to
that of a credible interval.

Certain model types require the data used for the model be entered as an argument.
These are: \code{nls, survreg,} and \code{coxph}. Additionally, if a model
includes certain functions (splines, factor, I), the data needs to be provided.

If running into convergence issues with nls models, it is advised to refit the
nls model with starting values equal to the estimates provided by the model
and then try rerunning \code{resi}.
}
\section{Methods (by class)}{
\itemize{
\item \code{resi(default)}: RESI point and interval estimation for models

\item \code{resi(glm)}: RESI point and interval estimation for models

\item \code{resi(lm)}: RESI point and interval estimation for lm models

\item \code{resi(lmrob)}: RESI point and interval estimation for lmrob models (robustbase)

\item \code{resi(glmrob)}: RESI point and interval estimation for glmrob models (robustbase)

\item \code{resi(nls)}: RESI point and interval estimation for nls models

\item \code{resi(survreg)}: RESI point and interval estimation for survreg models

\item \code{resi(coxph)}: RESI point and interval estimation for coxph models

\item \code{resi(hurdle)}: RESI point and interval estimation for hurdle models

\item \code{resi(zeroinfl)}: RESI point and interval estimation for zeroinfl models

\item \code{resi(geeglm)}: RESI point and interval estimation for GEE models

\item \code{resi(glmgee)}: RESI point and interval estimation for GEE models in glmtoolbox

\item \code{resi(gee)}: RESI point and interval estimation for GEE models

\item \code{resi(lme)}: RESI point and interval estimation for LME (nlme) models

\item \code{resi(lmerMod)}: RESI point and interval estimation for lmerMod models

\item \code{resi(glmmTMB)}: RESI point and interval estimation for glmmTMB models - Gaussian only

}}
\examples{
## for timing purposes, a small number of bootstrap replicates is used in the
## examples. Run them with a higher or default `nboot` argument for better performance

## RESI on a linear model
# fit linear model
mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)

# run resi on fitted model with desired number of bootstrap replicates
# store bootstrap results for calculating different CIs later
resi_obj = resi(mod, nboot = 50, store.boot = TRUE)
# print output
resi_obj

# fit a reduced model for comparison
mod_red = lm(charges ~ bmi, data = RESI::insurance)

# running resi and including the reduced model will provide almost the exact same
# output as not including a reduced model. The difference is that the "overall"
# portion of the output will compare the full model to the reduced model.
# The "summary" and "anova" RESI estimates will be the same. (The bootstrapped
# confidence intervals may differ.)
resi(model.full = mod, model.reduced = mod_red, nboot = 10)

# used stored bootstrap results to get a different alpha-level confidence interval
summary(resi_obj, alpha = c(0.01, 0.1))
car::Anova(resi_obj, alpha = c(0.01, 0.1))

# the result of resi, as well as the summary or Anova of a `resi` object can be plotted
# if the resi object was created with the store.boot = `TRUE` option, any alpha
# can be specified
plot(resi_obj, alpha = 0.01)
# if the variable names on the y-axis are too long, you can reduce their size with
# the ycex.axis argument (or use regular common solutions like changing the margins)
plot(resi_obj, alpha = 0.01, ycex.axis = 0.5)

# for some model types and formula structures, data argument is required
if(requireNamespace("splines")){
  # fit logistic regression model with splines
  mod = glm(smoker ~ splines::ns(age, df = 3) + region, data = RESI::insurance,
    family = "binomial")

  # specify additional arguments to the variance-covariance function via vcov.args
  resi_obj = resi(mod, data = RESI::insurance, alpha = 0.01,
    vcov.args = list(type = "HC0"), nboot = 25)
  summary(resi_obj)
  car::Anova(resi_obj)}


## RESI on a survival model with alternate Z2S
if(requireNamespace("survival")){
  # fit coxph model on example data from survival package
  # Note: for survival models, you need to specify robust variance in the model
  # creation. resi will ignore the vcovfunc argument for this reason.
  mod.coxph =  survival::coxph(survival::Surv(time, status) ~ age + sex + wt.loss,
   data=survival::lung, robust = TRUE)

  # run resi on the model
  # to use the alternative Z to RESI formula (which is equal in absolute value to the
  # chi-square to RESI (S) formula), specify unbiased = FALSE.
  resi(mod.coxph, data = survival::lung, unbiased = FALSE, nboot = 10)}

}
\references{
Vandekar S, Tao R, Blume J. A Robust Effect Size Index. \emph{Psychometrika}. 2020 Mar;85(1):232-246. doi: 10.1007/s11336-020-09698-2.

Kang, K., Armstrong, K., Avery, S., McHugo, M., Heckers, S., & Vandekar, S. (2021). Accurate confidence interval estimation for non-centrality parameters and effect size indices. \emph{arXiv preprint arXiv:2111.05966}.

Jones, M., Kang, K., Vandekar, S. (2025). \emph{Journal of Statistical Software}. RESI: An R Package for Robust Effect Sizes.<doi:10.18637/jss.v112.i03>
}
\seealso{
\code{\link{resi_pe}}, \link[sandwich]{vcovHC},
\code{\link{f2S}}, \code{\link{chisq2S}}, \code{\link{z2S}}, \code{\link{t2S}}
}
\concept{RESI functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/omnibus.R
\name{omnibus}
\alias{omnibus}
\title{Omnibus (Overall) Wald Test for resi objects}
\usage{
omnibus(object, alpha = NULL, ...)
}
\arguments{
\item{object}{an object resulting from resi function}

\item{alpha}{an optional new specification for the confidence level. Can be vector-valued}

\item{...}{ignored}
}
\value{
Returns a `omnibus_resi` object containing the computed omnibus Wald test
}
\description{
After running the \code{\link{resi}} function on a fitted model, this function
can be used to print the overall Wald test component. If the resi function was run
with the `store.boot = TRUE` option to store the full matrix of bootstrapped estimates,
the user can specify a different alpha level for this function's confidence intervals.
}
\examples{
# fit a model
mod = lm(charges ~ bmi + sex, data = RESI::insurance)

# run resi with the store.boot = TRUE option (ci.method must be "boot")
resi_obj = resi(mod, nboot = 100, store.boot = TRUE, alpha = 0.01, ci.method = "boot")

# run summary, specifying a different alpha level if desired
omnibus(resi_obj, alpha = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resi_pe.R
\name{resi_pe}
\alias{resi_pe}
\alias{resi_pe.default}
\alias{resi_pe.glm}
\alias{resi_pe.lm}
\alias{resi_pe.nls}
\alias{resi_pe.survreg}
\alias{resi_pe.coxph}
\alias{resi_pe.hurdle}
\alias{resi_pe.zeroinfl}
\alias{resi_pe.lmrob}
\alias{resi_pe.glmrob}
\alias{resi_pe.geeglm}
\alias{resi_pe.glmgee}
\alias{resi_pe.gee}
\alias{resi_pe.lme}
\alias{resi_pe.lmerMod}
\alias{resi_pe.glmmTMB}
\alias{resi_pe.emmGrid}
\title{Robust Effect Size Index (RESI) Point Estimation}
\usage{
resi_pe(...)

\method{resi_pe}{default}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  vcovfunc = sandwich::vcovHC,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  waldtype = 0,
  ...
)

\method{resi_pe}{glm}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  vcovfunc = sandwich::vcovHC,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  waldtype = 0,
  ...
)

\method{resi_pe}{lm}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  vcovfunc = sandwich::vcovHC,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{nls}(
  model.full,
  model.reduced = NULL,
  data,
  coefficients = TRUE,
  anova = FALSE,
  vcovfunc = r_nlshc,
  vcov.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{survreg}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  vcovfunc = vcov,
  Anova.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{coxph}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  vcovfunc = vcov,
  Anova.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{hurdle}(
  model.full,
  model.reduced = NULL,
  data,
  coefficients = TRUE,
  anova = TRUE,
  vcovfunc = sandwich::sandwich,
  vcov.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{zeroinfl}(
  model.full,
  model.reduced = NULL,
  data,
  coefficients = TRUE,
  anova = TRUE,
  vcovfunc = sandwich::sandwich,
  vcov.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{lmrob}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  vcovfunc = stats::vcov,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{glmrob}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  vcovfunc = stats::vcov,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  overall = TRUE,
  ...
)

\method{resi_pe}{geeglm}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  unbiased = TRUE,
  ...
)

\method{resi_pe}{glmgee}(
  model.full,
  model.reduced = NULL,
  data,
  anova = TRUE,
  coefficients = TRUE,
  overall = TRUE,
  unbiased = TRUE,
  ...
)

\method{resi_pe}{gee}(model.full, data, unbiased = TRUE, ...)

\method{resi_pe}{lme}(
  model.full,
  anova = TRUE,
  vcovfunc = clubSandwich::vcovCR,
  Anova.args = list(),
  vcov.args = list(),
  ...
)

\method{resi_pe}{lmerMod}(
  model.full,
  anova = TRUE,
  vcovfunc = clubSandwich::vcovCR,
  Anova.args = list(),
  vcov.args = list(),
  unbiased = TRUE,
  ...
)

\method{resi_pe}{glmmTMB}(
  model.full,
  anova = TRUE,
  vcovfunc = clubSandwich::vcovCR,
  Anova.args = list(),
  vcov.args = list(),
  ...
)

\method{resi_pe}{emmGrid}(object, model, N = NULL, unbiased = TRUE, ...)
}
\arguments{
\item{...}{Ignored.}

\item{model.full}{\code{lm, glm, nls, survreg, coxph, hurdle, zeroinfl, gee, geeglm} or \code{lme} model object.}

\item{model.reduced}{Fitted model object of same type as model.full. By default `NULL`; the same model as the full model but only having intercept.}

\item{data}{Data.frame or object coercible to data.frame of model.full data (required for some model types).}

\item{anova}{Logical, whether to produce an Anova table with the RESI columns added. By default = `TRUE`.}

\item{coefficients}{Logical, whether to produce a coefficients (summary) table with the RESI columns added. By default = `TRUE`.}

\item{overall}{Logical, whether to produce an overall Wald test comparing full to reduced model with RESI columns added. By default = `TRUE`.}

\item{vcovfunc}{The variance estimator function for constructing the Wald test statistic. By default, sandwich::vcovHC (the robust (sandwich) variance estimator).}

\item{Anova.args}{List, additional arguments to be passed to Anova function.}

\item{vcov.args}{List, additional arguments to be passed to vcovfunc.}

\item{unbiased}{Logical, whether to use the unbiased or alternative T/Z statistic to RESI conversion. By default, `TRUE`. See details.}

\item{waldtype}{Numeric, indicates which function to use for overall Wald test. 0 (default) = lmtest::waldtest Chi-square, 1 = lmtest::waldtest F, 2 = aod::wald.test}

\item{object}{emmGrid object}

\item{model}{model used to generate emmeans}

\item{N}{sample size}
}
\value{
Returns a list containing RESI point estimates
}
\description{
This function will estimate the robust effect size (RESI) from Vandekar, Tao, & Blume (2020).
The overall RESI is estimated via a Wald test. RESI is (optionally) estimated for each factor in coefficients-style table.
RESI is (optionally) estimated for each variable/interaction in an Anova-style table
for models with existing Anova methods. This function is the building block for the \code{\link{resi}} function.
}
\details{
The Robust Effect Size Index (RESI) is an effect size measure based on M-estimators.
This function is called by \code{\link{resi}} a specified number of times to
form bootstrapped confidence intervals. Called by itself, this function will
only calculate point estimates.

The RESI, denoted as S, is applicable across many model types. It is a unitless
index and can be easily be compared across models. The RESI can also be
converted to Cohen's \emph{d} (\code{\link{S2d}}) under model homoskedasticity.

The RESI is related to the non-centrality parameter
of the test statistic. The RESI estimate is consistent for all four
(Chi-square, F, T, and Z) types of statistics used. The Chi-square and F-based
calculations rely on asymptotic theory, so they may be biased in small samples.
When possible, the T and Z statistics are used. There are two formulas for both
the T and Z statistic conversion. The first (default, unbiased = TRUE)
are based on solving the expected value of the T or Z statistic for the RESI.
The alternative is based on squaring the T or Z statistic and using the
F or Chi-square statistic conversion. Both of these methods are consistent, but
the alternative exhibits a notable amount of finite sample bias. The alternative
may be appealing because its absolute value will be equal to the RESI based on
the F or Chi-square statistic. The RESI based on the Chi-Square and F statistics
is always greater than or equal to 0. The type of statistic
used is listed with the output. See \code{\link{f2S}}, \code{\link{chisq2S}},
\code{\link{t2S}}, and \code{\link{z2S}} for more details on the formulas.

For GEE (\code{geeglm}) models, a longitudinal RESI (L-RESI) and a cross-sectional,
per-measurement RESI (CS-RESI) is estimated. The longitudinal RESI takes the
specified clustering into account, while the cross-sectional RESI is estimated
using a model where each measurement is its own cluster.
}
\section{Methods (by class)}{
\itemize{
\item \code{resi_pe(default)}: RESI point estimation

\item \code{resi_pe(glm)}: RESI point estimation for generalized linear models

\item \code{resi_pe(lm)}: RESI point estimation for linear models

\item \code{resi_pe(nls)}: RESI point estimation for nonlinear least squares models

\item \code{resi_pe(survreg)}: RESI point estimation for survreg

\item \code{resi_pe(coxph)}: RESI point estimation for coxph models

\item \code{resi_pe(hurdle)}: RESI point estimation for hurdle models

\item \code{resi_pe(zeroinfl)}: RESI point estimation for zeroinfl models

\item \code{resi_pe(lmrob)}: RESI point estimation for lmrob objects (robustbase package)

\item \code{resi_pe(glmrob)}: RESI point estimation for glmrob objects (robustbase package)

\item \code{resi_pe(geeglm)}: RESI point estimation for geeglm object

\item \code{resi_pe(glmgee)}: RESI point estimation for glmgee object

\item \code{resi_pe(gee)}: RESI point estimation for gee object

\item \code{resi_pe(lme)}: RESI point estimation for lme object

\item \code{resi_pe(lmerMod)}: RESI point estimation for lmerMod object

\item \code{resi_pe(glmmTMB)}: RESI point estimation for glmmTMB object - Gaussian only

\item \code{resi_pe(emmGrid)}: RESI point estimation for emmeans object

}}
\examples{
# This function produces point estimates for the RESI. The resi function will
# provide the same point estimates but adds confidence intervals. See resi for
# more detailed examples.

## resi_pe for a linear model
# fit linear model
mod <- lm(charges ~ region * age + bmi + sex, data = RESI::insurance)
# run resi_pe on the model
resi_pe(mod)

# if you want to have RESI estimates in the coefficient table that are equal in absolute
# value to those in the Anova table (except for those with >1 df and/or included in other
# interaction terms), you can specify unbiased = FALSE to use the alternate conversion.
resi_pe(mod, unbiased = FALSE)
}
\references{
Vandekar S, Tao R, Blume J. A Robust Effect Size Index. \emph{Psychometrika}. 2020 Mar;85(1):232-246. doi: 10.1007/s11336-020-09698-2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{simEstimatorFigures}
\alias{simEstimatorFigures}
\title{Estimator Comparison Figures: t2S/f2S vs z2S/chisq2S}
\usage{
simEstimatorFigures(
  sim.dir = "resiBootSim",
  figures.dir = NULL,
  alpha = 0.05,
  fixed.knots = FALSE
)
}
\arguments{
\item{sim.dir}{Character, directory containing simulation output with a
\code{sim_raw/} sub-directory.  Default \code{"resiBootSim"}.}

\item{figures.dir}{Character, output directory for PDFs.
Default \code{file.path(sim.dir, "figures", "estimator_compare")}.}

\item{alpha}{Numeric, nominal level (used only in figure titles). Default 0.05.}

\item{fixed.knots}{Logical. Must match the value used in the original
\code{\link{insurancePlasmodeSim}} call. Default \code{FALSE}.}
}
\value{
Invisibly returns the combined estimator-comparison \code{data.frame}.
}
\description{
Reads the per-replicate raw \code{.rds} files from one simulation directory and
produces Bias + MSE comparison figures contrasting the RESI estimators actually
used for \code{lm} models against the simpler z-statistic / chi-squared alternatives:
\itemize{
  \item \strong{Coefficients table}: \code{\link{t2S}} (used) vs \code{\link{z2S}} (alternative)
  \item \strong{Anova table}: \code{\link{f2S}} (used) vs \code{\link{chisq2S}} (alternative)
}
Only \code{lm} models are included; GLM models are skipped because z2S and
chisq2S are the natural estimators there.
}
\seealso{
\code{\link{simCompareMethodsFigures}}, \code{\link{simRecomputeSummary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S2fsq.R
\name{S2fsq}
\alias{S2fsq}
\title{Covert S to Cohen's \emph{f}^2}
\usage{
S2fsq(S)
}
\arguments{
\item{S}{Numeric,the robust effect size index.}
}
\value{
Returns an estimate of Cohen's \emph{f}^2 based on the RESI
}
\description{
Converts robust effect size index (S) to Cohen's \emph{f}^2
(effect size for multiple regression) using the formula from Vandekar, Tao, & Blume (2020).
}
\details{
The formula for the conversion is:

\eqn{ f^2 = S^2}
}
\examples{
# fit a linear regression model with continuous outcome and predictor
mod = lm(charges ~ age, data = RESI::insurance)

# obtain t value for calculating RESI
t = summary(mod)$coefficients[2, "t value"]

# calculate RESI
S = t2S(t, n = 1338, rdf = 1336)

# convert to f^2
S2fsq(S)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f2S.R
\name{f2S}
\alias{f2S}
\title{Compute the robust effect size index estimate from F-statistic}
\usage{
f2S(f, df, rdf, n)
}
\arguments{
\item{f}{The F statistic for the parameter of interest.}

\item{df}{Number of degrees of freedom of the F statistic.}

\item{rdf}{Model residual degrees of freedom.}

\item{n}{Number of independent samples.}
}
\value{
Returns a scalar or vector argument of the robust effect size index estimate.
}
\description{
This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
}
\details{
The formula for converting an F statistic to S is:

\eqn{ S = \sqrt(max(0, (f * df * (rdf - 2)/rdf - df)/n))}

The estimator is derived by setting the statistic equal to the expected value of
the test statistic and solving for S.
}
\examples{

# to obtain example F values, first fit a lm
mod = lm(charges ~ region * age + bmi + sex, data = RESI::insurance)

# run Anova, using a robust variance-covariance function
# get the F values and Df values
fs = car::Anova(mod, vcov. = sandwich::vcovHC)[1:5, "F"]
dfs = car::Anova(mod, vcov. = sandwich::vcovHC)[1:5, "Df"]

# get RESI estimates
f2S(fs, df = dfs, rdf = mod$df.residual, n = nrow(RESI::insurance))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot.R
\name{ggplot.resi}
\alias{ggplot.resi}
\title{Plotting RESI Estimates and CIs}
\usage{
\method{ggplot}{resi}(data, mapping, alpha = NULL, error.bars = TRUE, ..., environment)
}
\arguments{
\item{data}{Object of `resi`, `summary_resi`, or `anova_resi` class}

\item{mapping}{Ignored, included for consistency with `ggplot` generic}

\item{alpha}{Numeric, desired alpha level for confidence intervals}

\item{error.bars}{Logical, whether to include end caps on the confidence intervals. Default = `TRUE`}

\item{...}{Ignored}

\item{environment}{Ignored, included for consistency with `ggplot` generic}
}
\value{
Returns a ggplot of RESI point estimates
}
\description{
This function uses ggplot2 graphics to plot robust effect size (RESI) estimates and confidence intervals
from `resi`, `summary_resi`, and `anova_resi` objects.
}
\examples{
# create a resi object
resi_obj <- resi(lm(charges ~ region * age + bmi + sex, data = RESI::insurance),
nboot = 10)

# plot ANOVA table
ggplot2::ggplot(anova(resi_obj))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S2Rsq.R
\name{S2Rsq}
\alias{S2Rsq}
\title{Covert S to R^2}
\usage{
S2Rsq(S)
}
\arguments{
\item{S}{Numeric, the robust effect size index.}
}
\value{
Returns an estimate of R^2 based on the RESI
}
\description{
Converts robust effect size index (S) to R^2, the partial
coefficient of determination, using the formula from Vandekar, Tao, & Blume (2020).
}
\details{
The formula for the conversion is:

\eqn{ R^2 = S^2 / (1 + S^2)}
}
\examples{
# fit a simple linear regression with a binary predictor
mod = lm(charges ~ sex, data = RESI::insurance)

# calculate t-value
t = summary(mod)$coefficients[2, "t value"]

# calculate RESI (S)
S = t2S(t, n = 1338, rdf = 1336)

# convert S to R^2
S2Rsq(S)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/z2S.R
\name{z2S}
\alias{z2S}
\title{Compute the robust effect size index estimate from Z statistic}
\usage{
z2S(z, n, unbiased = TRUE)
}
\arguments{
\item{z}{The Z statistic for the parameter of interest.}

\item{n}{Number of independent samples.}

\item{unbiased}{Logical, whether to use unbiased or alternative estimator. See details.}
}
\value{
Returns a scalar or vector argument of the robust effect size index estimate.
}
\description{
This function computes the robust effect size index from Vandekar, Tao, & Blume (2020).
Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
}
\details{
This function computes S, the RESI, from a Z statistic. The formula for the
unbiased estimator (default) is derived by solving the expected value of the
Z statistic for S. It is unbiased and consistent.

The formula for the unbiased conversion is:

\eqn{S = Z/\sqrt(n)}

The formula for the alternative estimator is derived by squaring the Z statistic
and using the \code{\link{chisq2S}} formula. This estimator may be appealing for its
intuitive relationship to the Chi-square statistic; the absolute value of RESI estimates
using this formula will be equal to a RESI estimate using a Chi-square statistic for
the same model. However, this estimator does have finite sample bias, which is an
important consideration for the coverage of the bootstrapping that \code{resi} uses.

The formula for the alternative conversion is:

\eqn{ \sqrt(max(0, (Z^2 - 1)/n)) * sign(Z)}
}
\examples{
# to obtain example z values, first fit a glm
mod = glm(charges ~ region * age + bmi + sex, data = RESI::insurance)
# run coeftest to get z values using a robust variance-covariance function
zs = lmtest::coeftest(mod, vcov. = sandwich::vcovHC)[,'z value']

# get RESI estimates using unbiased estimator
z2S(zs, n = nrow(RESI::insurance))

# get RESI estimates usng alternative estimator
z2S(zs, n = nrow(RESI::insurance), unbiased = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{insurance}
\alias{insurance}
\title{US Health Insurance Data}
\format{
A data frame with 1338 rows and 7 variables:
\describe{
\item{age}{age of primary beneficiary in years}
\item{sex}{insurance contractor sex, male/female}
\item{bmi}{body mass index}
\item{children}{number of dependents}
\item{smoker}{smoker/non-smoker}
\item{region}{beneficiary's region of US}
\item{charges}{individual medical costs billed by health insurance}
}
}
\source{
Kaggle dataset teertha ushealthinsurancedataset. url checker thinks the link is broken.
}
\usage{
insurance
}
\description{
A dataset with 1338 observations on health insurance charges and demographic factors.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsq2S.R
\name{fsq2S}
\alias{fsq2S}
\title{Covert Cohen's \emph{f}^2 to S}
\usage{
fsq2S(fsq)
}
\arguments{
\item{fsq}{Numeric, value of Cohen's \emph{f}^2.}
}
\value{
Returns an estimate the robust effect size index
}
\description{
Converts Cohen's \emph{f}^2 to robust effect size index (S)
using the formula from Vandekar, Tao, & Blume (2020).
}
\details{
The formula for the conversion is:

\eqn{S =  \sqrt(f^2)}
}
\examples{
# consider a moderate effect size of f^2 = 0.3
fsq2S(0.3)
# This corresponds to a RESI of 0.5477226
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/d2S.R
\name{d2S}
\alias{d2S}
\title{Covert Cohen's \emph{d} to |S|}
\usage{
d2S(d, pi = 0.5)
}
\arguments{
\item{d}{Numeric, value of Cohen's \emph{d}.}

\item{pi}{Numeric, the sampling proportions.}
}
\value{
Returns an estimate the robust effect size index
}
\description{
Converts Cohen's \emph{d} robust effect size index (S) using the formula from
Vandekar, Tao, & Blume (2020).
}
\details{
The pi parameter comes from the fact that Cohen's d doesn't account
for unequal sample proportions in the population, but S does.

The default is set to a natural value 1/2, which corresponds to a case
control design, for example, where sampling proportions always are
controlled by the experimenter.

The formula to convert Cohen's \emph{d} to S is:

\eqn{S = d/\sqrt( 1/\pi + 1/ (1 - \pi))}
}
\examples{
# Consider an experiment with equal sampling proportions and a medium effect size
# corresponding to a Cohen's d of 0.5.
# convert to RESI (S)
d2S(d = 0.5)

# This corresponds to a RESI of 0.25.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S2chisq.R
\name{S2chisq}
\alias{S2chisq}
\title{Convert non-zero S to Chi-square statistic}
\usage{
S2chisq(S, df, n)
}
\arguments{
\item{S}{The value of the RESI estimate.}

\item{df}{Number of degrees of freedom of the chi-square statistic.}

\item{n}{Number of independent samples.}
}
\value{
Returns a scalar or vector argument of the Chi-square statistic.
}
\description{
Converts the robust effect size index (S) to Chi-square statistic, given that S is greater than 0.
For an S value of 0, only an upper bound on the Chi-square statistic can be computed.
Vector arguments are accepted. If different length arguments are passed they are dealt with in the usual way of R.
}
\details{
The formula for converting a RESI estimate above 0 to Chi-square statistic is:

\eqn{ chisq = n*S^2 + df}

If the RESI estimate is 0, all that is known is that the Chi-square statistic is less than or equal to the degrees of freedom.
}
\examples{
# convert S estimates with corresponding degrees of freedom to Chi-square estimates
S_ests = c(0.2, 0.4, 0.6)
dfs = c(2, 1, 3)
S2chisq(S = S_ests, df = dfs, n = 300)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{simCompareMethodsFigures}
\alias{simCompareMethodsFigures}
\title{Per-Term CI Method Comparison Figures}
\usage{
simCompareMethodsFigures(
  output.dirs = c(Bootstrap = "resiBootSim", Normal = "resiAsympNormalSim", QF =
    "resiAsympQFSim", CF = "resiAsympCFSim"),
  figures.dir = NULL,
  alpha = 0.05,
  fixed.knots = FALSE
)
}
\arguments{
\item{output.dirs}{Named character vector mapping CI method labels to their
simulation output directories. Default:
\code{c(boot = "resiBootSim", normal = "resiAsympNormalSim",
qf = "resiAsympQFSim", cf = "resiAsympCFSim")}.
Directories that do not exist are silently skipped.}

\item{figures.dir}{Character, directory where comparison figures are saved.
Default: \code{file.path(output.dirs[1], "figures", "method_comparison")}.}

\item{alpha}{Numeric, nominal CI level used for coverage reference lines.
Default 0.05.}

\item{fixed.knots}{Logical. Must match the value used in the original
\code{\link{insurancePlasmodeSim}} call. Default \code{FALSE}.}
}
\value{
Invisibly returns the combined summary \code{data.frame}. Saves
  \code{estimator_<model>_<vcov>_<table>.pdf},
  \code{compare_<model>_<vcov>_<table>.pdf}, and
  \code{covquant_<model>_<vcov>_<table>.pdf} to \code{figures.dir}.
}
\description{
Reads simulation summary tables from multiple output directories (one per CI
method) and produces PDF figures comparing CI methods across sample sizes.
For each (model type x variance estimator x table) combination two PDFs are
saved: an estimator comparison (Bias, MSE) and a CI comparison (SE
calibration, coverage, width).
}
\seealso{
\code{\link{insurancePlasmodeSim}}, \code{\link{simFigures}},
  \code{\link{simEstimatorFigures}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.resi}
\alias{plot.resi}
\title{Plotting RESI Estimates and CIs}
\usage{
\method{plot}{resi}(
  x,
  alpha = NULL,
  ycex.axis = NULL,
  yaxis.args = list(),
  automar = TRUE,
  ...
)
}
\arguments{
\item{x}{Object of `resi`, `summary_resi`, or `anova_resi` class}

\item{alpha}{Numeric, desired alpha level for confidence intervals}

\item{ycex.axis}{Numeric, scale specifically for the variable name labels}

\item{yaxis.args}{List, other arguments to be passed to \code{\link{axis}} for the y-axis}

\item{automar}{Logical, whether to automatically adjust the plotting margins to accommodate variable names. Default = `TRUE`}

\item{...}{Other graphical parameters passed to \code{\link{plot}} and \code{\link{lines}}}
}
\value{
Returns a plot of RESI point estimates
}
\description{
This function uses base graphics to plot robust effect size (RESI) estimates and confidence intervals
from `resi`, `summary_resi`, and `anova_resi` objects.
}
\details{
This function creates a forest-like plot with RESI estimates for each variable or factor.
The size of the left margin will be automatically adjusted (and returned to original after plotting)
unless `automar = FALSE`. Additional graphics parameters will be passed to the main
plot function, the confidence intervals. Arguments specifically for the y-axis (variable names)
can be specified using `yaxis.args`. To manually adjust the size of the y-axis labels without
affecting the x-axis, the user can specify a value for `ycex.axis`.
}
\examples{
# create a resi object
resi_obj <- resi(lm(charges ~ region * age + bmi + sex, data = RESI::insurance),
nboot = 10)

# plot coefficients table, changing size of labels for both axes in the usual way
plot(resi_obj, cex.axis = 0.7)

# plot ANOVA table, changing the size of just the y-axis
plot(resi_obj, ycex.axis = 0.8)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{depression}
\alias{depression}
\title{Depression Treatment Data}
\format{
A data frame with 1020 rows and 5 variables:
\describe{
\item{diagnose}{diagnosed depression severity}
\item{drug}{treatment; standard or new}
\item{id}{patient id}
\item{time}{time point of treatment}
\item{depression}{depression response at time of treatment. 1 = Normal, 0 = Abnormal}
}
}
\source{
\url{http://static.lib.virginia.edu/statlab/materials/data/depression.csv}
}
\usage{
depression
}
\description{
A longitudinal dataset comparing two treatments for depression.
}
\references{
Agresti, A. (2002). Categorical Data Analysis. Wiley, 2nd Edition.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rsq2S.R
\name{Rsq2S}
\alias{Rsq2S}
\title{Covert R^2 to S}
\usage{
Rsq2S(Rsq)
}
\arguments{
\item{Rsq}{Numeric, R^2}
}
\value{
Returns an estimate of R^2 based on the RESI
}
\description{
Converts R^2, the partial coefficient of determination, to
robust effect size index (S) using the formula from Vandekar, Tao, & Blume (2020).
}
\details{
The formula for the conversion is:

\eqn{S = \sqrt((-R^2)/(R^2 - 1))}
}
\examples{
# consider a moderate effect size of R^2 = 0.1
Rsq2S(0.1)
# this corresponds to a RESI of 0.333
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulations.R
\name{simRecomputeSummary}
\alias{simRecomputeSummary}
\title{Recompute Summary Table from Raw Simulation Output}
\usage{
simRecomputeSummary(
  output.dir = "resiBootSim",
  alpha = 0.05,
  fixed.knots = FALSE
)
}
\arguments{
\item{output.dir}{Character, directory containing simulation output.
Default \code{"resiBootSim"}.}

\item{alpha}{Numeric, CI significance level used in the original simulation.
Default 0.05.}

\item{fixed.knots}{Logical. Must match the value used in the original
\code{\link{insurancePlasmodeSim}} call so that the true RESI values are
computed from the same model formula. Default \code{FALSE}.}
}
\value{
Invisibly returns the updated summary \code{data.frame}.
  Overwrites \code{output.dir/summary_table.rds}.
}
\description{
Reads the per-replicate raw \code{.rds} files saved by
\code{\link{insurancePlasmodeSim}} and recomputes the summary metrics table
(bias, MSE, coverage, upper_coverage, lower_coverage, width).  Use this
whenever \code{.simComputeMetrics} has been updated (e.g. new metrics added)
without needing to rerun the full simulation.
}
\details{
True RESI values are re-estimated from the full \code{\link{insurance}} dataset
using the same formulae and variance functions as the original simulation.
}
\seealso{
\code{\link{insurancePlasmodeSim}}, \code{\link{simFigures}}
}
